@@ -1,22 +1,30 @@
+/*
+ * $Id: asingle.c,v 1.6.2.1 2002/03/12 11:50:54 srittau Exp $
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/file.h>
 #include <sys/time.h>
 #include <sys/param.h>
-#if BSD >= 199006
-# include <machine/endian.h>
-#else
-# include <netinet/in.h>
-#endif
-#include <strings.h>
-#include <syslog.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#include <time.h>
+#include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <atalk/adouble.h>
+#include <netatalk/endian.h>
+#include "asingle.h"
 #include "megatron.h"
 
-#define	DEBUG		0
-
 /*	String used to indicate standard input instead of a disk
 	file.  Should be a string not normally used for a file
  */
@@ -29,38 +37,11 @@
 #define NOWAY		0
 #define SURETHANG	1
 
-/*	AppleSingle Entry IDs.
- */
-#define ENTRYID_DATA		1
-#define ENTRYID_RESOURCE	2
-#define ENTRYID_NAME		3
-#define ENTRYID_COMMENT		4
-#define ENTRYID_ICN		5
-#define ENTRYID_CICN		6
-#define ENTRYID_FILEINFO	7
-#define ENTRYID_DATES		8
-#define ENTRYID_FINDERINFO	9
-#define ENTRYID_MACFINFO	10
-#define ENTRYID_PRODOSFINFO	11
-#define ENTRYID_MSDOSFINFO	12
-#define ENTRYID_AFPSHORTNAME	13
-#define ENTRYID_AFPFINFO	14
-#define ENTRYID_AFPDIRID	15
-#define ENTRYID_MAX		16
-
-#define ENTRY_INFO_SIZE	12
-
-#define HEADERSIZ	26
-
 /*	This structure holds an entry description, consisting of three
 	four byte entities.  The first is the Entry ID, the second is
 	the File Offset and the third is the Length.
  */
 
-struct entry_info {
-    long		offset;
-    long		length;
-};
 
 /*	Both input and output routines use this struct and the
 	following globals; therefore this module can only be used
@@ -68,12 +49,12 @@ struct entry_info {
  */
 struct single_file_data {
     int			filed;
-    char		path[ MAXPATHLEN ];
-    struct entry_info	entry[ ENTRYID_MAX ];
+    char		path[ MAXPATHLEN + 1];
+    struct ad_entry	entry[ ADEID_MAX ];
 } 		single;
 
 extern char	*forkname[];
-u_char		header_buf[ HEADERSIZ ];
+u_char		header_buf[ AD_HEADER_LEN ];
 
 /* 
  * single_open must be called first.  pass it a filename that is supposed
@@ -81,12 +62,11 @@ u_char		header_buf[ HEADERSIZ ];
  * somewhat initialized; single_filed is set.
  */
 
-single_open( singlefile, flags, fh )
+int single_open( singlefile, flags, fh, options )
     char		*singlefile;
-    int			flags;
+    int			flags, options;
     struct FHeader	*fh;
 {
-    int			maxlen;
     int			rc;
 
     if ( flags == O_RDONLY ) {
@@ -99,7 +79,7 @@ single_open( singlefile, flags, fh )
 	strncpy( single.path, singlefile, MAXPATHLEN );
 #if DEBUG
 	fprintf( stderr, "opened %s for read\n", single.path );
-#endif
+#endif /* DEBUG */
 	if ((( rc = single_header_test()) > 0 ) && 
 		( single_header_read( fh, rc ) == 0 )) {
 	    return( 0 );
@@ -107,6 +87,7 @@ single_open( singlefile, flags, fh )
 	single_close( KEEP );
 	return( -1 );
     }
+    return( 0 );
 }
 
 /* 
@@ -115,7 +96,7 @@ single_open( singlefile, flags, fh )
  * Otherwise, a value of -1 is returned.
  */
 
-single_close( keepflag )
+int single_close( keepflag )
     int			keepflag;
 {
     if ( keepflag == KEEP ) {
@@ -135,7 +116,7 @@ single_close( keepflag )
  * bytes of the other two forks can be read, as well.
  */
 
-single_header_read( fh, version )
+int single_header_read( fh, version )
     struct FHeader	*fh;
     int			version;
 {
@@ -144,8 +125,8 @@ single_header_read( fh, version )
  * 	the actual entries of FILEINFO, FINDERINFO, and DATES.
  */
     u_char		entry_buf[ 16 ];
-    u_long		entry_id;
-    u_long		time_seconds;
+    u_int32_t		entry_id;
+    u_int32_t		time_seconds;
     u_short		mask = 0xfcee;
     u_short		num_entries;
     int			n;
@@ -159,39 +140,38 @@ single_header_read( fh, version )
  * in the array.
  */
 
-    for ( n = 0 ; n < ENTRYID_MAX; n++ ) {
-	single.entry[ n ].offset = 0;
-	single.entry[ n ].length = 0;
+    for ( n = 0 ; n < ADEID_MAX; n++ ) {
+	single.entry[ n ].ade_off = 0;
+	single.entry[ n ].ade_len = 0;
     }
-    bcopy( (char *)&header_buf[ 24 ], (char *)&num_entries,
-	    sizeof( num_entries ));
+    memcpy( &num_entries, header_buf + 24, sizeof( num_entries ));
     num_entries = ntohs( num_entries );
 #if DEBUG >= 2
     fprintf( stderr, "The number of entries is %d\n", num_entries );
-#endif
+#endif /* DEBUG */
     for ( ; num_entries > 0 ; num_entries-- ) {
-	if ( read( single.filed, (char *)entry_buf, ENTRY_INFO_SIZE )
-		!= ENTRY_INFO_SIZE ) {
+	if ( read( single.filed, (char *)entry_buf, AD_ENTRY_LEN )
+		!= AD_ENTRY_LEN ) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
 	}
-	bcopy( (char *)entry_buf, (char *)&entry_id, sizeof( entry_id ));
+	memcpy(&entry_id,  entry_buf, sizeof( entry_id ));
 	entry_id = ntohl( entry_id );
-	bcopy( (char *)&entry_buf[ 4 ],
-		(char *)&single.entry[ entry_id ].offset,
-		sizeof( single.entry[ entry_id ].offset ));
-	single.entry[ entry_id ].offset =
-		ntohl( single.entry[ entry_id ].offset );
-	bcopy( (char *)&entry_buf[ 8 ],
-		(char *)&single.entry[ entry_id ].length,
-		sizeof( single.entry[ entry_id ].length ));
-	single.entry[ entry_id ].length =
-		ntohl( single.entry[ entry_id ].length );
+	memcpy(&single.entry[ entry_id ].ade_off,
+	       entry_buf + 4,
+	       sizeof( single.entry[ entry_id ].ade_off ));
+	single.entry[ entry_id ].ade_off =
+		ntohl( single.entry[ entry_id ].ade_off );
+	memcpy(&single.entry[ entry_id ].ade_len,
+	       entry_buf + 8,
+	       sizeof( single.entry[ entry_id ].ade_len ));
+	single.entry[ entry_id ].ade_len =
+		ntohl( single.entry[ entry_id ].ade_len );
 #if DEBUG >= 2
 	fprintf( stderr, "entry_id\t%d\n", entry_id );
-	fprintf( stderr, "\toffset\t\t%d\n", single.entry[ entry_id ].offset );
-	fprintf( stderr, "\tlength\t\t%d\n", single.entry[ entry_id ].length );
-#endif
+	fprintf( stderr, "\toffset\t\t%d\n", single.entry[ entry_id ].ade_off );
+	fprintf( stderr, "\tlength\t\t%d\n", single.entry[ entry_id ].ade_len );
+#endif /* DEBUG */
     }
 
 /*
@@ -200,21 +180,21 @@ single_header_read( fh, version )
  */
 
     if ( version == 1 ) {
-	if ( single.entry[ ENTRYID_FILEINFO ].length > 0 ) {
-	    date_entry = ENTRYID_FILEINFO;
+	if ( single.entry[ ADEID_FILEI ].ade_len > 0 ) {
+	    date_entry = ADEID_FILEI;
 	} else {
 	    date_entry = 0;
 	}
     } else if ( version == 2 ) {
-	if ( single.entry[ ENTRYID_DATES ].length > 0 ) {
-	    date_entry = ENTRYID_DATES;
+	if ( single.entry[ ADEID_FILEDATESI ].ade_len > 0 ) {
+	    date_entry = ADEID_FILEDATESI;
 	} else {
 	    date_entry = 0;
 	}
     }
 #if DEBUG
     fprintf( stderr, "date_entry = %d\n", date_entry );
-#endif
+#endif /* DEBUG */
 
 /*
  * Go through and copy all the information you can get from 
@@ -223,47 +203,45 @@ single_header_read( fh, version )
  * the right place for the first read of the data fork.
  */
  
-    if ( single.entry[ ENTRYID_NAME ].offset == 0 ) {
+    if ( single.entry[ ADEID_NAME ].ade_off == 0 ) {
 	fprintf( stderr, "%s has no name for the mac file.\n", single.path );
 	return( -1 );
     } else {
-	pos = lseek( single.filed, single.entry[ ENTRYID_NAME ].offset,
-		L_SET );
-	readlen = single.entry[ ENTRYID_NAME ].length > NAMESIZ ? NAMESIZ
-		: single.entry[ ENTRYID_NAME ].length;
+	pos = lseek( single.filed, single.entry[ ADEID_NAME ].ade_off,
+		SEEK_SET );
+	readlen = single.entry[ ADEID_NAME ].ade_len > ADEDLEN_NAME ? 
+	      ADEDLEN_NAME : single.entry[ ADEID_NAME ].ade_len;
 	if ( read( single.filed, (char *)fh->name, readlen ) != readlen ) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
 	}
     }
-    if (( single.entry[ ENTRYID_FINDERINFO ].length < 16 ) ||
-	    ( single.entry[ ENTRYID_FINDERINFO ].offset <= HEADERSIZ )) {
+    if (( single.entry[ ADEID_FINDERI ].ade_len < 16 ) ||
+	    ( single.entry[ ADEID_FINDERI ].ade_off <= AD_HEADER_LEN )) {
 	fprintf( stderr, "%s has bogus FinderInfo.\n", single.path );
 	return( -1 );
     } else {
 	pos = lseek( single.filed,
-		single.entry[ ENTRYID_FINDERINFO ].offset, L_SET );
+		single.entry[ ADEID_FINDERI ].ade_off, SEEK_SET );
 	if ( read( single.filed, (char *)entry_buf, sizeof( entry_buf )) !=
 		sizeof( entry_buf )) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
 	}
-	bcopy( (char *)&entry_buf[ FINDERIOFF_TYPE ],
-		(char *)&fh->finder_info.fdType,
+	memcpy( &fh->finder_info.fdType, entry_buf + FINDERIOFF_TYPE,	
 		sizeof( fh->finder_info.fdType ));
-	bcopy( (char *)&entry_buf[ FINDERIOFF_CREATOR ],
-		(char *)&fh->finder_info.fdCreator,
+	memcpy( &fh->finder_info.fdCreator, entry_buf + FINDERIOFF_CREATOR,
 		sizeof( fh->finder_info.fdCreator ));
-	bcopy( (char *)&entry_buf[ FINDERIOFF_FLAGS ],
-		(char *)&fh->finder_info.fdFlags,
+	memcpy( &fh->finder_info.fdFlags, entry_buf +  FINDERIOFF_FLAGS,
 		sizeof( fh->finder_info.fdFlags ));
 	fh->finder_info.fdFlags = fh->finder_info.fdFlags & mask;
-	bcopy( (char *)&entry_buf[ FINDERIOFF_LOC ],
-		(char *)&fh->finder_info.fdLocation,
+	memcpy( &fh->finder_info.fdLocation, entry_buf + FINDERIOFF_LOC,
 		sizeof( fh->finder_info.fdLocation ));
-	bcopy( (char *)&entry_buf[ FINDERIOFF_FLDR ],
-		(char *)&fh->finder_info.fdFldr,
+	memcpy(&fh->finder_info.fdFldr, entry_buf + FINDERIOFF_FLDR,
 		sizeof( fh->finder_info.fdFldr ));
+	fh->finder_xinfo.fdScript = *(entry_buf + FINDERIOFF_SCRIPT);
+	fh->finder_xinfo.fdXFlags = *(entry_buf + FINDERIOFF_XFLAGS);
+
 #if DEBUG
 	{
 	    char		type[5];
@@ -271,19 +249,19 @@ single_header_read( fh, version )
  
 	    strncpy( type, &fh->finder_info.fdType, 4 );
 	    strncpy( creator, &fh->finder_info.fdCreator, 4 );
-	    type[5] = creator[5] = '\0';
+	    type[4] = creator[4] = '\0';
 	    fprintf( stderr, "type is %s, creator is %s\n", type, creator );
 	}
-#endif
+#endif /* DEBUG */
     }
-    if (( single.entry[ ENTRYID_COMMENT ].length == 0 ) || 
-	    ( single.entry[ ENTRYID_COMMENT ].offset <= HEADERSIZ )) {
+    if (( single.entry[ ADEID_COMMENT ].ade_len == 0 ) || 
+	    ( single.entry[ ADEID_COMMENT ].ade_off <= AD_HEADER_LEN )) {
 	fh->comment[0] = '\0';
     } else {
-	pos = lseek( single.filed, single.entry[ ENTRYID_COMMENT ].offset,
-		L_SET );
-	readlen = single.entry[ ENTRYID_COMMENT ].length > COMMENTSIZ 
-		? COMMENTSIZ : single.entry[ ENTRYID_COMMENT ].length;
+	pos = lseek( single.filed, single.entry[ ADEID_COMMENT ].ade_off,
+		SEEK_SET );
+	readlen = single.entry[ ADEID_COMMENT ].ade_len > ADEDLEN_COMMENT
+		? ADEDLEN_COMMENT : single.entry[ ADEID_COMMENT ].ade_len;
 	if ( read( single.filed, (char *)fh->comment, readlen ) != readlen ) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
@@ -297,61 +275,57 @@ single_header_read( fh, version )
  */
     if (( date_entry < 7 ) || ( date_entry > 8 )) {
 	if (( time_seconds = time( NULL )) == -1 ) {
-	    time_seconds = htonl( TIME_ZERO );
+	    time_seconds = AD_DATE_START;
 	} else {
-	    time_seconds = htonl( time_seconds + TIME_DIFF );
+	    time_seconds = AD_DATE_FROM_UNIX(time_seconds);
 	}
-	bcopy( (char *)&time_seconds, (char *)&fh->create_date,
-		sizeof( fh->create_date ));
-	bcopy( (char *)&time_seconds, (char *)&fh->mod_date,
-		sizeof( fh->mod_date ));
-	fh->backup_date = htonl( TIME_ZERO );
-    } else if ( single.entry[ date_entry ].length != 16 ) {
+	memcpy(&fh->create_date, &time_seconds, sizeof( fh->create_date ));
+	memcpy(&fh->mod_date, &time_seconds, sizeof( fh->mod_date ));
+	fh->backup_date = AD_DATE_START;
+    } else if ( single.entry[ date_entry ].ade_len != 16 ) {
 	fprintf( stderr, "%s has bogus FileInfo or File Dates Info.\n", 
 		single.path );
 	return( -1 );
-    } else if ( date_entry == ENTRYID_FILEINFO ) {
+    } else if ( date_entry == ADEID_FILEI ) {
 	pos = lseek( single.filed,
-		single.entry[ date_entry ].offset, L_SET );
+		single.entry[ date_entry ].ade_off, SEEK_SET );
 	if ( read( single.filed, (char *)entry_buf, sizeof( entry_buf )) !=
 		sizeof( entry_buf )) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
 	}
-	bcopy( (char *)&entry_buf[ 0 ], (char *)&fh->create_date,
+	memcpy( &fh->create_date, entry_buf + FILEIOFF_CREATE,
 		sizeof( fh->create_date ));
-	bcopy( (char *)&entry_buf[ 4 ], (char *)&fh->mod_date,
+	memcpy( &fh->mod_date, entry_buf + FILEIOFF_MODIFY,
 		sizeof( fh->mod_date ));
-	fh->backup_date = htonl( TIME_ZERO );
-    } else if ( date_entry == ENTRYID_DATES ) {
-	time_seconds = (u_long)946684800L + (u_long)TIME_DIFF;
+	memcpy( &fh->backup_date, entry_buf + FILEIOFF_BACKUP,
+		sizeof(fh->backup_date));
+    } else if ( date_entry == ADEID_FILEDATESI ) {
 	pos = lseek( single.filed,
-		single.entry[ date_entry ].offset, L_SET );
+		single.entry[ date_entry ].ade_off, SEEK_SET );
 	if ( read( single.filed, (char *)entry_buf, sizeof( entry_buf )) !=
 		sizeof( entry_buf )) {
 	    perror( "Premature end of file :" );
 	    return( -1 );
 	}
-	bcopy( (char *)&entry_buf[ 0 ], (char *)&fh->create_date,
+	memcpy( &fh->create_date, entry_buf + FILEIOFF_CREATE,
 		sizeof( fh->create_date ));
-	fh->create_date = htonl( time_seconds - 
-		( 0L - ntohl( fh->create_date )));
-	bcopy( (char *)&entry_buf[ 4 ], (char *)&fh->mod_date,
+	memcpy( &fh->mod_date, entry_buf + FILEIOFF_MODIFY,
 		sizeof( fh->mod_date ));
-	fh->mod_date = htonl( time_seconds - ( 0L - ntohl( fh->mod_date )));
-	fh->backup_date = htonl( TIME_ZERO );
+	memcpy( &fh->backup_date, entry_buf + FILEIOFF_BACKUP,
+		sizeof(fh->backup_date));
     }
-    if ( single.entry[ ENTRYID_RESOURCE ].offset == 0 ) {
-	fh->forklen[ RESOURCE ] = 0;
+    if ( single.entry[ ADEID_RFORK ].ade_off == 0 ) {
+	fh->forklen[ ADEID_RFORK ] = 0;
     } else {
-	fh->forklen[ RESOURCE ] =
-		htonl( single.entry[ ENTRYID_RESOURCE ].length );
+	fh->forklen[ ADEID_RFORK ] =
+		htonl( single.entry[ ADEID_RFORK ].ade_len );
     }
-    if ( single.entry[ ENTRYID_DATA ].offset == 0 ) {
+    if ( single.entry[ ADEID_DFORK ].ade_off == 0 ) {
 	fh->forklen[ DATA ] = 0;
     } else {
-	fh->forklen[ DATA ] = htonl( single.entry[ ENTRYID_DATA ].length );
-	pos = lseek( single.filed, single.entry[ ENTRYID_DATA ].offset, L_SET );
+	fh->forklen[ DATA ] = htonl( single.entry[ ADEID_DFORK ].ade_len );
+	pos = lseek( single.filed, single.entry[ ADEID_DFORK ].ade_off, SEEK_SET );
     }
 
     return( 0 );
@@ -370,19 +344,14 @@ single_header_read( fh, version )
  * contain hex 0x4d6163696e746f736820202020202020 which is ASCII
  * "Macintosh       " (that is seven blanks of padding).
  */
-#define ASMAGIC		0x00051600L
-#define VERSIONONE	0x00010000L
-#define VERSIONTWO	0x00020000L
 #define MACINTOSH	"Macintosh       "
 u_char		sixteennulls[] = { 0, 0, 0, 0, 0, 0, 0, 0,
 				    0, 0, 0, 0, 0, 0, 0, 0 };
 
-single_header_test()
+int single_header_test(void)
 {
     int			cc;
-    u_long		templong;
-    u_short		header_crc;
-    u_char		namelen;
+    u_int32_t		templong;
 
     cc = read( single.filed, (char *)header_buf, sizeof( header_buf ));
     if ( cc < sizeof( header_buf )) {
@@ -390,25 +359,25 @@ single_header_test()
 	return( -1 );
     }
 
-    bcopy( (char *)&header_buf[ 0 ], (char *)&templong, sizeof( templong ));
-    if ( ntohl( templong ) != ASMAGIC ) {
+    memcpy( &templong, header_buf, sizeof( templong ));
+    if ( ntohl( templong ) != AD_APPLESINGLE_MAGIC ) {
 	fprintf( stderr, "%s is not an AppleSingle file.\n", single.path );
 	return( -1 );
     }
 
-    bcopy( (char *)&header_buf[ 4 ], (char *)&templong, sizeof( templong ));
+    memcpy(&templong,  header_buf +  4, sizeof( templong ));
     templong = ntohl( templong );
-    if ( templong == VERSIONONE ) {
+    if ( templong == AD_VERSION1 ) {
 	cc = 1;
-	if ( bcmp( MACINTOSH, &header_buf[ 8 ], sizeof( MACINTOSH ) - 1 ) 
+	if ( memcmp( MACINTOSH, header_buf + 8, sizeof( MACINTOSH ) - 1 ) 
 		!= 0 ) {
 	    fprintf( stderr, "%s is not a Macintosh AppleSingle file.\n", 
 		    single.path );
 	    return( -1 );
 	}
-    } else if ( templong == VERSIONTWO ) {
+    } else if ( templong == AD_VERSION2 ) {
 	cc = 2;
-	if ( bcmp( sixteennulls, &header_buf[ 8 ], sizeof( sixteennulls ))
+	if ( memcmp( sixteennulls, header_buf + 8, sizeof( sixteennulls ))
 		!= 0 ) {
 	    fprintf( stderr, 
 		    "Warning:  %s may be a corrupt AppleSingle file.\n",
@@ -432,12 +401,12 @@ single_header_test()
  *
  */
 
-single_read( fork, buffer, length )
+int single_read( fork, buffer, length )
     int			fork;
     char		*buffer;
     int			length;
 {
-    u_long		entry_id;
+    u_int32_t		entry_id;
     char		*buf_ptr;
     int			readlen;
     int			cc = 1;
@@ -445,30 +414,30 @@ single_read( fork, buffer, length )
 
     switch ( fork ) {
 	case DATA :
-	    entry_id = ENTRYID_DATA;
+	    entry_id = ADEID_DFORK;
 	    break;
 	case RESOURCE :
-	    entry_id = ENTRYID_RESOURCE;
+	    entry_id = ADEID_RFORK;
 	    break;
 	default :
 	    return( -1 );
 	    break;
     }
 
-    if ( single.entry[ entry_id ].length < 0 ) {
+    if ( single.entry[ entry_id ].ade_len < 0 ) {
 	fprintf( stderr, "single_read: Trying to read past end of fork!\n" );
-	return( single.entry[ entry_id ].length );
+	return( single.entry[ entry_id ].ade_len );
     }
-    if ( single.entry[ entry_id ].length == 0 ) {
+    if ( single.entry[ entry_id ].ade_len == 0 ) {
 	if ( fork == DATA ) {
 	    pos = lseek( single.filed,
-		single.entry[ ENTRYID_RESOURCE ].offset, L_SET );
+		single.entry[ ADEID_RFORK ].ade_off, SEEK_SET );
 	}
 	return( 0 );
     }
 
-    if ( single.entry[ entry_id ].length < length ) {
-	readlen = single.entry[ entry_id ].length;
+    if ( single.entry[ entry_id ].ade_len < length ) {
+	readlen = single.entry[ entry_id ].ade_len;
     } else {
 	readlen = length;
     }
@@ -482,7 +451,7 @@ single_read( fork, buffer, length )
     }
     if ( cc >= 0 ) {
 	cc = buf_ptr - buffer;
-	single.entry[ entry_id ].length -= cc;
+	single.entry[ entry_id ].ade_len -= cc;
     }
 
     return( cc );
