@@ -1,21 +1,35 @@
+/*
+ * $Id: hqx.c,v 1.8.2.1 2002/03/12 14:12:40 srittau Exp $
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/file.h>
 #include <sys/time.h>
 #include <sys/param.h>
-#if BSD >= 199006
-# include <machine/endian.h>
-#else
-# include <netinet/in.h>
-#endif
-#include <strings.h>
-#include <syslog.h>
+
+#include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
+
+#include <unistd.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+
+#include <netinet/in.h>
+
 #include <atalk/adouble.h>
+#include <netatalk/endian.h>
+
 #include "megatron.h"
+#include "hqx.h"
 
-#define	DEBUG		0
 #define HEXOUTPUT	0
 
 /*	String used to indicate standard input instead of a disk
@@ -23,7 +37,7 @@
  */
 #ifndef	STDIN
 #	define	STDIN	"-"
-#endif
+#endif /* ! STDIN */
 
 /*	Yes and no
  */
@@ -42,25 +56,33 @@
 /*	These are field sizes in bytes of various pieces of the
 	binhex header
  */
-#define	VERSION		1
-#define	TCSIZ		8
-#define	FLAGSIZ		2
-#define	DATASIZ		4
-#define	RESSIZ		4
-#define CRCSIZ		2
-#define HEADSIZ		21
+#define	BHH_VERSION		1
+#define	BHH_TCSIZ		8
+#define	BHH_FLAGSIZ		2
+#define	BHH_DATASIZ		4
+#define	BHH_RESSIZ		4
+#define BHH_CRCSIZ		2
+#define BHH_HEADSIZ		21
 
-char		*malloc();
 u_short		updcrc();
 
+/*	Forward declarations.
+ */
+int skip_junk(int line);
+int hqx_close(int keepflag);
+int hqx_header_read(struct FHeader *fh);
+int hqx_header_write(struct FHeader *fh);
+int hqx_7tobin(char *outbuf, int datalen);
+int hqx7_fill(u_char *hqx7_ptr);
+
 #if HEXOUTPUT
 FILE		*rawhex, *expandhex;
-#endif
+#endif /* HEXOUTPUT */
 
 struct hqx_file_data {
-    long		forklen[ NUMFORKS ];
+    u_int32_t		forklen[ NUMFORKS ];
     u_short		forkcrc[ NUMFORKS ];
-    char		path[ MAXPATHLEN ];
+    char		path[ MAXPATHLEN + 1];
     u_short		headercrc;
     int			filed;
 } 		hqx;
@@ -78,22 +100,22 @@ somewhat initialized; hqx_fd is set.  skip_junk is called from
 here; skip_junk leaves hqx7_first and hqx7_last set.
  */
 
-hqx_open( hqxfile, flags, fh )
+int hqx_open( hqxfile, flags, fh, options )
     char		*hqxfile;
-    int			flags;
+    int			flags, options;
     struct FHeader	*fh;
 {
     int			maxlen;
 
 #if DEBUG
     fprintf( stderr, "megatron: entering hqx_open\n" );
-#endif
+#endif /* DEBUG */
     if ( flags == O_RDONLY ) {
 
 #if HEXOUTPUT
 	rawhex = fopen( "rawhex.unhex", "w" );
 	expandhex = fopen( "expandhex.unhex", "w" );
-#endif
+#endif /* HEXOUTPUT */
 
 	first_flag = 0;
 
@@ -109,9 +131,9 @@ hqx_open( hqxfile, flags, fh )
 #if DEBUG
 		off_t	pos;
 
-		pos = lseek( hqx.filed, 0, L_INCR );
+		pos = lseek( hqx.filed, 0, SEEK_CUR );
 		fprintf( stderr, "megatron: current position is %ld\n", pos );
-#endif
+#endif /* DEBUG */
 		return( 0 );
 	    }
 	}
@@ -142,7 +164,7 @@ hqx_open( hqxfile, flags, fh )
  * Otherwise, a value of -1 is returned.
  */
 
-hqx_close( keepflag )
+int hqx_close( keepflag )
     int			keepflag;
 {
     if ( keepflag == KEEP ) {
@@ -164,7 +186,7 @@ hqx_close( keepflag )
  * return zero and no more than that.
  */
 
-hqx_read( fork, buffer, length )
+int hqx_read( fork, buffer, length )
     int			fork;
     char		*buffer;
     int			length;
@@ -176,12 +198,12 @@ hqx_read( fork, buffer, length )
 #if DEBUG >= 3
     {
 	off_t	pos;
-	pos = lseek( hqx.filed, 0, L_INCR );
+	pos = lseek( hqx.filed, 0, SEEK_CUR );
 	fprintf( stderr, "hqx_read: current position is %ld\n", pos );
     }
     fprintf( stderr, "hqx_read: fork is %s\n", forkname[ fork ] );
     fprintf( stderr, "hqx_read: remaining length is %d\n", hqx.forklen[fork] );
-#endif
+#endif /* DEBUG >= 3 */
 
     if ( hqx.forklen[ fork ] < 0 ) {
 	fprintf( stderr, "This should never happen, dude!\n" );
@@ -195,7 +217,7 @@ hqx_read( fork, buffer, length )
 #if DEBUG >= 4
     fprintf( stderr, "hqx_read: storedcrc\t\t%x\n", storedcrc );
     fprintf( stderr, "hqx_read: observed crc\t\t%x\n\n", hqx.forkcrc[fork] );
-#endif
+#endif /* DEBUG >= 4 */
 	    if ( storedcrc == hqx.forkcrc[ fork ] ) {
 		return( 0 );
 	    }
@@ -212,7 +234,7 @@ hqx_read( fork, buffer, length )
     }
 #if DEBUG >= 3
     fprintf( stderr, "hqx_read: readlen is %d\n", readlen );
-#endif
+#endif /* DEBUG >= 3 */
 
     cc = hqx_7tobin( buffer, readlen );
     if ( cc > 0 ) {
@@ -222,7 +244,7 @@ hqx_read( fork, buffer, length )
     }
 #if DEBUG >= 3
     fprintf( stderr, "hqx_read: chars read is %d\n", cc );
-#endif
+#endif /* DEBUG >= 3 */
     return( cc );
 }
 
@@ -234,11 +256,11 @@ hqx_read( fork, buffer, length )
  * to fill the hqx_header fields.
  */
 
-hqx_header_read( fh )
+int hqx_header_read( fh )
     struct FHeader	*fh;
 {
     char		*headerbuf, *headerptr;
-    u_long		time_seconds;
+    u_int32_t		time_seconds;
     u_short		mask;
     u_short		header_crc;
     char		namelen;
@@ -246,7 +268,7 @@ hqx_header_read( fh )
 #if HEXOUTPUT
     int		headerfork;
     headerfork = open( "headerfork", O_WRONLY|O_CREAT, 0622 );
-#endif
+#endif /* HEXOUTPUT */
 
     mask = htons( 0xfcee );
     hqx.headercrc = 0;
@@ -260,45 +282,45 @@ hqx_header_read( fh )
 
 #if HEXOUTPUT
     write( headerfork, &namelen, sizeof( namelen ));
-#endif
+#endif /* HEXOUTPUT */
 
     if (( headerbuf = 
-	    (char *)malloc( (unsigned int)( namelen + HEADSIZ ))) == 0 ) {
+	    (char *)malloc( (unsigned int)( namelen + BHH_HEADSIZ ))) == 0 ) {
 	return( -1 );
     }
-    if ( hqx_7tobin( headerbuf, ( namelen + HEADSIZ )) == 0 ) {
+    if ( hqx_7tobin( headerbuf, ( namelen + BHH_HEADSIZ )) == 0 ) {
 	free( headerbuf );
 	fprintf( stderr, "Premature end of file :" );
 	return( -2 );
     }
     headerptr = headerbuf;
     hqx.headercrc = updcrc( hqx.headercrc, 
-	    (u_char *)headerbuf, ( namelen + HEADSIZ - CRCSIZ ));
+	    (u_char *)headerbuf, ( namelen + BHH_HEADSIZ - BHH_CRCSIZ ));
 
 #if HEXOUTPUT
-    write( headerfork, headerbuf, ( namelen + HEADSIZ ));
-#endif
+    write( headerfork, headerbuf, ( namelen + BHH_HEADSIZ ));
+#endif /* HEXOUTPUT */
 
 /*
  * stuff from the hqx file header
  */
 
-    bcopy( headerptr, fh->name, (int)namelen );
+    memcpy( fh->name, headerptr, (int)namelen );
     headerptr += namelen;
-    headerptr += VERSION;
-    bcopy( headerptr, (char *)&fh->finder_info, TCSIZ );
-    headerptr += TCSIZ;
-    bcopy( headerptr, (char *)&fh->finder_info.fdFlags, FLAGSIZ );
+    headerptr += BHH_VERSION;
+    memcpy(&fh->finder_info,  headerptr, BHH_TCSIZ );
+    headerptr += BHH_TCSIZ;
+    memcpy(&fh->finder_info.fdFlags,  headerptr, BHH_FLAGSIZ );
     fh->finder_info.fdFlags = fh->finder_info.fdFlags & mask;
-    headerptr += FLAGSIZ;
-    bcopy( headerptr, (char *)&fh->forklen[ DATA ], DATASIZ );
+    headerptr += BHH_FLAGSIZ;
+    memcpy(&fh->forklen[ DATA ],  headerptr, BHH_DATASIZ );
     hqx.forklen[ DATA ] = ntohl( fh->forklen[ DATA ] );
-    headerptr += DATASIZ;
-    bcopy( headerptr, (char *)&fh->forklen[ RESOURCE ], RESSIZ );
+    headerptr += BHH_DATASIZ;
+    memcpy( &fh->forklen[ RESOURCE ], headerptr, BHH_RESSIZ );
     hqx.forklen[ RESOURCE ] = ntohl( fh->forklen[ RESOURCE ] );
-    headerptr += RESSIZ;
-    bcopy( headerptr, (char *)&header_crc, CRCSIZ );
-    headerptr += CRCSIZ;
+    headerptr += BHH_RESSIZ;
+    memcpy(&header_crc,  headerptr, BHH_CRCSIZ );
+    headerptr += BHH_CRCSIZ;
     header_crc = ntohs( header_crc );
 
 /*
@@ -306,8 +328,8 @@ hqx_header_read( fh )
  */
 
     fh->comment[0] = '\0';
-    bzero( &fh->finder_info.fdLocation, sizeof( fh->finder_info.fdLocation ));
-    bzero( &fh->finder_info.fdFldr, sizeof( fh->finder_info.fdFldr ));
+    fh->finder_info.fdLocation = 0;
+    fh->finder_info.fdFldr = 0;
 
 #if DEBUG >= 5
     {
@@ -322,8 +344,7 @@ hqx_header_read( fh )
 	fprintf( stderr, "creator\t\t\t%.*s\n", 
 		sizeof( fh->finder_info.fdCreator ), 
 		&fh->finder_info.fdCreator );
-	bcopy( (char *)&fh->finder_info.fdFlags, (char *)&flags, 
-		sizeof( flags ));
+	memcpy( &flags, &fh->finder_info.fdFlags, sizeof( flags ));
 	flags = ntohs( flags );
 	fprintf( stderr, "flags\t\t\t%x\n", flags );
 	fprintf( stderr, "data fork length\t%ld\n", hqx.forklen[DATA] );
@@ -332,26 +353,27 @@ hqx_header_read( fh )
 	fprintf( stderr, "observed crc\t\t%x\n", hqx.headercrc );
 	fprintf( stderr, "\n" );
     }
-#endif
+#endif /* DEBUG >= 5 */
 
 /*
  * create and modify times are figured from right now
  */
 
-    time_seconds = htonl( time( NULL ) + TIME_DIFF );
-    bcopy( (char *)&time_seconds, (char *)&fh->create_date, 
+    time_seconds = AD_DATE_FROM_UNIX(time( NULL ));
+    memcpy( &fh->create_date, &time_seconds, 
 	    sizeof( fh->create_date ));
-    bcopy( (char *)&time_seconds, (char *)&fh->mod_date, 
+    memcpy( &fh->mod_date, &time_seconds, 
 	    sizeof( fh->mod_date ));
-    fh->backup_date = htonl( TIME_ZERO );
+    fh->backup_date = AD_DATE_START;
 
 /*
  * stuff that should be zero'ed out
  */
 
     fh->comment[0] = '\0';
-    bzero( &fh->finder_info.fdLocation, sizeof( fh->finder_info.fdLocation ));
-    bzero( &fh->finder_info.fdFldr, sizeof( fh->finder_info.fdFldr ));
+    memset( &fh->finder_info.fdLocation, 0, 
+	    sizeof( fh->finder_info.fdLocation ));
+    memset( &fh->finder_info.fdFldr, 0, sizeof( fh->finder_info.fdFldr ));
 
     hqx.forkcrc[ DATA ] = 0;
     hqx.forkcrc[ RESOURCE ] = 0;
@@ -368,7 +390,7 @@ hqx_header_read( fh )
  * hqx_header_write.
  */
 
-hqx_header_write( fh )
+int hqx_header_write( fh )
     struct FHeader	*fh;
 {
     return( -1 );
@@ -381,7 +403,7 @@ hqx_header_write( fh )
  * it sets the pointers to the hqx7 buffer up to point to the valid data.
  */
 
-hqx7_fill( hqx7_ptr )
+int hqx7_fill( hqx7_ptr )
     u_char		*hqx7_ptr;
 {
     int			cc;
@@ -452,7 +474,7 @@ u_char hqxlookup[] = {
  * OTHER when looking for any subsequent line.
  */
 
-skip_junk( line )
+int skip_junk( line )
 int			line;
 {
     int			found = NOWAY;
@@ -531,7 +553,7 @@ int			line;
 	    if ( line == FIRST ) {
 		*hqx7_buf = ':';
 	    } else *hqx7_buf = '\n';
-	    bcopy( (char *)hqx7_first, (char *)&hqx7_buf[1], nc );
+	    memcpy(hqx7_buf + 1, hqx7_first, nc );
 	    hqx7_first = hqx7_buf + ( ++nc );
 	    if ( hqx7_fill( hqx7_first ) <= 0 ) {
 		fprintf( stderr, "Premature end of file :" );
@@ -554,25 +576,25 @@ int			line;
  * file is reached.
  */
 
-hqx_7tobin( outbuf, datalen ) 
+int hqx_7tobin( outbuf, datalen ) 
     char		*outbuf;
     int			datalen;
 {
     static u_char	hqx8[3];
-    static char		hqx8i;
+    static int		hqx8i;
     static u_char	prev_hqx8;
     static u_char	prev_out;
     static u_char	prev_hqx7;
     static int		eofflag;
     u_char		hqx7[4];
-    char		hqx7i = 0;
+    int			hqx7i = 0;
     char		*out_first;
     char		*out_last;
 
 #if DEBUG
     fprintf( stderr, "hqx_7tobin: datalen entering %d\n", datalen );
     fprintf( stderr, "hqx_7tobin: hqx8i entering %d\n", hqx8i );
-#endif
+#endif /* DEBUG */
 
     if ( first_flag == 0 ) {
 	prev_hqx8 = 0;
@@ -585,7 +607,7 @@ hqx_7tobin( outbuf, datalen )
 
 #if DEBUG
     fprintf( stderr, "hqx_7tobin: hqx8i entering %d\n", hqx8i );
-#endif
+#endif /* DEBUG */
 
     out_first = outbuf;
     out_last = out_first + datalen;
@@ -645,14 +667,14 @@ hqx_7tobin( outbuf, datalen )
 #if HEXOUTPUT
 	    putc( hqx8i, rawhex );
             putc( hqx8[ hqx8i ], rawhex );
-#endif
+#endif /* HEXOUTPUT */
 
 	    if ( prev_hqx8 == RUNCHAR ) {
 		if ( hqx8[ hqx8i ] == 0 ) {
 		    *out_first = prev_hqx8;
 #if HEXOUTPUT
 		    putc( *out_first, expandhex );
-#endif
+#endif /* HEXOUTPUT */
 		    prev_out = prev_hqx8;
 		    out_first++;
 		}
@@ -660,7 +682,7 @@ hqx_7tobin( outbuf, datalen )
 		    *out_first = prev_out;
 #if HEXOUTPUT
 		    putc( *out_first, expandhex );
-#endif
+#endif /* HEXOUTPUT */
 		    hqx8[ hqx8i ]--;
 		    out_first++;
 		}
@@ -676,7 +698,7 @@ hqx_7tobin( outbuf, datalen )
 		*out_first = prev_hqx8;
 #if HEXOUTPUT
 		putc( *out_first, expandhex );
-#endif
+#endif /* HEXOUTPUT */
 		prev_out = prev_hqx8;
 		out_first++;
 	    }
