@@ -1,28 +1,43 @@
+/*
+ * $Id: macbin.c,v 1.8.2.1 2002/03/12 13:39:55 srittau Exp $
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/file.h>
 #include <sys/time.h>
 #include <sys/param.h>
-#if BSD >= 199006
-# include <machine/endian.h>
-#else
-# include <netinet/in.h>
-#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#include <string.h>
 #include <strings.h>
-#include <syslog.h>
 #include <ctype.h>
 #include <stdio.h>
-#include <atalk/adouble.h>
-#include "megatron.h"
+#include <time.h>
 
-#define	DEBUG		0
+#include <atalk/adouble.h>
+#include <netatalk/endian.h>
+#include "megatron.h"
+#include "macbin.h"
+
+/* This allows megatron to generate .bin files that won't choke other
+   well-known converter apps. It also makes sure that checksums
+   always match. (RLB) */
+#define MACBINARY_PLAY_NICE_WITH_OTHERS
 
 /*	String used to indicate standard input instead of a disk
 	file.  Should be a string not normally used for a file
  */
 #ifndef	STDIN
 #	define	STDIN	"-"
-#endif
+#endif /* STDIN */
 
 /*	Yes and no
  */
@@ -33,17 +48,16 @@
  */
 #define HEADBUFSIZ	128
 
-u_short		updcrc();
-
 /*	Both input and output routines use this struct and the
 	following globals; therefore this module can only be used
 	for one of the two functions at a time.
  */
 struct bin_file_data {
-    long		forklen[ NUMFORKS ];
-    char		path[ MAXPATHLEN ];
+    u_int32_t		forklen[ NUMFORKS ];
+    char		path[ MAXPATHLEN + 1];
     int			filed;
     u_short		headercrc;
+    time_t              gmtoff; /* to convert from/to localtime */
 } 		bin;
 
 extern char	*forkname[];
@@ -55,19 +69,30 @@ u_char		head_buf[HEADBUFSIZ];
  * somewhat initialized; bin_filed is set.
  */
 
-bin_open( binfile, flags, fh )
+int bin_open( binfile, flags, fh, options )
     char		*binfile;
-    int			flags;
+    int			flags, options;
     struct FHeader	*fh;
 {
     int			maxlen;
     int			rc;
+    time_t              t;
+    struct tm           *tp;
 
 #if DEBUG
     fprintf( stderr, "entering bin_open\n" );
-#endif
+#endif /* DEBUG */
 
-    if ( flags == O_RDONLY ) {
+    /* call localtime so that we get the timezone offset */
+    bin.gmtoff = 0;
+#ifndef NO_STRUCT_TM_GMTOFF
+    time(&t);
+    tp = localtime(&t);
+    if (tp)
+        bin.gmtoff = tp->tm_gmtoff;
+#endif /* ! NO_STRUCT_TM_GMTOFF */
+
+    if ( flags == O_RDONLY ) { /* input */
 	if ( strcmp( binfile, STDIN ) == 0 ) {
 	    bin.filed = fileno( stdin );
 	} else if (( bin.filed = open( binfile, flags )) < 0 ) {
@@ -76,30 +101,35 @@ bin_open( binfile, flags, fh )
 	}
 #if DEBUG
 	fprintf( stderr, "opened %s for read\n", binfile );
-#endif
+#endif /* DEBUG */
 	if ((( rc = test_header() ) > 0 ) && 
 		( bin_header_read( fh, rc ) == 0 )) {
 	    return( 0 );
 	}
-	bin_close( TRASH );
-	fprintf( stderr, "%s\n", binfile );
+	fprintf( stderr, "%s is not a macbinary file.\n", binfile );
 	return( -1 );
-    } else {
-	maxlen = sizeof( bin.path ) - 1;
+    } else { /* output */
+        if (options & OPTION_STDOUT) 
+	  bin.filed = fileno(stdout);
+	else {
+	  maxlen = sizeof( bin.path ) - 1;
 #if DEBUG
-	fprintf( stderr, "sizeof bin.path\t\t\t%d\n", sizeof( bin.path ));
-	fprintf( stderr, "maxlen \t\t\t\t%d\n", maxlen );
-#endif
-	strncpy( bin.path, fh->name, maxlen );
-	strncpy( bin.path, mtoupath( bin.path ), maxlen );
-	strncat( bin.path, ".bin", maxlen - strlen( bin.path ));
-	if (( bin.filed = open( bin.path, flags, 0666 )) < 0 ) {
+	  fprintf( stderr, "sizeof bin.path\t\t\t%d\n", sizeof( bin.path ));
+	  fprintf( stderr, "maxlen \t\t\t\t%d\n", maxlen );
+#endif /* DEBUG */
+	  strncpy( bin.path, fh->name, maxlen );
+	  strncpy( bin.path, mtoupath( bin.path ), maxlen );
+	  strncat( bin.path, ".bin", maxlen - strlen( bin.path ));
+	  if (( bin.filed = open( bin.path, flags, 0666 )) < 0 ) {
 	    perror( bin.path );
 	    return( -1 );
-	}
+	  }
 #if DEBUG
-	fprintf( stderr, "opened %s for write\n", bin.path );
-#endif
+	  fprintf( stderr, "opened %s for write\n", 
+		   (options & OPTION_STDOUT) ? "(stdout)" : bin.path );
+#endif /* DEBUG */
+	}
+
 	if ( bin_header_write( fh ) != 0 ) {
 	    bin_close( TRASH );
 	    fprintf( stderr, "%s\n", bin.path );
@@ -115,12 +145,12 @@ bin_open( binfile, flags, fh )
  * Otherwise, a value of -1 is returned.
  */
 
-bin_close( keepflag )
+int bin_close( keepflag )
     int			keepflag;
 {
 #if DEBUG
     fprintf( stderr, "entering bin_close\n" );
-#endif
+#endif /* DEBUG */
     if ( keepflag == KEEP ) {
 	return( close( bin.filed ));
     } else if ( keepflag == TRASH ) {
@@ -140,7 +170,7 @@ bin_close( keepflag )
  * return zero and no more than that.
  */
 
-bin_read( fork, buffer, length )
+int bin_read( fork, buffer, length )
     int			fork;
     char		*buffer;
     int			length;
@@ -153,7 +183,7 @@ bin_read( fork, buffer, length )
 #if DEBUG >= 3
     fprintf( stderr, "bin_read: fork is %s\n", forkname[ fork ] );
     fprintf( stderr, "bin_read: remaining length is %d\n", bin.forklen[fork] );
-#endif
+#endif /* DEBUG >= 3 */
 
     if ( bin.forklen[ fork ] < 0 ) {
 	fprintf( stderr, "This should never happen, dude!\n" );
@@ -162,16 +192,17 @@ bin_read( fork, buffer, length )
 
     if ( bin.forklen[ fork ] == 0 ) {
 	if ( fork == DATA ) {
-	    pos = lseek( bin.filed, 0, L_INCR );
+	    pos = lseek( bin.filed, 0, SEEK_CUR );
 #if DEBUG
 	    fprintf( stderr, "current position is %ld\n", pos );
-#endif
-	    pos = pos % HEADBUFSIZ;
-	    pos = pos ? HEADBUFSIZ - pos : pos;
-	    pos = lseek( bin.filed, pos, L_INCR );
+#endif /* DEBUG */
+	    pos %= HEADBUFSIZ;
+	    if (pos != 0) {
+	      pos = lseek( bin.filed, HEADBUFSIZ - pos, SEEK_CUR );
+	    }
 #if DEBUG
 	    fprintf( stderr, "current position is %ld\n", pos );
-#endif
+#endif /* DEBUG */
 	}
 	return( 0 );
     }
@@ -184,14 +215,14 @@ bin_read( fork, buffer, length )
 #if DEBUG >= 3
     fprintf( stderr, "bin_read: readlen is %d\n", readlen );
     fprintf( stderr, "bin_read: cc is %d\n", cc );
-#endif
+#endif /* DEBUG >= 3 */
 
     buf_ptr = buffer;
     while (( readlen > 0 ) && ( cc > 0 )) {
 	if (( cc = read( bin.filed, buf_ptr, readlen )) > 0 ) {
 #if DEBUG >= 3
 	    fprintf( stderr, "bin_read: cc is %d\n", cc );
-#endif
+#endif /* DEBUG >= 3 */
 	    readlen -= cc;
 	    buf_ptr += cc;
 	}
@@ -203,7 +234,7 @@ bin_read( fork, buffer, length )
 
 #if DEBUG >= 3
     fprintf( stderr, "bin_read: chars read is %d\n", cc );
-#endif
+#endif /* DEBUG >= 3 */
     return( cc );
 }
 
@@ -211,7 +242,7 @@ bin_read( fork, buffer, length )
  * bin_write 
  */
 
-bin_write( fork, buffer, length )
+int bin_write( fork, buffer, length )
     int			fork;
     char		*buffer;
     int			length;
@@ -220,12 +251,14 @@ bin_write( fork, buffer, length )
     int			writelen;
     int			cc = 0;
     off_t		pos;
-    u_char		padchar = 0;
+    u_char		padchar = 0x7f;
+		/* Not sure why, but it seems this must be 0x7f to match
+		   other converters, not 0. (RLB) */
 
 #if DEBUG >= 3
     fprintf( stderr, "bin_write: fork is %s\n", forkname[ fork ] );
     fprintf( stderr, "bin_write: remaining length is %d\n", bin.forklen[fork] );
-#endif
+#endif /* DEBUG >= 3 */
 
     if (( fork == RESOURCE ) && ( bin.forklen[ DATA ] != 0 )) {
 	fprintf( stderr, "Forklength error.\n" );
@@ -242,7 +275,7 @@ bin_write( fork, buffer, length )
 
 #if DEBUG >= 3
     fprintf( stderr, "bin_write: write length is %d\n", writelen );
-#endif
+#endif /* DEBUG >= 3 */
 
     while (( writelen > 0 ) && ( cc >= 0 )) {
 	cc = write( bin.filed, buf_ptr, writelen );
@@ -265,25 +298,26 @@ bin_write( fork, buffer, length )
  */
 
     if ( bin.forklen[ fork ] == 0 ) {
-	pos = lseek( bin.filed, 0, L_INCR );
+	pos = lseek( bin.filed, 0, SEEK_CUR );
 #if DEBUG
 	fprintf( stderr, "current position is %ld\n", pos );
-#endif
-	pos = pos % HEADBUFSIZ;
-	pos = pos ? HEADBUFSIZ - pos : pos;
-	pos = lseek( bin.filed, pos - 1, L_INCR );
-	if ( write( bin.filed, &padchar, 1 ) != 1 ) {
+#endif /* DEBUG */
+	pos %= HEADBUFSIZ;
+	if (pos != 0) { /* pad only if we need to */
+	  pos = lseek( bin.filed, HEADBUFSIZ - pos - 1, SEEK_CUR );
+	  if ( write( bin.filed, &padchar, 1 ) != 1 ) {
 	    perror( "Couldn't write to macbinary file:" );
 	    return( -1 );
+	  }
 	}
 #if DEBUG
-	fprintf( stderr, "current position is %ld\n", pos );
-#endif
+	  fprintf( stderr, "current position is %ld\n", pos );
+#endif /* DEBUG */
     }
 
 #if DEBUG
 	fprintf( stderr, "\n" );
-#endif
+#endif /* DEBUG */
 
     return( length );
 }
@@ -294,7 +328,7 @@ bin_write( fork, buffer, length )
  * of the bytes of the other two forks can be read, as well.
  */
 
-bin_header_read( fh, revision )
+int bin_header_read( fh, revision )
     struct FHeader	*fh;
     int			revision;
 {
@@ -308,10 +342,10 @@ bin_header_read( fh, revision )
  */
 
     switch ( revision ) {
+        case 3:
 	case 2 :
 	    mask = htons( 0xfcee );
-	    bcopy( (char *)&head_buf[ 101 ], 
-		    ((char *)&fh->finder_info.fdFlags) + 1, 1 );
+	    memcpy(&fh->finder_info.fdFlags + 1, head_buf + 101,1 );
 	    break;
 	case 1 :
 	    mask = htons( 0xfc00 );
@@ -326,24 +360,40 @@ bin_header_read( fh, revision )
  * MacBinary header into the fh struct.  What fun!
  */
 
-    bcopy( (char *)&head_buf[ 2 ], (char *)fh->name, head_buf[ 1 ] );
-    bcopy( (char *)&head_buf[ 91 ], (char *)&fh->create_date, 4 );
-    bcopy( (char *)&head_buf[ 95 ], (char *)&fh->mod_date, 4 );
-    fh->backup_date = 0;
-    bcopy( (char *)&head_buf[ 65 ], (char *)&fh->finder_info, 8 );
-    bcopy( (char *)&head_buf[ 73 ], (char *)&fh->finder_info.fdFlags, 1 );
-    fh->finder_info.fdFlags = fh->finder_info.fdFlags & mask;
-    bcopy( (char *)&head_buf[ 75 ], (char *)&fh->finder_info.fdLocation, 4 );
-    bcopy( (char *)&head_buf[ 79 ], (char *)&fh->finder_info.fdFldr, 2 );
-    bcopy( (char *)&head_buf[ 83 ], (char *)&fh->forklen[ DATA ], 4 );
+    memcpy(fh->name, head_buf +  2, head_buf[ 1 ] );
+    memcpy(&fh->create_date, head_buf +  91, 4 );
+    fh->create_date = MAC_DATE_TO_UNIX(fh->create_date) - bin.gmtoff;
+    fh->create_date = AD_DATE_FROM_UNIX(fh->create_date);
+    memcpy( &fh->mod_date, head_buf +  95, 4 );
+    fh->mod_date = MAC_DATE_TO_UNIX(fh->mod_date) - bin.gmtoff;
+    fh->mod_date = AD_DATE_FROM_UNIX(fh->mod_date);
+    fh->backup_date = AD_DATE_START;
+    memcpy( &fh->finder_info, head_buf +  65, 8 );
+
+#ifndef MACBINARY_PLAY_NICE_WITH_OTHERS /* (RLB) */
+    memcpy( &fh->finder_info.fdFlags, head_buf + 73, 1 );
+    fh->finder_info.fdFlags &= mask;
+#else /* ! MACBINARY_PLAY_NICE_WITH_OTHERS */
+	memcpy( &fh->finder_info.fdFlags, head_buf + 73, 2 );
+#endif /* ! MACBINARY_PLAY_NICE_WITH_OTHERS */
+
+    memcpy(&fh->finder_info.fdLocation, head_buf + 75, 4 );
+    memcpy(&fh->finder_info.fdFldr, head_buf +  79, 2 );
+    memcpy(&fh->forklen[ DATA ],  head_buf + 83, 4 );
     bin.forklen[ DATA ] = ntohl( fh->forklen[ DATA ] );
-    bcopy( (char *)&head_buf[ 87 ], (char *)&fh->forklen[ RESOURCE ], 4 );
+    memcpy(&fh->forklen[ RESOURCE ],  head_buf +  87, 4 );
     bin.forklen[ RESOURCE ] = ntohl( fh->forklen[ RESOURCE ] );
     fh->comment[0] = '\0';
 
+    if (revision == 3) {
+      fh->finder_xinfo.fdScript = *(head_buf + 106);
+      fh->finder_xinfo.fdXFlags = *(head_buf + 107);
+    }
+
 #if DEBUG >= 5
     {
 	short		flags;
+	long		flags_long;
 
 	fprintf( stderr, "Values read by bin_header_read\n" );
 	fprintf( stderr, "name length\t\t%d\n", head_buf[ 1 ] );
@@ -354,15 +404,21 @@ bin_header_read( fh, revision )
 	fprintf( stderr, "creator\t\t\t%.*s\n", 
 		sizeof( fh->finder_info.fdCreator ), 
 		&fh->finder_info.fdCreator );
-	bcopy( (char *)&fh->finder_info.fdFlags, (char *)&flags, 
-		sizeof( flags ));
+	memcpy( &flags, &fh->finder_info.fdFlags, sizeof( flags ));
 	flags = ntohs( flags );
 	fprintf( stderr, "flags\t\t\t%x\n", flags );
+
+	/* Show fdLocation too (RLB) */
+	memcpy( &flags_long, &fh->finder_info.fdLocation,
+		sizeof( flags_long ));
+	flags_long = ntohl( flags_long );
+	fprintf( stderr, "location flags\t\t%lx\n", flags_long );
+
 	fprintf( stderr, "data fork length\t%ld\n", bin.forklen[DATA] );
 	fprintf( stderr, "resource fork length\t%ld\n", bin.forklen[RESOURCE] );
 	fprintf( stderr, "\n" );
     }
-#endif
+#endif /* DEBUG >= 5 */
 
     return( 0 );
 }
@@ -374,48 +430,77 @@ bin_header_read( fh, revision )
  * bin_header_write and bin_header_read are opposites.
  */
 
-bin_header_write( fh )
+int bin_header_write( fh )
     struct FHeader	*fh;
 {
     char		*write_ptr;
+    u_int32_t           t;
     int			wc;
     int			wr;
 
-    bzero( (char *)head_buf, sizeof( head_buf ));
+    memset(head_buf, 0, sizeof( head_buf ));
     head_buf[ 1 ] = (u_char)strlen( fh->name );
-    bcopy( (char *)fh->name, (char *)&head_buf[ 2 ], head_buf[ 1 ] );
-    bcopy( (char *)&fh->finder_info, (char *)&head_buf[ 65 ], 8 );
-    bcopy( (char *)&fh->finder_info.fdFlags, (char *)&head_buf[ 73 ], 1 );
-    bcopy( (char *)&fh->finder_info.fdLocation, (char *)&head_buf[ 75 ], 4 );
-    bcopy( (char *)&fh->finder_info.fdFldr, (char *)&head_buf[ 79 ], 2 );
-    bcopy( (char *)&fh->forklen[ DATA ], (char *)&head_buf[ 83 ], 4 );
-    bcopy( (char *)&fh->forklen[ RESOURCE ], (char *)&head_buf[ 87 ], 4 );
-    bcopy( (char *)&fh->create_date, (char *)&head_buf[ 91 ], 4 );
-    bcopy( (char *)&fh->mod_date, (char *)&head_buf[ 95 ], 4 );
-    bcopy( ((char *)&fh->finder_info.fdFlags) + 1,
-	    (char *)&head_buf[ 101 ], 1 );
-    head_buf[ 122 ] = 129;
+    memcpy( head_buf + 2, fh->name, head_buf[ 1 ] );
+    memcpy( head_buf + 65, &fh->finder_info, 8 );
+
+#ifndef MACBINARY_PLAY_NICE_WITH_OTHERS /* (RLB) */
+    memcpy( head_buf + 73, &fh->finder_info.fdFlags, 1 );
+#else /* ! MACBINARY_PLAY_NICE_WITH_OTHERS */
+    memcpy( head_buf + 73, &fh->finder_info.fdFlags, 2 );
+#endif /* ! MACBINARY_PLAY_NICE_WITH_OTHERS */
+
+    memcpy( head_buf + 75, &fh->finder_info.fdLocation, 4 );
+    memcpy( head_buf + 79, &fh->finder_info.fdFldr, 2 );
+    memcpy( head_buf + 83, &fh->forklen[ DATA ], 4 );
+    memcpy( head_buf + 87, &fh->forklen[ RESOURCE ], 4 );
+    t = AD_DATE_TO_UNIX(fh->create_date) + bin.gmtoff;
+    t = MAC_DATE_FROM_UNIX(t);
+    memcpy( head_buf + 91, &t, sizeof(t) );
+    t = AD_DATE_TO_UNIX(fh->mod_date) + bin.gmtoff;
+    t = MAC_DATE_FROM_UNIX(t);
+    memcpy( head_buf + 95, &t, sizeof(t) );
+    memcpy( head_buf + 101, &fh->finder_info.fdFlags + 1, 1);
+
+    /* macbinary III */
+    memcpy( head_buf + 102, "mBIN", 4);
+    *(head_buf + 106) = fh->finder_xinfo.fdScript;
+    *(head_buf + 107) = fh->finder_xinfo.fdXFlags;
+    head_buf[ 122 ] = 130;
+
     head_buf[ 123 ] = 129;
 
     bin.headercrc = htons( updcrc( (u_short) 0, head_buf, 124 ));
-    bcopy( (char *)&bin.headercrc, (char *)&head_buf[ 124 ], 
-	    sizeof( bin.headercrc ));
+    memcpy(head_buf + 124, &bin.headercrc, sizeof( bin.headercrc ));
 
     bin.forklen[ DATA ] = ntohl( fh->forklen[ DATA ] );
     bin.forklen[ RESOURCE ] = ntohl( fh->forklen[ RESOURCE ] );
 
 #if DEBUG >= 5
     {
+	short	flags;
+	long	flags_long;
+
 	fprintf( stderr, "Values written by bin_header_write\n" );
 	fprintf( stderr, "name length\t\t%d\n", head_buf[ 1 ] );
 	fprintf( stderr, "file name\t\t%s\n", (char *)&head_buf[ 2 ] );
 	fprintf( stderr, "type\t\t\t%.4s\n", (char *)&head_buf[ 65 ] );
 	fprintf( stderr, "creator\t\t\t%.4s\n", (char *)&head_buf[ 69 ] );
+
+	memcpy( &flags, &fh->finder_info.fdFlags, sizeof( flags ));
+	flags = ntohs( flags );
+	fprintf( stderr, "flags\t\t\t%x\n", flags );
+
+	/* Show fdLocation too (RLB) */
+	memcpy( &flags_long, &fh->finder_info.fdLocation,
+		sizeof( flags_long ));
+	flags_long = ntohl( flags_long );
+	fprintf( stderr, "location flags\t\t%ldx\n", flags_long );
+
 	fprintf( stderr, "data fork length\t%ld\n", bin.forklen[DATA] );
 	fprintf( stderr, "resource fork length\t%ld\n", bin.forklen[RESOURCE] );
 	fprintf( stderr, "\n" );
     }
-#endif
+#endif /* DEBUG >= 5 */
 
     write_ptr = (char *)head_buf;
     wc = sizeof( head_buf );
@@ -434,27 +519,33 @@ bin_header_write( fh )
 }
 
 /*
- * test_header is called from bin_open.  it checks certain values of the
- * first 128 bytes, determines if the file is a MacBinary, MacBinary II,
- * or non-MacBinary file, and returns a one, two, or negative one to 
- * indicate the file type.
+ * test_header is called from bin_open.  it checks certain values of
+ * the first 128 bytes, determines if the file is a MacBinary,
+ * MacBinary II, MacBinary III, or non-MacBinary file, and returns a
+ * one, two, three or negative one to indicate the file type.
  *
- * Bytes 0 and 74 must be zero for the file to be any type of MacBinary.
- * If the crc of bytes 0 through 123 equals the value at offset 124 then
- * it is a MacBinary II.  If not, then if byte 82 is zero, byte 2 is
- * a valid value for a mac filename length (between one and sixty-three),
- * and bytes 101 through 125 are all zero, then the file is a MacBinary.
+ * If the signature at 102 is equal to "mBIN," then it's a MacBinary
+ * III file. Bytes 0 and 74 must be zero for the file to be any type
+ * of MacBinary.  If the crc of bytes 0 through 123 equals the value
+ * at offset 124 then it is a MacBinary II.  If not, then if byte 82
+ * is zero, byte 2 is a valid value for a mac filename length (between
+ * one and sixty-three), and bytes 101 through 125 are all zero, then
+ * the file is a MacBinary. 
+ *
+ * NOTE: apple's MacBinary II files have a non-zero value at byte 74.
+ * so, the check for byte 74 isn't very useful.
  */
 
-test_header()
+int test_header(void)
 {
-    int			cc;
+    const char          zeros[25] = "";
+    u_int32_t		cc;
     u_short		header_crc;
     u_char		namelen;
 
 #if DEBUG
     fprintf( stderr, "entering test_header\n" );
-#endif
+#endif /* DEBUG */
 
     cc = read( bin.filed, (char *)head_buf, sizeof( head_buf ));
     if ( cc < sizeof( head_buf )) {
@@ -464,41 +555,59 @@ test_header()
 
 #if DEBUG
     fprintf( stderr, "was able to read HEADBUFSIZ bytes\n" );
-#endif
+#endif /* DEBUG */
 
-    if (( head_buf[ 0 ] != 0 ) || ( head_buf[ 74 ] != 0 )) {
-	return( -1 );
-    }
+    /* check for macbinary III header */
+    if (memcmp(head_buf + 102, "mBIN", 4) == 0)
+        return 3;
 
+    /* check for macbinary II even if only one of the bytes is zero */
+    if (( head_buf[ 0 ] == 0 ) || ( head_buf[ 74 ] == 0 )) {
 #if DEBUG
-    fprintf( stderr, "byte 0 and 74 are both zero\n" );
-#endif
-
-    bin.headercrc = updcrc( (u_short) 0, head_buf, 124 );
-    bcopy( &head_buf[ 124 ], (char *)&header_crc, sizeof( header_crc ));
-    header_crc = ntohs( header_crc );
-    if ( header_crc == bin.headercrc ) {
+      fprintf( stderr, "byte 0 and 74 are both zero\n" );
+#endif /* DEBUG */
+      bin.headercrc = updcrc( (u_short) 0, head_buf, 124 );
+      memcpy(&header_crc, head_buf + 124, sizeof( header_crc ));
+      header_crc = ntohs( header_crc );
+      if ( header_crc == bin.headercrc ) {
 	return( 2 );
-    }
+      }
 
 #if DEBUG
-    fprintf( stderr, "header crc didn't pan out\n" );
-#endif
+      fprintf( stderr, "header crc didn't pan out\n" );
+#endif /* DEBUG */
+    }
 
+    /* now see if we have a macbinary file. */
     if ( head_buf[ 82 ] != 0 ) {
 	return( -1 );
     }
-    bcopy( &head_buf[ 1 ], (char *)&namelen, sizeof( namelen ));
+    memcpy( &namelen, head_buf + 1, sizeof( namelen ));
 #if DEBUG
     fprintf( stderr, "name length is %d\n", namelen );
-#endif
+#endif /* DEBUG */
     if (( namelen < 1 ) || ( namelen > 63 )) {
 	return( -1 );
     }
 
+    /* bytes 101 - 125 should be zero */
+    if (memcmp(head_buf + 101, zeros, sizeof(zeros)) != 0)
+        return -1;
+
+    /* macbinary forks aren't larger than 0x7FFFFF */
+    memcpy(&cc, head_buf + 83, sizeof(cc));
+    cc = ntohl(cc);
+    if (cc > 0x7FFFFF)
+        return -1;
+    memcpy(&cc, head_buf + 87, sizeof(cc));
+    cc = ntohl(cc);
+    if (cc > 0x7FFFFF)
+        return -1;
+
+
 #if DEBUG
     fprintf( stderr, "byte 82 is zero and name length is cool\n" );
-#endif
+#endif /* DEBUG */
 
     return( 1 );
 }
