@@ -1,47 +1,56 @@
+/*
+ * $Id: nad.c,v 1.8.2.1 2002/03/12 13:39:55 srittau Exp $
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
-#include <sys/dir.h>
-#include <sys/file.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/uio.h>
-#if BSD >= 199006
-# include <machine/endian.h>
-#else
-# include <netinet/in.h>
-#endif
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
-#include <strings.h>
-#include <syslog.h>
-#include <atalk/adouble.h>
-#include "megatron.h"
+#include <string.h>
+#include <dirent.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
 
-#define	DEBUG		0
+#include <atalk/adouble.h>
+#include <netatalk/endian.h>
+#include "megatron.h"
+#include "nad.h"
 
 static char		hexdig[] = "0123456789abcdef";
 
 char *mtoupath( mpath )
     char	*mpath;
 {
-    static char	upath[ MAXNAMLEN ];
+    static char	upath[ MAXNAMLEN  + 1];
     char	*m, *u;
     int		i = 0;
 
     m = mpath;
     u = upath;
     while ( *m != '\0' ) {
+#if AD_VERSION == AD_VERSION1
 	if ( !isascii( *m ) || *m == '/' || ( i == 0 && *m == '.' )) {
+#else /* AD_VERSION == AD_VERSION1 */
+	if (!isprint(*m) || *m == '/' || ( i == 0 && (*m == '.' ))) {
+#endif /* AD_VERSION == AD_VERSION1 */
 	    *u++ = ':';
 	    *u++ = hexdig[ ( *m & 0xf0 ) >> 4 ];
 	    *u++ = hexdig[ *m & 0x0f ];
 	} else {
 #ifdef DOWNCASE
 	    *u++ = ( isupper( *m )) ? tolower( *m ) : *m;
-#else DOWNCASE
+#else /* DOWNCASE */
 	    *u++ = *m;
-#endif DOWNCASE
+#endif /* DOWNCASE */
 	}
 	i++;
 	m++;
@@ -50,20 +59,55 @@ char *mtoupath( mpath )
     return( upath );
 }
 
+
+#define hextoint( c )	( isdigit( c ) ? c - '0' : c + 10 - 'a' )
+#define islxdigit(x)	(!isupper(x)&&isxdigit(x))
+
+char *utompath( upath )
+    char	*upath;
+{
+    static char	mpath[ MAXNAMLEN  + 1];
+    char	*m, *u;
+    int h;
+
+    m = mpath;
+    u = upath;
+    while ( *u != '\0' ) {
+        if (*u == ':' && *(u + 1) != '\0' && islxdigit(*(u+1)) &&
+	    *(u+2) != '\0' && islxdigit(*(u+2))) {
+	  u++;
+	  h = hextoint(*u) << 4;
+	  u++;
+	  h |= hextoint(*u);
+	  *m = h;
+	} else {
+#ifdef DOWNCASE
+	  *m = diatolower(*u);
+#else /* DOWNCASE */
+	  *m = *u;
+#endif /* DOWNCASE */
+	}
+	u++;
+	m++;
+    }
+    *m = '\0';
+    return( mpath );
+}
+
 #if HEXOUTPUT
     int			hexfork[ NUMFORKS ];
-#endif
+#endif /* HEXOUTPUT */
 
 struct nad_file_data {
-    char		macname[ MAXPATHLEN ];
-    char		adpath[ 2 ][ MAXPATHLEN ];
+    char		macname[ MAXPATHLEN + 1 ];
+    char		adpath[ 2 ][ MAXPATHLEN + 1];
     int			offset[ NUMFORKS ];
     struct adouble	ad;
 } nad;
 
-nad_open( path, openflags, fh )
+int nad_open( path, openflags, fh, options )
     char		*path;
-    int			openflags;
+    int			openflags, options;
     struct FHeader	*fh;
 {
     struct stat		st;
@@ -74,7 +118,7 @@ nad_open( path, openflags, fh )
  * is for write, then stat the current directory to get its mode.
  * Open the file.  Either fill or grab the adouble information.
  */
-
+    memset(&nad.ad, 0, sizeof(nad.ad));
     if ( openflags == O_RDONLY ) {
 	strcpy( nad.adpath[0], path );
 	strcpy( nad.adpath[1], 
@@ -93,9 +137,9 @@ nad_open( path, openflags, fh )
 #if DEBUG
     fprintf(stderr, "%s is adpath[0]\n", nad.adpath[0]);
     fprintf(stderr, "%s is adpath[1]\n", nad.adpath[1]);
-#endif
+#endif /* DEBUG */
 	if ( ad_open( nad.adpath[ 0 ], ADFLAGS_DF|ADFLAGS_HF,
-		openflags, (int)( st.st_mode & 0666 ), &nad.ad ) < 0 ) {
+		openflags, (int)( st.st_mode & 0666 ), &nad.ad) < 0 ) {
 	    perror( nad.adpath[ 0 ] );
 	    return( -1 );
 	}
@@ -110,14 +154,14 @@ nad_open( path, openflags, fh )
     fprintf(stderr, "%s\n", nad.macname);
     fprintf(stderr, "%s is adpath[0]\n", nad.adpath[0]);
     fprintf(stderr, "%s is adpath[1]\n", nad.adpath[1]);
-#endif
+#endif /* DEBUG */
 	if ( stat( ".", &st ) < 0 ) {
 	    perror( "stat of . failed" );
 	    return( -1 );
 	}
 	(void)umask( 0 );
 	if ( ad_open( nad.adpath[ 0 ], ADFLAGS_DF|ADFLAGS_HF,
-		openflags, (int)( st.st_mode & 0666 ), &nad.ad ) < 0 ) {
+		openflags, (int)( st.st_mode & 0666 ), &nad.ad) < 0 ) {
 	    perror( nad.adpath[ 0 ] );
 	    return( -1 );
 	}
@@ -125,16 +169,21 @@ nad_open( path, openflags, fh )
     }
 }
 
-nad_header_read( fh )
+int nad_header_read( fh )
     struct FHeader	*fh;
 {
-    long		temptime;
+    u_int32_t		temptime;
     struct stat		st;
 
-    bcopy( ad_entry( &nad.ad, ADEID_NAME ), nad.macname, 
+    memcpy( nad.macname, ad_entry( &nad.ad, ADEID_NAME ), 
 	    ad_getentrylen( &nad.ad, ADEID_NAME ));
     nad.macname[ ad_getentrylen( &nad.ad, ADEID_NAME ) ] = '\0';
     strcpy( fh->name, nad.macname );
+
+    /* just in case there's nothing in macname */
+    if (*fh->name == '\0')
+      strcpy(fh->name, utompath(nad.adpath[DATA]));
+
     if ( stat( nad.adpath[ DATA ], &st ) < 0 ) {
 	perror( "stat of datafork failed" );
 	return( -1 );
@@ -151,49 +200,49 @@ nad_header_read( fh )
     fprintf( stderr, "size of resource fork\t\t%d\n", 
 	    ntohl( fh->forklen[ RESOURCE ] ));
     fprintf( stderr, "get info comment\t\t\"%s\"\n", fh->comment );
-#endif
+#endif /* DEBUG */
 
-    bcopy(( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_CREATE ), &temptime,
-	    sizeof( temptime ));
-    temptime = htonl( ntohl( temptime ) + TIME_DIFF );
-    bcopy( &temptime, &fh->create_date, sizeof( temptime ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_MODIFY ), &temptime,
-	    sizeof( temptime ));
-    temptime = htonl( ntohl( temptime ) + TIME_DIFF );
-    bcopy( &temptime, &fh->mod_date, sizeof( temptime ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_BACKUP ), &temptime,
-	    sizeof( temptime ));
-    temptime = htonl( ntohl( temptime ) + TIME_DIFF );
-    bcopy( &temptime, &fh->backup_date, sizeof( temptime ));
+    ad_getdate(&nad.ad, AD_DATE_CREATE, &temptime);
+    memcpy( &fh->create_date, &temptime, sizeof( temptime ));
+    ad_getdate(&nad.ad, AD_DATE_MODIFY, &temptime);
+    memcpy( &fh->mod_date, &temptime, sizeof( temptime ));
+    ad_getdate(&nad.ad, AD_DATE_BACKUP, &temptime);
+    memcpy( &fh->backup_date, &temptime, sizeof( temptime ));
 
 #if DEBUG
-    bcopy( &fh->create_date, &temptime, sizeof( temptime ));
-    temptime = ntohl( temptime );
+    memcpy( &temptime, &fh->create_date, sizeof( temptime ));
+    temptime = AD_DATE_TO_UNIX(temptime);
     fprintf( stderr, "create_date seconds\t\t%lu\n", temptime );
-    bcopy( &fh->mod_date, &temptime, sizeof( temptime ));
-    temptime = ntohl( temptime );
+    memcpy( &temptime, &fh->mod_date, sizeof( temptime ));
+    temptime = AD_DATE_TO_UNIX(temptime);
     fprintf( stderr, "mod_date seconds\t\t%lu\n", temptime );
-    bcopy( &fh->backup_date, &temptime, sizeof( temptime ));
-    temptime = ntohl( temptime );
+    memcpy( &temptime, &fh->backup_date, sizeof( temptime ));
+    temptime = AD_DATE_TO_UNIX(temptime);
     fprintf( stderr, "backup_date seconds\t\t%lu\n", temptime );
     fprintf( stderr, "size of finder_info\t\t%d\n", sizeof( fh->finder_info ));
-#endif
+#endif /* DEBUG */
 
-    bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_TYPE ),
-	    (char *)&fh->finder_info.fdType,
-	    sizeof( fh->finder_info.fdType ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_CREATOR ),
-	    (char *)&fh->finder_info.fdCreator,
-	    sizeof( fh->finder_info.fdCreator ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLAGS ),
-	    (char *)&fh->finder_info.fdFlags,
-	    sizeof( fh->finder_info.fdFlags ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_LOC ),
-	    (char *)&fh->finder_info.fdLocation,
-	    sizeof( fh->finder_info.fdLocation ));
-    bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLDR ),
-	    (char *)&fh->finder_info.fdFldr,
-	    sizeof( fh->finder_info.fdFldr ));
+    memcpy(&fh->finder_info.fdType,
+	    ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_TYPE,
+	   sizeof( fh->finder_info.fdType ));
+    memcpy(&fh->finder_info.fdCreator,
+	   ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_CREATOR,
+	   sizeof( fh->finder_info.fdCreator ));
+    memcpy(&fh->finder_info.fdFlags,
+	   ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLAGS,
+	   sizeof( fh->finder_info.fdFlags ));
+    memcpy(&fh->finder_info.fdLocation,
+	   ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_LOC,
+	   sizeof( fh->finder_info.fdLocation ));
+    memcpy(&fh->finder_info.fdFldr,
+	  ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLDR,
+	  sizeof( fh->finder_info.fdFldr ));
+    memcpy(&fh->finder_xinfo.fdScript,
+	   ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_SCRIPT,
+	   sizeof(fh->finder_xinfo.fdScript));
+    memcpy(&fh->finder_xinfo.fdXFlags,
+	   ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_XFLAGS,
+	   sizeof(fh->finder_xinfo.fdXFlags));
 
 #if DEBUG
     {
@@ -207,13 +256,14 @@ nad_header_read( fh )
 		sizeof( fh->finder_info.fdType ) + 
 		sizeof( fh->finder_info.fdCreator ), 
 		ad_entry( &nad.ad, ADEID_FINDERI ));
-	bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + 
-		sizeof( fh->finder_info.fdType ) + 
-		sizeof( fh->finder_info.fdCreator ) ), &flags, sizeof( flags ));
+	memcpy(&flags, ad_entry( &nad.ad, ADEID_FINDERI ) + 
+	       FINDERIOFF_FLAGS, sizeof( flags ));
 	fprintf( stderr, "nad.ad flags\t\t\t%x\n", flags );
 	fprintf( stderr, "fh flags\t\t\t%x\n", fh->finder_info.fdFlags );
+	fprintf(stderr, "fh script\t\t\t%x\n", fh->finder_xinfo.fdScript);
+	fprintf(stderr, "fh xflags\t\t\t%x\n", fh->finder_xinfo.fdXFlags);
     }
-#endif
+#endif /* DEBUG */
 
     nad.offset[ DATA ] = nad.offset[ RESOURCE ] = 0;
 
@@ -221,14 +271,17 @@ nad_header_read( fh )
 
 }
 
-nad_header_write( fh )
+int nad_header_write( fh )
     struct FHeader	*fh;
 {
-    long		temptime;
+    u_int32_t		temptime;
 
     ad_setentrylen( &nad.ad, ADEID_NAME, strlen( nad.macname ));
-    bcopy( nad.macname, ad_entry( &nad.ad, ADEID_NAME ), 
+    memcpy( ad_entry( &nad.ad, ADEID_NAME ), nad.macname, 
 	    ad_getentrylen( &nad.ad, ADEID_NAME ));
+    ad_setentrylen( &nad.ad, ADEID_COMMENT, strlen( fh->comment ));
+    memcpy( ad_entry( &nad.ad, ADEID_COMMENT ), fh->comment, 
+	    ad_getentrylen( &nad.ad, ADEID_COMMENT ));
     ad_setentrylen( &nad.ad, ADEID_RFORK, ntohl( fh->forklen[ RESOURCE ] ));
 
 #if DEBUG
@@ -242,61 +295,60 @@ nad_header_write( fh )
     fprintf( stderr, "ad_entry of ADEID_NAME\t\t%.*s\n",
 	    ad_getentrylen( &nad.ad, ADEID_NAME ), 
 	    ad_entry( &nad.ad, ADEID_NAME ));
-#endif
+    fprintf( stderr, "ADEID_COMMENT\t\t\t%d\n",
+	     ad_getentrylen( &nad.ad, ADEID_COMMENT ));
+#endif /* DEBUG */
 
-    bcopy( &fh->create_date, &temptime, sizeof( temptime ));
-    temptime = htonl( ntohl( temptime ) - TIME_DIFF );
-    bcopy( &temptime, ( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_CREATE ), 
-	    sizeof( temptime ));
-    bcopy( &fh->mod_date, &temptime, sizeof( temptime ));
-    temptime = htonl( ntohl( temptime ) - TIME_DIFF );
-    bcopy( &temptime, ( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_MODIFY ),
-	    sizeof( temptime ));
+    memcpy( &temptime, &fh->create_date, sizeof( temptime ));
+    ad_setdate(&nad.ad, AD_DATE_CREATE, temptime);
+    memcpy( &temptime, &fh->mod_date, sizeof( temptime ));
+    ad_setdate(&nad.ad, AD_DATE_MODIFY, temptime);
 
 #if DEBUG
-    bcopy(( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_CREATE ), 
-	    &temptime, sizeof( temptime ));
-    temptime = ntohl( temptime );
+    ad_getdate(&nad.ad, AD_DATE_CREATE, &temptime);
+    temptime = AD_DATE_TO_UNIX(temptime);
     fprintf(stderr, "FILEIOFF_CREATE seconds\t\t%ld\n", temptime );
-    bcopy(( ad_entry( &nad.ad, ADEID_FILEI ) + FILEIOFF_MODIFY ), 
-	    &temptime, sizeof( temptime ));
-    temptime = ntohl( temptime );
+    ad_getdate(&nad.ad, AD_DATE_MODIFY, &temptime);
+    temptime = AD_DATE_TO_UNIX(temptime);
     fprintf(stderr, "FILEIOFF_MODIFY seconds\t\t%ld\n", temptime );
-#endif
+#endif /* DEBUG */
+
+    memset( ad_entry( &nad.ad, ADEID_FINDERI ), 0, ADEDLEN_FINDERI );
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_TYPE, 
+	    &fh->finder_info.fdType, sizeof( fh->finder_info.fdType ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_CREATOR,
+	   &fh->finder_info.fdCreator, sizeof( fh->finder_info.fdCreator ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLAGS,
+	    &fh->finder_info.fdFlags, sizeof( fh->finder_info.fdFlags ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_LOC,
+	    &fh->finder_info.fdLocation,sizeof( fh->finder_info.fdLocation ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLDR,
+	    &fh->finder_info.fdFldr, sizeof( fh->finder_info.fdFldr ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_SCRIPT,
+	    &fh->finder_xinfo.fdScript, sizeof( fh->finder_xinfo.fdScript ));
+    memcpy( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_XFLAGS,
+	    &fh->finder_xinfo.fdXFlags, sizeof( fh->finder_xinfo.fdXFlags));
 
-    bzero( ad_entry( &nad.ad, ADEID_FINDERI ), ADEDLEN_FINDERI );
-    bcopy( (char *)&fh->finder_info.fdType, 
-	    ad_entry( &nad.ad, ADEID_FINDERI ), 
-	    sizeof( fh->finder_info.fdType ));
-    bcopy( (char *)&fh->finder_info.fdCreator, 
-	    ad_entry( &nad.ad, ADEID_FINDERI ) + 
-	    sizeof( fh->finder_info.fdType ), 
-	    sizeof( fh->finder_info.fdCreator ));
-    bcopy( (char *)&fh->finder_info.fdFlags, 
-	    ad_entry( &nad.ad, ADEID_FINDERI ) + 
-	    sizeof( fh->finder_info.fdType ) + 
-	    sizeof( fh->finder_info.fdCreator ), 
-	    sizeof( fh->finder_info.fdFlags ));
 
 #if DEBUG
     {
 	short		flags;
-	bcopy(( ad_entry( &nad.ad, ADEID_FINDERI ) + 
-		sizeof( fh->finder_info.fdType ) + 
-		sizeof( fh->finder_info.fdCreator ) ), &flags, sizeof( flags ));
+	memcpy(&flags, ( ad_entry( &nad.ad, ADEID_FINDERI ) + FINDERIOFF_FLAGS),
+		sizeof( flags ));
 	fprintf( stderr, "nad.ad flags\t\t\t%x\n", flags );
 	fprintf( stderr, "fh flags\t\t\t%x\n", fh->finder_info.fdFlags );
+	fprintf( stderr, "fh xflags\t\t\t%x\n", fh->finder_xinfo.fdXFlags );
 	fprintf( stderr, "type and creator\t\t%.*s\n\n", 
 		sizeof( fh->finder_info.fdType ) + 
 		sizeof( fh->finder_info.fdCreator ),
 		ad_entry( &nad.ad, ADEID_FINDERI ));
     }
-#endif
+#endif /* DEBUG */
 
 #if HEXOUTPUT
     hexfork[ DATA ] = open( "datafork", O_WRONLY|O_CREAT, 0622 );
     hexfork[ RESOURCE ] = open( "resfork", O_WRONLY|O_CREAT, 0622 );
-#endif
+#endif /* HEXOUTPUT */
 
     nad.offset[ DATA ] = nad.offset[ RESOURCE ] = 0;
     ad_flush( &nad.ad, ADFLAGS_DF|ADFLAGS_HF );
@@ -306,7 +358,7 @@ nad_header_write( fh )
 
 int			forkeid[] = { ADEID_DFORK, ADEID_RFORK };
 
-nad_read( fork, forkbuf, bufc )
+int nad_read( fork, forkbuf, bufc )
     int			fork;
     char		*forkbuf;
     int			bufc;
@@ -315,10 +367,10 @@ nad_read( fork, forkbuf, bufc )
 
 #if DEBUG
     fprintf( stderr, "Entering nad_read\n" );
-#endif
+#endif /* DEBUG */
 
     if (( cc = ad_read( &nad.ad, forkeid[ fork ], nad.offset[ fork ], 
-	    forkbuf, bufc )) < 0 )  {
+	    forkbuf, bufc)) < 0 )  {
 	perror( "Reading the appledouble file:" );
 	return( cc );
     }
@@ -326,12 +378,12 @@ nad_read( fork, forkbuf, bufc )
 
 #if DEBUG
     fprintf( stderr, "Exiting nad_read\n" );
-#endif
+#endif /* DEBUG */
 
     return( cc );
 }
 
-nad_write( fork, forkbuf, bufc )
+int nad_write( fork, forkbuf, bufc )
     int			fork;
     char		*forkbuf;
     int			bufc;
@@ -342,18 +394,18 @@ nad_write( fork, forkbuf, bufc )
 
 #if DEBUG
     fprintf( stderr, "Entering nad_write\n" );
-#endif
+#endif /* DEBUG */
 
 #if HEXOUTPUT
     write( hexfork[ fork ], forkbuf, bufc );
-#endif
+#endif /* HEXOUTPUT */
 
     writelen = bufc;
     buf_ptr = forkbuf;
 
     while (( writelen > 0 ) && ( cc >= 0 )) {
 	cc =  ad_write( &nad.ad, forkeid[ fork ], nad.offset[ fork ], 
-		0, buf_ptr, writelen );
+		0, buf_ptr, writelen);
 	nad.offset[ fork ] += cc;
 	buf_ptr += cc;
 	writelen -= cc;
@@ -366,7 +418,7 @@ nad_write( fork, forkbuf, bufc )
     return( bufc );
 }
 
-nad_close( status )
+int nad_close( status )
 int			status;
 {
     int			rv;
@@ -388,4 +440,5 @@ int			status;
 	}
 	return( 0 );
     } else return( -1 );
+    return( 0 );
 }
