@@ -1,19 +1,30 @@
 /*
+ * $Id: pap.c,v 1.7 2001/07/31 19:49:02 srittau Exp $
+ *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/uio.h>
-#include <sys/file.h>
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
 #include <atalk/atp.h>
 #include <atalk/pap.h>
 #include <atalk/nbp.h>
+#include <atalk/util.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
+#include <string.h>
+#include <stdlib.h>
 #include <errno.h>
 
 #define FUCKED
@@ -21,24 +32,35 @@
 #define _PATH_PAPRC	".paprc"
 char	*nbpfailure = "AppleTalk printer offline";
 
-#ifdef EBUG
-#define DEBUG(x)	(x)
-#else EBUG
-#define DEBUG(x)
-#endif EBUG
+/* Forward Declarations */
+void updatestatus(char *s, int len);
+int send_file(int fd, ATP atp, int lastfile);
 
-usage( path )
+/* if there is a less hacky way to do this, please do it... */
+#ifdef DEBUG
+#define EBUG
+#endif /* DEBUG */
+
+#undef DEBUG
+
+#ifdef EBUG
+#define DEBUG(x,y)	(x,y)
+#else /*EBUG*/
+#define DEBUG(x,y)
+#endif /*EBUG*/
+
+void usage( path )
     char	*path;
 {
     char	*p;
 
-    if (( p = rindex( path, '/' )) == NULL ) {
+    if (( p = strrchr( path, '/' )) == NULL ) {
 	p = path;
     } else {
 	p++;
     }
     fprintf( stderr,
-	"Usage:\t%s [ -p printername ] [ -s statusfile ] [ file ] ...\n", p );
+	"Usage:\t%s [ -A address ] [ -p printername ] [ -s statusfile ] [ file ] ...\n", p );
     exit( 2 );
 }
 
@@ -48,7 +70,6 @@ paprc()
     static char	s[ 32 + 1 + 32 + 1 + 32 ];
     char	*name = NULL;
     FILE	*f;
-    extern int	errno;
 
     if (( f = fopen( _PATH_PAPRC, "r" )) == NULL ) {
 	if ( errno == ENOENT ) {
@@ -82,243 +103,6 @@ char			cbuf[ 8 ];
 struct nbpnve		nn;
 ATP			satp;
 
-main( ac, av )
-    int		ac;
-    char	**av;
-{
-    ATP			atp;
-    struct atp_block	atpb;
-    int			c, err = 0, fd, cuts = 0;
-    char		*obj = NULL, *type = "LaserWriter", *zone = "*";
-    struct timeval	stv, tv;
-    char		rbuf[ ATP_MAXDATA ];
-    struct iovec	iov;
-    unsigned short	waiting, result;
-    int			connattempts = 10, lkupattempts;
-    extern char		*optarg;
-    extern int		optind, errno;
-
-    while (( c = getopt( ac, av, "wcep:s:E" )) != EOF ) {
-	switch ( c ) {
-#ifdef FUCKED
-	case 'w' :
-	    waitforprinter = 1;
-	    break;
-#endif FUCKED
-
-	case 'c' :
-	    cuts++;
-	    break;
-
-	case 'e' :	/* send stdout to stderr */
-	    dup2( 2, 1 );
-	    break;
-
-	case 'p' :
-	    printer = optarg;
-	    break;
-
-	case 's' :
-	    status = optarg;
-	    break;
-
-	case 'E' :
-	    noeof = 1;
-	    break;
-
-	default :
-	    err++;
-	}
-    }
-    if ( err ) {
-	usage( *av );
-    }
-    if ( printer == NULL && (( printer = paprc()) == NULL )) {
-	fprintf( stderr, "No printer specified and ./.paprc not found.\n" );
-	exit( 2 );
-    }
-
-    /*
-     * Open connection.
-     */
-    if ( nbp_name( printer, &obj, &type, &zone ) < 0 ) {
-	fprintf( stderr, "%s: Bad name\n", printer );
-	exit( 2 );
-    }
-    if ( obj == NULL ) {
-	fprintf( stderr, "%s: Bad name\n", printer );
-	exit( 2 );
-    }
-
-    for ( lkupattempts = 10; lkupattempts != 0; lkupattempts-- ) {
-	if ( nbp_lookup( obj, type, zone, &nn, 1 ) <= 0 ) {
-	    if ( errno != 0 ) {
-		perror( "nbp_lookup" );
-		exit( 2 );
-	    }
-	    updatestatus( nbpfailure, strlen( nbpfailure ));
-	    sleep( 10 );
-	} else {
-	    break;
-	}
-    }
-    if ( lkupattempts == 0 ) {
-	fprintf( stderr, "%s:%s@%s: NBP Lookup failed\n", obj, type, zone );
-	exit( 1 );
-    }
-
-    if ( isatty( 0 )) {
-	printf( "Trying %u.%d:%d ...\n", ntohs( nn.nn_sat.sat_addr.s_net ),
-		nn.nn_sat.sat_addr.s_node, nn.nn_sat.sat_port );
-    }
-
-    if (( atp = atp_open( 0 )) == NULL ) {
-	perror( "atp_open" );
-	exit( 2 );
-    }
-    if (( satp = atp_open( 0 )) == NULL ) {
-	perror( "atp_open" );
-	exit( 2 );
-    }
-
-    cbuf[ 0 ] = connid = getpid() & 0xff;
-    cbuf[ 1 ] = PAP_OPEN;
-    cbuf[ 2 ] = cbuf[ 3 ] = 0;
-    cbuf[ 4 ] = atp_sockaddr( atp )->sat_port;
-    cbuf[ 5 ] = oquantum;	/* flow quantum */
-    if ( gettimeofday( &stv, 0 ) < 0 ) {
-	perror( "gettimeofday" );
-	exit( 2 );
-    }
-    for (;;) {
-	if ( cuts ) {
-	    waiting = 0xffff;
-	} else {
-	    if ( gettimeofday( &tv, 0 ) < 0 ) {
-		perror( "gettimeofday" );
-		exit( 2 );
-	    }
-	    waiting = htons( tv.tv_sec - stv.tv_sec );
-	}
-	bcopy( &waiting, &cbuf[ 6 ], sizeof( waiting ));
-
-	atpb.atp_saddr = &nn.nn_sat;
-	atpb.atp_sreqdata = cbuf;
-	atpb.atp_sreqdlen = 8;		/* bytes in OpenConn request */
-	atpb.atp_sreqto = 2;		/* retry timer */
-	atpb.atp_sreqtries = 5;		/* retry count */
-	if ( atp_sreq( atp, &atpb, 1, ATP_XO ) < 0 ) {
-	    perror( "atp_sreq" );
-	    exit( 1 );
-	}
-
-DEBUG( printf( "OPEN >\n" ));
-
-	iov.iov_base = rbuf;
-	iov.iov_len = sizeof( rbuf );
-	atpb.atp_rresiov = &iov;
-	atpb.atp_rresiovcnt = 1;
-	if ( atp_rresp( atp, &atpb ) < 0 ) {
-	    perror( "atp_rresp" );
-	    if ( connattempts-- <= 0 ) {
-		fprintf( stderr, "Can't connect!\n" );
-		exit( 1 );
-	    }
-	    continue;
-	}
-
-	/* sanity */
-	if ( iov.iov_len < 8 || (unsigned char)rbuf[ 0 ] != connid ||
-		rbuf[ 1 ] != PAP_OPENREPLY ) {
-	    fprintf( stderr, "Bad response!\n" );
-	    continue;	/* This is weird, since TIDs must match... */
-	}
-
-DEBUG( printf( "< OPENREPLY\n" ));
-
-	if ( isatty( 1 )) {
-	    printf( "%.*s\n", iov.iov_len - 9, iov.iov_base + 9 );
-	}
-	updatestatus( iov.iov_base + 9, iov.iov_len - 9 );
-
-	bcopy( &rbuf[ 6 ], &result, sizeof( result ));
-	if ( result != 0 ) {
-	    sleep( 2 );
-	} else {
-	    bcopy( &nn.nn_sat, &sat, sizeof( struct sockaddr_at ));
-	    sat.sat_port = rbuf[ 4 ];
-	    quantum = rbuf[ 5 ];
-	    break;
-	}
-    }
-
-    if ( isatty( 1 )) {
-	printf( "Connected to %.*s:%.*s@%.*s.\n",
-		nn.nn_objlen, nn.nn_obj,
-		nn.nn_typelen, nn.nn_type,
-		nn.nn_zonelen, nn.nn_zone );
-    }
-
-    if ( optind == ac ) {
-	sendfile( 0, atp, 1 );
-    } else {
-	for (; optind < ac; optind++ ) {
-	    if ( strcmp( av[ optind ], "-" ) == 0 ) {
-		fd = 0;
-	    } else if (( fd = open( av[ optind ], O_RDONLY )) < 0 ) {
-		perror( av[ optind ] );
-		continue;
-	    }
-	    sendfile( fd, atp, ( optind == ac - 1 ) ? 1 : 0 );
-	    if ( fd != 0 ) {
-		close( fd );
-	    }
-	}
-    }
-
-    /*
-     * Close connection.
-     */
-    cbuf[ 0 ] = connid;
-    cbuf[ 1 ] = PAP_CLOSE;
-    cbuf[ 2 ] = cbuf[ 3 ] = 0;
-
-    atpb.atp_saddr = &sat;
-    atpb.atp_sreqdata = cbuf;
-    atpb.atp_sreqdlen = 4;		/* bytes in CloseConn request */
-    atpb.atp_sreqto = 2;		/* retry timer */
-    atpb.atp_sreqtries = 5;		/* retry count */
-    if ( atp_sreq( atp, &atpb, 1, ATP_XO ) < 0 ) {
-	perror( "atp_sreq" );
-	exit( 1 );
-    }
-
-DEBUG( printf( "CLOSE >\n" ));
-
-    iov.iov_base = rbuf;
-    iov.iov_len = sizeof( rbuf );
-    atpb.atp_rresiov = &iov;
-    atpb.atp_rresiovcnt = 1;
-    if ( atp_rresp( atp, &atpb ) < 0 ) {
-	perror( "atp_rresp" );
-	exit( 1 );
-    }
-
-    /* sanity */
-    if ( iov.iov_len != 4 || (unsigned char)rbuf[ 0 ] != connid ||
-	    rbuf[ 1 ] != PAP_CLOSEREPLY ) {
-	fprintf( stderr, "Bad response!\n" );
-	exit( 1 );	/* This is weird, since TIDs must match... */
-    }
-
-DEBUG( printf( "< CLOSEREPLY\n" ));
-
-    if ( isatty( 1 )) {
-	printf( "Connection closed.\n" );
-    }
-    exit( 0 );
-}
-
 char		fbuf[ PAP_MAXQUANTUM ][ 4 + PAP_MAXDATA ];
 struct iovec	rfiov[ PAP_MAXQUANTUM ] = {
     { fbuf[ 0 ] + 4,	0 },
@@ -363,31 +147,347 @@ struct iovec	sfiov[ PAP_MAXQUANTUM ] = {
     { nbuf[ 7 ] + 4,	0 },
 };
 
+int main( ac, av )
+    int		ac;
+    char	**av;
+{
+    ATP			atp;
+    struct atp_block	atpb;
+    int			c, err = 0, fd, cuts = 0;
+    char		*obj = NULL, *type = "LaserWriter", *zone = "*";
+    struct timeval	stv, tv;
+    char		rbuf[ ATP_MAXDATA ];
+    struct iovec	iov;
+    unsigned short	waiting, result;
+    int			connattempts = 10;
+    int			waitforidle = 0;
+    struct at_addr      addr;
+
+    extern char		*optarg;
+    extern int		optind;
+
+    memset(&addr, 0, sizeof(addr));
+    while (( c = getopt( ac, av, "Wwcep:s:EA:" )) != EOF ) {
+	switch ( c ) {
+#ifdef FUCKED
+	case 'w' :
+	    waitforprinter = 1;
+	    break;
+
+	case 'W' :
+	    waitforidle = 1;
+	    break;
+#endif /* FUCKED */
+
+	case 'c' :
+	    cuts++;
+	    break;
+
+	case 'e' :	/* send stdout to stderr */
+	    dup2( 2, 1 );
+	    break;
+
+	case 'p' :
+	    printer = optarg;
+	    break;
+
+	case 's' :
+	    status = optarg;
+	    break;
+
+	case 'E' :
+	    noeof = 1;
+	    break;
+	   
+	case 'A':
+	    if (!atalk_aton(optarg, &addr)) {
+	      fprintf(stderr, "Bad address.\n");
+	      exit(1);
+	    }
+	    break;
+
+	default :
+	    err++;
+	}
+    }
+    if ( err ) {
+	usage( *av );
+    }
+    if ( printer == NULL && (( printer = paprc()) == NULL )) {
+	fprintf( stderr, "No printer specified and ./.paprc not found.\n" );
+	exit( 2 );
+    }
+
+    /*
+     * Open connection.
+     */
+    if ( nbp_name( printer, &obj, &type, &zone ) < 0 ) {
+	fprintf( stderr, "%s: Bad name\n", printer );
+	exit( 2 );
+    }
+    if ( obj == NULL ) {
+	fprintf( stderr, "%s: Bad name\n", printer );
+	exit( 2 );
+    }
+
+    if ( nbp_lookup( obj, type, zone, &nn, 1, &addr ) <= 0 ) {
+	if ( errno != 0 ) {
+	    perror( "nbp_lookup" );
+	    exit( 2 );
+	}
+	fprintf( stderr, "%s:%s@%s: NBP Lookup failed\n", obj, type, zone );
+	exit( 1 );
+    }
+
+    if ( isatty( 0 )) {
+	printf( "Trying %u.%d:%d ...\n", ntohs( nn.nn_sat.sat_addr.s_net ),
+		nn.nn_sat.sat_addr.s_node, nn.nn_sat.sat_port );
+    }
+
+    if (( atp = atp_open( ATADDR_ANYPORT, &addr )) == NULL ) {
+	perror( "atp_open" );
+	exit( 2 );
+    }
+    if (( satp = atp_open( ATADDR_ANYPORT, &addr )) == NULL ) {
+	perror( "atp_open" );
+	exit( 2 );
+    }
+
+    while ( waitforidle ) {
+	char	st_buf[ 1024 ];	/* XXX too big */
+
+	cbuf[ 0 ] = 0;
+	cbuf[ 1 ] = PAP_SENDSTATUS;
+	cbuf[ 2 ] = cbuf[ 3 ] = 0;
+	atpb.atp_saddr = &nn.nn_sat;
+	atpb.atp_sreqdata = cbuf;
+	atpb.atp_sreqdlen = 4;	/* bytes in SendStatus request */
+	atpb.atp_sreqto = 2;		/* retry timer */
+	atpb.atp_sreqtries = 5;		/* retry count */
+	if ( atp_sreq( satp, &atpb, 1, 0 ) < 0 ) {
+	    perror( "atp_sreq" );
+	    exit( 1 );
+	}
+
+DEBUG( printf( "SENDSTATUS >\n" ), fflush( stdout ));
+
+	atpb.atp_saddr = &nn.nn_sat;
+	rniov[ 0 ].iov_len = PAP_MAXDATA + 4;
+	atpb.atp_rresiov = rniov;
+	atpb.atp_rresiovcnt = 1;
+	if ( atp_rresp( satp, &atpb ) < 0 ) {
+	    perror( "atp_rresp" );
+	    continue;
+	}
+
+#ifndef NONZEROSTATUS
+	/*
+	 * The stinking LaserWriter IINTX puts crap in this
+	 * field.
+	 */
+	if ( ((char *)rniov[ 0 ].iov_base)[ 0 ] != 0 ) {
+	    fprintf( stderr, "Bad status response!\n" );
+	    exit( 1 );
+	}
+#endif /* NONZEROSTATUS */
+
+	if ( ((char *)rniov[ 0 ].iov_base)[ 1 ] != PAP_STATUS ||
+		atpb.atp_rresiovcnt != 1 ) {
+	    fprintf( stderr, "Bad status response!\n" );
+	    exit( 1 );
+	}
+
+DEBUG( printf( "< STATUS\n" ), fflush( stdout ));
+
+	memcpy( st_buf, (char *) rniov[ 0 ].iov_base + 9, 
+		((char *)rniov[ 0 ].iov_base)[ 8 ] );
+	st_buf[ (int) ((char *)rniov[ 0 ].iov_base)[ 8 ]] = '\0';
+	if ( strstr( st_buf, "idle" ) != NULL ) {
+	    waitforidle = 0;
+	} else {
+	    updatestatus( (char *) rniov[ 0 ].iov_base + 9,
+		    ((char *)rniov[ 0 ].iov_base)[ 8 ] );
+	    sleep( 5 );
+	}
+    }
+
+    cbuf[ 0 ] = connid = getpid() & 0xff;
+    cbuf[ 1 ] = PAP_OPEN;
+    cbuf[ 2 ] = cbuf[ 3 ] = 0;
+    cbuf[ 4 ] = atp_sockaddr( atp )->sat_port;
+    cbuf[ 5 ] = oquantum;	/* flow quantum */
+    if ( gettimeofday( &stv, 0 ) < 0 ) {
+	perror( "gettimeofday" );
+	exit( 2 );
+    }
+    for (;;) {
+	if ( cuts ) {
+	    waiting = 0xffff;
+	} else {
+	    if ( gettimeofday( &tv, 0 ) < 0 ) {
+		perror( "gettimeofday" );
+		exit( 2 );
+	    }
+	    waiting = htons( tv.tv_sec - stv.tv_sec );
+	}
+	memcpy(cbuf +  6, &waiting, sizeof( waiting ));
+
+	atpb.atp_saddr = &nn.nn_sat;
+	atpb.atp_sreqdata = cbuf;
+	atpb.atp_sreqdlen = 8;		/* bytes in OpenConn request */
+	atpb.atp_sreqto = 2;		/* retry timer */
+	atpb.atp_sreqtries = 5;		/* retry count */
+	if ( atp_sreq( atp, &atpb, 1, ATP_XO ) < 0 ) {
+	    perror( "atp_sreq" );
+	    exit( 1 );
+	}
+
+DEBUG( printf( "OPEN >\n" ), fflush( stdout ));
+
+	iov.iov_base = rbuf;
+	iov.iov_len = sizeof( rbuf );
+	atpb.atp_rresiov = &iov;
+	atpb.atp_rresiovcnt = 1;
+	if ( atp_rresp( atp, &atpb ) < 0 ) {
+	    perror( "atp_rresp" );
+	    if ( connattempts-- <= 0 ) {
+		fprintf( stderr, "Can't connect!\n" );
+		exit( 1 );
+	    }
+	    continue;
+	}
+
+	/* sanity */
+	if ( iov.iov_len < 8 || (unsigned char)rbuf[ 0 ] != connid ||
+		rbuf[ 1 ] != PAP_OPENREPLY ) {
+	    fprintf( stderr, "Bad response!\n" );
+	    continue;	/* This is weird, since TIDs must match... */
+	}
+
+DEBUG( printf( "< OPENREPLY\n" ), fflush( stdout ));
+
+	if ( isatty( 1 )) {
+	    printf( "%.*s\n", (int)iov.iov_len - 9, (char *) iov.iov_base + 9 );
+	}
+	updatestatus( (char *) iov.iov_base + 9, iov.iov_len - 9 );
+
+	memcpy( &result, rbuf +  6,  sizeof( result ));
+	if ( result != 0 ) {
+	    sleep( 2 );
+	} else {
+	    memcpy( &sat, &nn.nn_sat, sizeof( struct sockaddr_at ));
+	    sat.sat_port = rbuf[ 4 ];
+	    quantum = rbuf[ 5 ];
+	    break;
+	}
+    }
+
+    if ( isatty( 1 )) {
+	printf( "Connected to %.*s:%.*s@%.*s.\n",
+		nn.nn_objlen, nn.nn_obj,
+		nn.nn_typelen, nn.nn_type,
+		nn.nn_zonelen, nn.nn_zone );
+    }
+
+    if ( optind == ac ) {
+	send_file( 0, atp, 1 );
+    } else {
+	for (; optind < ac; optind++ ) {
+	    if ( strcmp( av[ optind ], "-" ) == 0 ) {
+		fd = 0;
+	    } else if (( fd = open( av[ optind ], O_RDONLY )) < 0 ) {
+		perror( av[ optind ] );
+		continue;
+	    }
+	    send_file( fd, atp, ( optind == ac - 1 ) ? 1 : 0 );
+	    if ( fd != 0 ) {
+		close( fd );
+	    }
+	}
+    }
+
+    /*
+     * Close connection.
+     */
+    cbuf[ 0 ] = connid;
+    cbuf[ 1 ] = PAP_CLOSE;
+    cbuf[ 2 ] = cbuf[ 3 ] = 0;
+
+    atpb.atp_saddr = &sat;
+    atpb.atp_sreqdata = cbuf;
+    atpb.atp_sreqdlen = 4;		/* bytes in CloseConn request */
+    atpb.atp_sreqto = 2;		/* retry timer */
+    atpb.atp_sreqtries = 5;		/* retry count */
+    if ( atp_sreq( atp, &atpb, 1, ATP_XO ) < 0 ) {
+	perror( "atp_sreq" );
+	exit( 1 );
+    }
+
+DEBUG( printf( "CLOSE >\n" ), fflush( stdout ));
+
+    iov.iov_base = rbuf;
+    iov.iov_len = sizeof( rbuf );
+    atpb.atp_rresiov = &iov;
+    atpb.atp_rresiovcnt = 1;
+    if ( atp_rresp( atp, &atpb ) < 0 ) {
+	perror( "atp_rresp" );
+	exit( 1 );
+    }
+
+    /* sanity */
+    if ( iov.iov_len != 4 || rbuf[ 1 ] != PAP_CLOSEREPLY ) {
+	fprintf( stderr, "Bad response!\n" );
+	exit( 1 );
+    }
+
+#ifndef ZEROCONNID
+    /*
+     * The AGFA Viper Rip doesn't have the connection id in the close request.
+     */
+    if ((unsigned char)rbuf[ 0 ] != connid ) {
+	fprintf( stderr, "Bad connid in close!\n" );
+	exit( 1 );
+    }
+#endif /* ZEROCONNID */
+
+DEBUG( printf( "< CLOSEREPLY\n" ), fflush( stdout ));
+
+    if ( isatty( 1 )) {
+	printf( "Connection closed.\n" );
+    }
+    exit( 0 );
+}
+
 int		data = 0;
 unsigned char	port;
-unsigned short	seq = 0;
+u_int16_t       seq = 0, rseq = 1;
 
-sendfile( fd, atp, lastfile )
+int send_file( fd, atp, lastfile )
     int			fd;
     ATP			atp;
     int			lastfile;
 {
-    struct timeval	tv;
+    struct timeval	stv, tv;
     struct sockaddr_at	ssat;
     struct atp_block	atpb;
     fd_set		fds;
-    int			fiovcnt = 0, niovcnt = 0, eof = 0, senteof = 0, to = 0;
+    int			fiovcnt = 0, eof = 0, senteof = 0, to = 0;
     int			cc, i;
     unsigned short	netseq;
 
+    if ( gettimeofday( &stv, 0 ) < 0 ) {
+	perror( "gettimeofday" );
+	exit( 2 );
+    }
+
     /*
      * Ask for more data.
      */
     cbuf[ 0 ] = connid;
     cbuf[ 1 ] = PAP_READ;
-    if ( ++seq == 0xffff ) seq = 1;
+    if ( ++seq == 0 ) seq = 1;
     netseq = htons( seq );
-    bcopy( &netseq, &cbuf[ 2 ], sizeof( netseq ));
+    memcpy( cbuf +  2, &netseq, sizeof( netseq ));
     atpb.atp_saddr = &sat;
     atpb.atp_sreqdata = cbuf;
     atpb.atp_sreqdlen = 4;		/* bytes in SendData request */
@@ -398,32 +498,16 @@ sendfile( fd, atp, lastfile )
 	exit( 1 );
     }
 
-DEBUG( printf( "READ %d >\n", seq ));
+DEBUG( printf( "READ %d >\n", seq ), fflush( stdout ));
 
     for (;;) {
-	tv.tv_sec = 60;
-	tv.tv_usec = 0;
-
-	FD_ZERO( &fds );
-	if ( !waitforprinter && !eof && fiovcnt == 0 ) {
-	    FD_SET( fd, &fds );
-	}
-	FD_SET( atp_fileno( atp ), &fds );
-
-	if (( cc = select( FD_SETSIZE, &fds, 0, 0, &tv )) < 0 ) {
-	    perror( "select" );
+	if ( gettimeofday( &tv, 0 ) < 0 ) {
+	    perror( "gettimeofday" );
 	    exit( 2 );
 	}
 
-	/*
-	 * A timeout has occured. Keep track of it, and go ahead and
-	 * send a tickle.
-	 */
-	if ( cc == 0 ) {
-	    if ( to++ > 2 ) {
-		fprintf( stderr, "Connection timed out.\n" );
-		exit( 1 );
-	    }
+	if (( tv.tv_sec - stv.tv_sec ) >= 60 ) {
+	    stv = tv;
 
 	    /*
 	     * Send a tickle.
@@ -441,11 +525,32 @@ DEBUG( printf( "READ %d >\n", seq ));
 		exit( 1 );
 	    }
 
-DEBUG( printf( "TICKLE >\n" ));
+DEBUG( printf( "TICKLE >\n" ), fflush( stdout ));
+	}
 
+	tv.tv_sec = stv.tv_sec + 60 - tv.tv_sec;
+	tv.tv_usec = 0;
+
+	FD_ZERO( &fds );
+	if ( !waitforprinter && !eof && fiovcnt == 0 ) {
+	    FD_SET( fd, &fds );
+	}
+	FD_SET( atp_fileno( atp ), &fds );
+
+	if (( cc = select( FD_SETSIZE, &fds, 0, 0, &tv )) < 0 ) {
+	    perror( "select" );
+	    exit( 2 );
+	}
+
+	/*
+	 * A timeout has occured. Keep track of it.
+	 */
+	if ( cc == 0 ) {
+	    if ( to++ > 2 ) {
+		fprintf( stderr, "Connection timed out.\n" );
+		exit( 1 );
+	    }
 	    continue;
-	} else {
-	    to = 0;
 	}
 
 	/*
@@ -481,17 +586,27 @@ DEBUG( printf( "TICKLE >\n" ));
 		    perror( "atp_rreq" );
 		    exit( 1 );
 		}
-		/* sanity */
+
 		if ( (unsigned char)cbuf[ 0 ] != connid ) {
-		    fprintf( stderr, "Bad ATP request! (%X != %X)\n",
-			    cbuf[ 0 ], connid );
-		    exit( 1 );
+		    break;
 		}
 
+		/* reset timeout counter for all valid requests */
+		to = 0;
+
 		switch ( cbuf[ 1 ] ) {
 		case PAP_READ :
-
-DEBUG( printf( "< READ\n" ));
+		    memcpy( cbuf +  2, &netseq, sizeof( netseq ));
+DEBUG( printf( "< READ %d\n", ntohs( netseq )), fflush( stdout ));
+#ifdef notdef
+		    if ( netseq != 0 ) {
+			if ( rseq != ntohs( netseq )) {
+DEBUG( printf( "| DUP %d\n", rseq ), fflush( stdout ));
+			    break;
+			}
+			if ( rseq++ == 0xffff ) rseq = 1;
+		    }
+#endif /* notdef */
 
 		    data = 1;
 		    port = ssat.sat_port;
@@ -499,7 +614,7 @@ DEBUG( printf( "< READ\n" ));
 
 		case PAP_CLOSE :
 
-DEBUG( printf( "< CLOSE\n" ));
+DEBUG( printf( "< CLOSE\n" ), fflush( stdout ));
 
 		    /*
 		     * Respond to the close request, and fail.
@@ -516,14 +631,14 @@ DEBUG( printf( "< CLOSE\n" ));
 			exit( 1 );
 		    }
 
-DEBUG( printf( "CLOSEREPLY >\n" ));
+DEBUG( printf( "CLOSEREPLY >\n" ), fflush( stdout ));
 
 		    fprintf( stderr, "Connection closed by foreign host.\n" );
 		    exit( 1 );
 
 		case PAP_TICKLE :
 
-DEBUG( printf( "< TICKLE\n" ));
+DEBUG( printf( "< TICKLE\n" ), fflush( stdout ));
 
 		    break;
 		default :
@@ -533,6 +648,9 @@ DEBUG( printf( "< TICKLE\n" ));
 		break;
 
 	    case ATP_TRESP :
+		/* reset timeout counter for all valid requests */
+		to = 0;
+
 		atpb.atp_saddr = &ssat;
 		for ( i = 0; i < oquantum; i++ ) {
 		    rniov[ i ].iov_len = PAP_MAXDATA + 4;
@@ -553,7 +671,7 @@ DEBUG( printf( "< TICKLE\n" ));
 		    fprintf( stderr, "Bad data response!\n" );
 		    exit( 1 );
 		}
-#endif ZEROCONNID
+#endif /* ZEROCONNID */
 		if ( ((char *)rniov[ 0 ].iov_base)[ 1 ] != PAP_DATA ) {
 		    fprintf( stderr, "Bad data response!\n" );
 		    exit( 1 );
@@ -571,12 +689,12 @@ DEBUG( printf( "< TICKLE\n" ));
 		/* eof */
 		if ( ((char *)rniov[ 0 ].iov_base)[ 2 ] ) {
 
-DEBUG( printf( "< DATA (eof)\n" ));
+DEBUG( printf( "< DATA (eof)\n" ), fflush( stdout ));
 
 		    return( 0 );
 		}
 
-DEBUG( printf( "< DATA\n" ));
+DEBUG( printf( "< DATA\n" ), fflush( stdout ));
 
 
 		/*
@@ -584,9 +702,9 @@ DEBUG( printf( "< DATA\n" ));
 		 */
 		cbuf[ 0 ] = connid;
 		cbuf[ 1 ] = PAP_READ;
-		if ( ++seq == 0xffff ) seq = 1;
+		if ( ++seq == 0 ) seq = 1;
 		netseq = htons( seq );
-		bcopy( &netseq, &cbuf[ 2 ], sizeof( netseq ));
+		memcpy( cbuf +  2, &netseq, sizeof( netseq ));
 		atpb.atp_saddr = &sat;
 		atpb.atp_sreqdata = cbuf;
 		atpb.atp_sreqdlen = 4;		/* bytes in SendData request */
@@ -597,13 +715,13 @@ DEBUG( printf( "< DATA\n" ));
 		    exit( 1 );
 		}
 
-DEBUG( printf( "READ %d >\n", seq ));
+DEBUG( printf( "READ %d >\n", seq ), fflush( stdout ));
 
 		break;
 
 	    case 0:
 
-DEBUG( printf( "| RETRANS\n" ));
+DEBUG( printf( "| RETRANS\n" ), fflush( stdout ));
 
 		break;
 
@@ -642,7 +760,7 @@ DEBUG( printf( "| RETRANS\n" ));
 	    }
 	    data = fiovcnt = 0;
 
-DEBUG( printf( "DATA %s\n", eof ? "(eof) >" : ">" ));
+DEBUG( printf( "DATA %s\n", eof ? "(eof) >" : ">" ), fflush( stdout ));
 
 	    /*
 	     * The Apple LaserWriter IIf, the HP LWIIISi, and IV, don't
@@ -672,7 +790,7 @@ DEBUG( printf( "DATA %s\n", eof ? "(eof) >" : ">" ));
 		exit( 1 );
 	    }
 
-DEBUG( printf( "SENDSTATUS >\n" ));
+DEBUG( printf( "SENDSTATUS >\n" ), fflush( stdout ));
 
 	    atpb.atp_saddr = &nn.nn_sat;
 	    rniov[ 0 ].iov_len = PAP_MAXDATA + 4;
@@ -692,7 +810,7 @@ DEBUG( printf( "SENDSTATUS >\n" ));
 		fprintf( stderr, "Bad status response!\n" );
 		exit( 1 );
 	    }
-#endif NONZEROSTATUS
+#endif /* NONZEROSTATUS */
 
 	    if ( ((char *)rniov[ 0 ].iov_base)[ 1 ] != PAP_STATUS ||
 		    atpb.atp_rresiovcnt != 1 ) {
@@ -700,28 +818,28 @@ DEBUG( printf( "SENDSTATUS >\n" ));
 		exit( 1 );
 	    }
 
-DEBUG( printf( "< STATUS\n" ));
+DEBUG( printf( "< STATUS\n" ), fflush( stdout ));
 
 #ifdef FUCKED
 	    if ( waitforprinter ) {
 		char	st_buf[ 1024 ];	/* XXX too big */
 
-		bcopy( rniov[ 0 ].iov_base + 9, st_buf,
+		memcpy( st_buf, (char *) rniov[ 0 ].iov_base + 9, 
 			((char *)rniov[ 0 ].iov_base)[ 8 ] );
-		st_buf[ ((char *)rniov[ 0 ].iov_base)[ 8 ]] = '\0';
+		st_buf[ (int) ((char *)rniov[ 0 ].iov_base)[ 8 ]] = '\0';
 		if ( strstr( st_buf, "waiting" ) != NULL ) {
 		    waitforprinter = 0;
 		}
 	    }
-#endif FUCKED
+#endif /* FUCKED */
 
-	    updatestatus( rniov[ 0 ].iov_base + 9,
+	    updatestatus( (char *) rniov[ 0 ].iov_base + 9,
 		    ((char *)rniov[ 0 ].iov_base)[ 8 ] );
 	}
     }
 }
 
-updatestatus( s, len )
+void updatestatus( s, len )
     char	*s;
     int		len;
 {
