@@ -1,34 +1,78 @@
 /*
+ * $Id: config.c,v 1.6.2.1 2002/01/14 02:54:47 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <sys/syslog.h>
+#include <syslog.h>
 #include <sys/param.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <netatalk/at.h>
 #include <netatalk/endian.h>
 #include <atalk/paths.h>
+#include <atalk/util.h>
+#include <assert.h>
 #include <stdio.h>
-#include <strings.h>
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 #include <ctype.h>
+
+/* STDC check */
+#if STDC_HEADERS
+#include <string.h>
+#else /* STDC_HEADERS */
+#ifndef HAVE_STRCHR
+#define strchr index
+#define strrchr index
+#endif /* HAVE_STRCHR */
+char *strchr (), *strrchr ();
+#ifndef HAVE_MEMCPY
+#define memcpy(d,s,n) bcopy ((s), (d), (n))
+#define memmove(d,s,n) bcopy ((s), (d), (n))
+#endif /* ! HAVE_MEMCPY */
+#endif /* STDC_HEADERS */
+
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+
+#ifdef __svr4__
+#include <sys/sockio.h>
+#include <sys/stropts.h>
+#endif /* __svr4__ */
 
 #include "interface.h"
+#include "multicast.h"
 #include "rtmp.h"
 #include "zip.h"
 #include "list.h"
 
-int	seed(), phase(), net(), addr(), zone();
+#ifndef IFF_SLAVE /* a little backward compatibility */
+#define IFF_SLAVE 0
+#endif /* IFF_SLAVE */
 
-struct param {
+int	router(), dontroute(), seed(), phase(), net(), addr(), zone();
+
+static struct param {
     char	*p_name;
     int		(*p_func)();
 } params[] = {
+    { "router", router },
+    { "dontroute", dontroute },
     { "seed",	seed },
     { "phase",	phase },
     { "net",	net },
@@ -36,56 +80,101 @@ struct param {
     { "zone",	zone },
 };
 
-    char **
-parseline( line )
-    char	*line;
+#define ARGV_CHUNK_SIZE 128
+char **parseline(const char *line)
 {
-    char	*p;
-    int		argc = 0;
-    static char	*argv[ 128 ];
-    static char	buf[ 1024 ];
+    const char	 *p;
+    int		  argc = 0;
+    char	 *buffer, *tmpbuf;
+    char	**argv;
 
-    if ( *line == '#' ) {
-	return( NULL );
+    /* Ignore empty lines and lines with leading hash marks. */
+    p = line;
+    while ( isspace( *p ) ) {
+	p++;
+    }
+    if ( *p == '#' || *p == '\0' ) {
+	return NULL;
     }
-    argc = 0;
 
-    strcpy( buf, line );
-    p = buf;
+    buffer = (char *) malloc( strlen( p ) + 1 );
+    if ( !buffer ) {
+	/* FIXME: error handling */
+	return NULL;
+    }
+    strcpy( buffer, p );
+    tmpbuf = buffer;
+
+    argv = (char **) malloc( ARGV_CHUNK_SIZE * sizeof( char * ) );
+    if ( !argv ) {
+	/* FIXME: error handling */
+	free( buffer );
+	return NULL;
+    }
 
     /*
      * This parser should be made more powerful -- it should
      * handle various escapes, e.g. \" and \031.
      */
-    while ( *p != '\0' ) {
-	while ( *p != '\0' && *p != '"' && isspace( *p )) {
-	    p++;
-	}
-	if ( *p == '\0' ) {
-	    argv[ argc ] = 0;
-	    break;
-	}
-	if ( *p == '"' ) {
-	    argv[ argc ] = ++p;
-	    while ( *p != '\0' && *p != '"' ) {
-		p++;
+    do {
+	if ( *tmpbuf == '"' ) {
+	    argv[ argc++ ] = ++tmpbuf;
+	    while ( *tmpbuf != '\0' && *tmpbuf != '"' ) {
+		tmpbuf++;
+	    }
+	    if ( *tmpbuf == '"' ) {
+		/* FIXME: error handling */
 	    }
 	} else {
-	    argv[ argc ] = p;
-	    while ( *p != '\0' && !isspace( *p )) {
-		p++;
+	    argv[ argc++ ] = tmpbuf;
+	    while ( *tmpbuf != '\0' && !isspace( *tmpbuf )) {
+		tmpbuf++;
 	    }
 	}
-	*p++ = '\0';
-	argc++;
-    }
-    if ( argv[ 0 ] == '\0' || *argv[ 0 ] == '#' ) {
-	return( NULL );
-    }
-    return( argv );
+	*tmpbuf++ = '\0';
+
+	/* Make room for a NULL pointer and our special pointer (s.b.) */
+	if ( (argc + 1) % ARGV_CHUNK_SIZE == 0 ) {
+	    char **tmp;
+	    tmp = (char **) realloc( argv, argc + 1 + ARGV_CHUNK_SIZE );
+	    if ( !tmp ) {
+		/* FIXME: error handling */
+		free( argv );
+		free( buffer );
+		return NULL;
+	    }
+	    argv = tmp;
+	}
+
+	/* Skip white spaces. */
+        while ( isspace( *tmpbuf ) ) {
+            tmpbuf++;
+        }
+    } while ( *tmpbuf != '\0' );
+
+    argv[ argc++ ] = NULL;
+    /* We store our buffer pointer in argv, too, so we can free it later.
+     * (But don't tell anyone.)
+     */
+    argv[ argc ] = buffer;
+
+    return argv;
 }
 
-writeconf( cf )
+void freeline( char **argv )
+{
+    char **tmp = argv;
+
+    if ( argv ) {
+	while ( *tmp ) {
+	    tmp++;
+	}
+	free( *++tmp );
+	free( argv );
+    }
+}
+
+int writeconf( cf )
     char	*cf;
 {
     struct stat		st;
@@ -111,10 +200,10 @@ writeconf( cf )
 	 mode = st.st_mode;
     }
 
-    if (( p = rindex( path, '/' )) == NULL ) {
+    if (( p = strrchr( path, '/' )) == NULL ) {
 	strcpy( newpath, _PATH_ATALKDTMP );
     } else {
-	sprintf( newpath, "%.*s/%s", p - path, path, _PATH_ATALKDTMP );
+	sprintf( newpath, "%.*s/%s", (int)(p - path), path, _PATH_ATALKDTMP );
     }
     if (( fd = open( newpath, O_WRONLY|O_CREAT|O_TRUNC, mode )) < 0 ) {
 	syslog( LOG_ERR, "%s: %m", newpath );
@@ -138,15 +227,22 @@ writeconf( cf )
 		syslog( LOG_ERR, "fputs: %m" );
 		return( -1 );
 	    }
+	    freeline( argv );
 	    continue;
 	}
 
 	/* write real lines */
 	if ( iface ) {
 	    fprintf( newconf, "%s", iface->i_name );
-	    if ( iface->i_flags & IFACE_SEED ) {
+	    if ( iface->i_flags & IFACE_RSEED ) {
+		fprintf( newconf, " -router" );
+	    } else if ( iface->i_flags & IFACE_SEED ) {
 		fprintf( newconf, " -seed" );
 	    }
+	    if ( iface->i_flags & IFACE_DONTROUTE) {
+	        fprintf( newconf, " -dontroute");
+	    }
+
 	    fprintf( newconf, " -phase %d",
 		    ( iface->i_flags & IFACE_PHASE1 ) ? 1 : 2 );
 	    fprintf( newconf, " -net %d", ntohs( iface->i_rt->rt_firstnet ));
@@ -203,7 +299,7 @@ writeconf( cf )
  * zone for an interface is the first zone encountered for that
  * interface.
  */
-readconf( cf )
+int readconf( cf )
     char		*cf;
 {
     struct ifreq	ifr;
@@ -218,40 +314,63 @@ readconf( cf )
 	p = cf;
     }
     if (( conf = fopen( p, "r" )) == NULL ) {
-	if ( cf ) {
-	    perror( cf );
-	    exit( 1 );
-	} else {
-	    return( -1 );
-	}
+        return( -1 );
     }
 
+#ifndef __svr4__
     if (( s = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
 	perror( "socket" );
-	exit( 1 );
+	fclose(conf);
+	return -1;
     }
+#endif /* __svr4__ */
 
     while ( fgets( line, sizeof( line ), conf ) != NULL ) {
 	if (( argv = parseline( line )) == NULL ) {
 	    continue;
 	}
 
+#ifndef __svr4__
 	/*
 	 * Check that av[ 0 ] is a valid interface.
+	 * Not possible under sysV.
 	 */
-	strcpy( ifr.ifr_name, argv[ 0 ] );
+	strncpy( ifr.ifr_name, argv[ 0 ], sizeof(ifr.ifr_name) );
+	ifr.ifr_name[sizeof(ifr.ifr_name) - 1] = '\0';
+
+	/* for devices that don't support appletalk */
+	if ((ioctl(s, SIOCGIFADDR, &ifr) < 0) && (errno == ENODEV)) {
+	  perror(argv[0]);
+	  goto read_conf_err;
+	}
+
 	if ( ioctl( s, SIOCGIFFLAGS, &ifr ) < 0 ) {
 	    perror( argv[ 0 ] );
-	    exit( 1 );
+	    goto read_conf_err;
 	}
-	if ( ifr.ifr_flags & ( IFF_LOOPBACK | IFF_POINTOPOINT )) {
+
+	if (ifr.ifr_flags & (IFF_LOOPBACK | IFF_POINTOPOINT |IFF_SLAVE)) {
 	    fprintf( stderr, "%s: can't configure.\n", ifr.ifr_name );
-	    exit( 1 );
+	    goto read_conf_err;
 	}
 
+#ifdef IFF_MULTICAST
+	if ((ifr.ifr_flags & IFF_MULTICAST) == 0)
+	    fprintf(stderr, "%s: multicast may not work properly.\n",
+		    ifr.ifr_name);
+#endif /* IFF_MULTICAST */
+
+	/* configure hw multicast for this interface. */
+	if (addmulti(ifr.ifr_name, NULL) < 0) {
+	  perror(ifr.ifr_name);
+	  fprintf(stderr, "Can't configure multicast.\n");
+	  goto read_conf_err;
+	}
+#endif /* __svr4__ */
+
 	if (( niface = newiface( argv[ 0 ] )) == NULL ) {
 	    perror( "newiface" );
-	    exit( 1 );
+	    goto read_conf_err;
 	}
 
 	for ( i = 1; argv[ i ]; i += cc ) {
@@ -262,13 +381,15 @@ readconf( cf )
 		if ( strcmp( argv[ i ], params[ j ].p_name ) == 0 ) {
 		    if ( params[ j ].p_func != NULL ) {
 			cc = (*params[ j ].p_func)( niface, &argv[ i + 1 ] );
+			if (cc < 0) 
+			  goto read_conf_err;
 			break;
 		    }
 		}
 	    }
 	    if ( j >= sizeof( params ) / sizeof( params[ 0 ] )) {
 		fprintf( stderr, "%s: attribute not found.\n", argv[ i ] );
-		exit( 1 );
+		goto read_conf_err;
 	    }
 	}
 
@@ -281,8 +402,8 @@ readconf( cf )
 	    }
 	}
 	if ( iface ) {	/* Already have this interface and phase */
-	    fprintf( stderr, "Specified %s twice.\n", niface->i_name );
-	    exit( 1 );
+	    fprintf( stderr, "%s already configured!\n", niface->i_name );
+	    goto read_conf_err;
 	}
 
 	if ( interfaces == NULL ) {
@@ -295,7 +416,10 @@ readconf( cf )
 	niface->i_next = NULL;
     }
 
+#ifndef __svr4__
     close( s );
+#endif /* __svr4__ */
+
     fclose( conf );
 
     /*
@@ -304,31 +428,79 @@ readconf( cf )
      */
     for ( iface = interfaces, cc = 0; iface; iface = iface->i_next, cc++ )
 	;
-    if ( cc > 1 ) {
+    if ( cc >= IFBASE ) {
 	return( 0 );
     } else {
 	return( -1 );
     }
+
+read_conf_err:
+#ifndef __svr4__
+    close(s);
+#endif /* __svr4__ */
+    fclose(conf);
+    return -1;
 }
 
 /*ARGSUSED*/
-seed( iface, av )
+int router( iface, av )
+    struct interface	*iface;
+    char		**av;
+{
+    /* make sure "-router" and "-dontroute" aren't both on the same line. */
+    if (iface->i_flags & IFACE_DONTROUTE) {
+	fprintf( stderr, "Can't specify both -router and -dontroute.\n");
+	return -1;
+    }
+
+    /*
+     * Check to be sure "-router" is before "-zone".
+     */
+    if ( iface->i_czt ) {
+	fprintf( stderr, "Must specify -router before -zone.\n");
+	return -1;
+    }
+
+    /* -router also implies -seed */
+    iface->i_flags |= IFACE_RSEED | IFACE_SEED | IFACE_ISROUTER;
+    return( 1 );
+}
+
+/*ARGSUSED*/
+int dontroute( iface, av )
+    struct interface	*iface;
+    char		**av;
+{
+    /* make sure "-router" and "-dontroute" aren't both on the same line. */
+    if (iface->i_flags & IFACE_RSEED) {
+	fprintf( stderr, "Can't specify both -router and -dontroute.\n");
+	return -1;
+    }
+
+    iface->i_flags |= IFACE_DONTROUTE;
+    return( 1 );
+}
+
+/*ARGSUSED*/
+int seed( iface, av )
     struct interface	*iface;
     char		**av;
 {
     /*
-     * Check to be sure "-seed" is before "-zone".
+     * Check to be sure "-seed" is before "-zone". we keep the old
+     * semantics of just ignoring this in a routerless world.
      */
     if ( iface->i_czt ) {
-	fprintf( stderr, "Must specify -seed before -zone.\n" );
-	exit( 1 );
+	fprintf( stderr, "Must specify -seed before -zone(%s).\n",
+		 iface->i_czt->zt_name);
+	return -1;
     }
 
     iface->i_flags |= IFACE_SEED;
     return( 1 );
 }
 
-phase( iface, av )
+int phase( iface, av )
     struct interface	*iface;
     char		**av;
 {
@@ -337,7 +509,7 @@ phase( iface, av )
 
     if (( pnum = av[ 0 ] ) == NULL ) {
 	fprintf( stderr, "No phase.\n" );
-	exit( 1 );
+	return -1;
     }
 
     switch ( n = atoi( pnum )) {
@@ -351,12 +523,12 @@ phase( iface, av )
 
     default :
 	fprintf( stderr, "No phase %d.\n", n );
-	exit( 1 );
+	return -1;
     }
     return( 2 );
 }
 
-net( iface, av )
+int net( iface, av )
     struct interface	*iface;
     char		**av;
 {
@@ -366,33 +538,33 @@ net( iface, av )
 
     if (( nrange = av[ 0 ] ) == NULL ) {
 	fprintf( stderr, "No network.\n" );
-	exit( 1 );
+	return -1;
     }
 
-    if (( stop = index( nrange, '-' )) != 0 ) {
+    if (( stop = strchr( nrange, '-' )) != 0 ) {
 	stop++;
     }
     net = atoi( nrange );
     if ( net < 0 || net >= 0xffff ) {
-	fprintf( stderr, "Bad network: %d\n" );
-	exit( 1 );
+	fprintf( stderr, "Bad network: %d\n", net );
+	return -1;
     }
 
-    if ( iface->i_rt == NULL && ( iface->i_rt = newrt()) == NULL ) {
+    if ( iface->i_rt == NULL && ( iface->i_rt = newrt(iface)) == NULL ) {
 	perror( "newrt" );
-	exit( 1 );
+	return -1;
     }
 
     if ( iface->i_flags & IFACE_PHASE1 ) {
 	if ( stop != 0 ) {
 	    fprintf( stderr, "Phase 1 doesn't use an address range.\n" );
-	    exit( 1 );
+	    return -1;
 	}
 	if ( iface->i_caddr.sat_addr.s_net != ATADDR_ANYNET &&
 		ntohs( iface->i_caddr.sat_addr.s_net ) != net ) {
 	    fprintf( stderr, "Net-range (%u) doesn't match net %u.\n",
 		    net, ntohs( iface->i_caddr.sat_addr.s_net ));
-	    exit( 1 );
+	    return -1;
 	}
 	iface->i_rt->rt_firstnet = iface->i_rt->rt_lastnet = htons( net );
     } else if ( iface->i_flags & IFACE_PHASE2 ) {
@@ -400,8 +572,8 @@ net( iface, av )
 	if ( stop != 0 ) {
 	    net = atoi( stop );
 	    if ( net < 0 || net >= 0xffff ) {
-		fprintf( stderr, "Bad network: %d\n" );
-		exit( 1 );
+		fprintf( stderr, "Bad network: %d\n", net );
+		return -1;
 	    }
 	}
 	iface->i_rt->rt_lastnet = htons( net );
@@ -414,29 +586,29 @@ net( iface, av )
 		    ntohs( iface->i_rt->rt_firstnet ),
 		    ntohs( iface->i_rt->rt_lastnet ),
 		    ntohs( iface->i_caddr.sat_addr.s_net ));
-	    exit( 1 );
+	    return -1;
 	}
 	if ( iface->i_rt->rt_firstnet != iface->i_rt->rt_lastnet ) {
 	    iface->i_rt->rt_flags |= RTMPTAB_EXTENDED;
 	}
     } else {
 	fprintf( stderr, "Must specify phase before networks.\n" );
-	exit( 1 );
+	return -1;
     }
     return( 2 );
 }
 
-addr( iface, av )
+int addr( iface, av )
     struct interface	*iface;
     char		**av;
 {
     if ( av[ 0 ] == NULL ) {
 	fprintf( stderr, "No address.\n" );
-	exit( 1 );
+	return -1;
     }
     if ( atalk_aton( av[ 0 ], &iface->i_caddr.sat_addr ) == 0 ) {
 	fprintf( stderr, "Bad address, %s\n", av[ 0 ] );
-	exit( 1 );
+	return -1;
     }
 
     if ( iface->i_rt ) {
@@ -448,12 +620,12 @@ addr( iface, av )
 		    ntohs( iface->i_caddr.sat_addr.s_net ),
 		    ntohs( iface->i_rt->rt_firstnet ),
 		    ntohs( iface->i_rt->rt_lastnet ));
-	    exit( 1 );
+	    return -1;
 	}
     } else {
-	if (( iface->i_rt = newrt()) == NULL ) {
+	if (( iface->i_rt = newrt(iface)) == NULL ) {
 	    perror( "newrt" );
-	    exit( 1 );
+	    return -1;
 	}
 	iface->i_rt->rt_firstnet = iface->i_rt->rt_lastnet =
 		iface->i_caddr.sat_addr.s_net;
@@ -462,7 +634,7 @@ addr( iface, av )
     return( 2 );
 }
 
-zone( iface, av )
+int zone( iface, av )
     struct interface	*iface;
     char		**av;
 {
@@ -471,12 +643,7 @@ zone( iface, av )
 
     if (( zname = av[ 0 ] ) == NULL ) {
 	fprintf( stderr, "No zone.\n" );
-	exit( 1 );
-    }
-
-    if ( iface->i_rt == NULL ) {
-	fprintf( stderr, "Must specify net-range before zones.\n" );
-	exit( 1 );
+	return -1;
     }
 
     /*
@@ -485,9 +652,13 @@ zone( iface, av )
      * check that the network has given us good zones.
      */
     if ( iface->i_flags & IFACE_SEED ) {
+        if ( iface->i_rt == NULL ) {
+  	    fprintf( stderr, "Must specify net-range before zones.\n" );
+	    return -1;
+        }
 	if (( zt = newzt( strlen( zname ), zname )) == NULL ) {
 	    perror( "newzt" );
-	    exit( 1 );
+	    return -1;
 	}
 	if ( iface->i_czt == NULL ) {
 	    iface->i_czt = zt;
@@ -503,37 +674,38 @@ zone( iface, av )
  * Get the configuration from the kernel. Only called if there's no
  * configuration.
  */
-getifconf()
+int getifconf()
 {
-    struct ifconf	ifc;
-    struct ifreq	ifrs[ 10 ], *ifr;
     struct interface	*iface, *niface;
+    struct ifreq        ifr;
+    char                **start, **list;
     int			s;
 
     if (( s = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
 	perror( "socket" );
-	exit( 1 );
+	return -1;
     }
 
-    ifc.ifc_len = sizeof( ifrs );
-    ifc.ifc_buf = (caddr_t)ifrs;
-    if ( ioctl( s, SIOCGIFCONF, &ifc ) < 0 ) {
-	perror( "gifconf" );
-	exit( 1 );
-    }
+    start = list = getifacelist();
+    while (list && *list) {
+        strncpy(ifr.ifr_name, *list, sizeof(ifr.ifr_name));
+	list++;
 
-    for ( ifr = ifc.ifc_req; ifc.ifc_len >= sizeof( struct ifreq );
-	    ifc.ifc_len -= sizeof( struct ifreq ), ifr++ ) {
-	if ( ioctl( s, SIOCGIFFLAGS, ifr ) < 0 ) {
-	    perror( ifr->ifr_name );
-	    exit( 1 );
-	}
-	if ( ifr->ifr_flags & ( IFF_LOOPBACK | IFF_POINTOPOINT )) {
-	    continue;
-	}
+	if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0)
+	  continue;
+
+	if (ifr.ifr_flags & (IFF_LOOPBACK | IFF_POINTOPOINT | IFF_SLAVE))
+	  continue;
+
+	if ((ifr.ifr_flags & IFF_UP) == 0)
+	  continue;
+
+	/* for devices that don't support appletalk */
+	if (ioctl(s, SIOCGIFADDR, &ifr) < 0 && (errno == ENODEV))
+	  continue;
 
 	for ( iface = interfaces; iface; iface = iface->i_next ) {
-	    if ( strcmp( iface->i_name, ifr->ifr_name ) == 0 ) {
+	    if ( strcmp( iface->i_name, ifr.ifr_name ) == 0 ) {
 		break;
 	    }
 	}
@@ -541,9 +713,23 @@ getifconf()
 	    continue;
 	}
 
-	if (( niface = newiface( ifr->ifr_name )) == NULL ) {
+
+#ifdef IFF_MULTICAST
+	if ((ifr.ifr_flags & IFF_MULTICAST) == 0)
+	  fprintf(stderr, "%s: multicast may not work correctly.\n",
+		  ifr.ifr_name);
+#endif /* IFF_MULTICAST */
+
+	if (addmulti(ifr.ifr_name, NULL) < 0) {
+	  fprintf(stderr, "%s: disabled.\n", ifr.ifr_name);
+	  continue;
+	}
+	
+	if (( niface = newiface( ifr.ifr_name )) == NULL ) {
 	    perror( "newiface" );
-	    exit( 1 );
+	    close(s);
+	    freeifacelist(start);
+	    return -1;
 	}
 	/*
 	 * Could try to get the address from the kernel...
@@ -558,11 +744,7 @@ getifconf()
 	}
 	niface->i_next = NULL;
     }
-    if ( ifc.ifc_len != 0 ) {
-	fprintf( stderr, "Funky gifconf return.\n" );
-	exit( 1 );
-    }
-
+    freeifacelist(start);
     (void)close( s );
     return( 0 );
 }
@@ -571,29 +753,75 @@ getifconf()
  * Allocate a new interface structure.  Centralized here so we can change
  * the interface structure and have it updated nicely.
  */
-    struct interface *
-newiface( name )
-    char		*name;
+
+struct interface *newiface( name )
+    const char		*name;
 {
     struct interface	*niface;
 
-    if (( niface = (struct interface *)malloc( sizeof( struct interface )))
+    if (( niface = (struct interface *)calloc(1, sizeof( struct interface )))
 	    == NULL ) {
 	return( NULL );
     }
-    niface->i_next = NULL;
-    strcpy( niface->i_name, name );
-    niface->i_flags = 0;
-    niface->i_time = 0;
+    strncpy( niface->i_name, name, sizeof(niface->i_name));
+#ifdef BSD4_4
+    niface->i_addr.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     niface->i_addr.sat_family = AF_APPLETALK;
-    niface->i_addr.sat_addr.s_net = htons( ATADDR_ANYNET );
-    niface->i_addr.sat_addr.s_node = ATADDR_ANYNODE;
+#ifdef BSD4_4
+    niface->i_caddr.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     niface->i_caddr.sat_family = AF_APPLETALK;
-    niface->i_caddr.sat_addr.s_net = htons( ATADDR_ANYNET );
-    niface->i_caddr.sat_addr.s_node = ATADDR_ANYNODE;
-    niface->i_rt = NULL;
-    niface->i_gate = NULL;
-    niface->i_ports = NULL;
-
     return( niface );
 }
+
+#ifdef __svr4__
+int plumb()
+{
+    struct interface	*iface;
+    char		device[ MAXPATHLEN + 1], *p;
+    int			fd, ppa;
+
+    for ( iface = interfaces; iface != NULL; iface = iface->i_next ) {
+	if ( strcmp( iface->i_name, LOOPIFACE ) == 0 ) {
+	    continue;
+	}
+
+	strcpy( device, "/dev/" );
+	strcat( device, iface->i_name );
+	if (( p = strpbrk( device, "0123456789" )) == NULL ) {
+	    syslog( LOG_ERR, "plumb: invalid device: %s", device );
+	    return -1;
+	}
+	ppa = atoi( p );
+	*p = '\0';
+
+	if (( fd = open( device, O_RDWR, 0 )) < 0 ) {
+	    syslog( LOG_ERR, "%s: %m", device );
+	    return -1;
+	}
+	if ( ioctl( fd, I_PUSH, "ddp" ) < 0 ) {
+	    syslog( LOG_ERR, "I_PUSH: %m" );
+	    close(fd);
+	    return -1;
+	}
+	if ( ioctl( fd, IF_UNITSEL, ppa ) < 0 ) {
+	    syslog( LOG_ERR, "IF_UNITSEL: %m" );
+	    close(fd);
+	    return -1;
+	}
+
+	/* configure multicast. */
+	if (addmulti(iface->i_name, NULL) < 0) {
+	  perror(iface->i_name);
+	  fprintf(stderr,"Can't configure multicast.\n");
+	  close(fd);
+	  return -1;
+	}
+
+	syslog( LOG_INFO, "plumbed %s%d", device, ppa );
+    }
+
+    return( 0 );
+}
+#endif /* __svr4__ */
