@@ -1,32 +1,69 @@
 /*
+ * $Id: main.c,v 1.10.2.2 2002/03/12 15:36:10 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
-#include <sys/types.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/param.h>
 #include <sys/socket.h>
+#if defined( sun ) && defined( __svr4__ )
+#include </usr/ucbinclude/sys/file.h>
+#else /* sun __svr4__ */
+#include <sys/file.h>
+#endif /* sun __svr4__ */
 #include <sys/time.h>
 #include <sys/resource.h>
-
-#include <sys/file.h>
 #include <sys/ioctl.h>
+
+/* POSIX.1 check */
+#include <sys/types.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
-#include <sys/errno.h>
+#endif /* HAVE_SYS_WAIT_H */
+#ifndef WEXITSTATUS 
+#define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
+#endif /* ! WEXITSTATUS */
+#ifndef WIFEXITED
+#define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
+#endif /* ! WIFEXITED */
+
+#include <errno.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <net/route.h>
-#include <netatalk/endian.h>
-#include <netatalk/at.h>
+
 #include <signal.h>
+#include <netinet/in.h>
 #include <syslog.h>
 #include <stdio.h>
-#include <strings.h>
+#include <stdlib.h>
+#include <string.h>
 #include <netdb.h>
+#include <fcntl.h>
+#include <unistd.h>
 
+#include <netatalk/endian.h>
+#include <netatalk/at.h>
+#include <atalk/compat.h>
 #include <atalk/zip.h>
 #include <atalk/rtmp.h>
 #include <atalk/ddp.h>
+#include <atalk/atp.h>
 #include <atalk/paths.h>
+#include <atalk/util.h>
+
+#ifdef __svr4__
+#include <sys/sockio.h>
+#include <termios.h>
+#endif /* __svr4__ */
 
 #include "interface.h"
 #include "gate.h"
@@ -34,22 +71,32 @@
 #include "rtmp.h"
 #include "zip.h"
 #include "atserv.h"
-#include "multicast.h"
+#include "main.h"
+
+/* Forward Declarations */
+int ifconfig(const char *iname, unsigned long cmd, struct sockaddr_at *sa);
+
+/* FIXME/SOCKLEN_T: socklen_t is a unix98 feature */
+#ifndef SOCKLEN_T
+#define SOCKLEN_T unsigned int
+#endif /* SOCKLEN_T */
 
 #ifndef WEXITSTATUS
 #define WEXITSTATUS(x)	((x).w_retcode)
-#endif WEXITSTATUS
+#endif /* WEXITSTATUS */
+
+/* linux has a special ioctl for appletalk device destruction.  as of
+ * 2.1.57, SIOCDIFADDR works w/ linux. okay, we need to deal with the
+ * fact that SIOCDIFADDR may be defined on linux despite the fact that
+ * it doesn't work. */
+#if !defined(SIOCDIFADDR) && defined(SIOCATALKDIFADDR)
+#define SIOCDIFADDR SIOCATALKDIFADDR
+#endif
 
 #define elements(a)	(sizeof(a)/sizeof((a)[0]))
 
 #define PKTSZ	1024
 
-#ifdef linux
-#define LOOPIFACE	"lo"
-#else linux
-#define LOOPIFACE	"lo0"
-#endif linux
-
 extern int	rtmp_packet();
 extern int	nbp_packet();
 extern int	aep_packet();
@@ -71,16 +118,49 @@ int		debug = 0, quiet = 0, chatty = 0;
 char		*configfile = NULL;
 int		ziptimeout = 0, transition = 0;
 int		stabletimer, stable = 0, newrtmpdata = 0, noparent = 0;
-int		ninterfaces;
+static int	ninterfaces;
 int		defphase = IFACE_PHASE2;
 int		nfds = 0;
 fd_set		fds;
 char		Packet[ PKTSZ ];
+char		*version = VERSION;
+static char     *pidfile = _PATH_ATALKDLOCK;
 
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif ibm032 _IBMR2
-as_timer()
+
+/* from config.c */
+
+int readconf( char * );
+int getifconf( void );
+int writeconf( char * );
+
+/* this is the messiest of the bunch as atalkd can exit pretty much
+ * everywhere. we delete interfaces here instead of in as_down. */
+static void atalkd_exit(const int i)
+{
+#ifdef SIOCDIFADDR
+  struct interface *iface;
+
+  for (iface = interfaces; iface; iface = iface->i_next) {
+    if (ifconfig( iface->i_name, SIOCDIFADDR, &iface->i_addr)) {
+#ifdef SIOCATALKDIFADDR
+#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+      if (!ifconfig(iface->i_name, SIOCATALKDIFADDR, &iface->i_addr)) 
+	continue;
+#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
+#endif /* SIOCATALKIFADDR */
+      syslog( LOG_ERR, "difaddr(%u.%u): %s", 
+	      ntohs(iface->i_addr.sat_addr.s_net), 
+	      iface->i_addr.sat_addr.s_node, strerror(errno));
+    }
+  }
+#endif /* SOPCDOFADDR */
+
+  server_unlock(pidfile);
+  exit(i);
+}
+
+
+void as_timer(void)
 {
     struct sockaddr_at	sat;
     struct ziphdr	zh;
@@ -91,12 +171,11 @@ as_timer()
     struct gate		*gate, *fgate = NULL;
     struct rtmptab	*rtmp, *frtmp;
     struct ziptab	*zt;
-    char		*data, *end, packet[ 587 ];
+    char		*data, *end, packet[ ATP_BUFSIZ ];
     int			sentzipq = 0;
-    int			n, cc, i;
-
-    bzero( (caddr_t)&sat, sizeof( struct sockaddr_at ));
+    int			n, cc;
 
+    memset(&sat, 0, sizeof( struct sockaddr_at ));
     for ( iface = interfaces; iface; iface = iface->i_next ) {
 	if ( iface->i_flags & IFACE_LOOPBACK ) {
 	    continue;
@@ -114,16 +193,22 @@ as_timer()
 		IFACE_ADDR ) {
 	    if ( iface->i_time < 3 ) {
 		if ( iface->i_flags & IFACE_PHASE1 ) {
-		    rtmp_request( iface );
+		  if (rtmp_request( iface ) < 0) {
+		      syslog(LOG_ERR, "rtmp_request: %s", strerror(errno));
+		      atalkd_exit(1);
+		  }
 		    newrtmpdata = 1;
 		} else {
-		    zip_getnetinfo( iface );
-		    sentzipq = 1;
+		  if (zip_getnetinfo( iface ) < 0) {
+		    syslog(LOG_ERR, "zip_getnetinfo: %s", strerror(errno));
+		    atalkd_exit(1);
+		  }
+		  sentzipq = 1;
 		}
 		iface->i_time++;
 	    } else {
 		iface->i_flags |= IFACE_NOROUTER;
-		if ( ninterfaces > IFBASE ) {
+		if ((iface->i_flags & IFACE_ISROUTER)) {
 		    if (( iface->i_flags & IFACE_SEED ) == 0 ) {
 			/*
 			 * No seed info, and we've got multiple interfaces.
@@ -143,7 +228,11 @@ as_timer()
 			 */
 			iface->i_flags |= IFACE_CONFIG;
 			for ( zt = iface->i_czt; zt; zt = zt->zt_next ) {
-			    addzone( iface->i_rt, zt->zt_len, zt->zt_name );
+			    if (addzone( iface->i_rt, zt->zt_len, 
+					 zt->zt_name) < 0) {
+			      syslog(LOG_ERR, "addzone: %s", strerror(errno));
+			      atalkd_exit(1);
+			    }
 			}
 			if ( iface->i_rt->rt_zt ) {
 			    iface->i_rt->rt_flags &= ~RTMPTAB_ZIPQUERY;
@@ -164,12 +253,13 @@ as_timer()
 				    iface->i_name );
 			}
 
-			if ( looproute( iface, RTMP_ADD ) < 0 ) {
+			if ( looproute( iface, RTMP_ADD )) { /* -1 or 1 */
 			    syslog( LOG_ERR,
-				    "as_timer: can't route %u.%u to loop: %m",
+				    "as_timer: can't route %u.%u to loop: %s",
 				    ntohs( iface->i_addr.sat_addr.s_net ),
-				    iface->i_addr.sat_addr.s_node );
-			    exit( 1 );
+				    iface->i_addr.sat_addr.s_node,
+				    strerror(errno) );
+			    atalkd_exit( 1 );
 			}
 			if ( iface == ciface ) {
 			    ciface = ciface->i_next;
@@ -182,6 +272,7 @@ as_timer()
 		     * to become available in rtmp_packet().
 		     */
 		    syslog( LOG_INFO, "config for no router" );
+		      
 		    if ( iface->i_flags & IFACE_PHASE2 ) {
 			iface->i_rt->rt_firstnet = 0;
 			iface->i_rt->rt_lastnet = htons( STARTUP_LASTNET );
@@ -190,12 +281,18 @@ as_timer()
 				iface->i_addr.sat_addr.s_node,
 				0, htons( STARTUP_LASTNET ));
 		    }
-		    if ( looproute( iface, RTMP_ADD ) < 0 ) {
+		    if ( looproute( iface, RTMP_ADD ) ) { /* -1 or 1 */
 			syslog( LOG_ERR,
-				"as_timer: can't route %u.%u to loopback: %m",
+				"as_timer: can't route %u.%u to loopback: %s",
 				ntohs( iface->i_addr.sat_addr.s_net ),
-				iface->i_addr.sat_addr.s_node );
-			exit( 1 );
+				iface->i_addr.sat_addr.s_node,
+				strerror(errno) );
+			atalkd_exit( 1 );
+		    }
+
+		    if ( iface == ciface ) {
+		      ciface = ciface->i_next;
+		      bootaddr( ciface );
 		    }
 		}
 	    }
@@ -231,7 +328,11 @@ as_timer()
 			rtmp_free( rtmp );
 		    } else {
 			rtmp->rt_hops = RTMPHOPS_POISON;
-			if ( rtmp_replace( rtmp ) > 0 ) {
+			if ((cc = rtmp_replace( rtmp )) < 0) {
+			  syslog(LOG_ERR, "rtmp_replace: %s", strerror(errno));
+			  atalkd_exit(1);
+			}
+			if (cc) {
 			    gate->g_state = rtmp->rt_state = RTMPTAB_GOOD;
 			}
 		    }
@@ -253,10 +354,22 @@ as_timer()
 		 * reconfigure our netrange.  By marking the interface
 		 * as having no router, we will notice when a router
 		 * comes back up.
+		 *
+		 * XXX: actually, we always reconfigure an interface
+		 * if we're not a seed router.
 		 */
-		if ( gate->g_iface->i_gate == 0 && ninterfaces == IFBASE ) {
+
+		if ( gate->g_iface->i_gate == 0 && 
+		     ((iface->i_flags & IFACE_SEED) == 0)) {
 		    gate->g_iface->i_flags |= IFACE_NOROUTER;
 		    gate->g_iface->i_flags &= ~IFACE_CONFIG;
+
+		    /* get rid of any zones associated with this iface */
+		    if (gate->g_iface->i_rt->rt_zt) {
+		      rtmp_delzonemap(gate->g_iface->i_rt);
+		      gate->g_iface->i_rt->rt_flags &= ~RTMPTAB_HASZONES;
+		    }
+
 		    syslog( LOG_INFO, "as_timer last gateway down" );
 
 		    /* Set netrange to 0-fffe.  */
@@ -279,7 +392,7 @@ as_timer()
 	     */
 	    if (( iface->i_rt->rt_flags & RTMPTAB_HASZONES ) == 0 ) {
 		iface->i_rt->rt_flags |= RTMPTAB_ZIPQUERY;
-		bcopy( &iface->i_rt->rt_firstnet, data, sizeof( u_short ));
+		memcpy( data, &iface->i_rt->rt_firstnet, sizeof( u_short ));
 		data += sizeof( u_short );
 		n++;
 	    }
@@ -311,9 +424,12 @@ as_timer()
 			    rtmp_free( rtmp );
 			} else {
 			    rtmp->rt_hops = RTMPHOPS_POISON;
-			    if ( rtmp_replace( rtmp ) > 0 ) {
-				rtmp->rt_state = RTMPTAB_GOOD;
+			    if ((cc = rtmp_replace( rtmp )) < 0) {
+			    	syslog(LOG_ERR, "rtmp_replace: %s", strerror(errno));
+			    	atalkd_exit(1);
 			    }
+		            if (cc)
+				rtmp->rt_state = RTMPTAB_GOOD;
 			}
 		    }
 		    rtmp = frtmp;
@@ -332,12 +448,12 @@ as_timer()
 			cc = data - packet;
 			data = packet;
 			*data++ = DDPTYPE_ZIP;
-			bcopy( &zh, data, sizeof( struct ziphdr ));
+			memcpy( data, &zh, sizeof( struct ziphdr ));
 
 			if ( sendto( zap->ap_fd, packet, cc, 0,
 				(struct sockaddr *)&sat,
 				sizeof( struct sockaddr_at )) < 0 ) {
-			    syslog( LOG_ERR, "as_timer sendto: %m" );
+			    syslog( LOG_ERR, "as_timer sendto: %s", strerror(errno) );
 			}
 			sentzipq = 1;
 
@@ -368,7 +484,7 @@ as_timer()
 			sentzipq = 1;
 		    }
 		    rtmp->rt_flags |= RTMPTAB_ZIPQUERY;
-		    bcopy( &rtmp->rt_firstnet, data, sizeof( u_short ));
+		    memcpy( data, &rtmp->rt_firstnet, sizeof( u_short ));
 		    data += sizeof( u_short );
 		    n++;
 		}
@@ -382,11 +498,11 @@ as_timer()
 		cc = data - packet;
 		data = packet;
 		*data++ = DDPTYPE_ZIP;
-		bcopy( &zh, data, sizeof( struct ziphdr ));
+		memcpy( data, &zh, sizeof( struct ziphdr ));
 
 		if ( sendto( zap->ap_fd, packet, cc, 0, (struct sockaddr *)&sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "as_timer sendto: %m" );
+		    syslog( LOG_ERR, "as_timer sendto: %s", strerror(errno) );
 		}
 	    }
 	}
@@ -396,12 +512,16 @@ as_timer()
 	}
 
 	/*
-	 * Send RTMP broadcasts.
+	 * Send RTMP broadcasts if we have multiple interfaces or our 
+	 * interface is configured as a router.  
 	 */
-	if ( ninterfaces > IFBASE ) {
+	if ((iface->i_flags & IFACE_ISROUTER)) {
+#ifdef BSD4_4
+	    sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
 	    sat.sat_family = AF_APPLETALK;
-	    sat.sat_addr.s_net = 0;
-	    sat.sat_addr.s_node = 0xff;
+	    sat.sat_addr.s_net = ATADDR_ANYNET;
+	    sat.sat_addr.s_node = ATADDR_BCAST;
 	    sat.sat_port = rap->ap_port;
 
 	    data = packet;
@@ -410,7 +530,7 @@ as_timer()
 	    rh.rh_net = iface->i_addr.sat_addr.s_net;
 	    rh.rh_nodelen = 8;
 	    rh.rh_node = iface->i_addr.sat_addr.s_node;
-	    bcopy( &rh, data, sizeof( struct rtmp_head ));
+	    memcpy( data, &rh, sizeof( struct rtmp_head ));
 	    data += sizeof( struct rtmp_head );
 	    n = 0;
 
@@ -418,23 +538,27 @@ as_timer()
 	    if ( iface->i_flags & IFACE_PHASE1 ) {
 		rt.rt_net = 0;
 		rt.rt_dist = 0x82;
-		bcopy( &rt, data, SZ_RTMPTUPLE );
+		memcpy( data, &rt, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 	    } else {
 		rt.rt_net = iface->i_rt->rt_firstnet;
 		rt.rt_dist = 0x80;
-		bcopy( &rt, data, SZ_RTMPTUPLE );
+		memcpy( data, &rt, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 
 		rt.rt_net = iface->i_rt->rt_lastnet;
 		rt.rt_dist = 0x82;
-		bcopy( &rt, data, SZ_RTMPTUPLE );
+		memcpy( data, &rt, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 	    }
 
 	    for ( iface2 = interfaces; iface2; iface2 = iface2->i_next ) {
-		if (( iface2->i_flags & IFACE_CONFIG ) == 0 ||
-			iface == iface2 ) {
+
+	      /* XXX: there used to be a bit checking against iface ==
+		 iface2. also, we don't want to send an rtmp broadcast
+		 to an interface that doesn't want it.  */
+	        if ((( iface2->i_flags & IFACE_CONFIG ) == 0) ||
+		    ((iface2->i_flags & IFACE_ISROUTER) == 0)) {
 		    continue;
 		}
 		/*
@@ -457,10 +581,11 @@ as_timer()
 			if ( sendto( rap->ap_fd, packet, data - packet, 0,
 				(struct sockaddr *)&sat,
 				sizeof( struct sockaddr_at )) < 0 ) {
-			    syslog( LOG_ERR, "as_timer sendto %u.%u (%u): %m",
+			    syslog( LOG_ERR, "as_timer sendto %u.%u (%u): %s",
 				    ntohs( sat.sat_addr.s_net ),
 				    sat.sat_addr.s_node,
-				    ntohs( iface->i_rt->rt_firstnet ));
+				    ntohs( iface->i_rt->rt_firstnet ),
+				    strerror(errno) );
 			}
 
 			if ( iface->i_flags & IFACE_PHASE2 ) {
@@ -478,13 +603,13 @@ as_timer()
 		    if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
 			rt.rt_dist |= 0x80;
 		    }
-		    bcopy( &rt, data, SZ_RTMPTUPLE );
+		    memcpy( data, &rt, SZ_RTMPTUPLE );
 		    data += SZ_RTMPTUPLE;
 
 		    if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
 			rt.rt_net = rtmp->rt_lastnet;
 			rt.rt_dist = 0x82;
-			bcopy( &rt, data, SZ_RTMPTUPLE );
+			memcpy( data, &rt, SZ_RTMPTUPLE );
 			data += SZ_RTMPTUPLE;
 		    }
 		    n++;
@@ -496,10 +621,11 @@ as_timer()
 		if ( sendto( rap->ap_fd, packet, data - packet, 0,
 			(struct sockaddr *)&sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "as_timer sendto %u.%u (%u): %m",
+		    syslog( LOG_ERR, "as_timer sendto %u.%u (%u): %s",
 			    ntohs( sat.sat_addr.s_net ),
 			    sat.sat_addr.s_node,
-			    ntohs( iface->i_rt->rt_firstnet ));
+			    ntohs( iface->i_rt->rt_firstnet ),
+			    strerror(errno) );
 		}
 	    }
 	}
@@ -534,7 +660,7 @@ as_timer()
 		 */
 		if ( kill( getpid(), SIGSTOP ) < 0 ) {
 		    syslog( LOG_ERR, "as_timer: kill-self failed!" );
-		    exit( 1 );
+		    atalkd_exit( 1 );
 		}
 	    }
 	}
@@ -542,9 +668,7 @@ as_timer()
 
 #ifdef DEBUG
     consistency();
-#endif DEBUG
-
-    return;
+#endif /* DEBUG */
 }
 
 #ifdef DEBUG
@@ -564,7 +688,7 @@ consistency()
 		syslog( LOG_ERR, "%.*s has %u-%u (unused)\n",
 			zt->zt_len, zt->zt_name, ntohs( rtmp->rt_firstnet ),
 			ntohs( rtmp->rt_lastnet ));
-		abort();
+		atalkd_exit(1);
 	    }
 	    for ( lz = rtmp->rt_zt; lz; lz = lz->l_next ) {
 		if ( zt == (struct ziptab *)lz->l_data ) {
@@ -576,16 +700,16 @@ consistency()
 			ntohs( rtmp->rt_firstnet ),
 			ntohs( rtmp->rt_lastnet ),
 			zt->zt_len, zt->zt_name );
-		abort();
+		atalkd_exit(1);
 	    }
 	}
     }
 }
-#endif DEBUG
+#endif /* DEBUG */
 
 #if !defined( ibm032 ) && !defined( _IBMR2 )
     void
-#endif ibm032 _IBMR2
+#endif /* ! ibm032 && ! _IBMR2 */
 as_debug()
 {
     struct interface	*iface;
@@ -596,8 +720,7 @@ as_debug()
     FILE		*rtmpdebug;
 
     if (( rtmpdebug = fopen( _PATH_ATALKDEBUG, "w" )) == NULL ) {
-	syslog( LOG_ERR, "rtmp: %m" );
-	return;
+	syslog( LOG_ERR, "rtmp: %s", strerror(errno) );
     }
 
     for ( iface = interfaces; iface; iface = iface->i_next ) {
@@ -610,9 +733,15 @@ as_debug()
 	if ( iface->i_flags & IFACE_PHASE2 ) {
 	    putc( '2', rtmpdebug );
 	}
+	if ( iface->i_flags & IFACE_RSEED ) {
+	    putc( 'R', rtmpdebug );
+	}
 	if ( iface->i_flags & IFACE_SEED ) {
 	    putc( 'S', rtmpdebug );
 	}
+	if ( iface->i_flags & IFACE_DONTROUTE ) {
+	    putc( 'D', rtmpdebug );
+	}
 	if ( iface->i_flags & IFACE_ADDR ) {
 	    putc( 'A', rtmpdebug );
 	}
@@ -677,7 +806,6 @@ as_debug()
     }
 
     fclose( rtmpdebug );
-    return;
 }
 
 /*
@@ -685,7 +813,7 @@ as_debug()
  */
 #if !defined( ibm032 ) && !defined( _IBMR2 )
     void
-#endif ibm032 _IBMR2
+#endif /* ! ibm032 && ! _IBMR2 */
 as_down()
 {
     struct interface	*iface;
@@ -697,43 +825,48 @@ as_down()
 	    for ( rt = gate->g_rt; rt; rt = rt->rt_next ) {
 		if ( rt->rt_iprev ) {
 		    if ( gateroute( RTMP_DEL, rt ) < 0 ) {
-			syslog( LOG_ERR, "as_down remove %u-%u failed: %m",
+			syslog( LOG_ERR, "as_down remove %u-%u failed: %s",
 				ntohs( rt->rt_firstnet ),
-				ntohs( rt->rt_lastnet ));
+				ntohs( rt->rt_lastnet ),
+				strerror(errno) );
 		    }
 		}
 	    }
 	}
 	if ( iface->i_flags & IFACE_LOOP ) {
-	    if ( looproute( iface, RTMP_DEL ) < 0 ) {
-		syslog( LOG_ERR, "as_down remove %s %u.%u failed: %m",
-			iface->i_name, ntohs( iface->i_addr.sat_addr.s_net ),
-			iface->i_addr.sat_addr.s_node );
-	    }
+	  if (looproute( iface, RTMP_DEL )) {
+	    syslog( LOG_ERR, "as_down remove %s %u.%u failed: %s",
+		    iface->i_name, ntohs( iface->i_addr.sat_addr.s_net ),
+		    iface->i_addr.sat_addr.s_node,
+		    strerror(errno) );
+	  }
 	}
     }
 
     syslog( LOG_INFO, "done" );
-    exit( 0 );
+    atalkd_exit( 0 );
 }
 
-main( ac, av )
+int main( ac, av )
     int		ac;
     char	**av;
 {
+    extern char         *optarg;
+    extern int          optind;
+
     struct sockaddr_at	sat;
     struct sigaction	sv;
     struct itimerval	it;
     struct interface	*iface;
-    union wait		status;
+    int			status;
     struct atport	*ap;
     fd_set		readfds;
-    int			fd, i, mask, c, fromlen;
+    int			i, mask, c;
+    SOCKLEN_T 		fromlen;
     char		*prog;
-    extern int		errno, optind;
-    extern char		*optarg;
+;
 
-    while (( c = getopt( ac, av, "12qsdtf:" )) != EOF ) {
+    while (( c = getopt( ac, av, "12qsdtf:P:v" )) != EOF ) {
 	switch ( c ) {
 	case '1' :
 	    defphase = IFACE_PHASE1;
@@ -763,6 +896,15 @@ main( ac, av )
 	    transition++;
 	    break;
 
+	case 'P' :	/* pid file */
+	    pidfile = optarg;
+	    break;
+
+	case 'v' :	/* version */
+	    printf( "atalkd (version %s)\n", version );
+	    exit ( 1 );
+	    break;
+
 	default :
 	    fprintf( stderr, "Unknown option -- '%c'\n", c );
 	    exit( 1 );
@@ -773,7 +915,7 @@ main( ac, av )
 	exit( 1 );
     }
 
-    if (( prog = rindex( av[ 0 ], '/' )) == NULL ) {
+    if (( prog = strrchr( av[ 0 ], '/' )) == NULL ) {
 	prog = av[ 0 ];
     } else {
 	prog++;
@@ -804,22 +946,39 @@ main( ac, av )
 	exit( 1 );
     }
 
+    /* we need to count up our interfaces so that we can simplify things
+     * later. we also need to figure out if we have more than one interface
+     * that is routing. */
+    for (i = 0, ninterfaces = 0, iface = interfaces; iface;
+	 iface=iface->i_next) {
+      if (iface->i_flags & IFACE_DONTROUTE)
+	i++;
+      ninterfaces++;
+    }
+    i = ninterfaces - i; /* number of routable interfaces */
+
     /*
      * At this point, we have (at least partially) initialized data
      * structures. Fill in what we can and verify that nothing is obviously
      * broken.
      */
-    for ( ninterfaces = 0, iface = interfaces; iface;
-	    iface = iface->i_next, ninterfaces++ ) {
+    for (iface = interfaces; iface; iface = iface->i_next) {
 	/* Apply the default phase */
 	if (( iface->i_flags & IFACE_PHASE1 ) == 0 &&
 		( iface->i_flags & IFACE_PHASE2 ) == 0 ) {
 	    iface->i_flags |= defphase;
 	}
 
+        /* set up router flag information. if we have multiple interfaces
+	 * and DONTROUTE isn't set, set up ROUTER. i is the number of 
+	 * interfaces that don't have the DONTROUTE flag set. */
+	if ((i > IFBASE) && ((iface->i_flags & IFACE_DONTROUTE) == 0)) {
+	  iface->i_flags |= IFACE_ISROUTER;
+	}
+
 	/* Set default addresses */
 	if ( iface->i_rt == NULL ) {
-	    if (( iface->i_rt = newrt()) == NULL ) {
+	    if (( iface->i_rt = newrt(iface)) == NULL ) {
 		perror( "newrt" );
 		exit( 1 );
 	    }
@@ -838,7 +997,7 @@ main( ac, av )
 		}
 	    }
 	}
-
+	
 	if (( iface->i_flags & IFACE_PHASE1 ) == 0 ) {
 	    iface->i_rt->rt_flags |= RTMPTAB_EXTENDED;
 	}
@@ -860,63 +1019,70 @@ main( ac, av )
 	exit( 1 );
     }
 
+    /* do this here so that we can use ifconfig */
+#ifdef __svr4__
+    if ( plumb() < 0 ) {
+	fprintf(stderr, "can't establish STREAMS plumbing, exiting.\n" );
+	atalkd_exit( 1 );
+    }
+#endif /* __svr4__ */
+
+    /* delete pre-existing interface addresses. */
+#ifdef SIOCDIFADDR
+    for (iface = interfaces; iface; iface = iface->i_next) {
+      if (ifconfig(iface->i_name, SIOCDIFADDR, &iface->i_addr)) {
+#ifdef SIOCATALKDIFADDR
+#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+	ifconfig(iface->i_name, SIOCATALKDIFADDR, &iface->i_addr);
+#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
+#endif /* SIOCATALKDIFADDR */
+      }
+    }
+#endif /* SIOCDIFADDR */
+
     /*
      * Disassociate. The child will send itself a signal when it is
      * stable. This indicates that other processes may begin using
      * AppleTalk.
      */
-    if ( !debug ) {
-	switch( i = fork()) {
-	case 0 :
-	    i = getdtablesize();
-	    for ( fd = 0; fd < i; fd++ ) {
-		(void)close( fd );
-	    }
-	    if (( fd = open( "/dev/tty", O_RDWR )) >= 0 ) {
-		(void)ioctl( fd, TIOCNOTTY, (char *)0 );
-		(void)close( fd );
-	    }
-	    (void)setpgrp( 0, getpid());
-	    break;
-
-	case -1 :
-	    perror( "fork" );
-	    exit( 1 );
-
-	default :
-	    /*
-	     * Wait for the child to send itself a SIGSTOP, after which
-	     * we send it a SIGCONT and exit ourself.
-	     */
-	    if ( wait3( &status, WUNTRACED, (struct rusage *)0 ) != i ) {
-		perror( "wait3" );	/* Child died? */
-		exit( 1 );
-	    }
-	    if ( !WIFSTOPPED( status )) {
-		fprintf( stderr, "AppleTalk not up!" );
-		if ( WIFEXITED( status )) {
-		    fprintf( stderr, " Child exited with %d.\n",
-			    WEXITSTATUS( status ));
-		} else {
-		    fprintf( stderr, " Child died.\n" );
-		}
-		exit( 1 );
-	    }
-	    if ( kill( i, SIGCONT ) < 0 ) {
-		perror( "kill" );
-		exit( 1 );
-	    }
-	    exit( 0 );
+    switch (i = server_lock("atalkd", pidfile, debug)) {
+    case -1:
+      exit(1);
+    case 0: /* child */
+      break;
+    default: /* parent */
+      /*
+       * Wait for the child to send itself a SIGSTOP, after which
+       * we send it a SIGCONT and exit ourself.
+       */
+      if ( wait3( &status, WUNTRACED, (struct rusage *)0 ) != i) {
+	perror( "wait3" );	/* Child died? */
+	atalkd_exit( 1 );
+      }
+      if ( !WIFSTOPPED( status )) {
+	fprintf( stderr, "AppleTalk not up! Check your syslog for the reason." );
+	if ( WIFEXITED( status )) {
+	  fprintf( stderr, " Child exited with %d.\n",
+		   WEXITSTATUS( status ));
+	} else {
+	  fprintf( stderr, " Child died.\n" );
 	}
+	atalkd_exit( 1 );
+      }
+      if ( kill(i, SIGCONT ) < 0 ) {
+	perror( "kill" );
+	atalkd_exit( 1 );
+      }
+      exit( 0 );
     }
 
 #ifdef ultrix
     openlog( prog, LOG_PID );
-#else ultrix
+#else /* ultrix */
     openlog( prog, LOG_PID, LOG_DAEMON );
-#endif ultrix
+#endif /* ultrix */
 
-    syslog( LOG_INFO, "restart" );
+    syslog( LOG_INFO, "restart (%s)", version );
 
     /*
      * Socket for use in routing ioctl()s. Can't add routes to our
@@ -924,65 +1090,74 @@ main( ac, av )
      */
 #ifdef BSD4_4
     if (( rtfd = socket( PF_ROUTE, SOCK_RAW, AF_APPLETALK )) < 0 ) {
-	syslog( LOG_ERR, "route socket" );
-	exit( 1 );
+	syslog( LOG_ERR, "route socket: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
     if ( shutdown( rtfd, 0 ) < 0 ) {
-	syslog( LOG_ERR, "route shutdown" );
-	exit( 1 );
+	syslog( LOG_ERR, "route shutdown: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
-#else BSD4_4
+#else /* BSD4_4 */
     if (( rtfd = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
-	syslog( LOG_ERR, "route socket" );
-	exit( 1 );
+	syslog( LOG_ERR, "route socket: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
-#endif BSD4_4
+#endif /* BSD4_4 */
 
+    memset(&sv, 0, sizeof(sv));
     sv.sa_handler = as_down;
-    sv.sa_mask = sigmask( SIGUSR1 ) | sigmask( SIGALRM ) | sigmask( SIGTERM );
-    sv.sa_flags = 0;
-    if ( sigaction( SIGTERM, &sv, (struct sigaction *)0 ) < 0 ) {
-	syslog( LOG_ERR, "sigterm: %m" );
-	exit( 1 );
+    sigemptyset( &sv.sa_mask );
+    sigaddset( &sv.sa_mask, SIGUSR1 );
+    sigaddset( &sv.sa_mask, SIGALRM );
+    sigaddset( &sv.sa_mask, SIGTERM );
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGTERM, &sv, NULL) < 0 ) {
+	syslog( LOG_ERR, "sigterm: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
 
     sv.sa_handler = as_debug;
-    sv.sa_mask = sigmask( SIGUSR1 ) | sigmask( SIGALRM ) | sigmask( SIGTERM );
-    sv.sa_flags = 0;
-    if ( sigaction( SIGUSR1, &sv, (struct sigaction *)0 ) < 0 ) {
-	syslog( LOG_ERR, "sigusr1: %m" );
-	exit( 1 );
+    sigemptyset( &sv.sa_mask );
+    sigaddset( &sv.sa_mask, SIGUSR1 );
+    sigaddset( &sv.sa_mask, SIGALRM );
+    sigaddset( &sv.sa_mask, SIGTERM );
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGUSR1, &sv, NULL) < 0 ) {
+	syslog( LOG_ERR, "sigusr1: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
 
     sv.sa_handler = as_timer;
-    sv.sa_mask = sigmask( SIGUSR1 ) | sigmask( SIGALRM ) | sigmask( SIGTERM );
-    sv.sa_flags = 0;
-    if ( sigaction( SIGALRM, &sv, (struct sigaction *)0 ) < 0 ) {
-	syslog( LOG_ERR, "sigalrm: %m" );
-	exit( 1 );
+    sigemptyset( &sv.sa_mask );
+    sigaddset( &sv.sa_mask, SIGUSR1 );
+    sigaddset( &sv.sa_mask, SIGALRM );
+    sigaddset( &sv.sa_mask, SIGTERM );
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGALRM, &sv, NULL) < 0 ) {
+	syslog( LOG_ERR, "sigalrm: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
 
     it.it_interval.tv_sec = 10L;
     it.it_interval.tv_usec = 0L;
     it.it_value.tv_sec = 10L;
     it.it_value.tv_usec = 0L;
-    if ( setitimer( ITIMER_REAL, &it, (struct itimerval *)0 ) < 0 ) {
-	syslog( LOG_ERR, "setitimer: %m" );
-	exit( 1 );
+    if ( setitimer( ITIMER_REAL, &it, NULL) < 0 ) {
+	syslog( LOG_ERR, "setitimer: %s", strerror(errno) );
+	atalkd_exit( 1 );
     }
 
     ciface = interfaces;
     bootaddr( ciface );
     for (;;) {
 	readfds = fds;
-	if ( select( nfds, &readfds, (fd_set *)0, (fd_set *)0,
-		(struct timeval *)0 ) < 0 ) {
+	if ( select( nfds, &readfds, NULL, NULL, NULL) < 0 ) {
 	    if ( errno == EINTR ) {
 		errno = 0;
 		continue;
 	    } else {
-		syslog( LOG_ERR, "select: %m" );
-		exit( 1 );
+		syslog( LOG_ERR, "select: %s", strerror(errno) );
+		atalkd_exit( 1 );
 	    }
 	}
 
@@ -993,7 +1168,7 @@ main( ac, av )
 			fromlen = sizeof( struct sockaddr_at );
 			if (( c = recvfrom( ap->ap_fd, Packet, sizeof( Packet ),
 				0, (struct sockaddr *)&sat, &fromlen )) < 0 ) {
-			    syslog( LOG_ERR, "recvfrom: %m" );
+			    syslog( LOG_ERR, "recvfrom: %s", strerror(errno) );
 			    continue;
 			}
 #ifdef DEBUG
@@ -1004,15 +1179,32 @@ main( ac, av )
 				    iface->i_flags, ap->ap_port, ap->ap_fd );
 			    bprint( Packet, c );
 			}
-#endif DEBUG
+#endif /* DEBUG */
+#ifdef __svr4__
+			if ( sighold( SIGALRM ) || sighold( SIGUSR1 )) {
+			    syslog( LOG_ERR, "sighold: %s", strerror(errno) );
+			    atalkd_exit( 1 );
+			}
+#else /* __svr4__ */
 			mask = sigsetmask( sigmask( SIGALRM ) |
 				sigmask( SIGUSR1 ));
-			( *ap->ap_packet )( ap, &sat, Packet, c );
+#endif /* __svr4__ */
+			if (( *ap->ap_packet )( ap, &sat, Packet, c ) < 0) {
+			  syslog(LOG_ERR, "ap->ap_packet: %s", strerror(errno));
+			  atalkd_exit(1);
+			}
 
 #ifdef DEBUG
 			consistency();
-#endif DEBUG
+#endif /* DEBUG */
+#ifdef __svr4__
+			if ( sigrelse( SIGUSR1 ) || sigrelse( SIGALRM )) {
+			    syslog( LOG_ERR, "sigrelse: %s", strerror(errno) );
+			    atalkd_exit( 1 );
+			}
+#else /* __svr4__ */
 			sigsetmask( mask );
+#endif /* __svr4__ */
 		    }
 		}
 	    }
@@ -1025,11 +1217,9 @@ main( ac, av )
  * and rtmp_packet()) to set the initial "bootstrapping" address
  * on an interface.
  */
-bootaddr( iface )
+void bootaddr( iface )
     struct interface	*iface;
 {
-    struct sockaddr	sa;
-
     if ( iface == 0 ) {
 	return;
     }
@@ -1037,7 +1227,7 @@ bootaddr( iface )
     /* consistency */
     if ( iface->i_flags & IFACE_ADDR ) {
 	syslog( LOG_ERR, "bootaddr OOPS!" );
-	abort();
+	atalkd_exit(1);
     }
 
     if ( iface->i_flags & IFACE_PHASE1 ) {
@@ -1050,8 +1240,10 @@ bootaddr( iface )
 		ciface = ciface->i_next;
 		bootaddr( ciface );
 	    }
-	} else {
-	    rtmp_request( iface );
+
+	} else if (rtmp_request( iface ) < 0) {
+	  syslog(LOG_ERR, "bootaddr (rtmp_request): %s", strerror(errno));
+	  atalkd_exit(1);
 	}
 
     } else {
@@ -1065,34 +1257,25 @@ bootaddr( iface )
 		ciface = ciface->i_next;
 		bootaddr( ciface );
 	    }
-	} else {
-	    /* configure multicast for this interface */
-	    bzero( &sa, sizeof( struct sockaddr ));
-	    bcopy( ethermulti, sa.sa_data, sizeof( ethermulti ));
-	    if ( ifconfig( iface->i_name, SIOCADDMULTI, &sa )) {
-		syslog( LOG_ERR, "addmulti: %m" );
-		exit( 1 );
-	    }
-
-	    zip_getnetinfo( iface );
+	    
+	} else if (zip_getnetinfo( iface ) < 0) {
+	  syslog(LOG_ERR, "bootaddr (zip_getnetinfo): %s", strerror(errno));
+	  atalkd_exit(1);
 	}
     }
     ++iface->i_time;
     iface->i_flags |= IFACE_ADDR;
     stabletimer = UNSTABLE;
-    return;
 }
 
+
 /*
  * Change setaddr()
  * to manage the i_ports field and the fds for select().
  */
-setaddr( iface, phase, net, node, first, last )
-    struct interface	*iface;
-    u_char		phase;
-    u_short		net;
-    u_char		node;
-    u_short		first, last;
+void setaddr(struct interface *iface,
+             u_int8_t  phase, u_int16_t net, u_int8_t node,
+	     u_int16_t first, u_int16_t last)
 {
     int			i;
     struct atserv	*as;
@@ -1110,8 +1293,8 @@ setaddr( iface, phase, net, node, first, last )
 	    }
 	    if (( ap = (struct atport *)malloc( sizeof( struct atport ))) ==
 		    NULL ) {
-		syslog( LOG_ERR, "malloc: %m" );
-		exit( 1 );
+		syslog( LOG_ERR, "malloc: %s", strerror(errno) );
+		atalkd_exit( 1 );
 	    }
 	    ap->ap_fd = 0;
 	    ap->ap_next = iface->i_ports;
@@ -1127,6 +1310,9 @@ setaddr( iface, phase, net, node, first, last )
 	}
     }
 
+#ifdef BSD4_4
+    iface->i_addr.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     iface->i_addr.sat_family = AF_APPLETALK;
     iface->i_addr.sat_addr.s_net = net;
     iface->i_addr.sat_addr.s_node = node;
@@ -1134,25 +1320,36 @@ setaddr( iface, phase, net, node, first, last )
     nr.nr_phase = phase;
     nr.nr_firstnet = first;
     nr.nr_lastnet = last;
-    bcopy( &nr, iface->i_addr.sat_zero, sizeof( struct netrange ));
+    memcpy( iface->i_addr.sat_zero, &nr, sizeof( struct netrange ));
 
     if ( ifconfig( iface->i_name, SIOCSIFADDR, &iface->i_addr )) {
-	syslog( LOG_ERR, "setifaddr: %m" );
-	exit( 1 );
+      syslog( LOG_ERR, "setifaddr: %s (%u-%u): %s. try specifying a \
+smaller net range.", iface->i_name, ntohs(first), ntohs(last), strerror(errno));
+	atalkd_exit( 1 );
     }
     if ( ifconfig( iface->i_name, SIOCGIFADDR, &iface->i_addr )) {
-	syslog( LOG_ERR, "getifaddr: %m" );
-	exit( 1 );
+	syslog( LOG_ERR, "getifaddr: %s: %s", iface->i_name, strerror(errno) );
+	atalkd_exit( 1 );
     }
 
     /* open ports */
+    i = 1; /* enable broadcasts */
+#ifdef __svr4__ 
+    syslog(LOG_INFO, "setsockopt incompatible w/ Solaris STREAMS module.");
+#endif /* __svr4__ */
     for ( ap = iface->i_ports; ap; ap = ap->ap_next ) {
 	if (( ap->ap_fd = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
-	    syslog( LOG_ERR, "socket: %m" );
-	    exit( 1 );
+	    syslog( LOG_ERR, "socket: %s", strerror(errno) );
+	    atalkd_exit( 1 );
 	}
+#ifndef __svr4__
+	setsockopt(ap->ap_fd, SOL_SOCKET, SO_BROADCAST, &i, sizeof(i));
+#endif /* ! __svr4 */
 
-	bzero( &sat, sizeof( struct sockaddr_at ));
+	memset( &sat, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+	sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
 	sat.sat_family = AF_APPLETALK;
 	sat.sat_addr.s_net = iface->i_addr.sat_addr.s_net;
 	sat.sat_addr.s_node = iface->i_addr.sat_addr.s_node;
@@ -1160,9 +1357,22 @@ setaddr( iface, phase, net, node, first, last )
 
 	if ( bind( ap->ap_fd, (struct sockaddr *)&sat,
 		sizeof( struct sockaddr_at )) < 0 ) {
-	    syslog( LOG_ERR, "bind %u.%u:%u: %m", ntohs( sat.sat_addr.s_net ),
-		    sat.sat_addr.s_node, sat.sat_port );
-	    exit( 1 );
+	    syslog( LOG_ERR, "bind %u.%u:%u: %s",
+		    ntohs( sat.sat_addr.s_net ),
+		    sat.sat_addr.s_node, sat.sat_port, strerror(errno) );
+#ifdef SIOCDIFADDR
+	    /* remove all interfaces if we have a problem with bind */
+	    for (iface = interfaces; iface; iface = iface->i_next) {
+	      if (ifconfig( iface->i_name, SIOCDIFADDR, &iface->i_addr )) {
+#ifdef SIOCATALKDIFADDR
+#if (SIOCDIFADDR != SIOCATALKDIFADDR)
+		ifconfig( iface->i_name, SIOCATALKDIFADDR, &iface->i_addr );
+#endif /* SIOCDIFADDR != SIOCATALKDIFADDR */
+#endif /* SIOCATALKDIFADDR */
+	      }
+	    }
+#endif /* SIOCDIFADDR */
+	    atalkd_exit( 1 );
 	}
     }
 
@@ -1177,18 +1387,17 @@ setaddr( iface, phase, net, node, first, last )
 	}
     }
     nfds++;
-
-    return;
 }
 
-ifconfig( iname, cmd, sa )
-    char		*iname;
-    int			cmd;
+int ifconfig( iname, cmd, sa )
+    const char		*iname;
+    unsigned long	cmd;
     struct sockaddr_at	*sa;
 {
     struct ifreq	ifr;
     int			s;
 
+    memset(&ifr, 0, sizeof(ifr));
     strcpy( ifr.ifr_name, iname );
     ifr.ifr_addr = *(struct sockaddr *)sa;
 
@@ -1196,6 +1405,7 @@ ifconfig( iname, cmd, sa )
 	return( 1 );
     }
     if ( ioctl( s, cmd, &ifr ) < 0 ) {
+	close(s);
 	return( 1 );
     }
     close( s );
@@ -1205,15 +1415,21 @@ ifconfig( iname, cmd, sa )
     return( 0 );
 }
 
-dumpconfig( iface )
+void dumpconfig( iface )
     struct interface	*iface;
 {
     struct list		*l;
 
     printf( "%s", iface->i_name );
-    if ( iface->i_flags & IFACE_SEED ) {
+    if ( iface->i_flags & IFACE_RSEED ) {
+	printf( " -router" );
+    } else if ( iface->i_flags & IFACE_SEED ) {
 	printf( " -seed" );
     }
+
+    if ( iface->i_flags & IFACE_DONTROUTE) 
+        printf( " -dontroute");
+
     printf( " -phase" );
     if ( iface->i_flags & IFACE_PHASE1 ) {
 	printf( " 1" );
@@ -1236,7 +1452,7 @@ dumpconfig( iface )
 }
 
 #ifdef DEBUG
-dumproutes()
+void dumproutes()
 {
     struct interface	*iface;
     struct rtmptab	*rtmp;
@@ -1284,7 +1500,7 @@ dumproutes()
     fflush( stdout );
 }
 
-dumpzones()
+void dumpzones()
 {
     struct interface	*iface;
     struct rtmptab	*rtmp;
@@ -1311,4 +1527,4 @@ dumpzones()
     printf( "\n" );
     fflush( stdout );
 }
-#endif DEBUG
+#endif /* DEBUG */
