@@ -1,16 +1,36 @@
 /*
+ * $Id: nbp.c,v 1.5.2.2 2002/02/07 23:58:46 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 #include <sys/syslog.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <netatalk/at.h>
 #include <atalk/ddp.h>
+#include <atalk/atp.h>
 #include <atalk/nbp.h>
+#include <atalk/util.h>
+
+#ifdef __svr4__
+#include <sys/sockio.h>
+#endif /* __svr4__ */
 
 #include "atserv.h"
 #include "interface.h"
@@ -21,12 +41,34 @@
 #include "nbp.h"
 #include "multicast.h"
 
-extern u_char	*zone_mcast();
 extern int	transition;
 
-struct nbptab	*nbptab = 0;
+struct nbptab	*nbptab = NULL;
 
-nbp_packet( ap, from, data, len )
+static 
+void nbp_ack( fd, nh_op, nh_id, to )
+    int			fd;
+    int			nh_op;
+    int			nh_id;
+    struct sockaddr_at	*to;
+{
+    struct nbphdr	nh;
+    char		*data, packet[ SZ_NBPHDR + 1 ];
+
+    nh.nh_op = nh_op;
+    nh.nh_cnt = 0;
+    nh.nh_id = nh_id;
+    data = packet;
+    *data++ = DDPTYPE_NBP;
+    memcpy( data, &nh, SZ_NBPHDR );
+    data += SZ_NBPHDR;
+    if ( sendto( fd, packet, data - packet, 0, (struct sockaddr *)to,
+	    sizeof( struct sockaddr_at )) < 0 ) {
+	syslog( LOG_ERR, "sendto: %s", strerror(errno) );
+    }
+}
+
+int nbp_packet( ap, from, data, len )
     struct atport	*ap;
     struct sockaddr_at	*from;
     char		*data;
@@ -35,82 +77,91 @@ nbp_packet( ap, from, data, len )
     struct nbphdr	nh;
     struct nbptuple	nt;
     struct nbpnve	nn;
-    struct sockaddr	sa;
     struct sockaddr_at	sat;
     struct nbptab	*ntab;
     struct ziptab	*zt;
     struct interface	*iface;
     struct list		*l;
     struct rtmptab	*rtmp;
-    char		*end, *nbpop, *zonep, packet[ 587 ];
-    int			n, i, cc, locallkup = 0;
-    u_short		tonet;
+    char		*end, *nbpop, *zonep, packet[ ATP_BUFSIZ ];
+    int			n, i, cc, locallkup;
+    u_char		tmplen;
 
     end = data + len;
     if ( data >= end ) {
 	syslog( LOG_INFO, "nbp_packet malformed packet" );
-	return;
+	return 1;
     }
     if ( *data++ != DDPTYPE_NBP ) {
 	syslog( LOG_INFO, "nbp_packet bad ddp type" );
-	return;
+	return 1;
     }
 
     if ( data + SZ_NBPHDR + SZ_NBPTUPLE > end ) {
 	syslog( LOG_INFO, "nbp_packet: malformed packet" );
-	return;
+	return 1;
     }
-    bcopy( data, &nh, SZ_NBPHDR );
+    memcpy( &nh, data, SZ_NBPHDR );
     nbpop = data;			/* remember for fwd and brrq */
     data += SZ_NBPHDR;
     if ( nh.nh_cnt != 1 ) {
 	syslog( LOG_INFO, "nbp_packet: bad tuple count (%d/%d)", nh.nh_cnt,
 		nh.nh_op );
-	return;
+	return 1;
     }
 
-    bcopy( data, &nt, SZ_NBPTUPLE );
+    memcpy( &nt, data, SZ_NBPTUPLE );
     data += SZ_NBPTUPLE;
 
-    bzero( &nn.nn_sat, sizeof( struct sockaddr_at ));
+    memset( &nn.nn_sat, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    nn.nn_sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     nn.nn_sat.sat_family = AF_APPLETALK;
     nn.nn_sat.sat_addr.s_net = nt.nt_net;
     nn.nn_sat.sat_addr.s_node = nt.nt_node;
     nn.nn_sat.sat_port = nt.nt_port;
 
     /* object */
-    if ( data >= end || ( *data < 0 || *data > 32 ) || data + *data > end ) {
+    tmplen = (u_char) *data;
+    if ( data >= end || tmplen > 32 || data + tmplen > end ) {
 	syslog( LOG_INFO, "nbp_packet: malformed packet" );
-	return;
+	return 1;
     }
-    nn.nn_objlen = *data++;
-    bcopy( data, nn.nn_obj, nn.nn_objlen );
+    nn.nn_objlen = tmplen;
+    data++;
+    memcpy( nn.nn_obj, data, nn.nn_objlen );
     data += nn.nn_objlen;
 
     /* type */
-    if ( data >= end || ( *data < 0 || *data > 32 ) || data + *data > end ) {
+    tmplen = (u_char) *data;
+    if ( data >= end || tmplen > 32 || data + tmplen > end ) {
 	syslog( LOG_INFO, "nbp_packet: malformed packet" );
-	return;
+	return 1;
     }
-    nn.nn_typelen = *data++;
-    bcopy( data, nn.nn_type, nn.nn_typelen );
+    nn.nn_typelen = tmplen;
+    data++;
+    memcpy( nn.nn_type, data, nn.nn_typelen );
     data += nn.nn_typelen;
 
     /* zone */
-    if ( data >= end || ( *data < 0 || *data > 32 ) || data + *data > end ) {
+    tmplen = (u_char) *data;
+    if ( data >= end || tmplen > 32 || data + tmplen > end ) {
 	syslog( LOG_INFO, "nbp_packet: malformed packet" );
-	return;
+	return 1;
     }
     zonep = data;			/* remember for fwd */
-    nn.nn_zonelen = *data++;
-    bcopy( data, nn.nn_zone, nn.nn_zonelen );
+    nn.nn_zonelen = tmplen;
+    data++;
+    memcpy( nn.nn_zone, data, nn.nn_zonelen );
     data += nn.nn_zonelen;
 
     if ( data != end ) {
 	syslog( LOG_INFO, "nbp_packet: malformed packet" );
-	return;
+	return 1;
     }
 
+    locallkup = 0;
     switch ( nh.nh_op ) {
 
     case NBPOP_RGSTR :
@@ -133,7 +184,7 @@ nbp_packet( ap, from, data, len )
 	    }
 	    if ( zt == 0 ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
-		return;
+		return 0;
 	    }
 	}
 
@@ -157,19 +208,14 @@ nbp_packet( ap, from, data, len )
 	    }
 	    if ( iface == 0 ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
-		return;
+		return 0;
 	    }
 
 	    /* calculate and save multicast address */
-	    if (( zt->zt_bcast =
-		    (u_char *)malloc( sizeof( ethermulti ))) == 0 ) {
-		syslog( LOG_ERR, "nbp rgstr malloc: %m" );
-		exit( 1 );
+	    if (zone_bcast(zt) < 0) {
+	        syslog(LOG_ERR, "nbp_packet: zone_bcast");
+		return -1;
 	    }
-	    bcopy( zone_mcast( zt->zt_name, zt->zt_len ), zt->zt_bcast,
-		    sizeof( ethermulti ));
-	    bzero( &sa, sizeof( struct sockaddr ));
-	    bcopy( zt->zt_bcast, sa.sa_data, sizeof( ethermulti ));
 
 	    for ( iface = interfaces; iface; iface = iface->i_next ) {
 		if (( iface->i_flags & IFACE_PHASE2 ) == 0 ) {
@@ -177,10 +223,11 @@ nbp_packet( ap, from, data, len )
 		}
 		for ( l = iface->i_rt->rt_zt; l; l = l->l_next ) {
 		    if ( zt == (struct ziptab *)l->l_data ) {
-			/* add multicast */
-			if ( ifconfig( iface->i_name, SIOCADDMULTI, &sa )) {
-			    syslog( LOG_ERR, "addmulti: %m" );
-			    exit( 1 );
+		        /* add multicast */
+		        if (addmulti(iface->i_name, zt->zt_bcast) < 0) {
+			    syslog( LOG_ERR, "nbp_packet: addmulti: %s",
+				    strerror(errno) );
+			    return -1;
 			}
 		    }
 		}
@@ -189,13 +236,14 @@ nbp_packet( ap, from, data, len )
 
 	if (( ntab = (struct nbptab *)malloc( sizeof( struct nbptab )))
 		== 0 ) {
-	    syslog( LOG_ERR, "malloc: %m" );
-	    exit( 1 );
+	    syslog( LOG_ERR, "nbp_packet: malloc: %s", strerror(errno) );
+	    return -1;
 	}
-	bcopy( &nn, &ntab->nt_nve, sizeof( struct nbpnve ));
+	memcpy( &ntab->nt_nve, &nn, sizeof( struct nbpnve ));
+	ntab->nt_iface = ap->ap_iface;
 	ntab->nt_next = nbptab;
 	ntab->nt_prev = 0;
-	if ( nbptab != 0 ) {
+	if ( nbptab ) {
 	    nbptab->nt_prev = ntab;
 	}
 	nbptab = ntab;
@@ -204,6 +252,17 @@ nbp_packet( ap, from, data, len )
 	break;
 
     case NBPOP_UNRGSTR :
+        /* deal with local zone info */
+        if (( nn.nn_zonelen == 1 && *nn.nn_zone == '*' ) ||
+	    ( nn.nn_zonelen == 0 )) {
+	  locallkup = 1;
+	  if ( interfaces->i_next->i_rt->rt_zt ) {
+	    zt = (struct ziptab *)interfaces->i_next->i_rt->rt_zt->l_data;
+	  } else {
+	    zt = NULL;
+	  }
+	}
+
 	/* remove from our data, perhaps removing a multicast address */
 	for ( ntab = nbptab; ntab; ntab = ntab->nt_next ) {
 	    if ( ntab->nt_nve.nn_objlen != nn.nn_objlen ||
@@ -218,23 +277,37 @@ nbp_packet( ap, from, data, len )
 	    }
 	    /*
 	     * I *think* we really do check the zone, here.
+	     *
+	     * i changed it to better handle local zone cases as well.
+	     * -- asun
 	     */
-	    if ((( ntab->nt_nve.nn_zonelen == 1 &&
-		    *ntab->nt_nve.nn_zone == '*' ) ||
-		    ntab->nt_nve.nn_zonelen == 0 ) &&
-		    (( nn.nn_zonelen == 1 && *nn.nn_zone == '*' ) ||
-		    nn.nn_zonelen == 0 )) {
+	    
+	    /* match local zones */
+	    if (locallkup) {
+	      /* ntab is also local zone */
+	      if (( ntab->nt_nve.nn_zonelen == 1 &&
+		    *ntab->nt_nve.nn_zone == '*' ) || 
+		  (ntab->nt_nve.nn_zonelen == 0))
 		break;
+	      
+	      /* ntab is default zone */
+	      if (zt && (zt->zt_len == ntab->nt_nve.nn_zonelen) &&
+		  (strndiacasecmp(ntab->nt_nve.nn_zone, zt->zt_name,
+				  zt->zt_len) == 0)) {
+		break;
+	      }
 	    }
-	    if ( ntab->nt_nve.nn_zonelen != nn.nn_zonelen ||
-		    strndiacasecmp( ntab->nt_nve.nn_zone, nn.nn_zone,
-		    nn.nn_zonelen )) {
+	    
+	    /* match particular zone */
+	    if ((ntab->nt_nve.nn_zonelen == nn.nn_zonelen) &&
+		(strndiacasecmp( ntab->nt_nve.nn_zone, nn.nn_zone,
+				nn.nn_zonelen ) == 0)) {
 		break;
 	    }
 	}
 	if ( ntab == 0 ) {
 	    nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
-	    return;
+	    return 0;
 	}
 
 	if ( ntab->nt_next != 0 ) {
@@ -273,7 +346,7 @@ nbp_packet( ap, from, data, len )
 	    } else if ( interfaces->i_next->i_rt->rt_zt ) {
 		zt = (struct ziptab *)interfaces->i_next->i_rt->rt_zt->l_data;
 	    } else {
-		zt = 0;
+		zt = NULL;
 	    }
 
 	    /*
@@ -282,11 +355,11 @@ nbp_packet( ap, from, data, len )
 	     * of this packet.
 	     */
 	    if ( zt ) {
-		bcopy( data - len, packet, len );
+		memcpy( packet, data - len, len );
 		nbpop = packet + ( len - ( data - nbpop ));
 		data = packet + ( len - ( data - zonep ));
 		*data++ = zt->zt_len;
-		bcopy( zt->zt_name, data, zt->zt_len );
+		memcpy( data, zt->zt_name, zt->zt_len );
 		data += zt->zt_len;
 		len = data - packet;
 	    }
@@ -299,33 +372,36 @@ nbp_packet( ap, from, data, len )
 	    }
 	    if ( zt == 0 ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
-		return;
+		return 0;
 	    }
 	}
 
-	locallkup = 0;
-
 	/*
 	 * If we've got no zones, send out LKUP on the local net.
 	 * Otherwise, look through the zone table.
 	 */
 	if ( zt == 0 ) {
+#ifdef BSD4_4
+	    sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
 	    sat.sat_family = AF_APPLETALK;
 	    sat.sat_port = ap->ap_port;
 
 	    nh.nh_op = NBPOP_LKUP;
-	    bcopy( &nh, nbpop, SZ_NBPHDR );
+	    memcpy( nbpop, &nh, SZ_NBPHDR );
 	    sat.sat_addr.s_net = 0;			/* XXX */
 	    sat.sat_addr.s_node = ATADDR_BCAST;
 
 	    /* Find the first non-loopback ap */
 	    for ( iface = interfaces; iface; iface = iface->i_next ) {
-		if (( iface->i_flags & IFACE_LOOPBACK ) == 0 ) {
+		if ((( iface->i_flags & IFACE_LOOPBACK ) == 0) &&
+		    (iface == ap->ap_iface || 
+		    (iface->i_flags & IFACE_ISROUTER))) {
 		    break;
 		}
 	    }
 	    if ( iface == 0 ) {
-		return;
+		return 0;
 	    }
 	    for ( ap = iface->i_ports; ap; ap = ap->ap_next ) {
 		if ( ap->ap_packet == nbp_packet ) {
@@ -335,13 +411,16 @@ nbp_packet( ap, from, data, len )
 
 	    if ( sendto( ap->ap_fd, data - len, len, 0, (struct sockaddr *)&sat,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp brrq sendto: %m" );
+		syslog( LOG_ERR, "nbp brrq sendto: %s", strerror(errno) );
 	    }
 
 	    locallkup = 1;
 	} else {
-	    sat.sat_port = ap->ap_port;
+#ifdef BSD4_4
+	    sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
 	    sat.sat_family = AF_APPLETALK;
+	    sat.sat_port = ap->ap_port;
 	    for ( l = zt->zt_rt; l; l = l->l_next ) {
 		rtmp = (struct rtmptab *)l->l_data;
 
@@ -353,21 +432,23 @@ nbp_packet( ap, from, data, len )
 			}
 		    }
 		    if ( !iface ) {
-			syslog( LOG_ERR, "Can't find route's interface!" );
-			abort();
-		    }
-		    for ( ap = iface->i_ports; ap; ap = ap->ap_next ) {
-			if ( ap->ap_packet == nbp_packet ) {
-			    break;
-			}
-		    }
-		    if ( !ap ) {
-			syslog( LOG_ERR, "Can't find port!" );
-			abort();
+			syslog( LOG_ERR, "nbp_packet: \
+Can't find route's interface!" );
+			return -1;
 		    }
+		    ap = iface->i_ports;
 		} else {
 		    ap = rtmp->rt_gate->g_iface->i_ports;
 		}
+		for ( ; ap; ap = ap->ap_next ) {
+		    if ( ap->ap_packet == nbp_packet ) {
+			break;
+		    }
+		}
+		if ( !ap ) {
+		    syslog( LOG_ERR, "nbp_packet: Can't find port!" );
+		    return -1;
+		}
 
 		if ( transition &&
 			( rtmp->rt_flags & RTMPTAB_EXTENDED ) == 0 ) {
@@ -375,19 +456,19 @@ nbp_packet( ap, from, data, len )
 			locallkup = 1;
 		    }
 		    nh.nh_op = NBPOP_LKUP;
-		    bcopy( &nh, nbpop, SZ_NBPHDR );
+		    memcpy( nbpop, &nh, SZ_NBPHDR );
 		    sat.sat_addr.s_net = rtmp->rt_firstnet;
 		    sat.sat_addr.s_node = ATADDR_BCAST;
 		} else {
 		    if ( rtmp->rt_gate == 0 ) {
 			nh.nh_op = NBPOP_LKUP;
-			bcopy( &nh, nbpop, SZ_NBPHDR );
+			memcpy( nbpop, &nh, SZ_NBPHDR );
 			sat.sat_addr.s_net = 0;
 			sat.sat_addr.s_node = ATADDR_BCAST;
 			locallkup = 1;
 		    } else {
 			nh.nh_op = NBPOP_FWD;
-			bcopy( &nh, nbpop, SZ_NBPHDR );
+			memcpy( nbpop, &nh, SZ_NBPHDR );
 			sat.sat_addr.s_net = rtmp->rt_firstnet;
 			sat.sat_addr.s_node = 0;
 		    }
@@ -396,8 +477,9 @@ nbp_packet( ap, from, data, len )
 		if ( sendto( ap->ap_fd, data - len, len, 0,
 			(struct sockaddr *)&sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "nbp brrq sendto %u.%u: %m",
-			    ntohs( sat.sat_addr.s_net ), sat.sat_addr.s_node );
+		    syslog( LOG_ERR, "nbp brrq sendto %u.%u: %s",
+			    ntohs( sat.sat_addr.s_net ), sat.sat_addr.s_node,
+			    strerror(errno) );
 		    continue;
 		}
 	    }
@@ -409,18 +491,18 @@ nbp_packet( ap, from, data, len )
 	/*FALL THROUGH*/
 
     case NBPOP_FWD :
-	/* send lkup on net */
-	/* should we check that we're a router? XXX */
-	if ( !locallkup ) {
+        /* send lkup on net. we need to make sure we're a router. */
+        if ( !locallkup && (ap->ap_iface->i_flags & IFACE_ISROUTER)) {
 	    nh.nh_op = NBPOP_LKUP;
-	    bcopy( &nh, nbpop, SZ_NBPHDR );
+	    memcpy( nbpop, &nh, SZ_NBPHDR );
 	    from->sat_addr.s_net = 0;
-	    from->sat_addr.s_node = 0xff;
+	    from->sat_addr.s_node = ATADDR_BCAST;
 	    if ( sendto( ap->ap_fd, data - len, len, 0, (struct sockaddr *)from,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp fwd sendto %u.%u: %m",
-			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-		return;
+		syslog( LOG_ERR, "nbp fwd sendto %u.%u: %s",
+			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
+			strerror(errno) );
+		return 0;
 	    }
 	}
 	/*FALL THROUGH*/
@@ -432,6 +514,12 @@ nbp_packet( ap, from, data, len )
 	end = packet + sizeof( packet );
 
 	for ( ntab = nbptab; ntab; ntab = ntab->nt_next ) {
+	    /* don't send out entries if we don't want to route. */
+	    if ((ap->ap_iface != ntab->nt_iface) && 
+		(ntab->nt_iface->i_flags & IFACE_ISROUTER) == 0) {
+	      continue;
+	    }
+
 	    if ( nn.nn_objlen != 1 || *nn.nn_obj != '=' ) {
 		if ( ntab->nt_nve.nn_objlen != nn.nn_objlen ||
 			strndiacasecmp( ntab->nt_nve.nn_obj, nn.nn_obj,
@@ -482,15 +570,16 @@ nbp_packet( ap, from, data, len )
 		cc = data - packet;
 		data = packet;
 		*data++ = DDPTYPE_NBP;
-		bcopy( &nh, data, SZ_NBPHDR );
+		memcpy( data, &nh, SZ_NBPHDR );
 
 		if ( sendto( ap->ap_fd, packet, cc, 0,
 			(struct sockaddr *)&nn.nn_sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "nbp lkup sendto %u.%u: %m",
+		    syslog( LOG_ERR, "nbp lkup sendto %u.%u: %s",
 			    ntohs( nn.nn_sat.sat_addr.s_net ),
-			    nn.nn_sat.sat_addr.s_node );
-		    return;
+			    nn.nn_sat.sat_addr.s_node,
+			    strerror(errno) );
+		    return 0;
 		}
 
 		n = 0;
@@ -510,15 +599,15 @@ nbp_packet( ap, from, data, len )
 	     */
 	    nt.nt_enum = i++;
 
-	    bcopy( &nt, data, SZ_NBPTUPLE );
+	    memcpy( data, &nt, SZ_NBPTUPLE );
 	    data += SZ_NBPTUPLE;
 
 	    *data++ = ntab->nt_nve.nn_objlen;
-	    bcopy( ntab->nt_nve.nn_obj, data, ntab->nt_nve.nn_objlen );
+	    memcpy( data, ntab->nt_nve.nn_obj, ntab->nt_nve.nn_objlen );
 	    data += ntab->nt_nve.nn_objlen;
 
 	    *data++ = ntab->nt_nve.nn_typelen;
-	    bcopy( ntab->nt_nve.nn_type, data, ntab->nt_nve.nn_typelen );
+	    memcpy(data, ntab->nt_nve.nn_type, ntab->nt_nve.nn_typelen );
 	    data += ntab->nt_nve.nn_typelen;
 
 	    /*
@@ -528,7 +617,7 @@ nbp_packet( ap, from, data, len )
 	     */
 	    if ( ntab->nt_nve.nn_zonelen ) {
 		*data++ = ntab->nt_nve.nn_zonelen;
-		bcopy( ntab->nt_nve.nn_zone, data, ntab->nt_nve.nn_zonelen );
+		memcpy( data, ntab->nt_nve.nn_zone, ntab->nt_nve.nn_zonelen );
 		data += ntab->nt_nve.nn_zonelen;
 	    } else {
 		*data++ = 1;
@@ -544,43 +633,24 @@ nbp_packet( ap, from, data, len )
 	    cc = data - packet;
 	    data = packet;
 	    *data++ = DDPTYPE_NBP;
-	    bcopy( &nh, data, SZ_NBPHDR );
+	    memcpy( data, &nh, SZ_NBPHDR );
 
 	    if ( sendto( ap->ap_fd, packet, cc, 0,
 		    (struct sockaddr *)&nn.nn_sat,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp lkup sendto %u.%u: %m",
+		syslog( LOG_ERR, "nbp lkup sendto %u.%u: %s",
 			ntohs( nn.nn_sat.sat_addr.s_net ),
-			nn.nn_sat.sat_addr.s_node );
-		return;
+			nn.nn_sat.sat_addr.s_node,
+			strerror(errno) );
+		return 0;
 	    }
 	}
 	break;
 
     default :
 	syslog( LOG_INFO, "nbp_packet: bad op (%d)", nh.nh_op );
+	return 1;
     }
-}
 
-nbp_ack( fd, nh_op, nh_id, to )
-    int			fd;
-    int			nh_op;
-    int			nh_id;
-    struct sockaddr_at	*to;
-{
-    struct nbphdr	nh;
-    char		*data, packet[ SZ_NBPHDR + 1 ];
-
-    nh.nh_op = nh_op;
-    nh.nh_cnt = 0;
-    nh.nh_id = nh_id;
-    data = packet;
-    *data++ = DDPTYPE_NBP;
-    bcopy( &nh, data, SZ_NBPHDR );
-    data += SZ_NBPHDR;
-    if ( sendto( fd, packet, data - packet, 0, (struct sockaddr *)to,
-	    sizeof( struct sockaddr_at )) < 0 ) {
-	syslog( LOG_ERR, "sendto: %m" );
-	return;
-    }
+    return 0;
 }
