@@ -1,18 +1,37 @@
 /*
+ * $Id: rtmp.c,v 1.8.2.2 2002/03/12 15:39:06 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
-#include <sys/syslog.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
+#include <syslog.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <net/route.h>
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
 
+#ifdef __svr4__
+#include <sys/sockio.h>
+#endif /* __svr4__ */
+
 #include <atalk/ddp.h>
+#include <atalk/atp.h>
 #include <atalk/rtmp.h>
 
 #include "interface.h"
@@ -21,10 +40,370 @@
 #include "zip.h"
 #include "list.h"
 #include "atserv.h"
+#include "route.h"
+#include "main.h"
 
-extern int	ninterfaces;
+void rtmp_delzonemap(rtmp)
+    struct rtmptab *rtmp;
+{
+    struct list		*lz, *flz, *lr, *flr;
+    struct ziptab	*zt;
 
-rtmp_packet( ap, from, data, len )
+    lz = rtmp->rt_zt;
+    while ( lz ) {					/* for each zone */
+	zt = (struct ziptab *)lz->l_data;
+	lr = zt->zt_rt;
+	while ( lr ) {					/* for each route */
+	    if ( (struct rtmptab *)lr->l_data == rtmp ) {
+		if ( lr->l_prev == NULL ) {		/* head */
+		    if ( lr->l_next == NULL ) {		/* last route in zone */
+			if ( zt->zt_prev == NULL ) {
+			    ziptab = zt->zt_next;
+			} else {
+			    zt->zt_prev->zt_next = zt->zt_next;
+			}
+			if ( zt->zt_next == NULL ) {
+			    ziplast = zt->zt_prev;
+			} else {
+			    zt->zt_next->zt_prev = zt->zt_prev;
+			}
+			free( zt->zt_bcast );
+			free( zt->zt_name );
+			free( zt );
+		    } else {
+			zt->zt_rt = lr->l_next;
+		    }
+		} else {
+		    lr->l_prev->l_next = lr->l_next;
+		}
+		if ( lr->l_next != NULL ) {
+		    lr->l_next->l_prev = lr->l_prev;
+		}
+		flr = lr;
+		lr = lr->l_next;
+		free( flr );
+	    } else {
+		lr = lr->l_next;
+	    }
+	}
+	flz = lz;
+	lz = lz->l_next;
+	free( flz );
+    }
+    rtmp->rt_zt = NULL;
+}
+
+
+/*
+ * Complete configuration for phase 1 interface using RTMP information.
+ */
+static int rtmp_config( rh, iface )
+    struct rtmp_head	*rh;
+    struct interface	*iface;
+{
+    extern int		stabletimer;
+    int cc;
+
+    /*
+     * If we're configuring a phase 2 interface, don't complete
+     * configuration with RTMP.
+     */
+    if ( iface->i_flags & IFACE_PHASE2 ) {
+	syslog( LOG_INFO, "rtmp_config ignoring data" );
+	return 0;
+    }
+
+    /*
+     * Check our seed information, and reconfigure.
+     */
+    if ( rh->rh_net != iface->i_addr.sat_addr.s_net ) {
+	if (( iface->i_flags & IFACE_SEED ) &&
+		rh->rh_net != iface->i_caddr.sat_addr.s_net) {
+	    syslog( LOG_ERR, "rtmp_config net mismatch %u != %u",
+		    ntohs( rh->rh_net ),
+		    ntohs( iface->i_addr.sat_addr.s_net ));
+	    return 1;
+	}
+	iface->i_addr.sat_addr.s_net = rh->rh_net;
+
+	/*
+	 * It is possible that we will corrupt our route database
+	 * by just forcing this change.  XXX
+	 */
+	iface->i_rt->rt_firstnet = iface->i_rt->rt_lastnet = rh->rh_net;
+
+	setaddr( iface, IFACE_PHASE1, iface->i_addr.sat_addr.s_net,
+		iface->i_addr.sat_addr.s_node, rh->rh_net, rh->rh_net );
+	stabletimer = UNSTABLE;
+    }
+
+    /* add addr to loopback route */
+    if ((cc = looproute( iface, RTMP_ADD )) < 0 )
+      return -1;
+
+    if (cc) {
+	syslog( LOG_ERR, "rtmp_config: can't route %u.%u to loopback: %s",
+		ntohs( iface->i_addr.sat_addr.s_net ),
+		iface->i_addr.sat_addr.s_node,
+		strerror(errno) );
+    }
+
+    syslog( LOG_INFO, "rtmp_config configured %s", iface->i_name );
+    iface->i_flags |= IFACE_CONFIG;
+    if ( iface == ciface ) {
+	ciface = ciface->i_next;
+	bootaddr( ciface );
+    }
+
+    return 0;
+}
+
+/*
+ * Delete rtmp from the per-interface in-use table, remove all
+ * zone references, and remove the route from the kernel.
+ */
+static void rtmp_delinuse( rtmp )
+    struct rtmptab	*rtmp;
+{
+    struct rtmptab	*irt;
+
+    irt = rtmp->rt_gate->g_iface->i_rt;
+    if ( irt->rt_inext == rtmp ) {			/* first */
+	if ( rtmp->rt_iprev == rtmp ) {			/* only */
+	    irt->rt_inext = NULL;
+	} else {
+	    irt->rt_inext = rtmp->rt_inext;
+	    rtmp->rt_inext->rt_iprev = rtmp->rt_iprev;
+	}
+    } else {
+	if ( rtmp->rt_inext == NULL ) {			/* last */
+	    rtmp->rt_iprev->rt_inext = NULL;
+	    irt->rt_inext->rt_iprev = rtmp->rt_iprev;
+	} else {
+	    rtmp->rt_iprev->rt_inext = rtmp->rt_inext;
+	    rtmp->rt_inext->rt_iprev = rtmp->rt_iprev;
+	}
+    }
+    rtmp->rt_iprev = NULL;
+    rtmp->rt_inext = NULL;
+
+    /* remove zone map */
+    rtmp_delzonemap(rtmp);
+
+    /* remove old route */
+    gateroute( RTMP_DEL, rtmp );
+}
+
+/*
+ * Add rtmp to the per-interface in-use table.  No verification is done...
+ */
+static void rtmp_addinuse( rtmp )
+    struct rtmptab	*rtmp;
+{
+    struct rtmptab	*irt;
+
+    gateroute( RTMP_ADD, rtmp );
+
+    irt = rtmp->rt_gate->g_iface->i_rt;
+    if ( irt->rt_inext == NULL ) {	/* empty list */
+	rtmp->rt_inext = NULL;
+	rtmp->rt_iprev = rtmp;
+	irt->rt_inext = rtmp;
+    } else {
+	rtmp->rt_inext = irt->rt_inext;
+	rtmp->rt_iprev = irt->rt_inext->rt_iprev;
+	irt->rt_inext->rt_iprev = rtmp;
+	irt->rt_inext = rtmp;
+    }
+}
+
+
+/*
+ * Change the zone mapping to replace "from" with "to".  This code assumes
+ * the consistency of both the route -> zone map and the zone -> route map.
+ * This is probably a bad idea.  How can we insure that the data is good
+ * at this point?  What do we do if we get several copies of a route in
+ * an RTMP packet?
+ */
+static int rtmp_copyzones( to, from )
+    struct rtmptab	*to, *from;
+{
+    struct list		*lz, *lr;
+
+    to->rt_zt = from->rt_zt;
+    from->rt_zt = NULL;
+    if ( from->rt_flags & RTMPTAB_HASZONES ) {
+	to->rt_flags |= RTMPTAB_HASZONES;
+    }
+    for ( lz = to->rt_zt; lz; lz = lz->l_next ) {
+	for ( lr = ((struct ziptab *)lz->l_data)->zt_rt; lr; lr = lr->l_next ) {
+	    if ( (struct rtmptab *)lr->l_data == from ) {
+		lr->l_data = (void *)to;	/* cast BS */
+		break;
+	    }
+	}
+	if ( lr == NULL ) {
+	    syslog( LOG_ERR, "rtmp_copyzones z -> r without r -> z, abort" );
+	    return -1;
+	}
+    }
+
+    return 0;
+}
+
+
+/*
+ * Remove rtmp from the in-use table and the per-gate table.
+ * Free any associated space.
+ */
+void rtmp_free( rtmp )
+    struct rtmptab	*rtmp;
+{
+    struct gate		*gate;
+
+    syslog(LOG_INFO, "rtmp_free: %u-%u", ntohs(rtmp->rt_firstnet),
+	   ntohs(rtmp->rt_lastnet));
+    if ( rtmp->rt_iprev ) {
+	rtmp_delinuse( rtmp );
+    }
+
+    /* remove from per-gate */
+    gate = rtmp->rt_gate;
+    if ( gate->g_rt == rtmp ) {				/* first */
+	if ( rtmp->rt_prev == rtmp ) {			/* only */
+	    gate->g_rt = NULL;
+	} else {
+	    gate->g_rt = rtmp->rt_next;
+	    rtmp->rt_next->rt_prev = rtmp->rt_prev;
+	}
+    } else {
+	if ( rtmp->rt_next == NULL ) {			/* last */
+	    rtmp->rt_prev->rt_next = NULL;
+	    gate->g_rt->rt_prev = rtmp->rt_prev;
+	} else {
+	    rtmp->rt_prev->rt_next = rtmp->rt_next;
+	    rtmp->rt_next->rt_prev = rtmp->rt_prev;
+	}
+    }
+
+    free( rtmp );
+}
+
+
+/*
+ * Find a replacement for "replace".  If we can't find a replacement,
+ * return 1.  If we do find a replacement, return 0. -1 on error.
+ */
+int rtmp_replace( replace )
+    struct rtmptab	*replace;
+{
+    struct interface	*iface;
+    struct gate		*gate;
+    struct rtmptab	*rtmp, *found = NULL;
+
+    syslog(LOG_INFO, "rtmp_replace %u-%u", ntohs(replace->rt_firstnet),
+	   ntohs(replace->rt_lastnet));
+    for ( iface = interfaces; iface; iface = iface->i_next ) {
+        if ((replace->rt_iface != iface) && 
+	    ((iface->i_flags & IFACE_ISROUTER) == 0))
+	  continue;
+
+	for ( gate = iface->i_gate; gate; gate = gate->g_next ) {
+	    for ( rtmp = gate->g_rt; rtmp; rtmp = rtmp->rt_next ) {
+		if ( rtmp->rt_firstnet == replace->rt_firstnet &&
+			rtmp->rt_lastnet == replace->rt_lastnet ) {
+		    if ( found == NULL || rtmp->rt_hops < found->rt_hops ) {
+			found = rtmp;
+		    }
+		    break;
+		}
+	    }
+	}
+    }
+
+    if ( found != replace ) {
+	if (rtmp_copyzones( found, replace ) < 0)
+	  return -1;
+	rtmp_delinuse( replace );
+	rtmp_addinuse( found );
+	if ( replace->rt_state == RTMPTAB_BAD ) {
+	    rtmp_free( replace );
+	}
+	return( 0 );
+    } else {
+	if ( replace->rt_hops == RTMPHOPS_POISON ) {
+	    gateroute( RTMP_DEL, replace );
+	}
+	return( 1 );
+    }
+}
+
+
+static int rtmp_new( rtmp )
+    struct rtmptab	*rtmp;
+{
+    struct interface	*i;
+    struct rtmptab	*r;
+    extern int		newrtmpdata;
+
+    newrtmpdata = 1;
+
+    /*
+     * Do we already have a gateway for this route?
+     */
+    for ( i = interfaces; i; i = i->i_next ) {
+        if ((rtmp->rt_iface != i) && 
+	    ((i->i_flags & IFACE_ISROUTER) == 0))
+	  continue;
+
+	for ( r = i->i_rt; r; r = r->rt_inext ) {
+	    /* Should check RTMPTAB_EXTENDED here. XXX */
+	    if (( ntohs( r->rt_firstnet ) <= ntohs( rtmp->rt_firstnet ) &&
+		    ntohs( r->rt_lastnet ) >= ntohs( rtmp->rt_firstnet )) ||
+		    ( ntohs( r->rt_firstnet ) <= ntohs( rtmp->rt_lastnet ) &&
+		    ntohs( r->rt_lastnet ) >= ntohs( rtmp->rt_lastnet ))) {
+		break;
+	    }
+	}
+	if ( r ) {
+	    break;
+	}
+    }
+
+    /*
+     * This part of this routine is almost never run.
+     */
+    if ( i ) {	/* can we get here without r being set? */
+	if ( r->rt_firstnet != rtmp->rt_firstnet ||
+		r->rt_lastnet != rtmp->rt_lastnet ) {
+	    syslog( LOG_INFO, "rtmp_new netrange mismatch %u-%u != %u-%u",
+		    ntohs( r->rt_firstnet ), ntohs( r->rt_lastnet ),
+		    ntohs( rtmp->rt_firstnet ), ntohs( rtmp->rt_lastnet ));
+	    return 1;
+	}
+
+	/*
+	 * Note that our whole methodology is wrong, if we want to do
+	 * route "load balancing."  This entails changing our route
+	 * each time we receive a tuple of equal value.  In fact, we can't
+	 * do this, using our method, since we only check against in-use
+	 * routes when a tuple is new from a router.
+	 */
+	if ( r->rt_hops < rtmp->rt_hops ) {
+	    return 1;
+	}
+
+	if (rtmp_copyzones( rtmp, r ) < 0)
+	  return -1;
+	rtmp_delinuse( r );
+    }
+
+    rtmp_addinuse( rtmp );
+    return 0;
+}
+
+
+int rtmp_packet( ap, from, data, len )
     struct atport	*ap;
     struct sockaddr_at	*from;
     char		*data;
@@ -35,15 +414,14 @@ rtmp_packet( ap, from, data, len )
     struct gate		*gate;
     struct interface	*iface;
     struct rtmptab	*rtmp;
-    u_short		tonet;
-    char		*end, packet[ 587 ];
-    extern int		debug;
+    char		*end, packet[ ATP_BUFSIZ ];
+    int                 cc;
 
     end = data + len;
 
     if ( data >= end ) {
 	syslog( LOG_INFO, "rtmp_packet no data" );
-	return;
+	return 1;
     }
 
     iface = ap->ap_iface;
@@ -51,7 +429,7 @@ rtmp_packet( ap, from, data, len )
     /* ignore our own packets */
     if ( from->sat_addr.s_net == iface->i_addr.sat_addr.s_net &&
 	    from->sat_addr.s_node == iface->i_addr.sat_addr.s_node  ) {
-	return;
+	return 0;
     }
 
     switch( *data++ ) {
@@ -61,43 +439,53 @@ rtmp_packet( ap, from, data, len )
 	 */
 	if ( data + sizeof( struct rtmprdhdr ) > end ) {
 	    syslog( LOG_INFO, "rtmp_packet no data header" );
-	    return;
+	    return 1;
 	}
-	bcopy( data, &rh, sizeof( struct rtmprdhdr ));
+	memcpy( &rh, data, sizeof( struct rtmprdhdr ));
 	data += sizeof( struct rtmprdhdr );
 
 	/* check rh address against from address */
 	if ( rh.rh_nodelen != 8 ) {
 	    syslog( LOG_INFO, "rtmp_packet bad node len (%d)", rh.rh_nodelen );
-	    return;
+	    return 1;
 	}
-	if (( from->sat_addr.s_net != 0 && from->sat_addr.s_net != rh.rh_net )||
-		from->sat_addr.s_node != rh.rh_node ) {
+	if (( from->sat_addr.s_net != 0 && 
+	      from->sat_addr.s_net != rh.rh_net ) ||
+	      from->sat_addr.s_node != rh.rh_node ) {
 	    syslog( LOG_INFO, "rtmp_packet address mismatch" );
-	    return;
+	    return 1;
 	}
 
 	if (( iface->i_flags & ( IFACE_ADDR|IFACE_CONFIG )) == IFACE_ADDR ) {
 	    if ( iface->i_flags & IFACE_NOROUTER ) {
 		/* remove addr to loopback route */
-		if ( looproute( iface, RTMP_DEL ) < 0 ) {
-		    syslog( LOG_ERR, "rtmp_packet: can't remove loopback: %m" );
-		}
+  	        if ((cc = looproute( iface, RTMP_DEL )) < 0) {
+		  syslog(LOG_ERR, "rtmp_packet: looproute");
+		  return -1;
+	        } 
+
+		if (cc)
+		  syslog( LOG_ERR, "rtmp_packet: can't remove loopback: %s",
+			  strerror(errno) );
 
 		iface->i_flags &= ~IFACE_NOROUTER;
 		iface->i_time = 0;
 		syslog( LOG_INFO, "rtmp_packet router has become available" );
 	    }
 	    if ( iface->i_flags & IFACE_PHASE1 ) {
-		rtmp_config( &rh, iface );
-	    } else {
-		zip_getnetinfo( iface );
+	      if (rtmp_config( &rh, iface ) < 0) {
+		  syslog(LOG_ERR, "rtmp_packet: rtmp_config");
+		  return -1;
+	      }
+	    } else if (zip_getnetinfo( iface ) < 0) {
+	      syslog(LOG_ERR, "rtmp_packet: zip_getnetinfo");
+	      return -1;
 	    }
-	    return;
+	    return 0;
 	}
 
 	if (( iface->i_flags & IFACE_CONFIG ) == 0 ) {
-	    return;
+	    return 0;
 	}
 
 	/*
@@ -105,15 +493,15 @@ rtmp_packet( ap, from, data, len )
 	 */
 	if ( data + SZ_RTMPTUPLE > end ) {
 	    syslog( LOG_INFO, "rtmp_packet missing first tuple" );
-	    return;
+	    return 1;
 	}
-	bcopy( data, &rt, SZ_RTMPTUPLE );
+	memcpy( &rt, data, SZ_RTMPTUPLE );
 	data += SZ_RTMPTUPLE;
 
 	if ( rt.rt_net == 0 ) {
 	    if ( rt.rt_dist != 0x82 ) {
 		syslog( LOG_INFO, "rtmp_packet bad phase 1 version" );
-		return;
+		return 1;
 	    }
 
 	    /*
@@ -123,21 +511,21 @@ rtmp_packet( ap, from, data, len )
 	     */
 	    if ( data + SZ_RTMPTUPLE > end ) {
 		syslog( LOG_INFO, "rtmp_packet missing second tuple" );
-		return;
+		return 1;
 	    }
-	    bcopy( data, &rt, SZ_RTMPTUPLE );
+	    memcpy( &rt, data, SZ_RTMPTUPLE );
 	    data += SZ_RTMPTUPLE;
 	} else if ( rt.rt_dist & 0x80 ) {
 	    if ( data + SZ_RTMPTUPLE > end ) {
 		syslog( LOG_INFO, "rtmp_packet missing first range-end" );
-		return;
+		return 1;
 	    }
-	    bcopy( data, &xrt, SZ_RTMPTUPLE );
+	    memcpy( &xrt, data, SZ_RTMPTUPLE );
 	    data += SZ_RTMPTUPLE;
 
 	    if ( xrt.rt_dist != 0x82 ) {
 		syslog( LOG_INFO, "rtmp_packet bad phase 2 version" );
-		return;
+		return 1;
 	    }
 
 	    /*
@@ -146,7 +534,7 @@ rtmp_packet( ap, from, data, len )
 	    if ( rt.rt_net != iface->i_rt->rt_firstnet ||
 		    xrt.rt_net != iface->i_rt->rt_lastnet ) {
 		syslog( LOG_INFO, "rtmp_packet interface mismatch" );
-		return;
+		return 1;
 	    }
 	} else {
 #ifdef PHASE1NET
@@ -157,13 +545,13 @@ rtmp_packet( ap, from, data, len )
 	    if ( rt.rt_net != iface->i_rt->rt_firstnet ||
 		    rt.rt_net != iface->i_rt->rt_lastnet ) {
 		syslog( LOG_INFO, "rtmp_packet phase 1 interface mismatch" );
-		return;
+		return 1;
 	    }
 	     */
-#else PHASE1NET
+#else /* PHASE1NET */
 	    syslog( LOG_INFO, "rtmp_packet bad first tuple" );
-	    return;
-#endif PHASE1NET
+	    return 1;
+#endif /* PHASE1NET */
 	}
 
 	/*
@@ -177,8 +565,8 @@ rtmp_packet( ap, from, data, len )
 	}
 	if ( !gate ) {	/* new gateway */
 	    if (( gate = (struct gate *)malloc( sizeof( struct gate ))) == 0 ) {
-		syslog( LOG_ERR, "malloc: %m" );
-		exit( 1 );
+		syslog( LOG_ERR, "rtmp_packet: malloc: %s", strerror(errno) );
+		return -1;
 	    }
 	    gate->g_next = iface->i_gate;
 	    gate->g_prev = 0;
@@ -227,29 +615,29 @@ rtmp_packet( ap, from, data, len )
 		if ( rtmp->rt_firstnet != rt.rt_net ) {
 		    syslog( LOG_INFO, "rtmp_packet firstnet mismatch %u!=%u",
 			    ntohs( rtmp->rt_firstnet ), ntohs( rt.rt_net ));
-		    return;
+		    return 1;
 		}
 		if ( rt.rt_dist & 0x80 ) {
 		    if (( rtmp->rt_flags & RTMPTAB_EXTENDED ) == 0 ) {
 			syslog( LOG_INFO, "rtmp_packet extended mismatch %u",
 				ntohs( rtmp->rt_firstnet ));
-			return;
+			return 1;
 		    }
 		    if ( rtmp->rt_lastnet != xrt.rt_net ) {
 			syslog( LOG_INFO, "rtmp_packet lastnet mismatch %u!=%u",
 			    ntohs( rtmp->rt_lastnet ), ntohs( xrt.rt_net ));
-			return;
+			return 1;
 		    }
 		} else {
 		    if ( rtmp->rt_flags & RTMPTAB_EXTENDED ) {
 			syslog( LOG_INFO, "rtmp_packet !extended mismatch %u",
 				ntohs( rtmp->rt_firstnet ));
-			return;
+			return 1;
 		    }
 		    if ( rtmp->rt_lastnet != rt.rt_net ) {
 			syslog( LOG_INFO, "rtmp_packet lastnet mismatch %u!=%u",
 			    ntohs( rtmp->rt_lastnet ), ntohs( rt.rt_net ));
-			return;
+			return 1;
 		    }
 		}
 
@@ -284,12 +672,18 @@ rtmp_packet( ap, from, data, len )
 			    } else {
 				rtmp->rt_hops = ( rt.rt_dist & 0x7f ) + 1;
 			    }
-			    (void)rtmp_replace( rtmp );
+			    if (rtmp_replace( rtmp ) < 0) {
+			      syslog(LOG_ERR, "rtmp_packet: rtmp_replace");
+			      return -1;
+			    }
 			}
 		    } else {			/* route not in use */
 			rtmp->rt_hops = ( rt.rt_dist & 0x7f ) + 1;
 			if ( rtmp->rt_hops > ( rt.rt_dist & 0x7f ) + 1 ) {
-			    rtmp_new( rtmp );
+			  if (rtmp_new( rtmp ) < 0) {
+			      syslog(LOG_ERR, "rtmp_packet: rtmp_new");
+			      return -1;
+			  }
 			}
 		    }
 		}
@@ -309,9 +703,9 @@ rtmp_packet( ap, from, data, len )
 			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
 			ntohs( rt.rt_net ));
 	    } else {		/* new for router */
-		if (( rtmp = newrt()) == 0 ) {
-		    syslog( LOG_ERR, "newrt: %m" );
-		    exit( 1 );
+		if (( rtmp = newrt(iface)) == NULL ) {
+		    syslog( LOG_ERR, "rtmp_packet: newrt: %s", strerror(errno) );
+		    return -1;
 		}
 		rtmp->rt_firstnet = rt.rt_net;
 		if ( rt.rt_dist & 0x80 ) {
@@ -335,21 +729,24 @@ rtmp_packet( ap, from, data, len )
 		    gate->g_rt->rt_prev->rt_next = rtmp;
 		    gate->g_rt->rt_prev = rtmp;
 		}
-
-		rtmp_new( rtmp );
+		
+		if (rtmp_new( rtmp ) < 0) {
+		    syslog(LOG_ERR, "rtmp_packet: rtmp_new");
+		    return -1;
+		}
 	    }
 
 	    if ( data + SZ_RTMPTUPLE > end ) {
 		break;
 	    }
-	    bcopy( data, &rt, SZ_RTMPTUPLE );
+	    memcpy( &rt, data, SZ_RTMPTUPLE );
 	    data += SZ_RTMPTUPLE;
 	    if ( rt.rt_dist & 0x80 ) {
 		if ( data + SZ_RTMPTUPLE > end ) {
 		    syslog( LOG_INFO, "rtmp_packet missing range-end" );
-		    return;
+		    return 1;
 		}
-		bcopy( data, &xrt, SZ_RTMPTUPLE );
+		memcpy( &xrt, data, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 	    }
 	}
@@ -366,9 +763,10 @@ rtmp_packet( ap, from, data, len )
 	/*
 	 * Request and RDR.
 	 */
-	if ( ninterfaces == IFBASE || iface->i_rt->rt_zt == 0 ||
-		( iface->i_flags & IFACE_CONFIG ) == 0 ) {
-	    return;
+        if (((iface->i_flags & IFACE_ISROUTER) == 0) ||
+	    iface->i_rt->rt_zt == 0 ||
+	    ( iface->i_flags & IFACE_CONFIG ) == 0 ) {
+	    return 0;
 	}
 	if ( *data == 1 ) {
 	    data = packet;
@@ -376,31 +774,31 @@ rtmp_packet( ap, from, data, len )
 	    rh.rh_net = iface->i_addr.sat_addr.s_net;
 	    rh.rh_nodelen = 8;
 	    rh.rh_node = iface->i_addr.sat_addr.s_node;
-	    bcopy( &rh, data, sizeof( struct rtmp_head ));
+	    memcpy( data, &rh, sizeof( struct rtmp_head ));
 	    data += sizeof( struct rtmp_head );
 
 	    if ( iface->i_flags & IFACE_PHASE2 ) {
 		rt.rt_net = iface->i_rt->rt_firstnet;
 		rt.rt_dist = 0x80;
-		bcopy( &rt, data, SZ_RTMPTUPLE );
+		memcpy( data, &rt, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 
 		rt.rt_net = iface->i_rt->rt_lastnet;
 		rt.rt_dist = 0x82;
-		bcopy( &rt, data, SZ_RTMPTUPLE );
+		memcpy( data, &rt, SZ_RTMPTUPLE );
 		data += SZ_RTMPTUPLE;
 	    }
 	    if ( sendto( ap->ap_fd, packet, data - packet, 0,
 		    (struct sockaddr *)from,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "as_timer sendto: %m" );
+		syslog( LOG_ERR, "as_timer sendto: %s", strerror(errno) );
 	    }
 	} else if ( *data == 2 || *data == 3 ) {
 #ifdef DEBUG
 	    printf( "rtmp_packet rdr (%d) from %u.%u\n",
 		    *data, ntohs( from->sat_addr.s_net ),
 		    from->sat_addr.s_node );
-#endif DEBUG
+#endif /* DEBUG */
 	} else {
 	    syslog( LOG_INFO, "rtmp_packet unknown request from %u.%u\n",
 		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
@@ -410,17 +808,18 @@ rtmp_packet( ap, from, data, len )
     default :
 	syslog( LOG_INFO, "rtmp_packet bad ddp type from %u.%u",
 		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-	return;
+	return 0;
     }
+
+    return 0;
 }
 
-rtmp_request( iface )
+int rtmp_request( iface )
     struct interface	*iface;
 {
     struct sockaddr_at	sat;
     struct atport	*ap;
     char		*data, packet[ 2 ];
-    int			i;
 
     syslog( LOG_INFO, "rtmp_request for %s", iface->i_name );
 
@@ -431,7 +830,7 @@ rtmp_request( iface )
     }
     if ( ap == 0 ) {
 	syslog( LOG_ERR, "rtmp_request can't find rtmp socket!" );
-	exit( 1 );
+	return -1;
     }
 
     data = packet;
@@ -441,104 +840,69 @@ rtmp_request( iface )
     /*
      * There is a problem with the net zero "hint" hack.
      */
-    bzero( &sat, sizeof( struct sockaddr_at ));
+    memset( &sat, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     sat.sat_family = AF_APPLETALK;
     sat.sat_addr.s_net = iface->i_addr.sat_addr.s_net;
     sat.sat_addr.s_node = ATADDR_BCAST;
     sat.sat_port = ap->ap_port;
     if ( sendto( ap->ap_fd, packet, data - packet, 0, (struct sockaddr *)&sat,
 	    sizeof( struct sockaddr_at )) < 0 ) {
-	syslog( LOG_ERR, "rtmp_request sendto: %m" );
-	exit( 1 );
+	syslog( LOG_ERR, "rtmp_request sendto: %s", strerror(errno) );
+	return -1;
     }
-    return;
+    return 0;
 }
 
-/*
- * Complete configuration for phase 1 interface using RTMP information.
- */
-rtmp_config( rh, iface )
-    struct rtmp_head	*rh;
+
+int looproute( iface, cmd )
     struct interface	*iface;
-{
-    extern int		stabletimer;
-
-    /*
-     * If we're configuring a phase 2 interface, don't complete
-     * configuration with RTMP.
-     */
-    if ( iface->i_flags & IFACE_PHASE2 ) {
-	syslog( LOG_INFO, "rtmp_config ignoring data" );
-	return;
-    }
-
-    /*
-     * Check our seed information, and reconfigure.
-     */
-    if ( rh->rh_net != iface->i_addr.sat_addr.s_net ) {
-	if (( iface->i_flags & IFACE_SEED ) &&
-		rh->rh_net != iface->i_caddr.sat_addr.s_net) {
-	    syslog( LOG_ERR, "rtmp_config net mismatch %u != %u",
-		    ntohs( rh->rh_net ),
-		    ntohs( iface->i_addr.sat_addr.s_net ));
-	    exit( 1 );
-	}
-	iface->i_addr.sat_addr.s_net = rh->rh_net;
-
-	/*
-	 * It is possible that we will corrupt our route database
-	 * by just forcing this change.  XXX
-	 */
-	iface->i_rt->rt_firstnet = iface->i_rt->rt_lastnet = rh->rh_net;
-
-	setaddr( iface, IFACE_PHASE1, iface->i_addr.sat_addr.s_net,
-		iface->i_addr.sat_addr.s_node, rh->rh_net, rh->rh_net );
-	stabletimer = UNSTABLE;
-    }
-
-    /* add addr to loopback route */
-    if ( looproute( iface, RTMP_ADD ) < 0 ) {
-	syslog( LOG_ERR, "rtmp_config: can't route %u.%u to loopback: %m",
-		ntohs( iface->i_addr.sat_addr.s_net ),
-		iface->i_addr.sat_addr.s_node );
-    }
-
-    syslog( LOG_INFO, "rtmp_config configured %s", iface->i_name );
-    iface->i_flags |= IFACE_CONFIG;
-    if ( iface == ciface ) {
-	ciface = ciface->i_next;
-	bootaddr( ciface );
-    }
-}
-
-looproute( iface, cmd )
-    struct interface	*iface;
-    int			cmd;
+    unsigned int	cmd;
 {
     struct sockaddr_at	dst, loop;
 
     if ( cmd == RTMP_DEL && ( iface->i_flags & IFACE_LOOP ) == 0 ) {
 	syslog( LOG_ERR, "looproute panic no route" );
-	abort();
+	return -1;
     }
 
     if ( cmd == RTMP_ADD && ( iface->i_flags & IFACE_LOOP )) {
 	syslog( LOG_ERR, "looproute panic two routes" );
-	abort();
+	return -1;
     }
 
-    bzero( &dst, sizeof( struct sockaddr_at ));
+    memset( &dst, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    dst.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     dst.sat_family = AF_APPLETALK;
     dst.sat_addr.s_net = iface->i_addr.sat_addr.s_net;
     dst.sat_addr.s_node = iface->i_addr.sat_addr.s_node;
-    bzero( &loop, sizeof( struct sockaddr_at ));
+    memset( &loop, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    loop.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     loop.sat_family = AF_APPLETALK;
     loop.sat_addr.s_net = htons( ATADDR_ANYNET );
     loop.sat_addr.s_node = ATADDR_ANYNODE;
 
-    if ( route( cmd, &dst, &loop, RTF_UP | RTF_HOST )) {
-	return( -1 );
+#ifndef BSD4_4
+    if ( route( cmd,
+		(struct sockaddr *) &dst,
+		(struct sockaddr *) &loop,
+		RTF_UP | RTF_HOST ) ) {
+	return( 1 );
     }
+#else /* ! BSD4_4 */
+    if ( route( cmd,
+	    	(struct sockaddr_at *) &dst,
+		(struct sockaddr_at *) &loop,
+		RTF_UP | RTF_HOST ) ) {
+	return ( 1);
+    }
+#endif /* BSD4_4 */
     if ( cmd == RTMP_ADD ) {
 	iface->i_flags |= IFACE_LOOP;
     }
@@ -548,8 +912,8 @@ looproute( iface, cmd )
     return( 0 );
 }
 
-gateroute( command, rtmp )
-    int			command;
+int gateroute( command, rtmp )
+    unsigned int	command;
     struct rtmptab	*rtmp;
 {
     struct sockaddr_at	dst, gate;
@@ -562,13 +926,17 @@ gateroute( command, rtmp )
 	return( -1 );
     }
 
+    net = ntohs( rtmp->rt_firstnet );
     /*
      * Since we will accept routes from gateways who advertise their
      * address as 0.YY, we must munge the gateway address we give to
      * the kernel.  Otherwise, we'll get a bunch of routes to the loop
      * back interface, and who wants that?
      */
-    bzero( &gate, sizeof( struct sockaddr_at ));
+    memset( &gate, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    gate.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     gate.sat_family = AF_APPLETALK;
     gate.sat_addr.s_net = rtmp->rt_gate->g_sat.sat_addr.s_net;
     gate.sat_addr.s_node = rtmp->rt_gate->g_sat.sat_addr.s_node;
@@ -576,18 +944,35 @@ gateroute( command, rtmp )
 	gate.sat_addr.s_net = net;
     }
 
-    bzero( &dst, sizeof( struct sockaddr_at ));
+    memset( &dst, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+    dst.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     dst.sat_family = AF_APPLETALK;
     dst.sat_addr.s_node = ATADDR_ANYNODE;
 
-    net = ntohs( rtmp->rt_firstnet );
     do {
 	dst.sat_addr.s_net = htons( net );
-	if ( route( command, &dst, &gate, RTF_UP | RTF_GATEWAY )) {
-	    syslog( LOG_ERR, "route: %u -> %u.%u: %m", net,
-		    ntohs( gate.sat_addr.s_net ), gate.sat_addr.s_node );
+#ifndef BSD4_4
+	if ( route( command,
+		    (struct sockaddr *) &dst,
+		    (struct sockaddr *) &gate,
+		    RTF_UP | RTF_GATEWAY )) {
+	    syslog( LOG_ERR, "route: %u -> %u.%u: %s", net,
+		    ntohs( gate.sat_addr.s_net ), gate.sat_addr.s_node,
+		    strerror(errno) );
 	    continue;
 	}
+#else /* ! BSD4_4 */
+	if ( route( command,
+		    (struct sockaddr_at *) &dst,
+		    (struct sockaddr_at *) &gate,
+		    RTF_UP | RTF_GATEWAY )) {
+	    syslog( LOG_ERR, "route: %u -> %u.%u: %m", net,
+	    	    ntohs( gate.sat_addr.s_net ), gate.sat_addr.s_node );
+	    continue;
+	}
+#endif /* ! BSD4_4 */
     } while ( net++ < ntohs( rtmp->rt_lastnet ));
 
     if ( command == RTMP_ADD ) {
@@ -600,287 +985,15 @@ gateroute( command, rtmp )
     return( 0 );
 }
 
-rtmp_new( rtmp )
-    struct rtmptab	*rtmp;
-{
-    struct interface	*i;
-    struct rtmptab	*r;
-    extern int		newrtmpdata;
-
-    newrtmpdata = 1;
-
-    /*
-     * Do we already have a gateway for this route?
-     */
-    for ( i = interfaces; i; i = i->i_next ) {
-	for ( r = i->i_rt; r; r = r->rt_inext ) {
-	    /* Should check RTMPTAB_EXTENDED here. XXX */
-	    if (( ntohs( r->rt_firstnet ) <= ntohs( rtmp->rt_firstnet ) &&
-		    ntohs( r->rt_lastnet ) >= ntohs( rtmp->rt_firstnet )) ||
-		    ( ntohs( r->rt_firstnet ) <= ntohs( rtmp->rt_lastnet ) &&
-		    ntohs( r->rt_lastnet ) >= ntohs( rtmp->rt_lastnet ))) {
-		break;
-	    }
-	}
-	if ( r ) {
-	    break;
-	}
-    }
-
-    /*
-     * This part of this routine is almost never run.
-     */
-    if ( i ) {	/* can we get here without r being set? */
-	if ( r->rt_firstnet != rtmp->rt_firstnet ||
-		r->rt_lastnet != rtmp->rt_lastnet ) {
-	    syslog( LOG_INFO, "rtmp_new netrange mismatch %u-%u != %u-%u",
-		    ntohs( r->rt_firstnet ), ntohs( r->rt_lastnet ),
-		    ntohs( rtmp->rt_firstnet ), ntohs( rtmp->rt_lastnet ));
-	    return;
-	}
-
-	/*
-	 * Note that our whole methodology is wrong, if we want to do
-	 * route "load balancing."  This entails changing our route
-	 * each time we receive a tuple of equal value.  In fact, we can't
-	 * do this, using our method, since we only check against in-use
-	 * routes when a tuple is new from a router.
-	 */
-	if ( r->rt_hops < rtmp->rt_hops ) {
-	    return;
-	}
-
-	rtmp_copyzones( rtmp, r );
-	rtmp_delinuse( r );
-    }
-
-    rtmp_addinuse( rtmp );
-}
-
-/*
- * Find a replacement for "replace".  If we can't find a replacement,
- * return 1.  If we do find a replacement, return 0.
- */
-rtmp_replace( replace )
-    struct rtmptab	*replace;
-{
-    struct interface	*iface;
-    struct gate		*gate;
-    struct rtmptab	*rtmp, *found = 0;
-
-    for ( iface = interfaces; iface; iface = iface->i_next ) {
-	for ( gate = iface->i_gate; gate; gate = gate->g_next ) {
-	    for ( rtmp = gate->g_rt; rtmp; rtmp = rtmp->rt_next ) {
-		if ( rtmp->rt_firstnet == replace->rt_firstnet &&
-			rtmp->rt_lastnet == replace->rt_lastnet ) {
-		    if ( found == 0 || rtmp->rt_hops < found->rt_hops ) {
-			found = rtmp;
-		    }
-		    break;
-		}
-	    }
-	}
-    }
-
-    if ( found != replace ) {
-	rtmp_copyzones( found, replace );
-	rtmp_delinuse( replace );
-	rtmp_addinuse( found );
-	if ( replace->rt_state == RTMPTAB_BAD ) {
-	    rtmp_free( replace );
-	}
-	return( 0 );
-    } else {
-	if ( replace->rt_hops == RTMPHOPS_POISON ) {
-	    gateroute( RTMP_DEL, replace );
-	}
-	return( 1 );
-    }
-}
-
-/*
- * Remove rtmp from the in-use table and the per-gate table.
- * Free any associated space.
- */
-rtmp_free( rtmp )
-    struct rtmptab	*rtmp;
-{
-    struct gate		*gate;
-
-    if ( rtmp->rt_iprev ) {
-	rtmp_delinuse( rtmp );
-    }
-
-    /* remove from per-gate */
-    gate = rtmp->rt_gate;
-    if ( gate->g_rt == rtmp ) {				/* first */
-	if ( rtmp->rt_prev == rtmp ) {			/* only */
-	    gate->g_rt = 0;
-	} else {
-	    gate->g_rt = rtmp->rt_next;
-	    rtmp->rt_next->rt_prev = rtmp->rt_prev;
-	}
-    } else {
-	if ( rtmp->rt_next == 0 ) {			/* last */
-	    rtmp->rt_prev->rt_next = 0;
-	    gate->g_rt->rt_prev = rtmp->rt_prev;
-	} else {
-	    rtmp->rt_prev->rt_next = rtmp->rt_next;
-	    rtmp->rt_next->rt_prev = rtmp->rt_prev;
-	}
-    }
-
-    free( rtmp );
-}
-
-/*
- * Delete rtmp from the per-interface in-use table, remove all
- * zone references, and remove the route from the kernel.
- */
-rtmp_delinuse( rtmp )
-    struct rtmptab	*rtmp;
-{
-    struct rtmptab	*irt;
-    struct ziptab	*zt;
-    struct list		*lz, *flz, *lr, *flr;
-
-    irt = rtmp->rt_gate->g_iface->i_rt;
-    if ( irt->rt_inext == rtmp ) {			/* first */
-	if ( rtmp->rt_iprev == rtmp ) {			/* only */
-	    irt->rt_inext = 0;
-	} else {
-	    irt->rt_inext = rtmp->rt_inext;
-	    rtmp->rt_inext->rt_iprev = rtmp->rt_iprev;
-	}
-    } else {
-	if ( rtmp->rt_inext == 0 ) {			/* last */
-	    rtmp->rt_iprev->rt_inext = 0;
-	    irt->rt_inext->rt_iprev = rtmp->rt_iprev;
-	} else {
-	    rtmp->rt_iprev->rt_inext = rtmp->rt_inext;
-	    rtmp->rt_inext->rt_iprev = rtmp->rt_iprev;
-	}
-    }
-    rtmp->rt_iprev = 0;
-    rtmp->rt_inext = 0;
-
-    /* remove zone map */
-    lz = rtmp->rt_zt;
-    while ( lz ) {					/* for each zone */
-	zt = (struct ziptab *)lz->l_data;
-	lr = zt->zt_rt;
-	while ( lr ) {					/* for each route */
-	    if ( (struct rtmptab *)lr->l_data == rtmp ) {
-		if ( lr->l_prev == 0 ) {		/* head */
-		    if ( lr->l_next == 0 ) {		/* last route in zone */
-			if ( zt->zt_prev == 0 ) {
-			    ziptab = zt->zt_next;
-			} else {
-			    zt->zt_prev->zt_next = zt->zt_next;
-			}
-			if ( zt->zt_next == 0 ) {
-			    ziplast = zt->zt_prev;
-			} else {
-			    zt->zt_next->zt_prev = zt->zt_prev;
-			}
-			free( zt->zt_bcast );
-			free( zt->zt_name );
-			free( zt );
-		    } else {
-			zt->zt_rt = lr->l_next;
-		    }
-		} else {
-		    lr->l_prev->l_next = lr->l_next;
-		}
-		if ( lr->l_next != 0 ) {
-		    lr->l_next->l_prev = lr->l_prev;
-		}
-		flr = lr;
-		lr = lr->l_next;
-		free( flr );
-	    } else {
-		lr = lr->l_next;
-	    }
-	}
-	flz = lz;
-	lz = lz->l_next;
-	free( flz );
-    }
-    rtmp->rt_zt = 0;
-
-    /* remove old route */
-    gateroute( RTMP_DEL, rtmp );
-
-    return;
-}
-
-/*
- * Add rtmp to the per-interface in-use table.  No verification is done...
- */
-rtmp_addinuse( rtmp )
-    struct rtmptab	*rtmp;
-{
-    struct rtmptab	*irt;
-
-    gateroute( RTMP_ADD, rtmp );
-
-    irt = rtmp->rt_gate->g_iface->i_rt;
-    if ( irt->rt_inext == 0 ) {	/* emtpy list */
-	rtmp->rt_inext = 0;
-	rtmp->rt_iprev = rtmp;
-	irt->rt_inext = rtmp;
-    } else {
-	rtmp->rt_inext = irt->rt_inext;
-	rtmp->rt_iprev = irt->rt_inext->rt_iprev;
-	irt->rt_inext->rt_iprev = rtmp;
-	irt->rt_inext = rtmp;
-    }
-    return;
-}
-
-/*
- * Change the zone mapping to replace "from" with "to".  This code assumes
- * the consistency of both the route -> zone map and the zone -> route map.
- * This is probably a bad idea.  How can we insure that the data is good
- * at this point?  What do we do if we get several copies of a route in
- * an RTMP packet?
- */
-rtmp_copyzones( to, from )
-    struct rtmptab	*to, *from;
-{
-    struct list		*lz, *lr;
-
-    to->rt_zt = from->rt_zt;
-    from->rt_zt = 0;
-    if ( from->rt_flags & RTMPTAB_HASZONES ) {
-	to->rt_flags |= RTMPTAB_HASZONES;
-    }
-    for ( lz = to->rt_zt; lz; lz = lz->l_next ) {
-	for ( lr = ((struct ziptab *)lz->l_data)->zt_rt; lr; lr = lr->l_next ) {
-	    if ( (struct rtmptab *)lr->l_data == from ) {
-		lr->l_data = (void *)to;	/* cast BS */
-		break;
-	    }
-	}
-	if ( lr == 0 ) {
-	    syslog( LOG_ERR, "rtmp_copyzones z -> r without r -> z, abort" );
-	    abort();
-	}
-    }
-
-    return;
-}
-
     struct rtmptab *
-newrt()
+newrt(const struct interface *iface)
 {
     struct rtmptab	*rtmp;
 
-    if (( rtmp = (struct rtmptab *)malloc(sizeof(struct rtmptab)))
-	    == 0 ) {
+    if (( rtmp = (struct rtmptab *)calloc(1, sizeof(struct rtmptab))) == 0 ) {
 	return( 0 );
     }
 
-    bzero( rtmp, sizeof( struct rtmptab ));
+    rtmp->rt_iface = iface;
     return( rtmp );
 }
