@@ -1,22 +1,40 @@
 /*
+ * $Id: zip.c,v 1.7.2.1 2001/12/31 20:05:20 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/syslog.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <net/route.h>
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
 
+#ifdef __svr4__
+#include <sys/sockio.h>
+#endif /* __svr4__ */
+
 #include <atalk/ddp.h>
 #include <atalk/zip.h>
 #include <atalk/atp.h>
+#include <atalk/util.h>
 
 #include "atserv.h"
 #include "interface.h"
@@ -25,13 +43,52 @@
 #include "rtmp.h"
 #include "list.h"
 #include "multicast.h"
+#include "main.h"
 
 struct ziptab	*ziptab = NULL, *ziplast = NULL;
 
-extern int	ninterfaces;
-u_char		*zone_mcast();
 
-zip_packet( ap, from, data, len )
+static int zonecheck( rtmp, iface )
+    struct rtmptab	*rtmp;
+    struct interface	*iface;
+{
+    struct list		*l;
+    struct ziptab	*czt, *zt;
+    int			cztcnt, ztcnt;
+
+    if (( iface->i_flags & IFACE_SEED ) == 0 ) {
+	return( 0 );
+    }
+
+    for ( cztcnt = 0, czt = iface->i_czt; czt; czt = czt->zt_next, cztcnt++ ) {
+	for ( l = rtmp->rt_zt; l; l = l->l_next ) {
+	    zt = (struct ziptab *)l->l_data;
+	    if ( czt->zt_len == zt->zt_len &&
+		    !strndiacasecmp( czt->zt_name, zt->zt_name, czt->zt_len )) {
+		break;
+	    }
+	}
+	if ( l == 0 ) {
+	    syslog( LOG_ERR, "zonecheck: %.*s not in zone list", czt->zt_len,
+		    czt->zt_name );
+	    return( -1 );	/* configured zone not found in net zones */
+	}
+    }
+
+    for ( ztcnt = 0, l = rtmp->rt_zt; l; l = l->l_next, ztcnt++ )
+	;
+
+    if ( cztcnt != ztcnt ) {
+	syslog( LOG_ERR, "zonecheck: %d configured zones, %d zones found",
+		cztcnt, ztcnt );
+	return( -1 );		/* more net zones than configured zones */
+    }
+
+    return( 0 );
+}
+
+
+int zip_packet( ap, from, data, len )
     struct atport	*ap;
     struct sockaddr_at	*from;
     char		*data;
@@ -44,17 +101,17 @@ zip_packet( ap, from, data, len )
     struct rtmptab	*rtmp;
     struct list		*l;
     struct ziptab	*zt;
-    u_short		firstnet, lastnet, tonet, index, nz;
-    char		*end, zname[ 32 ], packet[ 587 ], *nzones, *lastflag;
+    u_short		firstnet, lastnet, index, nz;
+    char		*end, zname[ 32 ], packet[ ATP_BUFSIZ ], *nzones, *lastflag;
     char		*reply, *rend, *ziphdr;
     int			zlen, n, zipop, rcnt, qcnt, zcnt, zsz;
-    extern int		debug, stabletimer;
+    extern int		stabletimer;
 
     end = data + len;
 
     if ( data >= end ) {
 	syslog( LOG_INFO, "zip_packet malformed packet" );
-	return;
+	return 1;
     }
 
     /* get interface */
@@ -64,9 +121,9 @@ zip_packet( ap, from, data, len )
     case DDPTYPE_ZIP :
 	if ( data + sizeof( struct ziphdr ) > end ) {
 	    syslog( LOG_INFO, "zip_packet malformed packet" );
-	    return;
+	    return 1;
 	}
-	bcopy( data, &zh, sizeof( struct ziphdr ));
+	memcpy( &zh, data, sizeof( struct ziphdr ));
 	data += sizeof( struct ziphdr );
 
 	switch ( zh.zh_op ) {
@@ -82,7 +139,7 @@ zip_packet( ap, from, data, len )
 	    qcnt = zh.zh_count;
 
 	    while ( data + sizeof( u_short ) <= end && qcnt-- > 0 ) {
-		bcopy( data, &firstnet, sizeof( u_short ));
+		memcpy( &firstnet, data, sizeof( u_short ));
 		data += sizeof( u_short );
 
 		/*
@@ -121,11 +178,12 @@ zip_packet( ap, from, data, len )
 		    if ( rcnt > 0 ) {
 			zh.zh_op = ZIPOP_REPLY;
 			zh.zh_cnt = rcnt;
-			bcopy( &zh, ziphdr, sizeof( struct ziphdr ));
+			memcpy( ziphdr, &zh, sizeof( struct ziphdr ));
 			if ( sendto( ap->ap_fd, packet, reply - packet, 0,
 				(struct sockaddr *)from,
 				sizeof( struct sockaddr_at )) < 0 ) {
-			    syslog( LOG_ERR, "zip reply sendto: %m" );
+			    syslog( LOG_ERR, "zip reply sendto: %s",
+				    strerror(errno) );
 			}
 
 			reply = packet + 3;
@@ -140,32 +198,34 @@ zip_packet( ap, from, data, len )
 				    rend ) {
 				zh.zh_op = ZIPOP_EREPLY;
 				zh.zh_cnt = zcnt;
-				bcopy( &zh, ziphdr, sizeof( struct ziphdr ));
+				memcpy( ziphdr, &zh, sizeof( struct ziphdr ));
 				if ( sendto( ap->ap_fd, packet, reply - packet,
 					0, (struct sockaddr *)from,
 					sizeof( struct sockaddr_at )) < 0 ) {
-				    syslog( LOG_ERR, "zip reply sendto: %m" );
+				    syslog( LOG_ERR, "zip reply sendto: %s",
+					    strerror(errno) );
 				}
 
 				reply = packet + 3;
 				rcnt = 0;
 			    }
 
-			    bcopy( &firstnet, reply, sizeof( u_short ));
+			    memcpy( reply, &firstnet, sizeof( u_short ));
 			    reply += sizeof( u_short );
 			    *reply++ = zt->zt_len;
-			    bcopy( zt->zt_name, reply, zt->zt_len );
+			    memcpy( reply, zt->zt_name, zt->zt_len );
 			    reply += zt->zt_len;
 			}
 
 			if ( rcnt > 0 ) {
 			    zh.zh_op = ZIPOP_EREPLY;
 			    zh.zh_cnt = zcnt;
-			    bcopy( &zh, ziphdr, sizeof( struct ziphdr ));
+			    memcpy( ziphdr, &zh, sizeof( struct ziphdr ));
 			    if ( sendto( ap->ap_fd, packet, reply - packet, 0,
 				    (struct sockaddr *)from,
 				    sizeof( struct sockaddr_at )) < 0 ) {
-				syslog( LOG_ERR, "zip reply sendto: %m" );
+				syslog( LOG_ERR, "zip reply sendto: %s",
+					strerror(errno) );
 			    }
 
 			    reply = packet + 3;
@@ -177,10 +237,10 @@ zip_packet( ap, from, data, len )
 
 		for ( l = rtmp->rt_zt; l; l = l->l_next, rcnt++ ) {
 		    zt = (struct ziptab *)l->l_data;
-		    bcopy( &firstnet, reply, sizeof( u_short ));
+		    memcpy( reply, &firstnet, sizeof( u_short ));
 		    reply += sizeof( u_short );
 		    *reply++ = zt->zt_len;
-		    bcopy( zt->zt_name, reply, zt->zt_len );
+		    memcpy( reply, zt->zt_name, zt->zt_len );
 		    reply += zt->zt_len;
 		}
 	    }
@@ -188,11 +248,12 @@ zip_packet( ap, from, data, len )
 	    if ( rcnt > 0 ) {
 		zh.zh_op = ZIPOP_REPLY;
 		zh.zh_cnt = rcnt;
-		bcopy( &zh, ziphdr, sizeof( struct ziphdr ));
+		memcpy( ziphdr, &zh, sizeof( struct ziphdr ));
 		if ( sendto( ap->ap_fd, packet, reply - packet, 0,
 			(struct sockaddr *)from,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "zip reply sendto: %m" );
+		    syslog( LOG_ERR, "zip reply sendto: %s",
+			    strerror(errno) );
 		}
 	    }
 	    break;
@@ -208,25 +269,25 @@ zip_packet( ap, from, data, len )
 	    if ( gate == NULL ) {
 		syslog( LOG_INFO, "zip reply from non-gateway %u.%u", 
 		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-		return;
+		return 1;
 	    }
 
-	    rtmp = 0;
+	    rtmp = NULL;
 
 	    do {
 		if ( data + sizeof( u_short ) + 1 > end ) {	/* + strlen */
 		    syslog( LOG_INFO, "zip reply short (%d)", len );
-		    return;
+		    return 1;
 		}
-		bcopy( data, &firstnet, sizeof( u_short ));
+		memcpy( &firstnet, data, sizeof( u_short ));
 		data += sizeof( u_short );
 
 		if ( rtmp && rtmp->rt_firstnet != firstnet ) {
 		    /* XXX */
-		    if ( rtmp->rt_gate == 0 &&
+		    if ( rtmp->rt_gate == NULL &&
 			    zonecheck( rtmp, gate->g_iface ) != 0 ) {
 			syslog( LOG_ERR, "zip_packet seed zonelist mismatch" );
-			exit( 1 );
+			return -1;
 		    }
 		    rtmp->rt_flags &= ~RTMPTAB_ZIPQUERY;
 		}
@@ -254,13 +315,13 @@ zip_packet( ap, from, data, len )
 		zlen = *data++;
 		if ( zlen > 32 || zlen <= 0 ) {
 		    syslog( LOG_INFO, "zip reply bad packet" );
-		    return;
+		    return 1;
 		}
 		if ( data + zlen > end ) {
 		    syslog( LOG_INFO, "zip reply short (%d)", len );
-		    return;
+		    return 1;
 		}
-		bcopy( data, zname, zlen );
+		memcpy( zname, data, zlen );
 		data += zlen;
 
 		/*
@@ -275,7 +336,7 @@ zip_packet( ap, from, data, len )
 		 * Check if the route is still in use (the iprev check is
 		 * no good if rtmp is the interface's route).
 		 */
-		} else if ( rtmp->rt_iprev == 0 && rtmp->rt_prev != 0 ) {
+		} else if ( rtmp->rt_iprev == NULL && rtmp->rt_prev != NULL ) {
 		    syslog( LOG_INFO,
 			    "zip skip reply %u-%u from %u.%u (rtmp not in use)",
 			    ntohs( rtmp->rt_firstnet ),
@@ -295,7 +356,10 @@ zip_packet( ap, from, data, len )
 			    ntohs( from->sat_addr.s_net ),
 			    from->sat_addr.s_node );
 		} else {
-		    addzone( rtmp, zlen, zname );
+		    if (addzone( rtmp, zlen, zname ) < 0) {
+		        syslog(LOG_ERR, "zip_packet: addzone");
+			return -1;
+		    }
 		    rtmp->rt_flags |= RTMPTAB_HASZONES;
 		}
 	    } while ( data < end );
@@ -305,7 +369,7 @@ zip_packet( ap, from, data, len )
 		if ( rtmp->rt_gate == 0 &&
 			zonecheck( rtmp, gate->g_iface ) != 0 ) {
 		    syslog( LOG_ERR, "zip_packet seed zonelist mismatch" );
-		    exit( 1 );
+		    return -1;
 		}
 		rtmp->rt_flags &= ~RTMPTAB_ZIPQUERY;
 	    }
@@ -322,7 +386,7 @@ zip_packet( ap, from, data, len )
 	    if ( gate == NULL ) {
 		syslog( LOG_INFO, "zip ereply from non-gateway %u.%u", 
 		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-		return;
+		return 1;
 	    }
 
 	    /*
@@ -331,9 +395,9 @@ zip_packet( ap, from, data, len )
 	     */
 	    if ( data + sizeof( u_short ) + 1 > end ) {	/* + strlen */
 		syslog( LOG_INFO, "zip ereply short (%d)", len );
-		return;
+		return 1;
 	    }
-	    bcopy( data, &firstnet, sizeof( u_short ));
+	    memcpy( &firstnet, data, sizeof( u_short ));
 
 	    /* Check if this is the interface's route. */
 	    if ( firstnet == gate->g_iface->i_rt->rt_firstnet ) {
@@ -348,7 +412,7 @@ zip_packet( ap, from, data, len )
 		    syslog( LOG_INFO, "zip ereply %u from %u.%u (no rtmp)",
 			    ntohs( firstnet ), ntohs( from->sat_addr.s_net ),
 			    from->sat_addr.s_node );
-		    return;
+		    return 1;
 		}
 		if ( rtmp->rt_iprev == 0 ) {
 		    syslog( LOG_INFO,
@@ -373,7 +437,7 @@ zip_packet( ap, from, data, len )
 			ntohs( rtmp->rt_lastnet ),
 			ntohs( from->sat_addr.s_net ),
 			from->sat_addr.s_node );
-		return;
+		return 0;
 	    }
 
 	    do {
@@ -383,29 +447,32 @@ zip_packet( ap, from, data, len )
 		 */
 		if ( data + sizeof( u_short ) + 1 > end ) {	/* + strlen */
 		    syslog( LOG_INFO, "zip ereply short (%d)", len );
-		    return;
+		    return 1;
 		}
-		bcopy( data, &firstnet, sizeof( u_short ));
+		memcpy( &firstnet, data, sizeof( u_short ));
 		data += sizeof( u_short );
 
 		/* check route */
 		if ( firstnet != rtmp->rt_firstnet ) {
 		    syslog( LOG_INFO, "zip ereply with multiple nets" );
-		    return;
+		    return 1;
 		}
 
 		zlen = *data++;
 		if ( zlen > 32 || zlen <= 0 ) {
 		    syslog( LOG_INFO, "zip ereply bad zone length (%d)", zlen );
-		    return;
+		    return 1;
 		}
 		if ( data + zlen > end ) {
 		    syslog( LOG_INFO, "zip ereply short (%d)", len );
-		    return;
+		    return 1;
 		}
-		bcopy( data, zname, zlen );
+		memcpy( zname, data, zlen );
 		data += zlen;
-		addzone( rtmp, zlen, zname );
+		if (addzone( rtmp, zlen, zname ) < 0) {
+		    syslog(LOG_ERR, "zip_packet: addzone");
+		    return -1;
+		}
 	    } while ( data < end );
 
 	    if ( rtmp ) {
@@ -420,7 +487,7 @@ zip_packet( ap, from, data, len )
 		    if ( rtmp->rt_gate == 0 &&
 			    zonecheck( rtmp, gate->g_iface ) != 0 ) {
 			syslog( LOG_ERR, "zip_packet seed zonelist mismatch" );
-			exit( 1 );
+			return -1;
 		    }
 		    rtmp->rt_flags &= ~RTMPTAB_ZIPQUERY;
 		}
@@ -431,31 +498,32 @@ zip_packet( ap, from, data, len )
 	    /*
 	     * Don't answer with bogus information.
 	     */
-	    if ( ninterfaces == IFBASE || iface->i_rt->rt_zt == 0 ||
-		    ( iface->i_flags & IFACE_CONFIG ) == 0 ) {
-		return;
+	    if (((iface->i_flags & IFACE_ISROUTER) == 0) ||
+		iface->i_rt->rt_zt == 0 ||
+		( iface->i_flags & IFACE_CONFIG ) == 0 ) {
+		return 0;
 	    }
 
 	    if ( zh.zh_zero != 0 || data + 2 * sizeof( u_short ) > end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
 
-	    bcopy( data, &firstnet, sizeof( u_short ));
+	    memcpy( &firstnet, data, sizeof( u_short ));
 	    data += sizeof( u_short );
-	    bcopy( data, &lastnet, sizeof( u_short ));
+	    memcpy( &lastnet, data, sizeof( u_short ));
 	    data += sizeof( u_short );
 	    if ( firstnet != 0 || lastnet != 0 || data >= end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
 
 	    zlen = *data++;
 	    if ( zlen < 0 || zlen > 32 ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
-	    bcopy( data, zname, zlen );
+	    memcpy( zname, data, zlen );
 
 	    data = packet;
 	    end = data + sizeof( packet );
@@ -466,13 +534,13 @@ zip_packet( ap, from, data, len )
 	     * Skip to the nets.  Fill in header when we're done.
 	     */
 	    data += 1 + sizeof( struct ziphdr );
-	    bcopy( &iface->i_rt->rt_firstnet, data, sizeof( u_short ));
+	    memcpy( data, &iface->i_rt->rt_firstnet, sizeof( u_short ));
 	    data += sizeof( u_short );
-	    bcopy( &iface->i_rt->rt_lastnet, data, sizeof( u_short ));
+	    memcpy( data, &iface->i_rt->rt_lastnet, sizeof( u_short ));
 	    data += sizeof( u_short );
 
 	    *data++ = zlen;
-	    bcopy( zname, data, zlen );
+	    memcpy( data, zname, zlen );
 	    data += zlen;
 
 	    /*
@@ -500,7 +568,11 @@ zip_packet( ap, from, data, len )
 
 	    /* multicast */
 	    *data++ = 6;	/* sizeof ??? */
-	    bcopy( zone_mcast( zt->zt_name, zt->zt_len ), data, 6 );
+	    if (zone_bcast(zt) < 0) {
+	      syslog(LOG_ERR, "zip_packet: zone_bcast");
+	      return -1;
+	    }
+	    memcpy(data, zt->zt_bcast, 6);
 	    data += 6;
 
 	    /*
@@ -508,13 +580,13 @@ zip_packet( ap, from, data, len )
 	     */
 	    if ( zh.zh_flags & ZIPGNI_INVALID ) {
 		*data++ = zt->zt_len;
-		bcopy( zt->zt_name, data, zt->zt_len );
+		memcpy( data, zt->zt_name, zt->zt_len );
 		data += zt->zt_len;
 	    }
 
 	    /* fill in header */
 	    *packet = DDPTYPE_ZIP;
-	    bcopy( &zh, packet + 1, sizeof( struct ziphdr ));
+	    memcpy( packet + 1, &zh, sizeof( struct ziphdr ));
 
 	    /*
 	     * If the address we received this request from isn't correct
@@ -525,15 +597,16 @@ zip_packet( ap, from, data, len )
 		    ntohs( from->sat_addr.s_net ) >
 		    ntohs( iface->i_rt->rt_lastnet )) {
 		from->sat_addr.s_net = 0;
-		from->sat_addr.s_node = 0xff;
+		from->sat_addr.s_node = ATADDR_BCAST;
 	    }
 
 	    if ( sendto( ap->ap_fd, packet, data - packet, 0,
 		    (struct sockaddr *)from,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "zip gni sendto %u.%u: %m",
-			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-		return;
+		syslog( LOG_ERR, "zip gni sendto %u.%u: %s",
+			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
+			strerror(errno) );
+		return 1;
 	    }
 	    break;
 
@@ -541,21 +614,23 @@ zip_packet( ap, from, data, len )
 	    /*
 	     * Ignore ZIP GNIReplys which are either late or unsolicited.
 	     */
+	    syslog( LOG_DEBUG, "zip gnireply from %u.%u (%s %x)",
+		    ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
+		    iface->i_name, iface->i_flags );
+
 	    if (( iface->i_flags & ( IFACE_CONFIG|IFACE_PHASE1 )) ||
 		    ( iface->i_flags & IFACE_ADDR ) == 0 ) {
-		syslog( LOG_INFO, "zip ignoring gnireply from %u.%u (%s %x)",
-			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
-			iface->i_name, iface->i_flags );
-		return;
+		syslog( LOG_DEBUG, "zip ignoring gnireply" );
+		return 1;
 	    }
 
 	    if ( data + 2 * sizeof( u_short ) > end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
-	    bcopy( data, &firstnet, sizeof( u_short ));
+	    memcpy( &firstnet, data, sizeof( u_short ));
 	    data += sizeof( u_short );
-	    bcopy( data, &lastnet, sizeof( u_short ));
+	    memcpy( &lastnet, data, sizeof( u_short ));
 	    data += sizeof( u_short );
 
 	    /*
@@ -564,23 +639,23 @@ zip_packet( ap, from, data, len )
 	     */
 	    if ( data >= end || data + *data > end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
 	    if ( *data++ != 0 ) {
 		syslog( LOG_INFO, "zip_packet unsolicited zone" );
-		return;
+		return 1;
 	    }
 
 	    /* skip multicast (should really check it) */
 	    if ( data >= end || data + *data > end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
 	    data += *data + 1;
 
 	    if ( data >= end || data + *data > end ) {
 		syslog( LOG_INFO, "zip_packet malformed packet" );
-		return;
+		return 1;
 	    }
 
 	    /*
@@ -600,13 +675,17 @@ zip_packet( ap, from, data, len )
 			    iface->i_czt->zt_len, iface->i_czt->zt_name,
 			    *data, data + 1 );
 		    syslog( LOG_ERR, "Seed error! Exiting!" );
-		    exit( 1 );
+		    return -1;
 		}
 	    }
-	    addzone( iface->i_rt, *data, data + 1 );
 
+	    if (addzone( iface->i_rt, *data, data + 1 ) < 0) {
+	        syslog(LOG_ERR, "zip_packet: addzone");
+	        return -1;
+	    }
+	    
 	    /*
-	     * The netrange we recieved from the router doesn't match the
+	     * The netrange we received from the router doesn't match the
 	     * range we have locally. This is not a problem, unless we
 	     * have seed information.
 	     */
@@ -620,7 +699,7 @@ zip_packet( ap, from, data, len )
 			    ntohs( iface->i_rt->rt_firstnet ),
 			    ntohs( iface->i_rt->rt_lastnet ));
 		    syslog( LOG_ERR, "Seed error! Exiting!" );
-		    exit( 1 );
+		    return -1;
 		}
 
 
@@ -647,12 +726,13 @@ zip_packet( ap, from, data, len )
 	    }
 
 	    /* add addr to loopback route */
-	    if ( looproute( iface, RTMP_ADD ) < 0 ) {
+	    if ( looproute( iface, RTMP_ADD )) { /* -1 or 1 */
 		syslog( LOG_ERR,
-			"zip_packet: can't route %u.%u to loopback: %m",
+			"zip_packet: can't route %u.%u to loopback: %s",
 			ntohs( iface->i_addr.sat_addr.s_net ),
-			iface->i_addr.sat_addr.s_node );
-		exit( 1 );
+			iface->i_addr.sat_addr.s_node,
+			strerror(errno) );
+		return -1;
 	    }
 
 	    syslog( LOG_INFO, "zip_packet configured %s from %u.%u",
@@ -669,7 +749,7 @@ zip_packet( ap, from, data, len )
 #ifdef DEBUG
 	    printf( "zip notify from %u.%u\n", ntohs( from->sat_addr.s_net ),
 		    from->sat_addr.s_node );
-#endif DEBUG
+#endif /* DEBUG */
 	    break;
 
 	default :
@@ -681,35 +761,35 @@ zip_packet( ap, from, data, len )
     case DDPTYPE_ATP :
 	if ( data + sizeof( struct atphdr ) > end ) {
 	    syslog( LOG_INFO, "zip atp malformed packet" );
-	    return;
+	    return 1;
 	}
-	bcopy( data, &ah, sizeof( struct atphdr ));
+	memcpy( &ah, data, sizeof( struct atphdr ));
 	data += sizeof( struct atphdr );
 	if ( ah.atphd_ctrlinfo != ATP_TREQ ) {
 	    syslog( LOG_INFO, "zip atp bad control" );
-	    return;
+	    return 1;
 	}
 	ah.atphd_ctrlinfo = ATP_TRESP | ATP_EOM;
 	if ( ah.atphd_bitmap != 1 ) {
 	    syslog( LOG_ERR, "zip atp bad bitmap" );
-	    return;
+	    return 1;
 	}
 	ah.atphd_bitmap = 0;
 
 	zipop = *data++;
 	data++;
-	bcopy( data, &index, sizeof( u_short ));
+	memcpy( &index, data, sizeof( u_short ));
 	data += sizeof( u_short );
 	index = ntohs( index );
 	if ( data != end ) {
 	    syslog( LOG_INFO, "zip atp malformed packet" );
-	    return;
+	    return 1;
 	}
 
 	data = packet;
 	end = data + sizeof( packet );
 	*data++ = DDPTYPE_ATP;
-	bcopy( &ah, data, sizeof( struct atphdr ));
+	memcpy( data, &ah, sizeof( struct atphdr ));
 	data += sizeof( struct atphdr );
 	lastflag = data++;		/* mark and space for last flag */
 	*data++ = 0;
@@ -720,7 +800,7 @@ zip_packet( ap, from, data, len )
 	case ZIPOP_GETMYZONE :
 	    if ( index != 0 ) {
 		syslog( LOG_INFO, "zip atp gmz bad index" );
-		return;
+		return 1;
 	    }
 
 	    if ( iface->i_flags & IFACE_LOOPBACK ) {
@@ -729,19 +809,19 @@ zip_packet( ap, from, data, len )
 		    ntohs( from->sat_addr.s_net ) ||
 		    ntohs( iface->i_rt->rt_lastnet ) <
 		    ntohs( from->sat_addr.s_net )) {
-		return;
+		return 0;
 	    }
 
 	    if ( iface->i_rt->rt_zt == 0 ) {
-		return;
+		return 0;
 	    }
 	    zt = (struct ziptab *)iface->i_rt->rt_zt->l_data;
 	    if ( data + 1 + zt->zt_len > end ) {
 		syslog( LOG_INFO, "zip atp gmz reply too long" );
-		return;
+		return 1;
 	    }
 	    *data++ = zt->zt_len;
-	    bcopy( zt->zt_name, data, zt->zt_len );
+	    memcpy( data, zt->zt_name, zt->zt_len );
 	    data += zt->zt_len;
 
 	    *lastflag = 0;
@@ -756,7 +836,7 @@ zip_packet( ap, from, data, len )
 		    break;
 		}
 		*data++ = zt->zt_len;
-		bcopy( zt->zt_name, data, zt->zt_len );
+		memcpy( data, zt->zt_name, zt->zt_len );
 		data += zt->zt_len;
 	    }
 
@@ -770,7 +850,7 @@ zip_packet( ap, from, data, len )
 		    ntohs( from->sat_addr.s_net ) ||
 		    ntohs( iface->i_rt->rt_lastnet ) <
 		    ntohs( from->sat_addr.s_net )) {
-		return;
+		return 0;
 	    }
 
 	    for ( l = iface->i_rt->rt_zt; l && ( index > 1 );
@@ -782,7 +862,7 @@ zip_packet( ap, from, data, len )
 		    break;
 		}
 		*data++ = zt->zt_len;
-		bcopy( zt->zt_name, data, zt->zt_len );
+		memcpy( data, zt->zt_name, zt->zt_len );
 		data += zt->zt_len;
 	    }
 
@@ -791,19 +871,20 @@ zip_packet( ap, from, data, len )
 
 	default :
 	    syslog( LOG_INFO, "zip atp bad option" );
-	    return;
+	    return 1;
 	}
 
 	/* send reply */
 	if ( nz > 0 ) {
 	    nz = htons( nz );
-	    bcopy( &nz, nzones, sizeof( u_short ));
+	    memcpy( nzones, &nz, sizeof( u_short ));
 	    if ( sendto( ap->ap_fd, packet, data - packet, 0,
 		    (struct sockaddr *)from,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "zip atp sendto %u.%u: %m",
-			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-		return;
+		syslog( LOG_ERR, "zip atp sendto %u.%u: %s",
+			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
+			strerror(errno) );
+		return 1;
 	    }
 	}
 	break;
@@ -811,11 +892,13 @@ zip_packet( ap, from, data, len )
     default :
 	syslog( LOG_INFO, "zip_packet bad ddp type from %u.%u\n",
 		ntohs( from->sat_addr.s_net ), from->sat_addr.s_node );
-	return;
+	return 1;
     }
+
+    return 0;
 }
 
-zip_getnetinfo( iface )
+int zip_getnetinfo( iface )
     struct interface	*iface;
 {
     struct atport	*ap;
@@ -823,7 +906,6 @@ zip_getnetinfo( iface )
     struct sockaddr_at	sat;
     char		*data, packet[ 40 ];
     u_short		net;
-    int			i;
 
     syslog( LOG_INFO, "zip_getnetinfo for %s", iface->i_name );
 
@@ -834,7 +916,7 @@ zip_getnetinfo( iface )
     }
     if ( ap == 0 ) {
 	syslog( LOG_ERR, "zip_getnetinfo can't find zip socket!" );
-	exit( 1 );
+	return -1;
     }
 
     data = packet;
@@ -843,12 +925,12 @@ zip_getnetinfo( iface )
 
     zh.zh_op = ZIPOP_GNI;
     zh.zh_zero = 0;
-    bcopy( &zh, data, sizeof( struct ziphdr ));
+    memcpy( data, &zh, sizeof( struct ziphdr ));
     data += sizeof( struct ziphdr );
     net = 0;
-    bcopy( &net, data, sizeof( u_short ));
+    memcpy( data, &net, sizeof( u_short ));
     data += sizeof( u_short );
-    bcopy( &net, data, sizeof( u_short ));
+    memcpy( data, &net, sizeof( u_short ));
     data += sizeof( u_short );
 
     /*
@@ -857,96 +939,49 @@ zip_getnetinfo( iface )
      */
     *data++ = 0;
 
+#ifdef BSD4_4
+    sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
     sat.sat_family = AF_APPLETALK;
     sat.sat_addr.s_net = 0;
-    sat.sat_addr.s_node = 0xff;
+    sat.sat_addr.s_node = ATADDR_BCAST;
     sat.sat_port = ap->ap_port;
 
     if ( sendto( ap->ap_fd, packet, data - packet, 0, (struct sockaddr *)&sat,
 	    sizeof( struct sockaddr_at )) < 0 ) {
-	syslog( LOG_ERR, "zip_getnetinfo sendto: %m" );
-	exit( 1 );
+	syslog( LOG_ERR, "zip_getnetinfo sendto: %s", strerror(errno) );
+	return -1;
     }
-    return;
+    return 0;
 }
 
     struct ziptab *
 newzt( len, name )
-    int		len;
-    char	*name;
+    const int		len;
+    const char	*name;
 {
     struct ziptab	*zt;
 
-    if (( zt = (struct ziptab *)malloc( sizeof( struct ziptab ))) == NULL ) {
+    if (( zt = (struct ziptab *)calloc(1, sizeof( struct ziptab ))) == NULL ) {
 	return( NULL );
     }
+
     zt->zt_len = len;
     if (( zt->zt_name = (char *)malloc( len )) == NULL ) {
+	free(zt);
 	return( NULL );
     }
-    bcopy( name, zt->zt_name, len );
-    zt->zt_rt = NULL;
-    zt->zt_bcast = NULL;
-    zt->zt_next = NULL;
-    zt->zt_prev = NULL;
 
+    memcpy( zt->zt_name, name, len );
     return( zt );
 }
 
-addzone( rt, len, zone )
-    struct rtmptab	*rt;
-    int			len;
-    char		*zone;
-{
-    struct ziptab	*zt;
-    int			exists = 0;
-
-    for ( zt = ziptab; zt; zt = zt->zt_next ) {
-	if ( zt->zt_len == len &&
-		strndiacasecmp( zt->zt_name, zone, len ) == 0 ) {
-	    break;
-	}
-    }
-    if ( zt == NULL ) {
-	if (( zt = newzt( len, zone )) == NULL ) {
-	    syslog( LOG_ERR, "addzone newzt: %m" );
-	    exit( 1 );
-	}
-	if ( ziptab == NULL ) {
-	    zt->zt_prev == NULL;
-	    ziptab = zt;
-	} else {
-	    zt->zt_prev = ziplast;
-	    ziplast->zt_next = zt;
-	}
-	ziplast = zt;
-    }
-
-    if ( ladd( &zt->zt_rt, rt ) < 0 ) {
-	exists++;
-    }
-
-    if ( ladd( &rt->rt_zt, zt ) < 0 ) {
-	if ( !exists ) {
-	    syslog( LOG_ERR, "addzone corrupted route/zone mapping" );
-	    abort();
-	}
-	/*
-	 * We get the repeat for local nets which have zone information
-	 * already: we ask anyway, just to make sure.
-	 */
-	return;
-    }
-    if ( exists ) {
-	syslog( LOG_ERR, "addzone corrupted zone/route mapping" );
-	abort();
-    }
-}
 
 /*
- * Insert at the end.  Return < 0 if a mapping already exists, 0 otherwise.
+ * Insert at the end.  Return 1 if a mapping already exists, 0 otherwise.
+ * -1 on error.
  */
-ladd( head, data )
+static int add_list( head, data )
     struct list	**head;
     void	*data;
 {
@@ -954,12 +989,12 @@ ladd( head, data )
 
     for ( l = *head; l; l = l->l_next ) {
 	if ( l->l_data == data ) {
-	    return( -1 );
+	    return( 1 );
 	}
     }
     if (( l = (struct list *)malloc( sizeof( struct list ))) == NULL ) {
-	syslog( LOG_ERR, "ladd malloc: %m" );
-	exit( 1 );
+	syslog( LOG_ERR, "add_list malloc: %s", strerror(errno) );
+	return -1;
     }
 
     l->l_data = data;
@@ -977,89 +1012,59 @@ ladd( head, data )
     return( 0 );
 }
 
-u_short
-atalk_cksum( data, len )
-    u_char	*data;
-    int		len;
+int addzone( rt, len, zone )
+    struct rtmptab	*rt;
+    int			len;
+    char		*zone;
 {
-    u_char	*end;
-    u_long	cksum = 0;
+    struct ziptab	*zt;
+    int			cc, exists = 0;
 
-    for ( end = data + len; data < end; data++ ) {
-	cksum = ( cksum + *data ) << 1;
-	if ( cksum & 0x00010000 ) {
-	    cksum++;
-	}
-	cksum &= 0x0000ffff;
-    }
-
-    if ( cksum == 0 ) {
-	cksum = 0x0000ffff;
-    }
-
-    return( (u_short)cksum );
-}
-
-#define elements( x )	(sizeof( x )/sizeof( (x)[0] ))
-
-/*
- * Return multicast for zone.  There is a general issue here:  how can
- * we tell the type of interface we're configuring for?  E.g.  Is it
- * ethernet, tokenring, or FDDI?  (Of course, FDDI and Ethernet look just
- * alike.)
- */
-u_char *
-zone_mcast( name, len )
-    char	*name;
-    u_char	len;
-{
-    extern u_char	_diacasemap[];
-    u_char		uname[ 32 ];
-    u_short		cksum;
-    int			i;
-
-    for ( i = 0; i < len; i++ ) {
-	uname[ i ] = _diacasemap[ name[ i ]];
-    }
-    cksum = atalk_cksum( uname, len );
-    return( ethermultitab[ cksum % elements( ethermultitab ) ] );
-}
-
-zonecheck( rtmp, iface )
-    struct rtmptab	*rtmp;
-    struct interface	*iface;
-{
-    struct list		*l;
-    struct ziptab	*czt, *zt;
-    int			cztcnt, ztcnt;
-
-    if (( iface->i_flags & IFACE_SEED ) == 0 ) {
-	return( 0 );
-    }
-
-    for ( cztcnt = 0, czt = iface->i_czt; czt; czt = czt->zt_next, cztcnt++ ) {
-	for ( l = rtmp->rt_zt; l; l = l->l_next ) {
-	    zt = (struct ziptab *)l->l_data;
-	    if ( czt->zt_len == zt->zt_len &&
-		    !strndiacasecmp( czt->zt_name, zt->zt_name, czt->zt_len )) {
-		break;
-	    }
-	}
-	if ( l == 0 ) {
-	    syslog( LOG_ERR, "zonecheck: %.*s not in zone list", czt->zt_len,
-		    czt->zt_name );
-	    return( -1 );	/* configured zone not found in net zones */
+    for ( zt = ziptab; zt; zt = zt->zt_next ) {
+	if ( zt->zt_len == len &&
+		strndiacasecmp( zt->zt_name, zone, len ) == 0 ) {
+	    break;
 	}
     }
-
-    for ( ztcnt = 0, l = rtmp->rt_zt; l; l = l->l_next, ztcnt++ )
-	;
-
-    if ( cztcnt != ztcnt ) {
-	syslog( LOG_ERR, "zonecheck: %d configured zones, %d zones found",
-		cztcnt, ztcnt );
-	return( -1 );		/* more net zones than configured zones */
+    if ( zt == NULL ) {
+	if (( zt = newzt( len, zone )) == NULL ) {
+	    syslog( LOG_ERR, "addzone newzt: %s", strerror(errno) );
+	    return -1;
+	}
+	if ( ziptab == NULL ) {
+	    zt->zt_prev = NULL;
+	    ziptab = zt;
+	} else {
+	    zt->zt_prev = ziplast;
+	    ziplast->zt_next = zt;
+	}
+	ziplast = zt;
     }
 
-    return( 0 );
+    if ((cc = add_list( &zt->zt_rt, rt )) < 0) 
+      return -1;
+
+    if (cc)
+      exists++;
+
+    if ((cc = add_list( &rt->rt_zt, zt )) < 0 )
+      return -1;
+
+    if (cc) {
+        if ( !exists ) {
+	    syslog( LOG_ERR, "addzone corrupted route/zone mapping" );
+	    return -1;
+	}
+	/*
+	 * We get the repeat for local nets which have zone information
+	 * already: we ask anyway, just to make sure.
+	 */
+	
+	return 0;
+    }
+    if ( exists ) {
+	syslog( LOG_ERR, "addzone corrupted zone/route mapping" );
+	return -1;
+    }
+    return 0;
 }
