@@ -1,4 +1,6 @@
 /*
+ * $Id: lp.c,v 1.10.2.1 2002/03/12 15:44:38 srittau Exp $
+ *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  *
@@ -39,12 +41,25 @@
  * Interface to lpr system.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/param.h>
-#include <sys/syslog.h>
-#include <sys/file.h>
+#include <syslog.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <ctype.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+
+#if defined( sun ) && defined( __svr4__ )
+#include </usr/ucbinclude/sys/file.h>
+#else /* sun && __svr4__ */
+#include <sys/file.h>
+#endif /* sun && __svr4__ */
 #include <sys/un.h>
 #include <netinet/in.h>
 #undef s_net
@@ -52,27 +67,32 @@
 #include <atalk/atp.h>
 #include <atalk/paths.h>
 
+#ifdef ABS_PRINT
+#include <math.h>
+#endif /* ABS_PRINT */
+
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <netdb.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#include <pwd.h>
 
 #include "printer.h"
 #include "file.h"
+#include "lp.h"
+
+/* These functions aren't used outside of lp.c */
+int lp_conn_inet();
+int lp_disconn_inet( int );
+int lp_conn_unix();
+int lp_disconn_unix( int );
 
 char	hostname[ MAXHOSTNAMELEN ];
 
-/* initialize printing interface */
-int	lp_init();
-/* cancel current job */
-int	lp_cancel();
-/* print current job */
-int	lp_print();
-
-/* open a file for spooling */
-int	lp_open();
-/* open a buffer to the current open file */
-int	lp_write();
-/* close current spooling file */
-int	lp_close();
+extern struct sockaddr_at *sat;
 
 struct lp {
     int			lp_flags;
@@ -89,7 +109,7 @@ struct lp {
 #define LP_CONNECT	(1<<3)
 #define LP_QUEUE	(1<<4)
 
-lp_person( person )
+void lp_person( person )
     char	*person;
 {
     if ( lp.lp_person != NULL ) {
@@ -102,7 +122,25 @@ lp_person( person )
     strcpy( lp.lp_person, person );
 }
 
-lp_host( host )
+#ifdef ABS_PRINT
+int lp_pagecost()
+{
+    char	cost[ 22 ];
+    char	balance[ 22 ];
+    int		err;
+
+    if ( lp.lp_person == NULL ) {
+	return( -1 );
+    }
+    err = ABS_canprint( lp.lp_person, printer->p_role, printer->p_srvid,
+	    cost, balance );
+    printer->p_pagecost = floor( atof( cost ) * 10000.0 );
+    printer->p_balance = atof( balance ) + atof( cost );
+    return( err < 0 ? -1 : 0 );
+}
+#endif /* ABS_PRINT */
+
+void lp_host( host )
     char	*host;
 {
     if ( lp.lp_host != NULL ) {
@@ -115,7 +153,7 @@ lp_host( host )
     strcpy( lp.lp_host, host );
 }
 
-lp_job( job )
+void lp_job( job )
     char	*job;
 {
     char	*p, *q;
@@ -128,7 +166,7 @@ lp_job( job )
 	exit( 1 );
     }
     for ( p = job, q = lp.lp_job; *p != '\0'; p++, q++ ) {
-	if ( !isascii( *p ) || !isprint( *p )) {
+	if ( !isascii( *p ) || !isprint( *p ) || *p == '\\' ) {
 	    *q = '.';
 	} else {
 	    *q = *p;
@@ -137,27 +175,78 @@ lp_job( job )
     *q = '\0';
 }
 
-lp_init( out )
+int lp_init( out, sat )
     struct papfile	*out;
+    struct sockaddr_at	*sat;
 {
     int		fd, n, len;
     char	*cp, buf[ BUFSIZ ];
     struct stat	st;
+    int		authenticated = 0;
+#ifdef ABS_PRINT
+    char	cost[ 22 ];
+    char	balance[ 22 ];
+#endif /* ABS_PRINT */
 
-    if (( printer->p_flags & P_AUTH ) && lp.lp_person == NULL ) {
-	syslog( LOG_ERR, "lp_init: must authenticate" );
-	spoolerror( out, "Authentication required." );
-	return( -1 );
-    }
+    if ( printer->p_flags & P_AUTH ) {
+	authenticated = 0;
+
+	/* cap style "log on to afp server before printing" authentication */
+
+	if ( printer->p_authprintdir && (printer->p_flags & P_AUTH_CAP) ) {
+	    int addr_net = ntohs( sat->sat_addr.s_net );
+	    int addr_node  = sat->sat_addr.s_node;
+	    char addr_filename[256];
+	    char username[32];
+	    struct stat cap_st;
+	    FILE *cap_file;
+
+	    sprintf(addr_filename, "%s/net%d.%dnode%d", 
+		printer->p_authprintdir, addr_net/256, addr_net%256, 
+		addr_node);
+	    if (stat(addr_filename, &cap_st) == 0) {
+		if ((cap_file = fopen(addr_filename, "r")) != NULL) {
+		    if (fscanf(cap_file, "%s", username) != EOF) {
+			if (getpwnam(username) != NULL ) {
+			    syslog(LOG_INFO, "CAP authenticated %s", username);
+			    lp_person(username);
+			    authenticated = 1;
+			} else {
+			    syslog(LOG_INFO, "CAP error: invalid username: '%s'", username);
+			}
+		    } else {
+			syslog(LOG_INFO, "CAP error: could not read username");
+		    }
+		} else {
+		    syslog(LOG_INFO, "CAP error: %m");
+		}
+	    } else {
+		syslog(LOG_INFO, "CAP error: %m");
+	    }
+	}
+
+	if ( printer->p_flags & P_AUTH_PSSP ) {
+	    if ( lp.lp_person != NULL ) {
+		authenticated = 1;
+	    }
+	}
+
+	if ( authenticated == 0 ) {
+	    syslog( LOG_ERR, "lp_init: must authenticate" );
+	    spoolerror( out, "Authentication required." );
+	    return( -1 );
+	}
 
 #ifdef ABS_PRINT
-    if (( printer->p_flags & P_ACCOUNT ) && printer->p_pagecost > 0 &&
-	    ! ABS_canprint( lp.lp_person )) {
-	syslog( LOG_ERR, "lp_init: no ABS funds" );
-	spoolerror( out, "No ABS funds available." );
-	return( -1 );
+	if (( printer->p_flags & P_ACCOUNT ) && printer->p_pagecost > 0 &&
+		! ABS_canprint( lp.lp_person, printer->p_role,
+		printer->p_srvid, cost, balance )) {
+	    syslog( LOG_ERR, "lp_init: no ABS funds" );
+	    spoolerror( out, "No ABS funds available." );
+	    return( -1 );
+	}
+#endif /* ABS_PRINT */
     }
-#endif ABS_PRINT
 
     if ( gethostname( hostname, sizeof( hostname )) < 0 ) {
 	syslog( LOG_ERR, "gethostname: %m" );
@@ -176,7 +265,7 @@ lp_init( out )
     if ( printer->p_flags & P_SPOOLED ) {
 	/* check if queuing is enabled: mode & 010 on lock file */
 	if ( stat( printer->p_lock, &st ) < 0 ) {
-	    syslog( LOG_ERR, "lp_init: can't stat %s", printer->p_lock );
+	    syslog( LOG_ERR, "lp_init: %s: %m", printer->p_lock );
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
@@ -191,6 +280,7 @@ lp_init( out )
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
+
 	if ( flock( fd, LOCK_EX ) < 0 ) {
 	    syslog( LOG_ERR, "lp_init: can't lock .seq" );
 	    spoolerror( out, NULL );
@@ -227,13 +317,15 @@ lp_init( out )
     return( 0 );
 }
 
-lp_open( out )
+int lp_open( out, sat )
     struct papfile	*out;
+    struct sockaddr_at	*sat;
 {
     char	name[ MAXPATHLEN ];
     int		fd;
+    struct passwd	*pwent;
 
-    if (( lp.lp_flags & LP_INIT ) == 0 && lp_init( out ) != 0 ) {
+    if (( lp.lp_flags & LP_INIT ) == 0 && lp_init( out, sat ) != 0 ) {
 	return( -1 );
     }
     if ( lp.lp_flags & LP_OPEN ) {
@@ -242,7 +334,21 @@ lp_open( out )
     }
 
     if ( lp.lp_flags & LP_PIPE ) {
-	/* open /tmp file?  go right to program? */
+	/* go right to program */
+	if (lp.lp_person != NULL) {
+	    if((pwent = getpwnam(lp.lp_person)) != NULL) {
+		if(setreuid(pwent->pw_uid, pwent->pw_uid) != 0) {
+		    syslog(LOG_INFO, "setreuid error: %m");
+		}
+	    } else {
+		syslog(LOG_INFO, "Error getting username (%s)", lp.lp_person);
+	    }
+	}
+	if (( lp.lp_stream = popen( printer->p_printer, "w" )) == NULL ) {
+	    syslog( LOG_ERR, "lp_open popen %s: %m", printer->p_printer );
+	    spoolerror( out, NULL );
+	    return( -1 );
+	}
     } else {
 	sprintf( name, "df%c%03d%s", lp.lp_letter++, lp.lp_seq, hostname );
 
@@ -251,29 +357,50 @@ lp_open( out )
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
+
+	if (lp.lp_person != NULL) {
+	    if ((pwent = getpwnam(lp.lp_person)) == NULL) {
+		syslog(LOG_ERR, "getpwnam %s: no such user", lp.lp_person);
+		spoolerror( out, NULL );
+		return( -1 );
+	    }
+	} else {
+	    if ((pwent = getpwnam(printer->p_operator)) == NULL) {
+		syslog(LOG_ERR, "getpwnam %s: no such user", printer->p_operator);
+		spoolerror( out, NULL );
+		return( -1 );
+	    }
+	}
+
+	if (fchown(fd, pwent->pw_uid, -1) < 0) {
+	    syslog(LOG_ERR, "chown %s %s: %m", pwent->pw_name, name);
+	    spoolerror( out, NULL );
+	    return( -1 );
+	}
+
 	if (( lp.lp_stream = fdopen( fd, "w" )) == NULL ) {
 	    syslog( LOG_ERR, "lp_open fdopen: %m" );
 	    spoolerror( out, NULL );
 	    return( -1 );
 	}
-	lp.lp_flags |= LP_OPEN;
     }
+    lp.lp_flags |= LP_OPEN;
 
     return( 0 );
 }
 
-lp_close()
+int lp_close()
 {
     if (( lp.lp_flags & LP_INIT ) == 0 || ( lp.lp_flags & LP_OPEN ) == 0 ) {
-	return;
+	return 0;
     }
     fclose( lp.lp_stream );
     lp.lp_stream = NULL;
     lp.lp_flags &= ~LP_OPEN;
-    return;
+    return 0;
 }
 
-lp_write( buf, len )
+int lp_write( buf, len )
     char	*buf;
     int		len;
 {
@@ -288,13 +415,13 @@ lp_write( buf, len )
     return( 0 );
 }
 
-lp_cancel()
+int lp_cancel()
 {
     char	name[ MAXPATHLEN ];
     char	letter;
 
     if (( lp.lp_flags & LP_INIT ) == 0 || lp.lp_letter == 'A' ) {
-	return;
+	return 0;
     }
 
     if ( lp.lp_flags & LP_OPEN ) {
@@ -308,7 +435,7 @@ lp_cancel()
 	}
     }
 
-    return;
+    return 0;
 }
 
 /*
@@ -317,7 +444,7 @@ lp_cancel()
  *
  * XXX piped?
  */
-lp_print()
+int lp_print()
 {
     char		buf[ MAXPATHLEN ];
     char		tfname[ MAXPATHLEN ];
@@ -327,94 +454,97 @@ lp_print()
     FILE		*cfile;
 
     if (( lp.lp_flags & LP_INIT ) == 0 || lp.lp_letter == 'A' ) {
-	return;
+	return 0;
     }
     lp_close();
 
-    sprintf( tfname, "tfA%03d%s", lp.lp_seq, hostname );
-    if (( fd = open( tfname, O_WRONLY|O_EXCL|O_CREAT, 0660 )) < 0 ) {
-	syslog( LOG_ERR, "lp_print %s: %m", tfname );
-	return;
-    }
-    if (( cfile = fdopen( fd, "w" )) == NULL ) {
-	syslog( LOG_ERR, "lp_print %s: %m", tfname );
-	return;
-    }
-    fprintf( cfile, "H%s\n", hostname );	/* XXX lp_host? */
+    if ( printer->p_flags & P_SPOOLED ) {
+	sprintf( tfname, "tfA%03d%s", lp.lp_seq, hostname );
+	if (( fd = open( tfname, O_WRONLY|O_EXCL|O_CREAT, 0660 )) < 0 ) {
+	    syslog( LOG_ERR, "lp_print %s: %m", tfname );
+	    return 0;
+	}
+	if (( cfile = fdopen( fd, "w" )) == NULL ) {
+	    syslog( LOG_ERR, "lp_print %s: %m", tfname );
+	    return 0;
+	}
+	fprintf( cfile, "H%s\n", hostname );	/* XXX lp_host? */
 
-    if ( lp.lp_person ) {
-	fprintf( cfile, "P%s\n", lp.lp_person );
-    } else {
-	fprintf( cfile, "P%s\n", printer->p_operator );
-    }
+	if ( lp.lp_person ) {
+	    fprintf( cfile, "P%s\n", lp.lp_person );
+	} else {
+	    fprintf( cfile, "P%s\n", printer->p_operator );
+	}
 
-    if ( lp.lp_job ) {
-	fprintf( cfile, "J%s\n", lp.lp_job );
-	fprintf( cfile, "T%s\n", lp.lp_job );
-    } else {
-	fprintf( cfile, "JMac Job\n" );
-	fprintf( cfile, "TMac Job\n" );
-    }
+	if ( lp.lp_job && *lp.lp_job ) {
+	    fprintf( cfile, "J%s\n", lp.lp_job );
+	    fprintf( cfile, "T%s\n", lp.lp_job );
+	} else {
+	    fprintf( cfile, "JMac Job\n" );
+	    fprintf( cfile, "TMac Job\n" );
+	}
 
-    fprintf( cfile, "C%s\n", hostname );	/* XXX lp_host? */
+	fprintf( cfile, "C%s\n", hostname );	/* XXX lp_host? */
 
-    if ( lp.lp_person ) {
-	fprintf( cfile, "L%s\n", lp.lp_person );
-    } else {
-	fprintf( cfile, "L%s\n", printer->p_operator );
-    }
+	if ( lp.lp_person ) {
+	    fprintf( cfile, "L%s\n", lp.lp_person );
+	} else {
+	    fprintf( cfile, "L%s\n", printer->p_operator );
+	}
 
-    for ( letter = 'A'; letter < lp.lp_letter; letter++ ) {
-	fprintf( cfile, "fdf%c%03d%s\n", letter, lp.lp_seq, hostname );
-	fprintf( cfile, "Udf%c%03d%s\n", letter, lp.lp_seq, hostname );
-    }
+	for ( letter = 'A'; letter < lp.lp_letter; letter++ ) {
+	    fprintf( cfile, "fdf%c%03d%s\n", letter, lp.lp_seq, hostname );
+	    fprintf( cfile, "Udf%c%03d%s\n", letter, lp.lp_seq, hostname );
+	}
 
-    if ( lp.lp_job ) {
-	fprintf( cfile, "N%s\n", lp.lp_job );
-    } else {
-	fprintf( cfile, "NMac Job\n" );
-    }
-    fclose( cfile );
+	if ( lp.lp_job && *lp.lp_job ) {
+	    fprintf( cfile, "N%s\n", lp.lp_job );
+	} else {
+	    fprintf( cfile, "NMac Job\n" );
+	}
+	fclose( cfile );
 
-    sprintf( cfname, "cfA%03d%s", lp.lp_seq, hostname );
-    if ( link( tfname, cfname ) < 0 ) {
-	syslog( LOG_ERR, "lp_print can't link %s to %s: %m", cfname, tfname );
-	return;
-    }
-    unlink( tfname );
+	sprintf( cfname, "cfA%03d%s", lp.lp_seq, hostname );
+	if ( link( tfname, cfname ) < 0 ) {
+	    syslog( LOG_ERR, "lp_print can't link %s to %s: %m", cfname,
+		    tfname );
+	    return 0;
+	}
+	unlink( tfname );
 
-    if (( s = lp_conn_unix()) < 0 ) {
-	syslog( LOG_ERR, "lp_print: lp_conn_unix: %m" );
-	return;
-    }
+	if (( s = lp_conn_unix()) < 0 ) {
+	    syslog( LOG_ERR, "lp_print: lp_conn_unix: %m" );
+	    return 0;
+	}
 
-    sprintf( buf, "\1%s\n", printer->p_printer );
-    n = strlen( buf );
-    if ( write( s, buf, n ) != n ) {
-	syslog( LOG_ERR, "lp_print write: %m" );
-	return;
-    }
-    if ( read( s, buf, 1 ) != 1 ) {
-	syslog( LOG_ERR, "lp_print read: %m" );
-	return;
-    }
+	sprintf( buf, "\1%s\n", printer->p_printer );
+	n = strlen( buf );
+	if ( write( s, buf, n ) != n ) {
+	    syslog( LOG_ERR, "lp_print write: %m" );
+	    return 0;
+	}
+	if ( read( s, buf, 1 ) != 1 ) {
+	    syslog( LOG_ERR, "lp_print read: %m" );
+	    return 0;
+	}
 
-    lp_disconn_unix( s );
+	lp_disconn_unix( s );
 
-    if ( buf[ 0 ] != '\0' ) {
-	syslog( LOG_ERR, "lp_print lpd said %c: %m", buf[ 0 ] );
-	return;
+	if ( buf[ 0 ] != '\0' ) {
+	    syslog( LOG_ERR, "lp_print lpd said %c: %m", buf[ 0 ] );
+	    return 0;
+	}
     }
     syslog( LOG_INFO, "lp_print queued" );
-    return;
+    return 0;
 }
 
-lp_disconn_unix( fd )
+int lp_disconn_unix( fd )
 {
     return( close( fd ));
 }
 
-lp_conn_unix()
+int lp_conn_unix()
 {
     int			s;
     struct sockaddr_un	saun;
@@ -423,7 +553,7 @@ lp_conn_unix()
 	syslog( LOG_ERR, "lp_conn_unix socket: %m" );
 	return( -1 );
     }
-    bzero( &saun, sizeof( struct sockaddr_un ));
+    memset( &saun, 0, sizeof( struct sockaddr_un ));
     saun.sun_family = AF_UNIX;
     strcpy( saun.sun_path, _PATH_DEVPRINTER );
     if ( connect( s, (struct sockaddr *)&saun,
@@ -436,12 +566,12 @@ lp_conn_unix()
     return( s );
 }
 
-lp_disconn_inet( fd )
+int lp_disconn_inet( int fd )
 {
     return( close( fd ));
 }
 
-lp_conn_inet()
+int lp_conn_inet()
 {
     int			privfd, port = IPPORT_RESERVED - 1;
     struct sockaddr_in	sin;
@@ -469,10 +599,10 @@ lp_conn_inet()
 	return( -1 );
     }
 
-    bzero( &sin, sizeof( struct sockaddr_in ));
+    memset( &sin, 0, sizeof( struct sockaddr_in ));
     sin.sin_family = AF_INET;
 /*    sin.sin_addr.s_addr = htonl( INADDR_LOOPBACK ); */
-    bcopy( hp->h_addr, &sin.sin_addr, hp->h_length );
+    memcpy( &sin.sin_addr, hp->h_addr, hp->h_length );
     sin.sin_port = sp->s_port;
 
     if ( connect( privfd, (struct sockaddr *)&sin,
@@ -485,7 +615,7 @@ lp_conn_inet()
     return( privfd );
 }
 
-lp_rmjob( job )
+int lp_rmjob( job )
     int		job;
 {
     char	buf[ 1024 ];
@@ -525,10 +655,11 @@ char	*tag_files = "files: ";
 char	*tag_size = "size: ";
 char	*tag_status = "status: ";
 
-lp_queue( out )
+int lp_queue( out )
     struct papfile	*out;
 {
     char			buf[ 1024 ], *start, *stop, *p, *q;
+    int				linelength, crlflength;
     static struct papfile	pf;
     int				n, len, s;
 	
@@ -547,19 +678,20 @@ lp_queue( out )
     pf.pf_state = PF_BOT;
 
     while (( n = read( s, buf, sizeof( buf ))) > 0 ) {
-	APPEND( &pf, buf, n );
+	append( &pf, buf, n );
     }
 
     for (;;) {
-	if ( markline( &start, &stop, &pf ) > 0 ) {
+	if ( markline( &pf, &start, &linelength, &crlflength ) > 0 ) {
 	    /* parse */
+	    stop = start + linelength;
 	    for ( p = start; p < stop; p++ ) {
 		if ( *p == ' ' || *p == '\t' ) {
 		    break;
 		}
 	    }
 	    if ( p >= stop ) {
-		consumetomark( start, stop, &pf );
+		CONSUME( &pf , linelength + crlflength);
 		continue;
 	    }
 
@@ -570,15 +702,15 @@ lp_queue( out )
 	    len = p - start;
 	    if ( len == strlen( kw_rank ) &&
 		    strncmp( kw_rank, start, len ) == 0 ) {
-		consumetomark( start, stop, &pf );
+		CONSUME( &pf, linelength + crlflength );
 		continue;
 	    }
 	    if (( len == strlen( kw_active ) &&
 		    strncmp( kw_active, start, len ) == 0 ) ||
 		    isdigit( *start )) {		/* a job line */
-		APPEND( out, tag_rank, strlen( tag_rank ));
-		APPEND( out, start, p - start );
-		APPEND( out, "\n", 1 );
+		append( out, tag_rank, strlen( tag_rank ));
+		append( out, start, p - start );
+		append( out, "\n", 1 );
 
 		for ( ; p < stop; p++ ) {
 		    if ( *p != ' ' && *p != '\t' ) {
@@ -591,13 +723,13 @@ lp_queue( out )
 		    }
 		}
 		if ( p >= stop ) {
-		    APPEND( out, ".\n", 2 );
-		    consumetomark( start, stop, &pf );
+		    append( out, ".\n", 2 );
+		    CONSUME( &pf, linelength + crlflength );
 		    continue;
 		}
-		APPEND( out, tag_owner, strlen( tag_owner ));
-		APPEND( out, q, p - q );
-		APPEND( out, "\n", 1 );
+		append( out, tag_owner, strlen( tag_owner ));
+		append( out, q, p - q );
+		append( out, "\n", 1 );
 
 		for ( ; p < stop; p++ ) {
 		    if ( *p != ' ' && *p != '\t' ) {
@@ -610,13 +742,13 @@ lp_queue( out )
 		    }
 		}
 		if ( p >= stop ) {
-		    APPEND( out, ".\n", 2 );
-		    consumetomark( start, stop, &pf );
+		    append( out, ".\n", 2 );
+		    CONSUME( &pf , linelength + crlflength );
 		    continue;
 		}
-		APPEND( out, tag_job, strlen( tag_job ));
-		APPEND( out, q, p - q );
-		APPEND( out, "\n", 1 );
+		append( out, tag_job, strlen( tag_job ));
+		append( out, q, p - q );
+		append( out, "\n", 1 );
 
 		for ( ; p < stop; p++ ) {
 		    if ( *p != ' ' && *p != '\t' ) {
@@ -639,35 +771,35 @@ lp_queue( out )
 		    }
 		}
 		if ( p <= q ) {
-		    APPEND( out, ".\n", 2 );
-		    consumetomark( start, stop, &pf );
+		    append( out, ".\n", 2 );
+		    CONSUME( &pf, linelength + crlflength );
 		    continue;
 		}
-		APPEND( out, tag_files, strlen( tag_files ));
-		APPEND( out, q, p - q );
-		APPEND( out, "\n", 1 );
+		append( out, tag_files, strlen( tag_files ));
+		append( out, q, p - q );
+		append( out, "\n", 1 );
 
 		for ( ; p < stop; p++ ) {
 		    if ( *p != ' ' && *p != '\t' ) {
 			break;
 		    }
 		}
-		APPEND( out, tag_size, strlen( tag_size ));
-		APPEND( out, p, stop - p );
-		APPEND( out, "\n.\n", 3 );
+		append( out, tag_size, strlen( tag_size ));
+		append( out, p, stop - p );
+		append( out, "\n.\n", 3 );
 
-		consumetomark( start, stop, &pf );
+		CONSUME( &pf, linelength + crlflength );
 		continue;
 	    }
 
 	    /* status */
-	    APPEND( out, tag_status, strlen( tag_status ));
-	    APPEND( out, start, stop - start );
-	    APPEND( out, "\n.\n", 3 );
+	    append( out, tag_status, strlen( tag_status ));
+	    append( out, start, linelength );
+	    append( out, "\n.\n", 3 );
 
-	    consumetomark( start, stop, &pf );
+	    CONSUME( &pf, linelength + crlflength );
 	} else {
-	    APPEND( out, "*\n", 2 );
+	    append( out, "*\n", 2 );
 	    lp_disconn_unix( s );
 	    return( 0 );
 	}
