@@ -1,33 +1,80 @@
 /*
+ * $Id: main.c,v 1.13.2.1 2002/03/12 15:44:38 srittau Exp $
+ *
  * Copyright (c) 1990,1995 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/param.h>
-#include <sys/types.h>
 #include <sys/time.h>
 #include <sys/uio.h>
-#include <sys/socket.h>
+#if defined( sun ) && defined( __svr4__ )
+#include </usr/ucbinclude/sys/file.h>
+#else /* sun && __svr4__ */
 #include <sys/file.h>
-#include <sys/syslog.h>
-#include <sys/ioctl.h>
-#include <sys/wait.h>
-#include <sys/errno.h>
+#endif /* sun && __svr4__ */
+#include <sys/socket.h>
+#include <syslog.h>
 
+/* POSIX.1 sys/wait.h check */
+#include <sys/types.h>
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif /* HAVE_SYS_WAIT_H */
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
+#endif /* ! WEXITSTATUS */
+#ifndef WIFEXITED
+#define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
+#endif /* ! WIFEXITED */
+
+#include <errno.h>
+
+/* STDC check */
+#if STDC_HEADERS
+#include <string.h>
+#else /* STDC_HEADERS */
+#ifndef HAVE_STRCHR
+#define strchr index
+#define strrchr index
+#endif /* HAVE_STRCHR */
+char *strchr (), *strrchr ();
+#ifndef HAVE_MEMCPY
+#define memcpy(d,s,n) bcopy ((s), (d), (n))
+#define memmove(d,s,n) bcopy ((s), (d), (n))
+#endif /* ! HAVE_MEMCPY */
+#endif /* STDC_HEADERS */
+
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <netdb.h>
+#include <fcntl.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+
+#include <netatalk/endian.h>
 #include <netatalk/at.h>
+#include <atalk/compat.h>
 #include <atalk/atp.h>
 #include <atalk/pap.h>
 #include <atalk/paths.h>
-
-#include <strings.h>
-#include <signal.h>
-#include <stdio.h>
+#include <atalk/util.h>
+#include <atalk/nbp.h>
 
 #include "printer.h"
+#include "printcap.h"
+#include "session.h"
+#include "uam_auth.h"
 
 #define _PATH_PAPDPPDFILE	".ppd"
 
-#define PIPED_STATUS	"status: print spooler processing job";
+#define PIPED_STATUS	"status: print spooler processing job"
 
 struct printer	defprinter;
 struct printer	*printers = NULL;
@@ -38,35 +85,57 @@ char		*printcap = _PATH_PAPDPRINTCAP;
 unsigned char	connid, quantum, sock, oquantum = PAP_MAXQUANTUM;
 char		*cannedstatus = PIPED_STATUS;
 struct printer	*printer = NULL;
+char		*version = VERSION;
+static char      *pidfile = _PATH_PAPDLOCK;
+
+char		*uamlist;
+char		*uampath = _PATH_PAPDUAMPATH;
+
+/* Prototypes for locally used functions */
+int getstatus( struct printer *pr, char *buf );
+int rprintcap( struct printer *pr );
+void getprinters( char *cf );
+
+
+/* this only needs to be used by the server process */
+static void papd_exit(const int i)
+{
+  server_unlock(pidfile);
+  auth_unload();
+  exit(i);
+}
 
 #if !defined( ibm032 ) && !defined( _IBMR2 )
     void
-#endif ibm032 _IBMR2
+#endif /* ! ibm032 && ! _IBMR2 */
 die( n )
     int			n;
 {
     struct printer	*pr;
+    struct at_addr	addr;
+
+    memset(&addr, 0, sizeof(addr));
 
     for ( pr = printers; pr; pr = pr->p_next ) {
 	if ( pr->p_flags & P_REGISTERED ) {
-	    if ( nbp_unrgstr( pr->p_name, pr->p_type, pr->p_zone ) < 0 ) {
+	    if ( nbp_unrgstr( pr->p_name, pr->p_type, pr->p_zone, &addr ) < 0 ) {
 		syslog( LOG_ERR, "can't unregister %s:%s@%s\n", pr->p_name,
 			pr->p_type, pr->p_zone );
-		exit( n + 1 );
+		papd_exit( n + 1 );
 	    }
 	    syslog( LOG_ERR, "unregister %s:%s@%s\n", pr->p_name, pr->p_type,
 		    pr->p_zone );
 	}
     }
-    exit( n );
+    papd_exit( n );
 }
 
 #if !defined( ibm032 ) && !defined( _IBMR2 )
     void
-#endif ibm032 _IBMR2
+#endif /* ! ibm032 && ! _IBMR2 */
 reap()
 {
-    union wait	status;
+    int		status;
     int		pid;
 
     while (( pid = wait3( &status, WNOHANG, 0 )) > 0 ) {
@@ -91,10 +160,12 @@ reap()
 
 char		rbuf[ 255 + 1 + 8 ];
 
-main( ac, av )
+int main( ac, av )
     int		ac;
     char	**av;
 {
+    extern char         *optarg;
+
     ATP			atp;
     struct atp_block	atpb;
     struct sockaddr_at	sat;
@@ -105,14 +176,12 @@ main( ac, av )
     char		*p, hostname[ MAXHOSTNAMELEN ];
     char		cbuf[ 8 ];
     int			c;
-    extern char		*optarg;
-    extern int		optind, errno;
 
     if ( gethostname( hostname, sizeof( hostname )) < 0 ) {
 	perror( "gethostname" );
 	exit( 1 );
     }
-    if (( p = index( hostname, '.' )) != 0 ) {
+    if (( p = strchr( hostname, '.' )) != 0 ) {
 	*p = '\0';
     }
     if (( defprinter.p_name = (char *)malloc( strlen( hostname ) + 1 ))
@@ -123,16 +192,26 @@ main( ac, av )
     strcpy( defprinter.p_name, hostname );
     defprinter.p_type = "LaserWriter";
     defprinter.p_zone = "*";
+    memset(&defprinter.p_addr, 0, sizeof(defprinter.p_addr));
     defprinter.p_ppdfile = _PATH_PAPDPPDFILE;
+#ifdef __svr4__
+    defprinter.p_flags = P_PIPED;
+    defprinter.p_printer = "/usr/bin/lp -T PS";
+#else /* __svr4__ */
     defprinter.p_flags = P_SPOOLED;
     defprinter.p_printer = "lp";
+#endif /* __svr4__ */
     defprinter.p_operator = "operator";
     defprinter.p_spool = _PATH_PAPDSPOOLDIR;
-    defprinter.p_pagecost = 0;
+#ifdef ABS_PRINT
+    defprinter.p_role = NULL;
+    defprinter.p_srvid = 0;
+#endif /* ABS_PRINT */
+    defprinter.p_pagecost = 200;		/* default cost */
     defprinter.p_pagecost_msg = NULL;
     defprinter.p_lock = "lock";
 
-    while (( c = getopt( ac, av, "adf:p:" )) != EOF ) {
+    while (( c = getopt( ac, av, "adf:p:P:v" )) != EOF ) {
 	switch ( c ) {
 	case 'a' :		/* for compatibility with old papd */
 	    break;
@@ -149,6 +228,15 @@ main( ac, av )
 	    printcap = optarg;
 	    break;
 
+	case 'P' :
+	    pidfile = optarg;
+	    break;
+
+	case 'v' :		/* version */
+	    printf( "papd (version %s)\n", VERSION );
+	    exit ( 1 );
+	    break;
+
 	default :
 	    fprintf( stderr,
 		    "Usage:\t%s [ -d ] [ -f conffile ] [ -p printcap ]\n",
@@ -159,59 +247,42 @@ main( ac, av )
 
     getprinters( conffile );
 
-    /*
-     * Disassociate from controlling tty.
-     */
-    if ( !debug ) {
-	int		i, dt;
-
-	switch ( fork()) {
-	case 0 :
-	    dt = getdtablesize();
-	    for ( i = 0; i < dt; i++ ) {
-		(void)close( i );
-	    }
-	    if (( i = open( "/dev/tty", O_RDWR )) >= 0 ) {
-		(void)ioctl( i, TIOCNOTTY, 0 );
-		setpgrp( 0, getpid());
-		(void)close( i );
-	    }
-	    if (( i = open( "/", O_RDONLY )) >= 0 ) {
-		dup2( i, 1 );
-		dup2( i, 2 );
-	    }
-	    break;
-	case -1 :
-	    perror( "fork" );
-	    die( 1 );
-	default :
-	    exit( 0 );
-	}
-    }
+    switch (server_lock("papd", pidfile, debug)) {
+    case 0: /* open a couple things again in the child */
+      if ((c = open("/", O_RDONLY)) >= 0) {
+	dup2(c, 1);
+	dup2(c, 2);
+      }
+      break;
+    case -1:
+      exit(1);
+    default:
+      exit(0);
+    }      
 
     /*
      * Start logging.
      */
-    if (( p = rindex( av[ 0 ], '/' )) == NULL ) {
+    if (( p = strrchr( av[ 0 ], '/' )) == NULL ) {
 	p = av[ 0 ];
     } else {
 	p++;
     }
 #ifdef ultrix
     openlog( p, LOG_PID );
-#else ultrix
+#else /* ultrix */
     openlog( p, LOG_NDELAY|LOG_PID, LOG_LPR );
-#endif ultrix
+#endif /* ultrix */
 
-    syslog( LOG_INFO, "restart" );
+    syslog( LOG_INFO, "restart (%s)", version );
 
     for ( pr = printers; pr; pr = pr->p_next ) {
 	if (( pr->p_flags & P_SPOOLED ) && rprintcap( pr ) < 0 ) {
 	    syslog( LOG_ERR, "printcap problem: %s", pr->p_printer );
 	}
-	if (( pr->p_atp = atp_open( 0 )) == NULL ) {
+	if (( pr->p_atp = atp_open( ATADDR_ANYPORT, &pr->p_addr )) == NULL ) {
 	    syslog( LOG_ERR, "atp_open: %m" );
-	    exit( 1 );
+	    papd_exit( 1 );
 	}
 	if ( nbp_rgstr( atp_sockaddr( pr->p_atp ), pr->p_name, pr->p_type,
 		pr->p_zone ) < 0 ) {
@@ -219,27 +290,39 @@ main( ac, av )
 		    pr->p_zone );
 	    die( 1 );
 	}
+	if ( pr->p_flags & P_AUTH ) {
+		syslog( LOG_INFO, "Authentication enabled: %s", pr->p_name );
+	}
+	else {
+		syslog( LOG_INFO, "Authentication disabled: %s", pr->p_name );
+	}
 	syslog( LOG_INFO, "register %s:%s@%s", pr->p_name, pr->p_type,
 		pr->p_zone );
 	pr->p_flags |= P_REGISTERED;
     }
 
+    memset(&sv, 0, sizeof(sv));
     sv.sa_handler = die;
-    sv.sa_mask = 0;
-    sv.sa_flags = 0;
+    sigemptyset( &sv.sa_mask );
+    sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
 	syslog( LOG_ERR, "sigaction: %m" );
-	exit( 1 );
+	papd_exit( 1 );
     }
 
     sv.sa_handler = reap;
-    sv.sa_mask = 0;
-    sv.sa_flags = 0;
+    sigemptyset( &sv.sa_mask );
+    sv.sa_flags = SA_RESTART;
     if ( sigaction( SIGCHLD, &sv, 0 ) < 0 ) {
 	syslog( LOG_ERR, "sigaction: %m" );
-	exit( 1 );
+	papd_exit( 1 );
     }
 
+    /*
+     * Load UAMS
+     */
+    auth_load(uampath, uamlist);
+
     /*
      * Begin accepting connections.
      */
@@ -253,12 +336,17 @@ main( ac, av )
 		continue;
 	    }
 	    syslog( LOG_ERR, "select: %m" );
-	    exit( 1 );
+	    papd_exit( 1 );
 	}
 
 	for ( pr = printers; pr; pr = pr->p_next ) {
 	    if ( FD_ISSET( atp_fileno( pr->p_atp ), &fdset )) {
-		bzero( &sat, sizeof( struct sockaddr_at ));
+		int		err = 0;
+
+		memset( &sat, 0, sizeof( struct sockaddr_at ));
+#ifdef BSD4_4
+		sat.sat_len = sizeof( struct sockaddr_at );
+#endif /* BSD4_4 */
 		sat.sat_family = AF_APPLETALK;
 		sat.sat_addr.s_net = ATADDR_ANYNET;
 		sat.sat_addr.s_node = ATADDR_ANYNODE;
@@ -279,12 +367,6 @@ main( ac, av )
 
 		switch( cbuf[ 1 ] ) {
 		case PAP_OPEN :
-		    if ( rprintcap( pr ) != 0 ) {
-			syslog( LOG_ERR, "printcap problem: %s",
-				pr->p_printer );
-			continue;
-		    }
-
 		    connid = (unsigned char)cbuf[ 0 ];
 		    sock = (unsigned char)cbuf[ 4 ];
 		    quantum = (unsigned char)cbuf[ 5 ];
@@ -292,14 +374,23 @@ main( ac, av )
 		    rbuf[ 1 ] = PAP_OPENREPLY;
 		    rbuf[ 2 ] = rbuf[ 3 ] = 0;
 
+		    if (( pr->p_flags & P_SPOOLED ) && rprintcap( pr ) != 0 ) {
+			syslog( LOG_ERR, "printcap problem: %s",
+				pr->p_printer );
+			rbuf[ 2 ] = rbuf[ 3 ] = 0xff;
+			err = 1;
+		    }
+
 		    /*
 		     * If this fails, we've run out of sockets. Rather than
 		     * just die(), let's try to continue. Maybe some sockets
 		     * will close, and we can continue;
 		     */
-		    if (( atp = atp_open( 0 )) == NULL ) {
+		    if (( atp = atp_open( ATADDR_ANYPORT, 
+					  &pr->p_addr)) == NULL ) {
 			syslog( LOG_ERR, "atp_open: %m" );
-			continue;
+			rbuf[ 2 ] = rbuf[ 3 ] = 0xff;
+			err = 1;
 		    }
 		    rbuf[ 4 ] = atp_sockaddr( atp )->sat_port;
 		    rbuf[ 5 ] = oquantum;
@@ -317,12 +408,16 @@ main( ac, av )
 			continue;
 		    }
 
+		    if ( err ) {
+			continue;
+		    }
+
 		    switch ( c = fork()) {
 		    case -1 :
 			syslog( LOG_ERR, "fork: %m" );
 			continue;
 
-		    case 0 :
+		    case 0 : /* child */
 			printer = pr;
 
 			if (( printer->p_flags & P_SPOOLED ) &&
@@ -332,8 +427,8 @@ main( ac, av )
 			}
 
 			sv.sa_handler = SIG_DFL;
-			sv.sa_mask = 0;
-			sv.sa_flags = 0;
+			sigemptyset( &sv.sa_mask );
+			sv.sa_flags = SA_RESTART;
 			if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
 			    syslog( LOG_ERR, "sigaction: %m" );
 			    exit( 1 );
@@ -350,7 +445,7 @@ main( ac, av )
 			exit( 0 );
 			break;
 
-		    default :
+		    default : /* parent */
 			syslog( LOG_INFO, "child %d for \"%s\" from %u.%u",
 				c, pr->p_name, ntohs( sat.sat_addr.s_net ),
 				sat.sat_addr.s_node);
@@ -388,7 +483,7 @@ main( ac, av )
 		/*
 		 * Sometimes the child process will send its first READ
 		 * before the parent has sent the OPEN REPLY.  Moving this
-		 * into the OPEN/STATUS switch fixes this problem.
+		 * code into the OPEN/STATUS switch fixes this problem.
 		 */
 		iov.iov_base = rbuf;
 		iov.iov_len = 8 + getstatus( pr, &rbuf[ 8 ] );
@@ -400,27 +495,31 @@ main( ac, av )
 		if ( atp_sresp( pr->p_atp, &atpb ) < 0 ) {
 		    syslog( LOG_ERR, "atp_sresp: %m" );
 		}
-#endif notdef
+#endif /* notdef */
 	    }
 	}
     }
+    return 0;
 }
 
 /*
  * We assume buf is big enough for 255 bytes of data and a length byte.
  */
-    int
-getstatus( pr, buf )
+
+int getstatus( pr, buf )
     struct printer	*pr;
     char		*buf;
 {
     char		path[ MAXPATHLEN ];
-    int			fd, rc;
+    int			fd = -1, rc;
 
-    strcpy( path, pr->p_spool );
-    strcat( path, "/status" );
+    if ( pr->p_flags & P_SPOOLED && ( pr->p_spool != NULL )) {
+	strcpy( path, pr->p_spool );
+	strcat( path, "/status" );
+	fd = open( path, O_RDONLY);
+    }
 
-    if (( pr->p_flags & P_PIPED ) || ( fd = open( path, O_RDONLY, 0 )) < 0 ) {
+    if (( pr->p_flags & P_PIPED ) || ( fd < 0 )) {
 	*buf = strlen( cannedstatus );
 	strncpy( &buf[ 1 ], cannedstatus, *buf );
 	return( *buf + 1 );
@@ -429,6 +528,9 @@ getstatus( pr, buf )
 	    rc = 0;
 	}
 	close( fd );
+	if ( rc && buf[ rc ] == '\n' ) {	/* remove trailing newline */
+	    rc--;
+	}
 	*buf = rc;
 	return( rc + 1 );
     }
@@ -437,7 +539,7 @@ getstatus( pr, buf )
 char	*pgetstr();
 char	*getpname();
 
-getprinters( cf )
+void getprinters( cf )
     char	*cf;
 {
     char		buf[ 1024 ], area[ 1024 ], *a, *p, *name, *type, *zone;
@@ -459,7 +561,7 @@ getprinters( cf )
 	    perror( "malloc" );
 	    exit( 1 );
 	}
-	bzero( pr, sizeof( struct printer ));
+	memset( pr, 0, sizeof( struct printer ));
 
 	name = defprinter.p_name;
 	type = defprinter.p_type;
@@ -519,7 +621,14 @@ getprinters( cf )
 	 */
 	if (( p = pgetstr( "pr", &a )) == NULL ) {
 	    pr->p_printer = defprinter.p_printer;
+	    pr->p_flags = defprinter.p_flags;
 	} else {
+	    if ( *p == '|' ) {
+		p++;
+		pr->p_flags = P_PIPED;
+	    } else {
+		pr->p_flags = P_SPOOLED;
+	    }
 	    if (( pr->p_printer = (char *)malloc( strlen( p ) + 1 )) == NULL ) {
 		perror( "malloc" );
 		exit( 1 );
@@ -527,10 +636,30 @@ getprinters( cf )
 	    strcpy( pr->p_printer, p );
 	}
 
-	if ( *pr->p_printer == '|' ) {
-	    pr->p_flags = P_PIPED;
-	} else {
-	    pr->p_flags = P_SPOOLED;
+	/*
+	 * Do we want authenticated printing?
+	 */
+	if ((p = pgetstr( "ca", &a )) != NULL ) {
+	    if ((pr->p_authprintdir = (char *)malloc(strlen(p)+1)) == NULL) {
+		perror( "malloc" );
+		exit(1);
+	    }
+	    strcpy( pr->p_authprintdir, p );
+	    pr->p_flags |= P_AUTH;
+	    pr->p_flags |= P_AUTH_CAP;
+	} else { pr->p_authprintdir = NULL; }
+
+	if ( pgetflag( "sp" ) == 1 ) {
+	    pr->p_flags |= P_AUTH;
+	    pr->p_flags |= P_AUTH_PSSP;
+	}
+
+	if ((p = pgetstr("am", &a)) != NULL ) {
+		if ((uamlist = (char *)malloc(strlen(p)+1)) == NULL ) {
+			perror("malloc");
+			exit(1);
+		}
+		strcpy(uamlist, p);
 	}
 
 	if ( pr->p_flags & P_SPOOLED ) {
@@ -549,6 +678,12 @@ getprinters( cf )
 	    }
 	}
 
+	/* get printer's appletalk address. */
+	if (( p = pgetstr( "pa", &a )) == NULL ) 
+	    memcpy(&pr->p_addr, &defprinter.p_addr, sizeof(pr->p_addr));
+	else 
+	    atalk_aton(p, &pr->p_addr);
+
 	pr->p_next = printers;
 	printers = pr;
     }
@@ -559,7 +694,7 @@ getprinters( cf )
     }
 }
 
-rprintcap( pr )
+int rprintcap( pr )
     struct printer	*pr;
 {
     char		buf[ 1024 ], area[ 1024 ], *a, *p;
@@ -599,8 +734,31 @@ rprintcap( pr )
 	    pr->p_flags &= ~P_ACCOUNT;
 	} else {
 	    pr->p_flags |= P_ACCOUNT;
+#ifdef ABS_PRINT
+	    if ( pr->p_role != NULL && pr->p_role != defprinter.p_role ) {
+		free( pr->p_role );
+	    }
+	    a = area;
+	    if (( p = pgetstr( "ro", &a )) == NULL ) {
+		pr->p_role = defprinter.p_role;
+	    } else {
+		if (( pr->p_role =
+			(char *)malloc( strlen( p ) + 1 )) == NULL ) {
+		    syslog( LOG_ERR, "malloc: %m" );
+		    exit( 1 );
+		}
+		strcpy( pr->p_role, p );
+	    }
+
+	    if (( c = pgetnum( "si" )) < 0 ) {
+		pr->p_srvid = defprinter.p_srvid;
+	    } else {
+		pr->p_srvid = c;
+	    }
+#endif /* ABS_PRINT */
 	}
 
+
 	/*
 	 * Cost of printer.
 	 */
@@ -619,7 +777,7 @@ rprintcap( pr )
 	    pr->p_pagecost = 0;
 	} else if ( pr->p_flags & P_ACCOUNT ) {
 	    if (( c = pgetnum( "pc" )) < 0 ) {
-		pr->p_pagecost = 200;		/* default cost */
+		pr->p_pagecost = defprinter.p_pagecost;
 	    } else {
 		pr->p_pagecost = c;
 	    }
@@ -643,14 +801,16 @@ rprintcap( pr )
 	    strcpy( pr->p_lock, p );
 	}
 
+#ifdef KRB
 	/*
 	 * Must Kerberos authenticate?
 	 */
 	if ( pgetflag( "ka" ) == 1 ) {
-	    pr->p_flags |= P_AUTH;
+	    pr->p_flags |= P_KRB;
 	} else {
-	    pr->p_flags &= ~P_AUTH;
+	    pr->p_flags &= ~P_KRB;
 	}
+#endif /* KRB */
 
 	endprent();
     }
