@@ -1,9 +1,9 @@
 /*
+ * $Id: printcap.c,v 1.5.2.1 2002/02/07 23:59:33 srittau Exp $
+ *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
- */
-
-/*
+ *
  * Copyright (c) 1983 Regents of the University of California.
  * All rights reserved.
  *
@@ -36,17 +36,32 @@
  * SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #ifndef lint
 static char sccsid[] = "@(#)printcap.c	5.7 (Berkeley) 3/4/91";
 #endif /* not lint */
 
 #include <ctype.h>
 #include <stdio.h>
+#include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#include <sys/types.h>
+#include <sys/stat.h>
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
 #include <atalk/paths.h>
 
+#include "printcap.h"
+
 #ifndef BUFSIZ
 #define	BUFSIZ	1024
-#endif
+#endif /* ! BUFSIZ */
 #define MAXHOP	32	/* max number of tc= indirections */
 
 /*
@@ -76,14 +91,14 @@ static char sccsid[] = "@(#)printcap.c	5.7 (Berkeley) 3/4/91";
 #define tnchktc	pnchktc
 #define	tnamatch pnamatch
 #define V6
-#endif
+#endif /* PRINTCAP */
 
 static	FILE *pfp = NULL;	/* printcap data base file pointer */
 static	char *tbuf;
 static	int hopcount;		/* detect infinite loops in termcap, init 0 */
-char	*tskip();
+static char	*tskip();
 char	*tgetstr();
-char	*tdecode();
+static char	*tdecode();
 char	*getenv();
 
 /*
@@ -93,7 +108,7 @@ char	*getenv();
  * Added a "cap" parameter, so we can use these calls for printcap
  * and papd.conf.
  */
-getprent( cap, bp)
+int getprent( cap, bp)
 	register char *cap;
 	register char *bp;
 {
@@ -135,7 +150,7 @@ getprent( cap, bp)
 	}
 }
 
-endprent()
+void endprent()
 {
 	if (pfp != NULL)
 		fclose(pfp);
@@ -149,16 +164,17 @@ endprent()
  * Added a "cap" parameter, so we can use these calls for printcap
  * and papd.conf.
  */
-tgetent( cap, bp, name)
+int tgetent( cap, bp, name)
 	char *cap, *bp, *name;
 {
 	register char *cp;
 	register int c;
 	register int i = 0, cnt = 0;
 	char ibuf[BUFSIZ];
-	char *cp2;
 	int tf;
+	int skip;
 
+	hopcount = 0;
 	tbuf = bp;
 	tf = 0;
 #ifndef V6
@@ -175,7 +191,7 @@ tgetent( cap, bp, name)
 			cp2 = getenv("TERM");
 			if (cp2==(char *) 0 || strcmp(name,cp2)==0) {
 				strcpy(bp,cp);
-				return(tnchktc());
+				return(tnchktc(cap));
 			} else {
 				tf = open(cap, 0);
 			}
@@ -184,13 +200,14 @@ tgetent( cap, bp, name)
 	}
 	if (tf==0)
 		tf = open(cap, 0);
-#else
+#else /* V6 */
 	tf = open(cap, 0);
-#endif
+#endif /* V6 */
 	if (tf < 0)
 		return (-1);
 	for (;;) {
 		cp = bp;
+		skip = 0;
 		for (;;) {
 			if (i == cnt) {
 				cnt = read(tf, ibuf, BUFSIZ);
@@ -202,12 +219,20 @@ tgetent( cap, bp, name)
 			}
 			c = ibuf[i++];
 			if (c == '\n') {
-				if (cp > bp && cp[-1] == '\\'){
+				if (!skip && cp > bp && cp[-1] == '\\') {
 					cp--;
 					continue;
 				}
-				break;
+				skip = 0;
+				if (cp == bp)
+					continue;
+				else
+					break;
 			}
+			if (c == '#' && cp == bp)
+				skip++;
+			if (skip)
+				continue;
 			if (cp >= bp+BUFSIZ) {
 				write(2,"Termcap entry too long\n", 23);
 				break;
@@ -221,7 +246,7 @@ tgetent( cap, bp, name)
 		 */
 		if (tnamatch(name)) {
 			close(tf);
-			return(tnchktc());
+			return(tnchktc(cap));
 		}
 	}
 }
@@ -236,7 +261,7 @@ tgetent( cap, bp, name)
  * Added a "cap" parameter, so we can use these calls for printcap
  * and papd.conf.
  */
-tnchktc( cap )
+int tnchktc( cap )
     char *cap;
 {
 	register char *p, *q;
@@ -284,7 +309,7 @@ tnchktc( cap )
  * against each such name.  The normal : terminator after the last
  * name (before the first field) stops us.
  */
-tnamatch(np)
+int tnamatch(np)
 	char *np;
 {
 	register char *Np, *Bp;
@@ -330,7 +355,7 @@ tskip(bp)
  * a # character.  If the option is not found we return -1.
  * Note that we handle octal numbers beginning with 0.
  */
-tgetnum(id)
+int tgetnum(id)
 	char *id;
 {
 	register int i, base;
@@ -363,7 +388,7 @@ tgetnum(id)
  * of the buffer.  Return 1 if we find the option, or 0 if it is
  * not given.
  */
-tgetflag(id)
+int tgetflag(id)
 	char *id;
 {
 	register char *bp = tbuf;
