@@ -1,9 +1,9 @@
 /*
+ * $Id: psf.c,v 1.5.2.2 2002/03/12 16:04:25 srittau Exp $
+ *
  * Copyright (c) 1990,1995 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
- */
-
-/*
+ *
  * PostScript Filter, psf.
  *
  * Handles both PostScript files and text files.  Files with the
@@ -17,17 +17,43 @@
  * a troff or dvi to PostScript converter.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #define FUCKED
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#include <sys/time.h>
+
+/* POSIX.1 sys/wait.h check */
 #include <sys/types.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif /* HAVE_SYS_WAIT_H */
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
+#endif /* ! WEXITSTATUS */
+#ifndef WIFEXITED
+#define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
+#endif /* ! WIFEXITED */
+
 #include <sys/file.h>
-#include <sys/syslog.h>
+#include <syslog.h>
 #include <atalk/paths.h>
 #include <stdio.h>
-#include <strings.h>
+#include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <signal.h>
+#include <errno.h>
+
+/* Forward Declarations */
+int pexecv(char *path, char *argv[]);
+int copyio();
+int textps();
 
 char		psapath[] = _PATH_PSA;
 char		*psaargv[] = { "psa", 0, 0, 0, 0 };
@@ -52,25 +78,32 @@ int		inlen;
 
 FILE		*acctfile = NULL;
 int		literal;
-#ifdef USEA4
-int		width = 80, length = 70, indent = 0;
-#else USEA4
 int		width = 80, length = 66, indent = 0;
-#endif USEA4
 char		*prog, *name, *host;
 
-main( ac, av ) 
+struct papersize {
+    int		width;
+    int		length;
+   float	win;
+    float	lin;
+} papersizes[] = {
+    { 80, 66, 8.5, 11.0 },			/* US Letter */
+    { 80, 70, 8.27, 11.69 },			/* A4 */
+};
+
+int main( ac, av ) 
     int		ac;
     char	**av;
 {
-    int			c, rc, i, children = 0;
+    int			c, rc, children = 0;
 #ifdef FUCKED
-    int			psafileno, multiconn = 0, waitidle = 0;
-#endif FUCKED
-    union wait		status;
+    int			psafileno, multiconn = 0, waitidle = 0, waitidle2 = 0;
+#endif /* FUCKED */
+    int			status;
     extern char		*optarg;
-    extern int		optind;
+    extern int		optind, opterr;
 
+    opterr = 0;
     if (( prog = rindex( av[ 0 ], '/' )) == NULL ) {
 	prog = av[ 0 ];
     } else {
@@ -78,11 +111,11 @@ main( ac, av )
     }
 #ifdef ultrix
     openlog( prog, LOG_PID );
-#else ultrix
+#else /* ultrix */
     openlog( prog, LOG_PID, LOG_LPR );
-#endif ultrix
+#endif /* ultrix */
 
-    while (( c = getopt( ac, av, "P:C:D:x:y:n:h:w:l:i:c" )) != EOF ) {
+    while (( c = getopt( ac, av, "P:C:D:F:L:J:x:y:n:h:w:l:i:c" )) != EOF ) {
 	switch ( c ) {
 	case 'n' :
 	    name = optarg;
@@ -101,7 +134,7 @@ main( ac, av )
 	    if ( width == 0 ) {
 		width = 80;
 	    }
-#endif ZEROWIDTH
+#endif /* ZEROWIDTH */
 	    break;
 
 	case 'l' :
@@ -116,6 +149,10 @@ main( ac, av )
 	    literal++;
 	    break;
 
+	case 'F' :
+	case 'L' :
+	case 'J' :
+	case 'P' :
 	case 'x' :
 	case 'y' :
 	    break;
@@ -124,7 +161,7 @@ main( ac, av )
 	default :
 	    syslog( LOG_ERR, "bad option: %c", c );
 	    exit( 2 );
-#endif notdef
+#endif /* notdef */
 	}
     }
     if ( ac - optind > 1 ) {
@@ -135,16 +172,19 @@ main( ac, av )
     if ( index( prog, 'w' )) {
 	waitidle++;
     }
+    if ( index( prog, 'W' )) {
+	waitidle2++;
+    }
     if ( index( prog, 'm' )) {
 	multiconn++;
     }
-#endif FUCKED
+#endif /* FUCKED */
 
     syslog( LOG_INFO, "starting for %s", name ? name : "?" );
 
 restart:
     if (( inlen = read( 0, inbuf, sizeof( inbuf ))) < 0 ) {
-	syslog( LOG_ERR, "read: %m" );
+	syslog( LOG_ERR, "read: %s", strerror(errno) );
 	exit( 1 );
     }
     if ( inlen == 0 ) {	/* nothing to be done */
@@ -162,7 +202,7 @@ restart:
 	psaargv[ 2 ] = name;
 	psaargv[ 3 ] = host;
 	if (( c = pexecv( psapath, psaargv )) < 0 ) {
-	    syslog( LOG_ERR, "%s: %m", psapath );
+	    syslog( LOG_ERR, "%s: %s", psapath, strerror(errno) );
 	    exit( 2 );
 	}
 	children++;
@@ -180,7 +220,14 @@ restart:
 		psafileno--;
 		dup2( 1, psafileno );
 
-		if ( waitidle ) {
+		if ( waitidle2 ) {
+		    papargv[ 2 ] = "-W";
+		    papargv[ 3 ] = "-c";
+		    papargv[ 4 ] = "-E";
+		    papargv[ 5 ] = _PATH_PAGECOUNT;
+		    papargv[ 6 ] = "-";
+		    papargv[ 7 ] = 0;
+		} else if ( waitidle ) {
 		    papargv[ 2 ] = "-w";
 		    papargv[ 3 ] = "-c";
 		    papargv[ 4 ] = "-E";
@@ -204,7 +251,7 @@ restart:
 		papargv[ 5 ] = _PATH_PAGECOUNT;
 		papargv[ 6 ] = 0;
 	    }
-#endif FUCKED
+#endif /* FUCKED */
 	} else {
 	    papargv[ 2 ] = "-c";
 	    papargv[ 3 ] = "-E";
@@ -212,7 +259,7 @@ restart:
 	}
 
 	if (( c = pexecv( pappath, papargv )) < 0 ) {
-	    syslog( LOG_ERR, "%s: %m", pappath );
+	    syslog( LOG_ERR, "%s: %s", pappath, strerror(errno) );
 	    exit( 2 );
 	}
 	children++;
@@ -225,7 +272,7 @@ restart:
      */
     if ( strstr( prog, "rev" ) != NULL ) {
 	if (( c = pexecv( revpath, revargv )) < 0 ) {
-	    syslog( LOG_ERR, "%s: %m", revpath );
+	    syslog( LOG_ERR, "%s: %s", revpath, strerror(errno) );
 	    exit( 2 );
 	}
 	syslog( LOG_INFO, "sending to rev[%d]", c );
@@ -239,7 +286,7 @@ restart:
     if ( *prog != 'i' && *prog != 'o' && *( prog + 1 ) == 'f' ) {
 	filtargv[ 0 ] = filtargv[ 1 ] = prog;
 	if (( c = pexecv( _PATH_PSFILTER, filtargv )) < 0 ) {
-	    syslog( LOG_ERR, "%s: %m", _PATH_PSFILTER );
+	    syslog( LOG_ERR, "%s: %s", _PATH_PSFILTER, strerror(errno) );
 	    exit( 2 );
 	}
 	syslog( LOG_INFO, "external filter[%d]", c );
@@ -249,6 +296,9 @@ restart:
 	if ( inlen >= 2 && inbuf[ 0 ] == '%' && inbuf[ 1 ] == '!' ) {
 	    syslog( LOG_INFO, "PostScript" );
 	    rc = copyio();	/* PostScript */
+	} else if ( inlen >= 2 && inbuf[ 0 ] == '\033' && inbuf[ 1 ] == '%' ) {
+	    syslog( LOG_INFO, "PostScript w/PJL" );
+	    rc = copyio();      /* PostScript */
 	} else {
 	    syslog( LOG_INFO, "straight text" );
 	    rc = textps();	/* straight text */
@@ -260,7 +310,11 @@ restart:
 	dup2( psafileno, 1 );
 	close( psafileno );
 	papargv[ 2 ] = "-c";
-	if ( waitidle ) {
+	if ( waitidle2 ) {
+	    papargv[ 3 ] = "-W";
+	    papargv[ 4 ] = _PATH_PAGECOUNT;
+	    papargv[ 5 ] = 0;
+	} else if ( waitidle ) {
 	    papargv[ 3 ] = "-w";
 	    papargv[ 4 ] = _PATH_PAGECOUNT;
 	    papargv[ 5 ] = 0;
@@ -270,26 +324,26 @@ restart:
 	}
 
 	if (( c = pexecv( pappath, papargv )) < 0 ) {
-	    syslog( LOG_ERR, "%s: %m", pappath );
+	    syslog( LOG_ERR, "%s: %s", pappath, strerror(errno) );
 	    exit( 2 );
 	}
 	children++;
 	syslog( LOG_INFO, "pagecount with pap[%d]", c );
     }
-#endif FUCKED
+#endif /* FUCKED */
 
     if ( children ) {
 	close( 1 );
     }
     while ( children ) {
 	if (( c = wait3( &status, 0, 0 )) < 0 ) {
-	    syslog( LOG_ERR, "wait3: %m" );
+	    syslog( LOG_ERR, "wait3: %s", strerror(errno) );
 	    exit( 1 );
 	}
 	if ( WIFEXITED( status )) {
 #ifndef WEXITSTATUS
 #define WEXITSTATUS(x)	((x).w_status)
-#endif WEXITSTATUS
+#endif /* WEXITSTATUS */
 	    if ( WEXITSTATUS( status ) != 0 ) {
 		syslog( LOG_ERR, "%d died with %d", c, WEXITSTATUS( status ));
 		exit( WEXITSTATUS( status ));
@@ -314,28 +368,100 @@ restart:
     exit( rc );
 }
 
-copyio()
+int copyio()
 {
+    /* implement the FSM needed to do the suspend. Note that
+     * the last characters will be \031\001 so don't worry
+     * Fun things: 1. \031\001 should not be written to output device
+     *  2. The \031 can be last char of one read, \001 first of next
+     *      - we need to write \031 if not followed by \001
+     */
+    struct timeval	tv;
+    fd_set		fdset;
+    int			ctl = 0;
+
+notdone:
     do {
-	if ( write( 1, inbuf, inlen ) != inlen ) {
-	    syslog( LOG_ERR, "write: %m" );
+	/*
+	 * First, \031 and \001 *must* be the last things in the buffer
+	 * (\001 can be the first thing in the next buffer).  There's no
+	 * need to scan any of the intervening bytes.  Second, if there's
+	 * more input, the escape sequence was bogus, and we should keep
+	 * reading.
+	 */
+	if ( inlen == 1 ) {
+	    if ( ctl == 1 ) {
+		if ( inbuf[ 0 ] == '\001' ) {
+		    ctl = 2;
+		    break;
+		}
+		if ( write( 1, "\031", 1 ) != 1 ) {
+		    syslog( LOG_ERR, "write: %s", strerror(errno) );
+		    return( 1 );
+		}
+		ctl = 0;
+	    }
+
+	    if ( inbuf[ 0 ] == '\031' ) {
+		ctl = 1;
+	    }
+
+	} else {
+	    if ( ctl == 1 ) {
+		if ( write( 1, "\031", 1 ) != 1 ) {
+		    syslog( LOG_ERR, "write: %s", strerror(errno) );
+		    return( 1 );
+		}
+	    }
+	    ctl = 0;
+	    if ( inbuf[ inlen - 2 ] == '\031' &&
+		    inbuf[ inlen - 1 ] == '\001' ) {
+		ctl = 2;
+	    } else if ( inbuf[ inlen - 1 ] == '\031' ) {
+		ctl = 1;
+	    }
+	}
+
+	inlen -= ctl;
+	if (( inlen > 0 ) && ( write( 1, inbuf, inlen ) != inlen )) {
+	    syslog( LOG_ERR, "write: %s", strerror(errno) );
 	    return( 1 );
 	}
+	if ( ctl == 2 ) {
+	    break;
+	}
     } while (( inlen = read( 0, inbuf, sizeof( inbuf ))) > 0 );
+
+    if ( ctl == 2 ) {
+	tv.tv_sec = 2;
+	tv.tv_usec = 0;
+	FD_ZERO( &fdset );
+	FD_SET( 0, &fdset );
+	if ( select( 1, &fdset, NULL, NULL, &tv ) != 0 ) {
+	    if (( inlen = read( 0, inbuf, sizeof( inbuf ))) > 0 ) {
+		goto notdone;
+	    }
+	}
+    }
+
     if ( inlen < 0 ) {
-	syslog( LOG_ERR, "read: %m" );
+	syslog( LOG_ERR, "read: %s", strerror(errno) );
 	return( 1 );
     }
+
+    if ( ctl == 1 ) {
+	if ( write( 1, "\031", 1 ) != 1 ) {
+	    syslog( LOG_ERR, "write: %s", strerror(errno) );
+	    return( 1 );
+	}
+    } else if ( ctl == 2 ) {
+	return( 3 );
+    }
     return( 0 );
 }
 
 char		*font = "Courier";
 int		point = 11;
-#ifdef USEA4
-float		win = 8.27, hin = 11.69;
-#else USEA4
-float		win = 8.5, hin = 11;
-#endif USEA4
 
 char		pspro[] = "\
 /GSV save def						% global VM\n\
@@ -344,9 +470,8 @@ char		pspro[] = "\
 	dup /H exch def					% save font height\n\
 	exch findfont exch scalefont setfont		% select font\n\
 	( ) stringwidth pop /W exch def			% save font width\n\
-	72 mul exch 2 sub H mul add 2 div /CY exch def	% save start Y\n\
-	72 mul exch 1 add W mul sub 2 div\n\
-		exch W mul 2 div add /CX exch def	% save start X\n\
+	0.5 sub 72 mul /CY exch def			% save start Y\n\
+	pop 0.5 add 72 mul /CX exch def			% save start X\n\
 	CX CY moveto					% make current point\n\
 } bind def\n\
 /S /show load def\n\
@@ -357,10 +482,23 @@ char		pspro[] = "\
 /EP { SV restore showpage } bind def\n\
 %%EndProlog\n";
 
-textps()
+int textps()
 {
-    int		state = 0, line = 0, col = 0, npages = 0, rc;
-    char	*p, *end;
+    struct papersize	papersize;
+    int			state = 0, line = 0, col = 0, npages = 0, rc, i;
+    char		*p, *end;
+
+#define elements(x)	(sizeof(x)/sizeof((x)[0]))
+    for ( i = 0; i < elements( papersizes ); i++ ) {
+	if ( width == papersizes[ 0 ].width &&
+		length == papersizes[ 0 ].length ) {
+	    papersize = papersizes[ i ];
+	    break;
+	}
+    }
+    if ( i >= elements( papersizes )) {
+	papersize = papersizes[ 0 ];		/* default */
+    }
 
 #define ST_AVAIL		(1<<0)
 #define ST_CONTROL		(1<<1)
@@ -383,7 +521,7 @@ textps()
 		    /* output postscript prologue: */
 		    if ( write( 1, pspro, sizeof( pspro ) - 1 ) !=
 			    sizeof( pspro ) - 1 ) {
-			syslog( LOG_ERR, "write prologue: %m" );
+			syslog( LOG_ERR, "write prologue: %s", strerror(errno) );
 			return( 1 );
 		    }
 		    if ( name && host ) {
@@ -393,8 +531,8 @@ textps()
 		}
 
 		printf( "%%%%Page: ? %d\n", ++npages );
-		printf( "%d %d %f %d %f /%s %d SP\n",
-			indent, width, win, length, hin, font, point );
+		printf( "%d %f %f /%s %d SP\n", indent,
+			papersize.win, papersize.lin, font, point );
 		state |= ST_PAGE;
 	    }
 	    if ( state & ST_CONTROL && *p != '\001' ) {
@@ -497,7 +635,7 @@ textps()
 				(unsigned char)*p );
 			return( 2 );	/* Toss job */
 		    }
-		    printf( "\\%o", (unsigned)*p );
+		    printf( "\\%o", (unsigned char)*p );
 		} else {
 		    putchar( *p );
 		}
@@ -508,7 +646,7 @@ textps()
 	}
     } while (( inlen = read( 0, inbuf, sizeof( inbuf ))) > 0 );
     if ( inlen < 0 ) {
-	syslog( LOG_ERR, "read: %m" );
+	syslog( LOG_ERR, "read: %s", strerror(errno) );
 	return( 1 );
     }
     rc = 0;
@@ -538,7 +676,7 @@ out:
  * Manipulates file descriptors 0, 1, and 2, such that the new child
  * is reading from the parent's output.
  */
-pexecv( path, argv )
+int pexecv( path, argv )
     char	*path, *argv[];
 {
     int		fd[ 2 ], c;
@@ -547,7 +685,7 @@ pexecv( path, argv )
 	return( -1 );
     }
 
-    switch ( c = vfork()) {
+    switch ( c = fork()) {
     case -1 :
 	return( -1 );
 	/* NOTREACHED */
