@@ -1,43 +1,75 @@
 /*
- * Copyright (c) 1990,1991 Regents of The University of Michigan.
- * All Rights Reserved.
+ * $Id: atp_rsel.c,v 1.3.2.1 2002/02/08 00:03:27 srittau Exp $
  *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appears in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of The University
- * of Michigan not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior
- * permission. This software is supplied as is without expressed or
- * implied warranties of any kind.
- *
- *	Research Systems Unix Group
- *	The University of Michigan
- *	c/o Mike Clark
- *	535 W. William Street
- *	Ann Arbor, Michigan
- *	+1-313-763-0525
- *	netatalk@itd.umich.edu
+ * Copyright (c) 1990,1997 Regents of The University of Michigan.
+ * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <string.h>
 #include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/time.h>
-#include <sys/errno.h>
 #include <sys/uio.h>
 #include <signal.h>
+#include <errno.h>
 
+#include <netatalk/endian.h>
 #include <netatalk/at.h>
+
+#include <atalk/netddp.h>
+#include <atalk/compat.h>
 #include <atalk/atp.h>
+#include <atalk/util.h>
+
 #include "atp_internals.h"
 
-extern int	errno;
-
-static int	got_alarm = 0;
 #ifdef DROP_ATPTREL
 static int	release_count = 0;
-#endif DROP_ATPTREL
+#endif /* DROP_ATPTREL */
+
+
+int
+resend_request( ah )
+    ATP		ah;
+{
+    /*
+     * update bitmap and send request packet
+     */
+    struct atphdr	req_hdr;
+
+#ifdef EBUG
+    printf( "\n<%d> resend_request: resending %ld byte request packet",
+	    getpid(), ah->atph_reqpkt->atpbuf_dlen );
+    atp_print_addr( " to", &ah->atph_reqpkt->atpbuf_addr );
+    putchar( '\n' );
+    bprint( ah->atph_reqpkt->atpbuf_info.atpbuf_data,
+	    ah->atph_reqpkt->atpbuf_dlen );
+#endif /* EBUG */
+
+    memcpy( &req_hdr, ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, 
+	sizeof( struct atphdr ));
+    req_hdr.atphd_bitmap = ah->atph_rbitmap;
+    memcpy( ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, &req_hdr, 
+	    sizeof( struct atphdr ));
+
+    gettimeofday( &ah->atph_reqtv, (struct timezone *)0 );
+    if ( netddp_sendto( ah->atph_socket,
+	    ah->atph_reqpkt->atpbuf_info.atpbuf_data,
+	    ah->atph_reqpkt->atpbuf_dlen, 0,
+	    (struct sockaddr *) &ah->atph_reqpkt->atpbuf_addr,
+	    sizeof( struct sockaddr_at )) != ah->atph_reqpkt->atpbuf_dlen ) {
+	return( -1 );
+    }
+
+    if ( ah->atph_reqtries > 0 ) {
+	--(ah->atph_reqtries);
+    }
+
+    return( 0 );
+}
 
 int
 atp_rsel( ah, faddr, func )
@@ -48,15 +80,16 @@ atp_rsel( ah, faddr, func )
 {
     struct atpbuf	*abuf, *pb, *cb;
     struct atphdr	req_hdr, resp_hdr;
-    int			i, recvlen, requesting, mask;
-    u_char		rfunc;
-    u_short		tid;
+    fd_set		fds;
+    int			i, recvlen, requesting, mask, c;
+    u_int8_t		rfunc;
+    u_int16_t		tid;
     struct timeval	tv;
     struct sockaddr_at	saddr;
 
 #ifdef EBUG
-    print_bufuse( ah, "atp_rsel at top" );
-#endif
+    atp_print_bufuse( ah, "atp_rsel at top" );
+#endif /* EBUG */
     if ( func == 0 ) {
 	func = ATP_FUNCANY;
     }
@@ -71,42 +104,54 @@ atp_rsel( ah, faddr, func )
 	return( ATP_TRESP );
     }
 
-    if (( abuf = alloc_buf()) == NULL ) {
+    if (( abuf = atp_alloc_buf()) == NULL ) {
 	return( -1 );
     }
 
     if ( requesting ) {
 #ifdef EBUG
 	printf( "<%d> atp_rsel: request pending\n", getpid());
-#endif
+#endif /* EBUG */
 	gettimeofday( &tv, (struct timezone *)0 );
 	if ( tv.tv_sec - ah->atph_reqtv.tv_sec > ah->atph_reqto ) {
 	    if ( resend_request( ah ) < 0 ) {
-		free_buf( abuf );
+		atp_free_buf( abuf );
 		return( -1 );
 	    }
 	}
-	save_timer();
-	set_alrm_catcher();
     }
 
     for ( ;; ) {
 	rfunc = func;
 	if ( requesting ) {
-	    set_timer( ah->atph_reqto );
+	    FD_ZERO( &fds );
+	    FD_SET( ah->atph_socket, &fds );
+	    tv.tv_sec = ah->atph_reqto;
+	    tv.tv_usec = 0;
+	    if (( c = select( ah->atph_socket + 1, &fds, NULL, NULL,
+		    &tv )) < 0 ) {
+		atp_free_buf( abuf );
+		return( -1 );
+	    }
+	    if ( c == 0 || FD_ISSET( ah->atph_socket, &fds ) == 0 ) {
+		recvlen = -1;
+		errno = EINTR;
+		goto timeout;
+	    }
 	}
-	bcopy( (char *)faddr, (char *)&saddr, sizeof( struct sockaddr_at ));
+	memcpy( &saddr, faddr, sizeof( struct sockaddr_at ));
 #ifdef EBUG
 	printf( "<%d> atp_rsel calling recv_atp,", getpid());
-	print_addr( " accepting from: ", &saddr );
+	atp_print_addr( " accepting from: ", &saddr );
 	putchar( '\n' );
-#endif EBUG
-	if (( recvlen = recv_atp( ah, &saddr, &rfunc, ATP_TIDANY,
+#endif /* EBUG */
+	if (( recvlen = atp_recv_atp( ah, &saddr, &rfunc, ATP_TIDANY,
 		abuf->atpbuf_info.atpbuf_data, 0 )) >= 0 ) {
 	    break;	/* we received something */
 	}
 
-	if ( !requesting || errno != EINTR || !got_alarm ) {
+timeout :
+	if ( !requesting || errno != EINTR ) {
 	    break;	/* error */
 	}
 	    
@@ -121,26 +166,20 @@ atp_rsel( ah, faddr, func )
 	}
     }
 
-    if ( requesting ) {
-	cancel_timer();
-	restore_timer();
-	reset_alrm_catcher();
-    }
-
     if ( recvlen <= 0 ) {	/* error */
-	free_buf( abuf );
+	atp_free_buf( abuf );
 	return( recvlen );
     }
 
 #ifdef EBUG
     printf( "<%d> atp_rsel: rcvd %d bytes", getpid(), recvlen );
-    print_addr( " from: ", &saddr );
+    atp_print_addr( " from: ", &saddr );
     putchar( '\n' );
     bprint( abuf->atpbuf_info.atpbuf_data, recvlen );
-#endif
+#endif /* EBUG */
 
-    abuf->atpbuf_dlen = recvlen;
-    bcopy( abuf->atpbuf_info.atpbuf_data + 1, (char *)&resp_hdr,
+    abuf->atpbuf_dlen = (size_t) recvlen;
+    memcpy( &resp_hdr, abuf->atpbuf_info.atpbuf_data + 1,
 	    sizeof( struct atphdr ));
 
     if ( rfunc == ATP_TREQ ) {
@@ -148,7 +187,7 @@ atp_rsel( ah, faddr, func )
 	 * we got a request: check to see if it is a duplicate (XO)
 	 * while we are at it, we expire old XO responses from sent list
 	 */
-	bcopy( abuf->atpbuf_info.atpbuf_data + 1, (char *)&req_hdr,
+	memcpy( &req_hdr, abuf->atpbuf_info.atpbuf_data + 1, 
 		sizeof( struct atphdr ));
 	tid = ntohs( req_hdr.atphd_tid );
 	gettimeofday( &tv, (struct timezone *)0 );
@@ -156,18 +195,18 @@ atp_rsel( ah, faddr, func )
 		pb = cb, cb = cb->atpbuf_next ) {
 #ifdef EBUG
 	    printf( "<%d>", getpid());
-	    print_addr( " examining", &cb->atpbuf_addr );
+	    atp_print_addr( " examining", &cb->atpbuf_addr );
 	    printf( " %hu", cb->atpbuf_info.atpbuf_xo.atpxo_tid );
-	    print_addr( " (looking for", &saddr );
+	    atp_print_addr( " (looking for", &saddr );
 	    printf( " %hu)\n", tid );
-#endif
+#endif /* EBUG */
 	    if ( tv.tv_sec - cb->atpbuf_info.atpbuf_xo.atpxo_tv.tv_sec
 		    > cb->atpbuf_info.atpbuf_xo.atpxo_reltime ) {
 		/* discard expired response */
 #ifdef EBUG
 		printf( "<%d> expiring tid %hu\n", getpid(),
 			cb->atpbuf_info.atpbuf_xo.atpxo_tid );
-#endif
+#endif /* EBUG */
 		if ( pb == NULL ) {
 		    ah->atph_sent = cb->atpbuf_next;
 		} else {
@@ -177,10 +216,10 @@ atp_rsel( ah, faddr, func )
 		for ( i = 0; i < 8; ++i ) {
 		    if ( cb->atpbuf_info.atpbuf_xo.atpxo_packet[ i ]
 			    != NULL ) {
-			free_buf( cb->atpbuf_info.atpbuf_xo.atpxo_packet[ i ] );
+			atp_free_buf( cb->atpbuf_info.atpbuf_xo.atpxo_packet[ i ] );
 		    }
 		}
-		free_buf( cb );
+		atp_free_buf( cb );
 
 		if (( cb = pb ) == NULL )
 		    break;
@@ -196,32 +235,29 @@ atp_rsel( ah, faddr, func )
 #ifdef EBUG
 	    printf( "<%d> duplicate request -- re-sending XO resp\n",
 		    getpid());
-#endif
+#endif /* EBUG */
 	    /* matches an old response -- just re-send and reset expire */
 	    cb->atpbuf_info.atpbuf_xo.atpxo_tv = tv;
 	    for ( i = 0; i < 8; ++i ) {
 		if ( cb->atpbuf_info.atpbuf_xo.atpxo_packet[i] != NULL &&
 			req_hdr.atphd_bitmap & ( 1 << i )) {
-		    sendto( ah->atph_socket,
+		    netddp_sendto( ah->atph_socket,
 			    cb->atpbuf_info.atpbuf_xo.atpxo_packet[i]->atpbuf_info.atpbuf_data,
 			    cb->atpbuf_info.atpbuf_xo.atpxo_packet[i]->atpbuf_dlen,
-			    0, (struct sockaddr *)&saddr,
-			    sizeof( struct sockaddr_at ));
+			    0, (struct sockaddr *) &saddr, sizeof( struct sockaddr_at));
 		}
 	    }
 	}
 
 	if ( cb == NULL ) {
 	    /* new request -- queue it and return */
-	    bcopy( (char *)&saddr, (char *)&abuf->atpbuf_addr,
-		    sizeof( struct sockaddr_at ));
-	    bcopy( (char *)&saddr, (char *)faddr,
-		    sizeof( struct sockaddr_at ));
+	    memcpy( &abuf->atpbuf_addr, &saddr, sizeof( struct sockaddr_at ));
+	    memcpy( faddr, &saddr, sizeof( struct sockaddr_at ));
 	    abuf->atpbuf_next = ah->atph_queue;
 	    ah->atph_queue = abuf;
 	    return( ATP_TREQ );
 	} else {
-	    free_buf( abuf );
+	    atp_free_buf( abuf );
 	    return( 0 );
 	}
     }
@@ -229,14 +265,14 @@ atp_rsel( ah, faddr, func )
     /*
      * we got a response: update bitmap
      */
-    bcopy( ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, (char *)&req_hdr,
+    memcpy( &req_hdr, ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, 
 	    sizeof( struct atphdr ));
     if ( requesting && ah->atph_rbitmap & ( 1<<resp_hdr.atphd_bitmap )
 		&& req_hdr.atphd_tid == resp_hdr.atphd_tid ) {
 	ah->atph_rbitmap &= ~( 1<<resp_hdr.atphd_bitmap );
 
 	if ( ah->atph_resppkt[ resp_hdr.atphd_bitmap ] != NULL ) {
-	    free_buf( ah->atph_resppkt[ resp_hdr.atphd_bitmap ] );
+	    atp_free_buf( ah->atph_resppkt[ resp_hdr.atphd_bitmap ] );
 	}
 	ah->atph_resppkt[ resp_hdr.atphd_bitmap ] = abuf;
 
@@ -246,9 +282,9 @@ atp_rsel( ah, faddr, func )
 #ifdef EBUG
 	    printf( "<%d> EOM -- seq num %d  current bitmap %d\n",
 		getpid(), resp_hdr.atphd_bitmap, ah->atph_rbitmap );
-#endif
+#endif /* EBUG */
 	    mask = 1 << resp_hdr.atphd_bitmap;
-	    ah->atph_rbitmap &= ( mask | mask-1 );
+	    ah->atph_rbitmap &= ( mask | (mask-1) );
 	}
 
 	/* if Send Trans. Status, send updated request
@@ -256,18 +292,17 @@ atp_rsel( ah, faddr, func )
 	if ( resp_hdr.atphd_ctrlinfo & ATP_STS ) {
 #ifdef EBUG
 	    puts( "STS" );
-#endif
+#endif /* EBUG */
 	    req_hdr.atphd_bitmap = ah->atph_rbitmap;
-	    bcopy( (char *)&req_hdr,
-		ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1,
-		sizeof( struct atphdr ));
-	    if ( sendto( ah->atph_socket,
+	    memcpy(ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1,
+		   &req_hdr, sizeof( struct atphdr ));
+	    if ( netddp_sendto( ah->atph_socket,
 		    ah->atph_reqpkt->atpbuf_info.atpbuf_data,
 		    ah->atph_reqpkt->atpbuf_dlen, 0,
 		    (struct sockaddr *) &ah->atph_reqpkt->atpbuf_addr,
 		    sizeof( struct sockaddr_at )) !=
 		    ah->atph_reqpkt->atpbuf_dlen ) {
-		free_buf( abuf );
+		atp_free_buf( abuf );
 		return( -1 );
 	    }
 	}
@@ -275,12 +310,12 @@ atp_rsel( ah, faddr, func )
 	/*
 	 * we are not expecting this response -- toss it
 	 */
-	free_buf( abuf );
+	atp_free_buf( abuf );
 #ifdef EBUG
 	printf( "atp_rsel: ignoring resp bm=%x tid=%d (expected %x/%d)\n",
 		resp_hdr.atphd_bitmap, ntohs( resp_hdr.atphd_tid ),
 		ah->atph_rbitmap, ah->atph_tid );
-#endif EBUG
+#endif /* EBUG */
     }
 
     if ( !ah->atph_rbitmap && ( req_hdr.atphd_ctrlinfo & ATP_XO )) {
@@ -289,24 +324,26 @@ atp_rsel( ah, faddr, func )
 	 * the release consists of DDP type byte + ATP header + 4 user bytes
 	 */
 	req_hdr.atphd_ctrlinfo = ATP_TREL;
-	bcopy( (char *)&req_hdr, ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1,
+	memcpy( ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, &req_hdr, 
 	    sizeof( struct atphdr ));
-	ah->atph_reqpkt->atpbuf_dlen = sizeof( struct atphdr ) + 5;
+	memset( ah->atph_reqpkt->atpbuf_info.atpbuf_data + ATP_HDRSIZE, 0, 4 );
+	ah->atph_reqpkt->atpbuf_dlen = sizeof( struct atphdr ) + ATP_HDRSIZE;
 #ifdef EBUG
 	printf( "<%d> sending TREL", getpid() );
 	bprint( ah->atph_reqpkt->atpbuf_info.atpbuf_data,
 		ah->atph_reqpkt->atpbuf_dlen );
-#endif
+#endif /* EBUG */
 #ifdef DROP_ATPTREL
 	if (( ++release_count % 10 ) != 0 ) {
-#endif DROP_ATPTREL
-	sendto( ah->atph_socket, ah->atph_reqpkt->atpbuf_info.atpbuf_data,
-		ah->atph_reqpkt->atpbuf_dlen, 0,
-		(struct sockaddr *) &ah->atph_reqpkt->atpbuf_addr,
-		sizeof( struct sockaddr_at));
+#endif /* DROP_ATPTREL */
+	netddp_sendto( ah->atph_socket, 
+		       ah->atph_reqpkt->atpbuf_info.atpbuf_data,
+		       ah->atph_reqpkt->atpbuf_dlen, 0,
+		       (struct sockaddr *) &ah->atph_reqpkt->atpbuf_addr,
+		       sizeof( struct sockaddr_at));
 #ifdef DROP_ATPTREL
 	}
-#endif DROP_ATPTREL
+#endif /* DROP_ATPTREL */
     }
 
     if ( ah->atph_rbitmap != 0 ) {
@@ -319,134 +356,6 @@ atp_rsel( ah, faddr, func )
 	}
     }
 
-    bcopy( (char *)&saddr, (char *)faddr, sizeof( struct sockaddr_at ));
+    memcpy( faddr, &saddr, sizeof( struct sockaddr_at ));
     return( ATP_TRESP );
 }
-
-
-#if !defined( ibm032 ) && !defined( _IBMR2 )
-    void
-#endif ibm032 _IBMR2
-catch_alrm()
-{
-    got_alarm = 1;
-}
-
-
-struct sigaction	old_alrm_sv;
-
-set_alrm_catcher()
-{
-    struct sigaction	sv;
-
-    sv.sa_handler = catch_alrm;
-    sv.sa_mask = 0;
-#ifdef ultrix
-    sv.sa_flags = 0;
-#else ultrix
-    sv.sa_flags = SA_INTERRUPT;
-#endif ultrix
-
-    if ( sigaction( SIGALRM, &sv, &old_alrm_sv ) < 0 ) {
-	perror( "sigaction in set_alrm_catcher" );
-	exit( -1 );
-    }
-}
-
-
-reset_alrm_catcher()
-{
-    if ( sigaction( SIGALRM, &old_alrm_sv, (struct sigaction *) 0 ) < 0 ) {
-	perror( "sigaction in reset_alrm_catcher" );
-	exit( -1 );
-    }
-}
-
-
-struct itimerval old_itv;
-
-#ifdef _IBMR2
-#define getitimer(a,b)		(setitimer( a, (struct itimerval *)0, b ))
-#endif _IBMR2
-
-save_timer()
-{
-    if ( getitimer( ITIMER_REAL, &old_itv ) < 0 ) {
-	perror( "getitimer" );
-	exit( -1 );
-    }
-}
-
-
-restore_timer()
-{
-    if ( setitimer( ITIMER_REAL, &old_itv, (struct itimerval *) 0 ) < 0 ) {
-	perror( "setitimer" );
-	exit( -1 );
-    }
-}
-
-set_timer( secs )
-    int		secs;
-{
-    struct itimerval itv;
-
-    itv.it_interval.tv_sec = 0L;
-    itv.it_interval.tv_usec = 0L;
-    itv.it_value.tv_sec = secs;
-    itv.it_value.tv_usec = 0L;
-
-    got_alarm = 0;
-    
-    if ( setitimer( ITIMER_REAL, &itv, (struct itimerval *) 0 ) < 0 ) {
-	perror( "setitimer" );
-	exit( -1 );
-    }
-}
-
-
-cancel_timer()
-{
-    set_timer( 0 );
-}
-
-
-int
-resend_request( ah )
-    ATP		ah;
-{
-    /*
-     * update bitmap and send request packet
-     */
-    struct atphdr	req_hdr;
-
-#ifdef EBUG
-    printf( "\n<%d> resend_request: resending %d byte request packet",
-	    getpid(), ah->atph_reqpkt->atpbuf_dlen );
-    print_addr( " to", &ah->atph_reqpkt->atpbuf_addr );
-    putchar( '\n' );
-    bprint( ah->atph_reqpkt->atpbuf_info.atpbuf_data,
-	    ah->atph_reqpkt->atpbuf_dlen );
-#endif
-
-    bcopy( ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1, (char *)&req_hdr,
-	sizeof( struct atphdr ));
-    req_hdr.atphd_bitmap = ah->atph_rbitmap;
-    bcopy( (char *)&req_hdr, ah->atph_reqpkt->atpbuf_info.atpbuf_data + 1,
-	    sizeof( struct atphdr ));
-
-    gettimeofday( &ah->atph_reqtv, (struct timezone *)0 );
-    if ( sendto( ah->atph_socket,
-	    ah->atph_reqpkt->atpbuf_info.atpbuf_data,
-	    ah->atph_reqpkt->atpbuf_dlen, 0,
-	    (struct sockaddr *)&ah->atph_reqpkt->atpbuf_addr,
-	    sizeof( struct sockaddr_at )) != ah->atph_reqpkt->atpbuf_dlen ) {
-	return( -1 );
-    }
-
-    if ( ah->atph_reqtries > 0 ) {
-	--(ah->atph_reqtries);
-    }
-
-    return( 0 );
-}
