@@ -1,25 +1,42 @@
 /*
+ * $Id: nbp_rgstr.c,v 1.3.2.1 2002/02/08 00:04:49 srittau Exp $
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <string.h>
+#include <errno.h>
+#include <signal.h>
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <sys/signal.h>
 #include <sys/time.h>
-#include <sys/errno.h>
+
 #include <netatalk/at.h>
 #include <netatalk/endian.h>
 #include <atalk/nbp.h>
 #include <atalk/ddp.h>
+#include <atalk/netddp.h>
 
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif /* HAVE_NETDB_H */
 #include  "nbp_conf.h"
 
-nbp_rgstr( sat, obj, type, zone )
+/* FIXME/SOCKLEN_T: socklen_t is a unix98 feature. */
+#ifndef SOCKLEN_T
+#define SOCKLEN_T unsigned int
+#endif /* ! SOCKLEN_T */
+
+int nbp_rgstr( sat, obj, type, zone )
     struct sockaddr_at	*sat;
-    char		*obj, *type, *zone;
+    const char		*obj, *type, *zone;
 {
     struct sockaddr_at	to;
     struct nbpnve	nn;
@@ -29,35 +46,37 @@ nbp_rgstr( sat, obj, type, zone )
     fd_set		readfd;
     struct servent	*se;
     char		*data;
-    int			s, cc, namelen;
-    extern int		errno;
+    int			s, cc;
+    SOCKLEN_T		namelen;
 
-    if ( nbp_lookup( obj, type, zone, &nn, 1 ) > 0 ) {
+    if ( nbp_lookup( obj, type, zone, &nn, 1, &sat->sat_addr ) > 0 ) {
+        errno = EADDRINUSE;
 	return( -1 );
     }
 
-    if (( s = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
-	return( -1 );
-    }
+    memset(&to, 0, sizeof(to));
+    if ((s = netddp_open(&to, NULL)) < 0)
+        return -1;
 
     data = nbp_send;
     *data++ = DDPTYPE_NBP;
     nh.nh_op = NBPOP_RGSTR;
     nh.nh_cnt = 1;
-    nh.nh_id = 0;
-    bcopy( &nh, data, SZ_NBPHDR );
+    nh.nh_id = ++nbp_id;
+    memcpy( data, &nh, SZ_NBPHDR );
     data += SZ_NBPHDR;
 
+    memset(&nt, 0, sizeof(nt));
     nt.nt_net = sat->sat_addr.s_net;
     nt.nt_node = sat->sat_addr.s_node;
     nt.nt_port = sat->sat_port;
-    bcopy( &nt, data, SZ_NBPTUPLE);
+    memcpy( data, &nt, SZ_NBPTUPLE);
     data += SZ_NBPTUPLE;
 
     if ( obj ) {
 	if (( cc = strlen( obj )) > NBPSTRLEN ) return( -1 );
 	*data++ = cc;
-	bcopy( obj, data, cc );
+	memcpy( data, obj, cc );
 	data += cc;
     } else {
 	*data++ = 0;
@@ -66,7 +85,7 @@ nbp_rgstr( sat, obj, type, zone )
     if ( type ) {
 	if (( cc = strlen( type )) > NBPSTRLEN ) return( -1 );
 	*data++ = cc;
-	bcopy( type, data, cc );
+	memcpy( data, type, cc );
 	data += cc;
     } else {
 	*data++ = 0;
@@ -75,16 +94,14 @@ nbp_rgstr( sat, obj, type, zone )
     if ( zone ) {
 	if (( cc = strlen( zone )) > NBPSTRLEN ) return( -1 );
 	*data++ = cc;
-	bcopy( zone, data, cc );
+	memcpy( data, zone, cc );
 	data += cc;
     } else {
-	*data++ = 0;
+	*data++ = 1;
+	*data++ = '*'; /* default zone */
     }
 
-    bzero( &to, sizeof( struct sockaddr_at ));
-    to.sat_family = AF_APPLETALK;;
-    to.sat_addr.s_net = ATADDR_ANYNET;
-    to.sat_addr.s_node = ATADDR_ANYNODE;
+    
     if ( nbp_port == 0 ) {
 	if (( se = getservbyname( "nbp", "ddp" )) == NULL ) {
 	    nbp_port = 2;
@@ -94,9 +111,10 @@ nbp_rgstr( sat, obj, type, zone )
     }
     to.sat_port = nbp_port;
 
-    if ( sendto( s, nbp_send, data - nbp_send, 0, &to,
-	    sizeof( struct sockaddr_at )) < 0 ) {
-	return( -1 );
+    if ( netddp_sendto( s, nbp_send, data - nbp_send, 0, 
+			(struct sockaddr *)&to,
+			sizeof( struct sockaddr_at )) < 0 ) {
+        goto register_err;
     }
 
     FD_ZERO( &readfd );
@@ -104,27 +122,32 @@ nbp_rgstr( sat, obj, type, zone )
     timeout.tv_sec = 2;
     timeout.tv_usec = 0;
     if (( cc = select( s + 1, &readfd, 0, 0, &timeout )) < 0 ) {
-	return( -1 );
+        goto register_err;
     }
     if ( cc == 0 ) {
 	errno = ETIMEDOUT;
-	return( -1 );
+	goto register_err;
     }
 
     namelen = sizeof( struct sockaddr_at );
-    if (( cc = recvfrom( s, nbp_recv, sizeof( nbp_recv ), 0, &to,
-	    &namelen )) < 0 ) {
-	return( -1 );
+    if (( cc = netddp_recvfrom( s, nbp_recv, sizeof( nbp_recv ), 0,
+			(struct sockaddr *)&to, &namelen )) < 0 ) {
+        goto register_err;
     }
-    close( s );
+
+    netddp_close( s );
 
     data = nbp_recv;
     if ( *data++ != DDPTYPE_NBP ) {
 	return( -1 );
     }
-    bcopy( data, &nh, SZ_NBPHDR );
+    memcpy( &nh, data, SZ_NBPHDR );
     if ( nh.nh_op != NBPOP_OK ) {
-	return( -1 );
+        return -1;
     }
     return( 0 );
+
+register_err:
+    netddp_close(s);
+    return -1;
 }
