@@ -1,54 +1,48 @@
 /*
- * Copyright (c) 1990,1991 Regents of The University of Michigan.
- * All Rights Reserved.
+ * $Id: nbp_util.c,v 1.3.2.1 2002/02/08 00:04:49 srittau Exp $
  *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appears in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of The University
- * of Michigan not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior
- * permission. This software is supplied as is without expressed or
- * implied warranties of any kind.
- *
- *	Research Systems Unix Group
- *	The University of Michigan
- *	c/o Mike Clark
- *	535 W. William Street
- *	Ann Arbor, Michigan
- *	+1-313-763-0525
- *	netatalk@itd.umich.edu
+ * Copyright (c) 1990,1997 Regents of The University of Michigan.
+ * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <string.h>
+#include <signal.h>
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <sys/signal.h>
 #include <sys/time.h>
+
 #include <netatalk/endian.h>
 #include <netatalk/at.h>
+
 #include <atalk/nbp.h>
 #include <atalk/ddp.h>
+#include <atalk/util.h>
 
-#include <sys/syslog.h>
-
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
-#include <strings.h>
+#endif /* HAVE_NETDB_H */
+
 #include  "nbp_conf.h"
 
 char		nbp_send[ 1024 ];
 char		nbp_recv[ 1024 ];
 u_char		nbp_port = 0;
+unsigned char   nbp_id = 0;
 
-nbp_parse( data, nn, len )
+int nbp_parse( data, nn, len )
     char		*data;
     struct nbpnve	*nn;
     int			len;
 {
     struct nbptuple	nt;
 
-    bcopy( data, &nt, SZ_NBPTUPLE);
+    memcpy( &nt, data, SZ_NBPTUPLE);
     data += SZ_NBPTUPLE;
     len -= SZ_NBPTUPLE;
     if ( len < 0 ) {
@@ -57,7 +51,7 @@ nbp_parse( data, nn, len )
 
 #ifdef BSD4_4
     nn->nn_sat.sat_len = sizeof( struct sockaddr_at );
-#endif BSD4_4
+#endif /* BSD4_4 */
     nn->nn_sat.sat_family = AF_APPLETALK;
     nn->nn_sat.sat_addr.s_net = nt.nt_net;
     nn->nn_sat.sat_addr.s_node = nt.nt_node;
@@ -71,7 +65,7 @@ nbp_parse( data, nn, len )
     if ( nn->nn_objlen > NBPSTRLEN ) {
 	return( -1 );
     }
-    bcopy( data, nn->nn_obj, nn->nn_objlen );
+    memcpy( nn->nn_obj, data, nn->nn_objlen );
     data += nn->nn_objlen;
 
     nn->nn_typelen = *data++;
@@ -82,7 +76,7 @@ nbp_parse( data, nn, len )
     if ( nn->nn_typelen > NBPSTRLEN ) {
 	return( 1 );
     }
-    bcopy( data, nn->nn_type, nn->nn_typelen );
+    memcpy( nn->nn_type, data, nn->nn_typelen );
 
     data += nn->nn_typelen;
     nn->nn_zonelen = *data++;
@@ -93,7 +87,7 @@ nbp_parse( data, nn, len )
     if ( nn->nn_zonelen > NBPSTRLEN ) {
 	return( 1 );
     }
-    bcopy( data, nn->nn_zone, nn->nn_zonelen );
+    memcpy( nn->nn_zone, data, nn->nn_zonelen );
 
     return( len );
 }
@@ -102,7 +96,7 @@ nbp_parse( data, nn, len )
 #define NBPM_TYPE	(1<<2)
 #define NBPM_ZONE	(1<<3)
 
-nbp_match( n1, n2, flags )
+int nbp_match( n1, n2, flags )
     struct nbpnve	*n1, *n2;
     int			flags;
 {
@@ -143,8 +137,9 @@ nbp_match( n1, n2, flags )
     return( 1 );
 }
 
-nbp_name( name, objp, typep, zonep )
-    char	*name, **objp, **typep, **zonep;
+int nbp_name( name, objp, typep, zonep )
+    const char  *name;
+    char	**objp, **typep, **zonep;
 {
     static char	buf[ 32 + 1 + 32 + 1 + 32 + 1 ];
     char	*p;
@@ -155,11 +150,11 @@ nbp_name( name, objp, typep, zonep )
 	}
 	strcpy( buf, name );
 
-	if (( p = rindex( buf, '@' )) != NULL ) {
+	if (( p = strrchr( buf, '@' )) != NULL ) {
 	    *p++ = '\0';
 	    *zonep = p;
 	}
-	if (( p = index( buf, ':' )) != NULL ) {
+	if (( p = strrchr( buf, ':' )) != NULL ) {
 	    *p++ = '\0';
 	    *typep = p;
 	}
