@@ -1,31 +1,33 @@
 /*
+ * $Id: at.h,v 1.2.2.1 2001/11/25 21:38:55 srittau Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
- * All Rights Reserved.
  *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appears in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of The University
- * of Michigan not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior
- * permission. This software is supplied as is without expressed or
- * implied warranties of any kind.
- *
- *	Research Systems Unix Group
- *	The University of Michigan
- *	c/o Mike Clark
- *	535 W. William Street
- *	Ann Arbor, Michigan
- *	+1-313-763-0525
- *	netatalk@itd.umich.edu
+ * All Rights Reserved.  See COPYRIGHT.
  */
 
 #ifndef __AT_HEADER__
 #define __AT_HEADER__
+
+#if defined(linux) /* pull in the linux header */
+#include <sys/socket.h>
+#include <asm/types.h>
+#include <linux/atalk.h>
+#else /* linux */
+
+#include <sys/types.h>
+#include <netinet/in.h> /* so that we can deal with sun's s_net #define */
+
+#ifdef MACOSX_SERVER
+#include <netat/appletalk.h>
+#endif /* MACOSX_SERVER */
+
 /*
  * Supported protocols
  */
+#ifdef ATPROTO_DDP
+#undef ATPROTO_DDP
+#endif /* ATPROTO_DDP */
 #define ATPROTO_DDP	0
 #define ATPROTO_AARP	254
 
@@ -48,21 +50,22 @@
  */
 #define ATPORT_FIRST	1
 #define ATPORT_RESERVED	128
-#define ATPORT_LAST	255
+#define ATPORT_LAST	254 /* 254 is reserved on ether/tokentalk networks */
 
 /*
  * AppleTalk address.
  */
+#ifndef MACOSX_SERVER
 struct at_addr {
+#ifdef s_net
+#undef s_net
+#endif /* s_net */
     u_short	s_net;
     u_char	s_node;
 };
+#endif /* MACOSX_SERVER */
 
-#ifdef BSD4_4
-#define ATADDR_ANYNET	(u_short)0xffff
-#else
 #define ATADDR_ANYNET	(u_short)0x0000
-#endif
 #define ATADDR_ANYNODE	(u_char)0x00
 #define ATADDR_ANYPORT	(u_char)0x00
 #define ATADDR_BCAST	(u_char)0xff		/* There is no BCAST for NET */
@@ -74,13 +77,14 @@ struct at_addr {
  * interface.  IFACE may be filled in by the client, and is filled in
  * by the kernel.
  */
+#ifndef MACOSX_SERVER
 struct sockaddr_at {
 #ifdef BSD4_4
     u_char		sat_len;
     u_char		sat_family;
-#else BSD4_4
+#else /* BSD4_4 */
     short		sat_family;
-#endif BSD4_4
+#endif /* BSD4_4 */
     u_char		sat_port;
     struct at_addr	sat_addr;
 #ifdef notdef
@@ -98,10 +102,11 @@ struct sockaddr_at {
 	    u_short		su_interface;
 	} sh_un;
     } sat_hints;
-#else notdef
+#else /* notdef */
     char		sat_zero[ 8 ];
-#endif notdef
+#endif /* notdef */
 };
+#endif /* MACOSX_SERVER */
 
 struct netrange {
     u_char		nr_phase;
@@ -112,6 +117,7 @@ struct netrange {
 #ifdef KERNEL
 extern struct domain	atalkdomain;
 extern struct protosw	atalksw[];
-#endif
+#endif /* KERNEL */
 
-#endif __AT_HEADER__
+#endif /* linux */
+#endif /* __AT_HEADER__ */
