@@ -1,19 +1,25 @@
 /*
+ * $Id: ddp_input.c,v 1.2.2.1 2002/03/12 16:24:50 srittau Exp $
+ *
  * Copyright (c) 1990,1994 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/mbuf.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
-#include <sys/syslog.h>
+#include <syslog.h>
 #include <net/if.h>
 #include <net/route.h>
 #ifdef _IBMR2
 #include <net/spl.h>
-#endif _IBMR2
+#endif /* _IBMR2 */
 
 #include "at.h"
 #include "at_var.h"
@@ -40,17 +46,17 @@ atintr()
     for (;;) {
 #ifndef _IBMR2
 	s = splimp();
-#endif _IBMR2
+#endif /* ! _IBMR2 */
 
 #ifdef BSD4_4
 	IF_DEQUEUE( &atintrq2, m );
-#else BSD4_4
+#else /* BSD4_4 */
 	IF_DEQUEUEIF( &atintrq2, m, ifp );
-#endif BSD4_4
+#endif /* BSD4_4 */
 
 #ifndef _IBMR2
 	splx( s );
-#endif _IBMR2
+#endif /* ! _IBMR2 */
 
 	if ( m == 0 ) {			/* no more queued packets */
 	    break;
@@ -58,7 +64,7 @@ atintr()
 
 #ifdef BSD4_4
 	ifp = m->m_pkthdr.rcvif;
-#endif BSD4_4
+#endif /* BSD4_4 */
 	for ( aa = at_ifaddr; aa; aa = aa->aa_next ) {
 	    if ( aa->aa_ifp == ifp && ( aa->aa_flags & AFA_PHASE2 )) {
 		break;
@@ -75,17 +81,17 @@ atintr()
     for (;;) {
 #ifndef _IBMR2
 	s = splimp();
-#endif _IBMR2
+#endif /* ! _IBMR2 */
 
 #ifdef BSD4_4
 	IF_DEQUEUE( &atintrq1, m );
-#else BSD4_4
+#else /* BSD4_4 */
 	IF_DEQUEUEIF( &atintrq1, m, ifp );
-#endif BSD4_4
+#endif /* BSD4_4 */
 
 #ifndef _IBMR2
 	splx( s );
-#endif _IBMR2
+#endif /* ! _IBMR2 */
 
 	if ( m == 0 ) {			/* no more queued packets */
 	    break;
@@ -93,7 +99,7 @@ atintr()
 
 #ifdef BSD4_4
 	ifp = m->m_pkthdr.rcvif;
-#endif BSD4_4
+#endif /* BSD4_4 */
 	for ( aa = at_ifaddr; aa; aa = aa->aa_next ) {
 	    if ( aa->aa_ifp == ifp && ( aa->aa_flags & AFA_PHASE2 ) == 0 ) {
 		break;
@@ -137,7 +143,7 @@ ddp_input( m, ifp, elh, phase )
     struct ddpehdr	*deh, ddpe;
 #ifndef BSD4_4
     struct mbuf		*mp;
-#endif BSD4_4
+#endif /* BSD4_4 */
     struct ddpcb	*ddp;
     int			dlen, mlen;
     u_short		cksum;
@@ -223,8 +229,8 @@ ddp_input( m, ifp, elh, phase )
 		}
 		if (( ntohs( to.sat_addr.s_net ) < ntohs( aa->aa_firstnet ) ||
 			ntohs( to.sat_addr.s_net ) > ntohs( aa->aa_lastnet )) &&
-			( ntohs( to.sat_addr.s_net ) < ntohs( 0xff00 ) ||
-			ntohs( to.sat_addr.s_net ) > ntohs( 0xfffe ))) {
+			( ntohs( to.sat_addr.s_net ) < 0xff00 ||
+			ntohs( to.sat_addr.s_net ) > 0xfffe)) {
 		    continue;
 		}
 		if ( to.sat_addr.s_node != AA_SAT( aa )->sat_addr.s_node &&
@@ -243,11 +249,11 @@ ddp_input( m, ifp, elh, phase )
      */
 #ifdef BSD4_4
     mlen = m->m_pkthdr.len;
-#else BSD4_4
+#else /* BSD4_4 */
     for ( mlen = 0, mp = m; mp; mp = mp->m_next ) {
 	mlen += mp->m_len;
     }
-#endif BSD4_4
+#endif /* BSD4_4 */
     if ( mlen < dlen ) {
 	ddpstat.ddps_toosmall++;
 	m_freem( m );
@@ -273,16 +279,16 @@ ddp_input( m, ifp, elh, phase )
 		to.sat_addr.s_node )) {
 #ifdef ultrix
 	    rtfree( forwro.ro_rt );
-#else ultrix
+#else /* ultrix */
 	    RTFREE( forwro.ro_rt );
-#endif ultrix
+#endif /* ultrix */
 	    forwro.ro_rt = (struct rtentry *)0;
 	}
 	if ( forwro.ro_rt == (struct rtentry *)0 ||
 	     forwro.ro_rt->rt_ifp == (struct ifnet *)0 ) {
 #ifdef BSD4_4
 	    forwro.ro_dst.sa_len = sizeof( struct sockaddr_at );
-#endif BSD4_4
+#endif /* BSD4_4 */
 	    forwro.ro_dst.sa_family = AF_APPLETALK;
 	    satosat( &forwro.ro_dst )->sat_addr.s_net = to.sat_addr.s_net;
 	    satosat( &forwro.ro_dst )->sat_addr.s_node = to.sat_addr.s_node;
@@ -296,7 +302,8 @@ ddp_input( m, ifp, elh, phase )
 	}
 
 	if ( ddp_firewall &&
-		( forwro.ro_rt == NULL || forwro.ro_rt->rt_ifp != ifp )) {
+		( forwro.ro_rt == NULL || ( forwro.ro_rt->rt_ifp != ifp &&
+		forwro.ro_rt->rt_ifp != at_ifaddr->aa_ifp ))) {
 	    m_freem( m );
 	    return;
 	}
@@ -314,7 +321,7 @@ ddp_input( m, ifp, elh, phase )
 
 #ifdef BSD4_4
     from.sat_len = sizeof( struct sockaddr_at );
-#endif BSD4_4
+#endif /* BSD4_4 */
     from.sat_family = AF_APPLETALK;
 
     if ( elh ) {
