@@ -1,32 +1,20 @@
 /*
+ * $Id: ddp_output.c,v 1.2.2.1 2002/03/12 16:24:50 srittau Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
- * All Rights Reserved.
- *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appears in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of The University
- * of Michigan not be used in advertising or publicity pertaining to
- * distribution of the software without specific, written prior
- * permission. This software is supplied as is without expressed or
- * implied warranties of any kind.
- *
- *	Research Systems Unix Group
- *	The University of Michigan
- *	c/o Mike Clark
- *	535 W. William Street
- *	Ann Arbor, Michigan
- *	+1-313-763-0525
- *	netatalk@itd.umich.edu
+ * All Rights Reserved. See COPYRIGHT.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/mbuf.h>
 #include <sys/socket.h>
 #include <sys/errno.h>
-#include <sys/syslog.h>
+#include <syslog.h>
 
 #include <net/if.h>
 #include <net/route.h>
@@ -51,7 +39,7 @@ ddp_output( ddp, m )
 #ifndef BSD4_4
     struct mbuf		*m0;
     int			len;
-#endif BSD4_4
+#endif /* ! BSD4_4 */
     struct ifnet	*ifp;
     struct at_ifaddr	*aa = NULL;
     struct ddpehdr	*deh;
@@ -59,7 +47,7 @@ ddp_output( ddp, m )
 
 #ifdef BSD4_4
     M_PREPEND( m, sizeof( struct ddpehdr ), M_WAIT );
-#else BSD4_4
+#else /* BSD4_4 */
     for ( len = 0, m0 = m; m; m = m->m_next ) {
 	len += m->m_len;
     }
@@ -69,13 +57,13 @@ ddp_output( ddp, m )
 	return( ENOBUFS );
     }
     m->m_next = m0;
-#endif BSD4_4
+#endif /* BSD4_4 */
 
 #ifndef BSD4_4
 # define align(a)	(((a)+3)&0xfc)
     m->m_off = MMINOFF + align( SZ_ELAPHDR );
     m->m_len = sizeof( struct ddpehdr );
-#endif BSD4_4
+#endif /* ! BSD4_4 */
 
     deh = mtod( m, struct ddpehdr *);
     deh->deh_pad = 0;
@@ -83,9 +71,9 @@ ddp_output( ddp, m )
 
 #ifdef BSD4_4
     deh->deh_len = m->m_pkthdr.len;
-#else BSD4_4
+#else /* BSD4_4 */
     deh->deh_len = len + sizeof( struct ddpehdr );
-#endif BSD4_4
+#endif /* BSD4_4 */
 
     deh->deh_dnet = ddp->ddp_fsat.sat_addr.s_net;
     deh->deh_dnode = ddp->ddp_fsat.sat_addr.s_node;
@@ -114,7 +102,7 @@ at_cksum( m, skip )
     int		skip;
 {
     u_char	*data, *end;
-    u_long	cksum = 0;
+    u_int32_t	cksum = 0;
 
     for (; m; m = m->m_next ) {
 	for ( data = mtod( m, u_char * ), end = data + m->m_len; data < end;
@@ -152,9 +140,9 @@ ddp_route( m, ro )
     if ( ro->ro_rt && ( ifp = ro->ro_rt->rt_ifp )) {
 #ifdef BSD4_4
 	net = satosat( ro->ro_rt->rt_gateway )->sat_addr.s_net;
-#else BSD4_4
+#else /* BSD4_4 */
 	net = satosat( &ro->ro_rt->rt_gateway )->sat_addr.s_net;
-#endif BSD4_4
+#endif /* BSD4_4 */
 	for ( aa = at_ifaddr; aa; aa = aa->aa_next ) {
 	    if ( aa->aa_ifp == ifp &&
 		    ntohs( net ) >= ntohs( aa->aa_firstnet ) &&
@@ -164,7 +152,6 @@ ddp_route( m, ro )
 	}
     }
     if ( aa == NULL ) {
-printf( "ddp_route: oops\n" );
 	m_freem( m );
 	return( EINVAL );
     }
@@ -189,11 +176,11 @@ printf( "ddp_route: oops\n" );
 	if ( m == NULL ) {
 	    return( ENOBUFS );
 	}
-#else BSD4_4
+#else /* BSD4_4 */
 	m->m_off -= SZ_ELAPHDR;
 	m->m_len += SZ_ELAPHDR;
-#endif BSD4_4
-# endif notdef
+#endif /* BSD4_4 */
+# endif /* notdef */
 
 	MGET( m0, M_WAIT, MT_HEADER );
 	if ( m0 == 0 ) {
@@ -216,9 +203,9 @@ printf( "ddp_route: oops\n" );
 	} else {
 #ifdef BSD4_4
 	    elh->el_dnode = satosat( ro->ro_rt->rt_gateway )->sat_addr.s_node;
-#else BSD4_4
+#else /* BSD4_4 */
 	    elh->el_dnode = satosat( &ro->ro_rt->rt_gateway )->sat_addr.s_node;
-#endif BSD4_4
+#endif /* BSD4_4 */
 	}
     }
 
@@ -230,9 +217,9 @@ printf( "ddp_route: oops\n" );
     } else {
 #ifdef BSD4_4
 	gate = *satosat( ro->ro_rt->rt_gateway );
-#else BSD4_4
+#else /* BSD4_4 */
 	gate = *satosat( &ro->ro_rt->rt_gateway );
-#endif BSD4_4
+#endif /* BSD4_4 */
     }
     ro->ro_rt->rt_use++;
 
@@ -249,11 +236,11 @@ printf( "ddp_route: oops\n" );
 #ifdef notdef
 	m->m_off += SZ_ELAPHDR;
 	m->m_len -= SZ_ELAPHDR;
-#endif notdef
+#endif /* notdef */
 	ddp_input( m, ifp, (struct elaphdr *)NULL, 2 );
 	return( 0 );
     }
-#endif ultrix
+#endif /* ultrix */
 
 #ifdef _IBMR2
     /*
@@ -275,6 +262,6 @@ printf( "ddp_route: oops\n" );
 	bcopy( &eh, (*(struct sockaddr *)&gate).sa_data,
 		sizeof( (*(struct sockaddr *)&gate).sa_data ));
     }
-#endif _IBMR2
+#endif /* _IBMR2 */
     return((*ifp->if_output)( ifp, m, &gate ));
 }
